//=============================================================================
// JPT_BattleEvent.js
//=============================================================================

var Imported = Imported || {};
Imported['JPT_BattleEvent'] = 1.00;

/*:ja
 * @plugindesc バトル中のイベントなどをカスタマイズします
 * @author Kawamura
 *
 * @param Damage From Enemy Switch Start ID
 * @desc 敵からダメージを受けたかどうかのスイッチの連番の開始IDを指定してください
 * バトルパーティ最大人数は4人です
 * ID指定 0の場合利用しません
 * @default 0
 *
 * @param Target From Enemy Switch Start ID
 * @desc 敵からターゲットされたかどうかのスイッチの連番の開始IDを指定してください
 * バトルパーティ最大人数は4人です
 * ID指定 0の場合利用しません
 * @default 0
 *
 * @param BattleVictoryCommonEventID
 * @desc バトル勝利時に呼ばれるコモンイベント
 * 0でなし
 * @default 0
 *
 * @help
 * バトル中のイベントなどをカスタマイズします
 *
 *=================================================================================
 * スイッチ
 *=================================================================================
 * バトル中の行動や結果に対してスイッチを切り替えます
 *
 * ターン開始時にすべてOFFになります
 *
 * ・ダメージ
 * 敵からダメージを受けたらスイッチONにします
 * バトルメンバー分の連番の開始IDを指定してください
 * バトルメンバー最大人数は4人です
 *
 *・ターゲット
 * 敵からターゲットされたらスイッチONにします
 * あくまでターゲットされたかどうかなので何回されたとか誰にとかは判定しません
 * バトルメンバー分の連番の開始IDを指定してください
 * バトルメンバー最大人数は4人です
 *
 *
 *=================================================================================
 * バトル勝利コモンイベント
 *=================================================================================
 *
 * バトル勝利時に実行するコモンイベントのIDを
 * BattleVictoryCommonEventIDに指定します
 * 0で実行なし
 *
 */

/*
 *
 */
(function() {

	'use strict';

	var parameters = PluginManager.parameters('JPT_BattleEvent');
	var damageSwitchID = parseInt(parameters['Damage From Enemy Switch Start ID'] || 0);
	var targetSwitchID = parseInt(parameters['Target From Enemy Switch Start ID'] || 0);
	var bv_eID = parseInt(parameters['BattleVictoryCommonEventID'] || 0);

	////////////////////////////////////////////////////////////////////////////////////
	// BattleManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_BattleEvent_BattleManager_initMembers = BattleManager.initMembers;
	BattleManager.initMembers = function() {
		_JPT_BattleEvent_BattleManager_initMembers.call(this);
	    this._checkBattleEvent = false;
	};

	BattleManager.switchInit = function() {
		for(var i=0; i<$gameParty.maxBattleMembers(); i++){
			if(damageSwitchID > 0)$gameSwitches.setValue( damageSwitchID + i, false );
			if(targetSwitchID > 0)$gameSwitches.setValue( targetSwitchID + i, false );
		}
	}

	BattleManager.damageSwitchOn = function(member_id) {
		if(damageSwitchID > 0)$gameSwitches.setValue( damageSwitchID + member_id, true );
	}

	BattleManager.targetSwitchOn = function(member_id) {
		if(targetSwitchID > 0)$gameSwitches.setValue( targetSwitchID + member_id, true );
	}


	var _JPT_BattleEvent_BattleManager_startInput = BattleManager.startInput;
	BattleManager.startInput = function() {
		_JPT_BattleEvent_BattleManager_startInput.call(this);
		this.switchInit();
	}

	var _JPT_BattleEvent_BattleManager_updateBattleEnd = BattleManager.updateBattleEnd;
	BattleManager.updateBattleEnd = function() {
		this.switchInit();
		_JPT_BattleEvent_BattleManager_updateBattleEnd.call(this);
	}


	var _JPT_BattleEvent_BattleManager_checkBattleEnd = BattleManager.checkBattleEnd;
	BattleManager.checkBattleEnd = function() {

		if (this._phase) {
	        if (this._checkBattleEvent) {
	        	_JPT_BattleEvent_BattleManager_processVictory.call(this);
	        	return true;
	        }
		}

	    return _JPT_BattleEvent_BattleManager_checkBattleEnd.call(this);
	};


	var _JPT_BattleEvent_BattleManager_processVictory = BattleManager.processVictory;
	BattleManager.processVictory = function() {
		if(!this._checkBattleEvent){
			if ($gameTroop.victoryCommonEvent()) {
	            this._checkBattleEvent = true;
	            return ;
	        }
		}
		_JPT_BattleEvent_BattleManager_processVictory.call(this);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Troop
	////////////////////////////////////////////////////////////////////////////////////
	Game_Troop.prototype.victoryCommonEvent = function() {
        if(bv_eID){
	        var event = $dataCommonEvents[bv_eID];
	        this._interpreter.setup(event.list, this.troop().id);
	        return true;
        }
	    return false;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Action
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_BattleEvent_Game_Action_apply = Game_Action.prototype.apply;
	Game_Action.prototype.apply = function(target) {
		 _JPT_BattleEvent_Game_Action_apply.call(this,target);
		 if (target.isActor() && this.subject().isEnemy()) {
			 if (this.isDamage() || this.isDrain()) {
				 BattleManager.damageSwitchOn(target.index());
			 }
		 }
	};

	var _JPT_BattleEvent_Game_Action_makeTargets = Game_Action.prototype.makeTargets;
	Game_Action.prototype.makeTargets = function() {
		var targets = _JPT_BattleEvent_Game_Action_makeTargets.call(this);

		if(targets){
			for(var i=0; i<targets.length; i++){
				 if (targets[i].isActor() && this.subject().isEnemy()) {
					 BattleManager.targetSwitchOn(targets[i].index());
				 }
			}
		}

		return targets;
	};

}() );
