//=============================================================================
// JPT_BattleUI
//=============================================================================

var Imported = Imported || {};
Imported['JPT_BattleUI'] = 1.00;

/*:ja
 * @plugindesc カスタムバトルUI
 * @author Kawamura
 *
 * @help
 *
 * 依存関係
 * JPT_MainMenu.js
 * YEP_BattleEngineCore.js
 * HIME_ActorBattleCommands.js
 * HIME_BattleCommandUseSkill.js
 * ChangeEquipBattle.js
 *
 *
 * <battle_face>
 * vid=12
 * images=battle_chara_006,battle_chara_012
 * </battle_face>
 *
 */



(function() {


	'use strict';

	var parameters = PluginManager.parameters('JPT_BattleUI');

	var eventTextMode = 0;

	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_BattleUI_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {
		if (!_JPT_BattleUI_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processActorNotetagsBattle($dataActors);
		return true;
	};


	//メモからデータ取得
	DataManager.processActorNotetagsBattle = function(group) {

		var noteBattleFace = /<battle_face:(.+)>/i;

		var noteBattleFaceStart = /<battle_face>/i;
		var noteBattleFaceEnd = /<\/battle_face>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			var mode = 'none';

			obj.battle_face = '';
			obj.battle_face_array = [];
			obj.battle_face_vid = 0;

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if(line.match(noteBattleFace)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.battle_face = tagStr.trim();
				}else if(line.match(noteBattleFaceStart)){
					mode = 'battle_face';
				}else if(line.match(noteBattleFaceEnd)){
					mode = 'end';
				}else if(mode == 'battle_face'){
					var tag = line.split(/=/)[0];
					var val = line.split(/=/)[1];

					switch(tag){
					case 'vid':
						obj.battle_face_vid = parseInt(val,10);
						break;
					case 'images':
						obj.battle_face_array = val.split(/,/);
						break;
					}
				}
			}
		}
	};



	////////////////////////////////////////////////////////////////////////////////////
	// BattleManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_BattleUI_BattleManager_setup = BattleManager.setup;
	BattleManager.setup = function(troopId, canEscape, canLose) {
		_JPT_BattleUI_BattleManager_setup.call(this,troopId, canEscape, canLose);
		$gameMessage.setPositionType(0);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Interpreter
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_BattleUI_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
    	_JPT_BattleUI_Game_Interpreter_pluginCommand.call(this, command, args);

    	if(command==='BattleEventMessageStart'){
    		eventTextMode=1;
    	}else if(command==='BattleEventMessageEnd'){
    		eventTextMode=2;
    	}
    };

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Battle
	////////////////////////////////////////////////////////////////////////////////////

	Scene_Battle.prototype.updateWindowPositions = function() {
		this._statusWindow.x = -400;
		this._statusWindow.y = -400;

		if(this._partyCommandWindow.isClosing() && this._actorCommandWindow.isOpening()){
			this._actorCommandWindow.hide();
			this._actorCommandWindow.openness_stop = true;
		}else if(this._actorCommandWindow.isOpening()){
			this._actorCommandWindow.show();
			this._actorCommandWindow.openness_stop = false;
		}else{
			this._actorCommandWindow.openness_stop = false;
		}

		if(this._partyCommandWindow.isOpening() && this._actorCommandWindow.isClosing()){
			this._partyCommandWindow.hide();
			this._partyCommandWindow.openness_stop = true;
		}else if(this._partyCommandWindow.isOpening()){
			this._partyCommandWindow.show();
			this._partyCommandWindow.openness_stop = false;
		}else{
			this._partyCommandWindow.openness_stop = false;
		}

	}

	var _JPT_BattleUI_Scene_Battle_createSpriteset = Scene_Battle.prototype.createSpriteset;
	Scene_Battle.prototype.createSpriteset = function() {
		_JPT_BattleUI_Scene_Battle_createSpriteset.call(this);
	    this.createSpriteSkillWindow();
	    this.createSpriteEquipWindow();
	};

	Scene_Battle.prototype.createSpriteSkillWindow = function() {
	    this._sprSkillWin = new Sprite_SkillWindowBase();
	    this.addChild(this._sprSkillWin);
	    this._sprSkillWin.hide();
	};

	Scene_Battle.prototype.createSpriteEquipWindow = function() {
	    this._sprEquipWin = new Sprite_EquipWindowBase();
	    this.addChild(this._sprEquipWin);
	    this._sprEquipWin.hide();
	};

	Scene_Battle.prototype.createStatusWindow = function() {
	    this._statusWindow = new Window_BattleStatus();
	    this.addWindow(this._statusWindow);
	    this._statusWindow.x = -400;
	    this._statusWindow.y = -400;
	};



	var _JPT_BattleUI_Scene_Battle_commandAttack = Scene_Battle.prototype.commandAttack;
	Scene_Battle.prototype.commandAttack = function() {
		_JPT_BattleUI_Scene_Battle_commandAttack.call(this);
		this._partyCommandWindow.close();
	    this._actorCommandWindow.close();
	};

	var _JPT_BattleUI_Scene_Battle_createActorWindow = Scene_Battle.prototype.createActorWindow;
	Scene_Battle.prototype.createActorWindow = function() {
		_JPT_BattleUI_Scene_Battle_createActorWindow.call(this);
	    this._actorWindow.setSpriteset(this._spriteset);
	};

	Scene_Battle.prototype.createMessageWindow = function() {
		eventTextMode=2;
	    this._messageWindow = new Window_BattleMessage();
	    this.addWindow(this._messageWindow);
	    this._messageWindow.subWindows().forEach(function(window) {
	        this.addWindow(window);
	    }, this);
	};

	Scene_Battle.prototype.createHelpWindow = function() {
	    this._helpWindow = new Window_BattleHelp();
	    this._helpWindow.visible = false;
	    this.addWindow(this._helpWindow);
	    this._helpWindow.setLogWindow(this._logWindow);
	};

	Scene_Battle.prototype.createSkillWindow = function() {
	    this._skillWindow = new Window_BattleSkill(0, 70, Graphics.boxWidth, 190);
	    this._skillWindow.setHelpWindow(this._helpWindow);
	    this._skillWindow.setBackWindow(this._sprSkillWin);
	    this._skillWindow.setHandler('ok',     this.onSkillOk.bind(this));
	    this._skillWindow.setHandler('cancel', this.onSkillCancel.bind(this));
	    this.addWindow(this._skillWindow);
	};

	Scene_Battle.prototype.createItemWindow = function() {
	    this._itemWindow = new Window_BattleItem(0, 70, Graphics.boxWidth, 190);
	    this._itemWindow.setHelpWindow(this._helpWindow);
	    this._itemWindow.setBackWindow(this._sprSkillWin);
	    this._itemWindow.setHandler('ok',     this.onItemOk.bind(this));
	    this._itemWindow.setHandler('cancel', this.onItemCancel.bind(this));
	    this.addWindow(this._itemWindow);
	};

    Scene_Battle.prototype.createEquipSlotWindow = function() {
        this._eSlotWindow = new Window_BattleEquipSlot(0, 70, Graphics.boxWidth, 40);
        this._eSlotWindow.setHelpWindow(this._helpWindow);
        this._eSlotWindow.setStatusWindow(this._eStatusWindow);
        this._eSlotWindow.setBackWindow(this._sprEquipWin);
        this._eSlotWindow.setHandler('ok',     this.onSlotOk.bind(this));
        this._eSlotWindow.setHandler('cancel',   this.onSlotCancel.bind(this));
        this._eSlotWindow.setHandler('release',  this.onSlotRelease.bind(this));
        this.addWindow(this._eSlotWindow);
        this._eSlotWindow.deactivate();
        this._eSlotWindow.hide();
    };

    Scene_Battle.prototype.createEquipStatusWindow = function() {
    	var wy = this._eSlotWindow.y + this._eSlotWindow.height;
        this._eStatusWindow = new Window_BattleEquipStatus(0, wy);
        this._eStatusWindow._pageIndex = 0;
        this.addWindow(this._eStatusWindow);
        this._eStatusWindow.hide();
        this._eStatusWindow._windowFrameSprite.alpha = 0;
	    this._eStatusWindow._windowBackSprite.alpha = 0;
    };

    Scene_Battle.prototype.createEquipItemWindow = function() {
        var wy = this._eStatusWindow.y + this._eStatusWindow.height;
        var wh = Graphics.boxHeight - wy;
        this._eItemWindow = new Window_BattleEquipItem(0, wy, Graphics.boxWidth, 170);
        this._eItemWindow.setHelpWindow(this._helpWindow);
        this._eItemWindow.setStatusWindow(this._eStatusWindow);
        this._eItemWindow.setHandler('ok',   this.onEItemOk.bind(this));
        this._eItemWindow.setHandler('cancel', this.onEItemCancel.bind(this));
        this._eItemWindow.setHandler('right',  this.eNextPage.bind(this));
        this._eItemWindow.setHandler('left',   this.ePrevPage.bind(this));
        this._eSlotWindow.setItemWindow(this._eItemWindow);
        this.addWindow(this._eItemWindow);
        this._eItemWindow.hide();
        this._eItemWindow._windowFrameSprite.alpha = 0;
	    this._eItemWindow._windowBackSprite.alpha = 0;
    };


	var _JPT_BattleUI_Scene_Battle_onSelectAction = Scene_Battle.prototype.onSelectAction;
	Scene_Battle.prototype.onSelectAction = function() {
		this._partyCommandWindow.close();
		var action = BattleManager.inputtingAction();
		if (!action.needsSelection()) {
		} else if (action.isForOpponent()) {
			this._actorCommandWindow.close();
		} else {
			this._actorCommandWindow.close();
		}
		_JPT_BattleUI_Scene_Battle_onSelectAction.call(this);
	};

	var _JPT_BattleUI_Scene_Battle_onEnemyCancel = Scene_Battle.prototype.onEnemyCancel;
	Scene_Battle.prototype.onEnemyCancel = function() {
		_JPT_BattleUI_Scene_Battle_onEnemyCancel.call(this);
	    switch (this._actorCommandWindow.currentSymbol()) {
	    case 'attack':
	        this._actorCommandWindow.open();
	        break;
	    case 'skill_type':
	        this._skillWindow.show();
	        this._skillWindow.activate();
	        break;
	    }
	};

	var _JPT_BattleUI_Scene_Battle_onActorCancel = Scene_Battle.prototype.onActorCancel;
	Scene_Battle.prototype.onActorCancel = function() {
		_JPT_BattleUI_Scene_Battle_onActorCancel.call(this);
	    switch (this._actorCommandWindow.currentSymbol()) {
	    case 'skill_type':
	        this._skillWindow.show();
	        this._skillWindow.activate();
	        break;
	    }
	};

    var _JPT_BattleUI_Scene_Battle_onSkillCancel = Scene_Battle.prototype.onSkillCancel;
    Scene_Battle.prototype.onSkillCancel = function() {
    	_JPT_BattleUI_Scene_Battle_onSkillCancel.call(this);
        this._actorCommandWindow.open();
    };


    var _JPT_BattleUI_Scene_Battle_onItemCancel = Scene_Battle.prototype.onItemCancel;
    Scene_Battle.prototype.onItemCancel = function() {
    	_JPT_BattleUI_Scene_Battle_onItemCancel.call(this);
        this._actorCommandWindow.open();
    };

	var _JPT_BattleUI_Scene_Battle_refreshStatus = Scene_Battle.prototype.refreshStatus;
	Scene_Battle.prototype.refreshStatus = function() {
		_JPT_BattleUI_Scene_Battle_refreshStatus.call(this);
		this._spriteset.refreshHud();
	};

	var _JPT_BattleUI_Scene_Battle_commandSkill = Scene_Battle.prototype.commandSkill;
	Scene_Battle.prototype.commandSkill = function() {
		_JPT_BattleUI_Scene_Battle_commandSkill.call(this);
	};

	Scene_Battle.prototype.commandItem = function() {
		this._itemWindow.setActor(BattleManager.actor());
	    this._itemWindow.refresh();
	    this._itemWindow.show();
	    this._itemWindow.activate();
	};

	 Scene_Battle.prototype.onEItemOk = function() {

		 var check = true;
			if(this._eItemWindow._slotId==1){
				if(!this._eItemWindow.item()){
					check = false;
				}
			}

        SoundManager.playEquip();
        var actor = BattleManager.actor();
        if(check)actor.changeEquip(this._eSlotWindow.slotIndex(), this._eItemWindow.item());
        this._eSlotWindow.activate();
        this._eSlotWindow.refresh();
        this._eItemWindow.deselect();
        this._eItemWindow.refresh();
        this._eStatusWindow.refresh();
    };


    Scene_Battle.prototype.commandEquip = function() {
        var actor = BattleManager.actor();
        this._eStatusWindow.setActor(actor);
        this._eSlotWindow.setActor(actor);
        this._eItemWindow.setActor(actor);
        this._eStatusWindow.show();
        this._eSlotWindow.show();
        this._eItemWindow.show();
        this._helpWindow.show();
        this._actorCommandWindow.deactivate();
        this._eSlotWindow.activate();
        this._eSlotWindow.select(0);
        this._activeEquip = true;

        this._eStatusWindow.refresh();
        this._eSlotWindow.refresh();
        this._eItemWindow.refresh();
        this._helpWindow.refresh();
    };

    Scene_Battle.prototype.onSlotRelease = function() {
        SoundManager.playEquip();
        var actor = BattleManager.actor();
        actor.changeEquip(this._eSlotWindow.slotIndex(), null);
        this._eSlotWindow.activate();
        this._eSlotWindow.refresh();
        this._eItemWindow.deselect();
        this._eItemWindow.refresh();
        this._eStatusWindow.refresh();
    };

    Scene_Battle.prototype.updateStatusWindow = function() {

    	if(eventTextMode==1){
    		this._messageWindow.setBattleMode(false);
    	}else if(eventTextMode==2){
    		this._messageWindow.setBattleMode(true);
    		$gameMessage.setPositionType(0);
    	}
    	eventTextMode = 0;

        if ($gameMessage.isBusy()) {
            this._statusWindow.close();
            this._partyCommandWindow.close();
            this._actorCommandWindow.close();
        } else {
        	if(!eventTextMode){
	        	if (this.isActive() && !this._messageWindow.isClosing()) {
	        		this._statusWindow.open();
	        	}
        	}else{
        		if (this.isActive() && !this._evMessageWindow.isClosing()) {
	        		this._statusWindow.open();
	        	}
        	}
        }
    };


    var _JPT_BattleUI_Scene_Battle_start = Scene_Battle.prototype.start;
    Scene_Battle.prototype.start = function() {
    	this.toneBuf = null;
    	this.toneBuf = $gameScreen.tone().clone();
    	$gameScreen.clearTone();
    	_JPT_BattleUI_Scene_Battle_start.call(this);
    };

    var _JPT_BattleUI_Scene_Battle_terminate = Scene_Battle.prototype.terminate;
    Scene_Battle.prototype.terminate = function() {
    	$gameScreen.startTint(this.toneBuf,0);
    	this.toneBuf = null;
    	_JPT_BattleUI_Scene_Battle_terminate.call(this);
    };




	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleCommand
	////////////////////////////////////////////////////////////////////////////////////

	function Window_BattleCommand() {
	    this.initialize.apply(this, arguments);
	}

	Window_BattleCommand.prototype = Object.create(Window_Command.prototype);
	Window_BattleCommand.prototype.constructor = Window_BattleCommand;

	Window_BattleCommand.prototype.initialize = function() {

		this.loadImage();
		this.createSprite();
	    this._command_icons = [];
	    this._h_ofs = 30;

	    var y = Graphics.boxHeight/2 - this.windowHeight()*0.8;
	    Window_Command.prototype.initialize.call(this, 0, y);
	    this.openness = 0;
	    this.deactivate();
	    this._windowFrameSprite.visible = false;
	    this._commandWidth = 0;
	    this._commandNum = 1;
	    this._commandTitleIndex = 0;

	    this.openness_stop = true;
	};

	Window_BattleCommand.prototype.windowWidth = function() {
	    return Graphics.boxWidth;
	};

	Window_BattleCommand.prototype.windowHeight = function() {
	    return 280 - this._h_ofs*2;
	};

	Window_BattleCommand.prototype.standardPadding = function() {
	    return 0;
	};

	Window_BattleCommand.prototype.setCommandIcons = function() {
		this._command_icons = [];
	};

	Window_BattleCommand.prototype.commandIcon = function(index) {
		return (this._command_icons.length > index) ? this._command_icons[index] : 0;
	};



	Window_BattleCommand.prototype.updateOpen = function() {
		if(this.openness_stop)return;
		Window_Command.prototype.updateOpen.call(this);
	};

	Window_BattleCommand.prototype.updateClose = function() {
		if(this.openness_stop)return;
		Window_Command.prototype.updateClose.call(this);
	};


	Window_BattleCommand.prototype.open = function() {
		Window_Command.prototype.open.call(this);
		this.openness_stop = true;
	};


	Window_BattleCommand.prototype.loadImage = function() {
		this._bmpPlate = ImageManager.loadBitmap("img/battlehud/", "battle_bg_command");
		this._bmpCommandTitle = ImageManager.loadBitmap("img/battlehud/", "battle_name");
		this._bmpCommandLine = ImageManager.loadBitmap("img/battlehud/", "battle_line");
		this._bmpCommandIcon = ImageManager.loadBitmap("img/battlehud/", "battle_command_icon");
		this._bmpCommandIconDot = ImageManager.loadBitmap("img/battlehud/", "battle_command_dot");
	}

	Window_BattleCommand.prototype.createSprite = function() {
		this._sprTitleSet = new Sprite();
		this._sprCommandArea = new Sprite();
		this._sprSelectCommand = new Sprite();
		this._sprSelectCommand.blendMode = PIXI.BLEND_MODES.SCREEN;
	}

	Window_BattleCommand.prototype.createOtherParts = function() {
		this._windowSpriteContainer.addChild(this._sprTitleSet);
		this._windowSpriteContainer.addChild(this._sprCommandArea);
		this._windowSpriteContainer.addChild(this._sprSelectCommand);
	}

	Window_BattleCommand.prototype.setTitleIndex = function(index) {
		this._commandTitleIndex = index;
	}

	Window_BattleCommand.prototype.setCommandWidth = function(width) {
		this._commandWidth = width;
	}

	Window_BattleCommand.prototype.setCommandNum = function(num) {
		this._commandNum = num;
		if(this._commandNum > 5 ){
			this.setCommandWidth(380 + (this._commandNum-5)*73);
		}else{
			this.setCommandWidth(380);
		}
	}

	Window_BattleCommand.prototype.commandRect = function(index) {
		var rect = new Rectangle();

		rect.y = 144 - this._h_ofs - 15;
		rect.width = 64;
		rect.height = 72;

		if(this._commandNum==2){
			rect.x = this.width/2 - rect.width/2 - rect.width + rect.width*2*index;
		}else{
			if(this._commandNum%2==0){
				rect.x = this.width/2 - Math.floor(this._commandNum/2)*rect.width + rect.width*index;
			}else{
				rect.x = this.width/2 - rect.width/2 - Math.floor(this._commandNum/2)*rect.width + rect.width*index;
			}
		}
		return rect;
	}

	Window_BattleCommand.prototype.update = function() {
		Window_Command.prototype.update.call(this);
		this.updateSelectIcon();
	};

	Window_BattleCommand.prototype._refreshBack = function() {
	    var m = 0;
	    var w = this._width - m * 2;
	    var h = this._height - m * 2;
	    var bitmap = new Bitmap(w, h);

	    this._windowBackSprite.alpha = 1;
	    this._windowBackSprite.bitmap = bitmap;
	    this._windowBackSprite.setFrame(0, 0, w, h);
	    this._windowBackSprite.move(m, m);

	    bitmap.blt(this._bmpPlate, 0, this._h_ofs+10, 8, h, 0, 0, w, h);
	    //bitmap.fillAll('#0000FF');

	};

	Window_BattleCommand.prototype.setup = function() {
	    this._refreshAllParts();
	};

	Window_BattleCommand.prototype.refresh = function() {
		if (this.contents) {
			//Window_Command.prototype.refresh.call(this);
			this.contents.clear();
			this.drawTitle();
			this.drawCommandDot();
			this.drawCommandIcons();
		}
	}

	Window_BattleCommand.prototype.drawTitle = function() {
		var w = this._commandWidth;
		var h = 36 + 8;
		this.changePaintOpacity(true);

		var bitmap = this.contents;
	    var x = this.width/2 - this._commandWidth/2;
	    var y = 100-8 - this._h_ofs - 15;

	    bitmap.blt(this._bmpCommandTitle, 0, 36*this._commandTitleIndex, 160, 36, x, y, 160, 36);

	    var dotW = 6;
	    var lineH = 8;
	    bitmap.blt(this._bmpCommandLine, dotW, 0, 10, lineH, x + dotW, y + h - lineH, this._commandWidth - dotW*2, lineH);
	    bitmap.blt(this._bmpCommandLine, 0, 0, dotW, lineH, x + 0, y + h - lineH, dotW, lineH);
	    bitmap.blt(this._bmpCommandLine, 60-dotW, 0, dotW, lineH, x +  this._commandWidth - dotW, y + h - lineH, dotW, lineH);
	}

	Window_BattleCommand.prototype.drawCommandDot = function() {
		var bitmap = this.contents;

		var rect = this.commandRect(0);

		var dotNum = this._commandNum + 1;
		if(this._commandNum==2) dotNum++;

		for(var i=0; i<dotNum; i++){
			var x = rect.x + i*rect.width - 6;
			bitmap.blt(this._bmpCommandIconDot, 0, 0, 12, rect.height, x , rect.y, 12, rect.height);
		}
	}

	Window_BattleCommand.prototype.drawCommandIcons = function() {
		if(this._command_icons.length <= 0)return;

		var bitmap = this.contents;
		for(var i=0; i<this._list.length; i++){
			var irect = this.iconRect(i,0);
			this.changePaintOpacity(this.isCommandEnabled(i));
			bitmap.blt(this._bmpCommandIcon,
					irect.sx, irect.sy,
					irect.pw, irect.ph,
					irect.px, irect.py,
					irect.pw, irect.ph
					);

			this.changePaintOpacity(true);
		}

		this._sprSelectCommand.bitmap = this._bmpCommandIcon;
	}

	Window_BattleCommand.prototype.iconRect = function(index,selected){
		var rect = this.commandRect(index);
		var pw = 60;
	    var ph = 64;
	    var sx = Math.floor( this._command_icons[index] / 7) * pw*2 + pw*selected;
		var sy = this._command_icons[index] % 7 * ph;
		return { sx: sx, sy: sy, px: rect.x + 2, py: rect.y+6, pw: pw, ph: ph };
	}

	Window_BattleCommand.prototype.updateSelectIcon = function() {

		this._sprSelectCommand.visible = (!this.isOpening() && !this.isClosing());

		if(this.index() < 0 || !this._sprSelectCommand.bitmap)return;

		var blinkCount = this._animationCount % 41;
	    var cursorOpacity = 255;
	    if (this.active) {
	        if (blinkCount < 20) {
	            cursorOpacity -= blinkCount * 4;
	        } else {
	            cursorOpacity -= (40 - blinkCount) * 4;
	        }
	    }
	    this._sprSelectCommand.alpha = cursorOpacity / 255;
	};

	Window_BattleCommand.prototype.itemRect = function(index) {
	    return this.commandRect(index);
	};

	Window_BattleCommand.prototype.updateCursor = function() {
		this.setCursorRect(0, 0, 0, 0);

		if(this.index() < 0 || !this._sprSelectCommand.bitmap)return;
		var irect = this.iconRect(this.index(),1);
		this._sprSelectCommand.setFrame(irect.sx,irect.sy,irect.pw, irect.ph);
		this._sprSelectCommand.move(irect.px, irect.py);

		if(this.isCommandEnabled(this.index())){
			this._sprSelectCommand.tint = 0xffffff;
			this._sprSelectCommand.z = 10;
			this._windowContentsSprite.z = 9;

		}else{

			this._sprSelectCommand.z = 9;
			this._windowContentsSprite.z = 10;
			this._sprSelectCommand.tint = 0xAAAAAA;
		}



	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_PartyCommand
	////////////////////////////////////////////////////////////////////////////////////
	Window_PartyCommand.prototype = Object.assign(Window_PartyCommand.prototype,Window_BattleCommand.prototype);
	Window_PartyCommand.prototype.constructor = Window_PartyCommand;

	Window_PartyCommand.prototype.initialize = function() {
	    Window_BattleCommand.prototype.initialize.call(this);
	    this.setTitleIndex(0);
	};

	Window_PartyCommand.prototype.maxCols = function() {
		return ( this._list.length > 0)? this._list.length : 1;
	};


	Window_PartyCommand.prototype.numVisibleRows = function() {
	    return 1;
	};

	Window_PartyCommand.prototype.setCommandIcons = function() {
		this._command_icons = [0,1];
	};

	Window_PartyCommand.prototype.makeCommandList = function() {
	    this.addCommand(TextManager.fight,  'fight');
	    this.addCommand(TextManager.escape, 'escape', BattleManager.canEscape());
	    this.setCommandNum(this._list.length);
	};

	Window_PartyCommand.prototype.setup = function() {
		Window_BattleCommand.prototype.setup.call(this);
	    this.clearCommandList();
	    this.makeCommandList();
	    this.setCommandIcons();
	    this.refresh();
	    this.select(0);
	    this.activate();
	    this.open();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_ActorCommand
	////////////////////////////////////////////////////////////////////////////////////

	Window_ActorCommand.prototype = Object.assign(Window_ActorCommand.prototype,Window_BattleCommand.prototype);
	Window_ActorCommand.prototype.constructor = Window_ActorCommand;

	Window_ActorCommand.prototype.initialize = function() {
	    Window_BattleCommand.prototype.initialize.call(this);
	};

	Window_ActorCommand.prototype.maxCols = function() {
		return ( this._list.length > 0)? this._list.length : 1;
	};

	Window_ActorCommand.prototype.numVisibleRows = function() {
	    return 1;
	};

	var _JPT_BattleUI_Window_ActorCommand_makeCommandList = Window_ActorCommand.prototype.makeCommandList;
	Window_ActorCommand.prototype.makeCommandList = function() {
		_JPT_BattleUI_Window_ActorCommand_makeCommandList.call(this);
	    this.setCommandNum(this._list.length);
	};

	Window_ActorCommand.prototype.setCommandIcons = function() {
		this._command_icons = [];
		var cmds = this._actor.battleCommands();
		var len = cmds.length;
		if (len > 0) {
			for (var i = 0; i < len; i++) {
				var cmd = cmds[i];
				if (this.isVisible(cmd)) {
					this._command_icons.push(cmd._icon);
				}
			}
		}
	};

	Window_ActorCommand.prototype.setup = function(actor) {
	    this._actor = actor;
	    Window_BattleCommand.prototype.setup.call(this);
	    this.setTitleIndex(actor.actor().class_icon + 1);
	    this.clearCommandList();
	    this.makeCommandList();
	    this.setCommandIcons();
	    this.refresh();
	    this.selectLast();
	    this.activate();
	    this.open();
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleStatus
	////////////////////////////////////////////////////////////////////////////////////

	Window_BattleStatus.prototype.maxCols = function() {
		return $gameParty.battleMembers().length;
	};

	Window_BattleStatus.prototype.numVisibleRows = function() {
	    return 1;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleActor
	////////////////////////////////////////////////////////////////////////////////////

	Window_BattleActor.prototype.setSpriteset = function(sprset) {
		this._sprset = sprset;
	}

	var _JPT_BattleUI_Window_BattleActor_processTouch = Window_BattleActor.prototype.processTouch;
	Window_BattleActor.prototype.processTouch = function() {
	    if (this.isOpenAndActive() && this._sprset) {
	    	if (TouchInput.isTriggered() && !this.isTouchedInsideFrame()) {
	    		if (this.getClickedActor() >= 0) {
	    			var index = this.getClickedActor();
	    			if (this.index() === index) {
	    				return this.processOk();
	    			} else {
	    				SoundManager.playCursor();
	    				return this.select(index);
	    			}
	    		}
	    	}
	    	if (TouchInput.isPressed() && !this.isTouchedInsideFrame()) {
	    		if (this.getClickedActor() >= 0) {
	    			var index = this.getClickedActor();
	    			if (this.index() !== index) {
	    				SoundManager.playCursor();
	    				return this.select(index);
	    			}
	    		}
	    	}
	    }
	    _JPT_BattleUI_Window_BattleActor_processTouch.call(this);
	};


	Window_BattleActor.prototype.getClickedActor = function() {
		for (var i = 0; i < $gameParty.battleMembers().length; ++i) {
			var actor = $gameParty.battleMembers().reverse()[i];
			if (!actor) continue;
			if (this.isClickedActor(actor)) {
				if (this._selectDead && !actor.isDead()) continue;
				if (this._inputLock && actor.index() !== this.index()) continue;
				return actor.index();
			}
		}
		return -1;
	};

	Window_BattleActor.prototype.isClickedActor = function(actor) {
		if (!actor) return false;
		if (!actor.isAppeared()) return false;

		if (!this.isEnabled(actor)) return false;

		var hud = this._sprset.getActorHud(actor);
		if ($gameTemp._disableMouseOverSelect) return false;
		if(!hud) return false;
		var rect = hud.touchRect();
		if(!rect)return false;
		var x = TouchInput.x;
		var y = TouchInput.y;
		return (x >= rect.x && y >= rect.y && x < rect.x + rect.width && y < rect.y + rect.height);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_Actor
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_BattleUI_Sprite_Actor_initialize = Sprite_Actor.prototype.initialize;
	Sprite_Actor.prototype.initialize = function(battler) {
		_JPT_BattleUI_Sprite_Actor_initialize.call(this,battler);
		this._sprHud = null;
	};

	Sprite_Actor.prototype.setHud = function(sprHud) {
		this._sprHud = sprHud;
	}

	Sprite_Actor.prototype.updatePosition = function() {
		if(this._sprHud){
			this.x = this._sprHud.x;
			this.y = this._sprHud.y - 60;
		}
	};


	Sprite_Actor.prototype.setupDamagePopup = function() {
		if (this._battler.isDamagePopupRequested()) {
			var sprite = new Sprite_Damage();
			sprite.x = this.x;
			sprite.y = this.y;
			sprite.z = this._sprHud.z + 3;
			sprite.setup(this._battler);
			this._damages.push(sprite);
			this.parent.addChild(sprite);
			this._battler.clearDamagePopup();
			this._battler.clearResult();
		};
	};

	Sprite_Actor.prototype.setupAnimation = function() {
		while (this._battler.isAnimationRequested()) {
			var data = this._battler.shiftAnimation();
			var animation = $dataAnimations[data.animationId];
			var mirror = data.mirror;
			var delay = animation.position === 3 ? 0 : data.delay;
			this.startAnimation(animation, mirror, delay);
			for (var i = 0; i < this._animationSprites.length; i++) {
				var sprite = this._animationSprites[i];
				sprite.visible = true;
				sprite.z = this._sprHud.z + 3;
			}
		}
	};
	////////////////////////////////////////////////////////////////////////////////////
	// Spriteset_Battle
	////////////////////////////////////////////////////////////////////////////////////

	// Enemy Select Arrow
	Spriteset_Battle.prototype.createESelectArrow = function() {
		this._sprite_es_arrow = [];
		for (var i = 0; i < $gameTroop.members().length; i++) {
			this._sprite_es_arrow[i] = new Sprite_SelectArrow();
			this._sprite_es_arrow[i].z = 0;
			this._battleField.addChild(this._sprite_es_arrow[i]);
	    };
	};

	Spriteset_Battle.prototype.removeESelectArrow = function() {
		for (var i = 0; i < this._sprite_es_arrow.length; i++) {
			this._battleField.removeChild(this._sprite_es_arrow[i]);
	    };
	};

	Spriteset_Battle.prototype.addESelectArrow = function() {
		for (var i = 0; i < this._enemySprites.length; i++) {
			this._enemySprites[i].addSelectArrowSprite(this._sprite_es_arrow[i]);
		};
	};

	// Enemy target Line
	Spriteset_Battle.prototype.createETragetLine = function() {
		this._sprite_et_lines = [];
		for (var i = 0; i < $gameTroop.members().length; i++) {
			this._sprite_et_lines[i] = [];
			for (var j = 0; j < $gameParty.maxBattleMembers(); j++) {
				this._sprite_et_lines[i][j] = new Sprite_TargetLine();
				this._sprite_et_lines[i][j].z = 0;
				this._battleField.addChild(this._sprite_et_lines[i][j]);
			}
	    };
	};

	Spriteset_Battle.prototype.removeETragetLine = function() {
		for (var i = 0; i < this._sprite_et_lines.length; i++) {
			for (var j = 0; j < this._sprite_et_lines[i].length; j++) {
				this._battleField.removeChild(this._sprite_et_lines[i][j]);
			}
	    };
	};

	Spriteset_Battle.prototype.addETragetLine = function() {
		for (var i = 0; i < this._enemySprites.length; i++) {
			 this._enemySprites[i].addTargetLineSprite(this._sprite_et_lines[i]);
		};
	};


	Spriteset_Battle.prototype.setSprAHud_ToEnemy = function(sprHuds) {
		for (var i = 0; i < this._enemySprites.length; i++) {
			 this._enemySprites[i].setActorHuds(sprHuds);
		};
	};


	var _JPT_BattleUI_Spriteset_Battle_createEnemies = Spriteset_Battle.prototype.createEnemies;
	Spriteset_Battle.prototype.createEnemies = function() {
	    this.createESelectArrow();
	    this.createETragetLine();
	    _JPT_BattleUI_Spriteset_Battle_createEnemies.call(this)
	    this.addESelectArrow();
	    this.addETragetLine();
	};

	// Actor Hud
	var _JPT_BattleUI_Spriteset_Battle_createActors = Spriteset_Battle.prototype.createActors;
	Spriteset_Battle.prototype.createActors = function() {
		_JPT_BattleUI_Spriteset_Battle_createActors.call(this);

		this._actorHudField = new Sprite_HudField();
		this._battleField.addChild(this._actorHudField);
		this._actorHud = [];
	    for (var i = 0; i < $gameParty.maxBattleMembers(); i++) {
	        this._actorHud[i] = new Sprite_ActorHud(i);
	        this._actorHud[i].setZ(this._actorHudField.z + 1);
	        this._battleField.addChild(this._actorHud[i]);
	        this._actorSprites[i].setHud(this._actorHud[i]);
	    }

	    this.setSprAHud_ToEnemy(this._actorHud);
	};

	Spriteset_Battle.prototype.getActorHud = function(actor) {
		var ret = false;
		for (var i = 0; i < $gameParty.maxBattleMembers(); i++) {
			if(this._actorHud[i]){
				if(this._actorHud[i]._battler==actor){
					ret = this._actorHud[i];
					break;
				}
			}
		}
		return ret;
	}

	Spriteset_Battle.prototype.refreshHud = function(){
		for (var i = 0; i < $gameParty.maxBattleMembers(); i++) {
			this._actorHud[i].statusRefresh();
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_HudFilter
	////////////////////////////////////////////////////////////////////////////////////
	function Sprite_HudField() {
	    this.initialize.apply(this, arguments);
	};

	Sprite_HudField.prototype = Object.create(Sprite_Base.prototype);
	Sprite_HudField.prototype.constructor = Sprite_HudField;

	Sprite_HudField.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
		this.anchor.set(0.5,1);
	    this.z = 10;
	    this.bitmap = ImageManager.loadBitmap("img/battlehud/", "battle_bg_footer");
		this.setFrame(0,0,1,272);
		this.scale.x = Graphics.boxWidth + 200;
		this.move(Graphics.boxWidth/2,Graphics.boxHeight);
		this.alpha = 198/255;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_ActorHud
	////////////////////////////////////////////////////////////////////////////////////
	function Sprite_ActorHud() {
	    this.initialize.apply(this, arguments);
	};

	Sprite_ActorHud.prototype = Object.create(Sprite_Base.prototype);
	Sprite_ActorHud.prototype.constructor = Sprite_ActorHud;

	Sprite_ActorHud.prototype.initialize = function(index) {
		Sprite_Base.prototype.initialize.call(this);
		this.anchor.set(0.5,1);
	    this._battler = null;
	    this._index = index;
	    this._sprFace = new Sprite();
	    this._sprStatus = new Sprite();
	    this.addChild(this._sprFace);
	    this.addChild(this._sprStatus);
	    this._sprSelect = new Sprite_SelectArrow();
	    this.addChild(this._sprSelect);
	    this.force_refresh = true;
	    this.z = 0;
	    this.status_refresh = false;
	    this._befHp = -1;
	    this._befMp = -1;
	    this._befSp = -1;
	    this._befIcons = [];
	    this.faceHomeX = 0;
	    this.faceHomeY = 0;
	};

	Sprite_ActorHud.prototype.isRefresh = function() {
		if (this._battler != $gameParty.battleMembers()[this._index])return true;
		return this.force_refresh;
	};

	Sprite_ActorHud.prototype.forceRefresh = function(){
		this.force_refresh = true;
	}


	Sprite_ActorHud.prototype.isStatusRefresh = function() {
		if(!this._battler)return false;
		var actor = this._battler;

		if(this._befHp!=actor.hp + actor.mhp){this.status_refresh = true}
		if(this._befMp!=actor.mp + actor.mmp){this.status_refresh = true}
		if(this._befSp!=actor.sp + actor.msp){this.status_refresh = true}
		if(JSON.stringify(this._befIcons)!=JSON.stringify(actor.allIcons())){this.status_refresh = true}

		this._befHp = actor.hp + actor.mhp;
		this._befMp = actor.mp + actor.mmp;
		this._befSp = actor.sp + actor.msp;
	    this._befIcons = actor.allIcons();
		return this.status_refresh;
	};

	Sprite_ActorHud.prototype.statusRefresh = function(){
		this.status_refresh = true;
	}


	Sprite_ActorHud.prototype.update = function() {
	    Sprite_Base.prototype.update.call(this);

	    if(this.isRefresh()){
		    this.force_refresh = false;
		    this.status_refresh = true;
		    this.setPosition();
	    	this.setBattler();
	    }

	    if(this.isStatusRefresh()){
		    this.status_refresh = false;
	    	this.updateStatus();
	    }

	    this.updateBitmap();
	    this.updateFrame();


	};

	Sprite_ActorHud.prototype.touchRect = function(){
		if(!this._battler)return false;
		var rect = new Rectangle();
		rect.width = this._sprFace.width;
		rect.height = this._sprFace.height + 54;
		rect.x = this.x - rect.width/2;
		rect.y = this.y - rect.height;
		return rect;
	};

	Sprite_ActorHud.prototype.updateBitmap = function() {
	};

	Sprite_ActorHud.prototype.updateFrame = function() {
		this.SelectedActor();
		this.setFaceFrame();

	};

	Sprite_ActorHud.prototype.SelectedActor = function(){
		var selected = false;
		var tint = 0xffffff;
		for (var i = 0; i < $gameParty.battleMembers().length; i++) {
			if($gameParty.battleMembers()[i].isSelected() ){
				selected = true;
			}
		}

		if(this._battler){
			if(!selected){
				if(BattleManager.actor()==null){
					tint = 0xffffff;
				}else{
					if(BattleManager.actor() == this._battler){
						tint = 0xffffff;
					}else{
						tint = 0x888888;
					}
				}
			}else{
				if(this._battler.isSelected()){
					tint = 0xffffff;
				}else{
					tint = 0x888888;
				}
			}
			if( this._battler.isSelected() ){
				this._sprSelect.show();
				this._sprSelect.setPosition(this._sprFace);
			}else{
				this._sprSelect.hide();
			}
		}

		this._sprFace.tint = tint;
		this._sprStatus.tint = tint;
	};

	Sprite_ActorHud.prototype.getFaceName = function(){
		var ret = this._battler.actor().battle_face;
		if(!ret && this._battler.actor().battle_face_vid > 0){
			var vid = $gameVariables.value(this._battler.actor().battle_face_vid);
			ret = this._battler.actor().battle_face_array[ vid ];
		}
		return ret;
	}

	Sprite_ActorHud.prototype.setFaceFrame = function(){
		if(!this._battler)return;
		var faceName = this.getFaceName();
		if(faceName){
			var fhud = this._battler.hudFace();
			var zoom = fhud.zoom();
			this._sprFace.scale.set(zoom,zoom);
			var pos = fhud.shake();
			this._sprFace.x = this.faceHomeX + pos.x;
			this._sprFace.y = this.faceHomeY + pos.y;
			this._sprFace.setFrame(204*fhud.facePattern(),0,204,200);
		}
	}

	Sprite_ActorHud.prototype.setZ = function(z) {
		this.z = z;
	};

	Sprite_ActorHud.prototype.setBattler = function(){
		this._battler = null;
		this._battler = $gameParty.battleMembers()[this._index];
		if(this._battler)this._battler.setHudFace(new Game_HudFace(this._battler));
		this.setFaceSpr();
		this.setStatusSpr();
	};

	Sprite_ActorHud.prototype.setFaceSpr = function(){
		if(!this._battler)return;
		var faceName = this.getFaceName();
		if(faceName){
			this.removeChild(this._sprFace);
			this._sprFace = new Sprite();
			this._sprFace.bitmap = ImageManager.loadBitmap("img/battlehud/", faceName);
			this._sprFace.setFrame(0,0,204,200);
			this._sprFace.anchor.set(0.5,1);
			this._sprFace.y = -54;
			this._sprFace.tint = 0xffffff;
			this.addChild(this._sprFace);

		    this.faceHomeX = this._sprFace.x;
		    this.faceHomeY = this._sprFace.y;
		}
	};

	Sprite_ActorHud.prototype.setStatusSpr = function(){
		if(!this._battler)return;
		this.removeChild(this._sprStatus);
		this._sprStatus = new Sprite();
		var w = 204;
		var h = 200;
		var bitmap = new Bitmap(w,h);
		bitmap.fontBold = false;
		bitmap.fontItalic = false;
		bitmap.outlineWidth = 0;
		bitmap.outlineColor = 'rgba(0, 0, 0, 0)';

		this._sprStatus.bitmap = bitmap;
		this._sprStatus.setFrame(0,0,w,h);
		this._sprStatus.anchor.set(0.5,1);
		this.addChild(this._sprStatus);
	};

	Sprite_ActorHud.prototype.updateStatus = function(){
		if(!this._battler)return;

		this._sprStatus.bitmap.clear();
		//this._sprStatus.bitmap.fillAll('#0000FF');

		var actor = this._battler;
		var bitmap = this._sprStatus.bitmap;
		var width = 204 - 36;
		var x = this._sprStatus.width/2 - width/2;
		var y = this._sprStatus.height - 36;
		this.setSpGauge(x,y,width,actor,bitmap);
		y -= 27;
		this.setMpGauge(x,y,width,actor,bitmap);
		y -= 27;
		this.setHpGauge(x,y,width,actor,bitmap);

		x = this._sprStatus.width/2 - width/2 + width - Window_Base._iconWidth;
		y -= Window_Base._iconHeight+2;
		this.drawStatusIcons(x,y,actor,bitmap);

	};

	Sprite_ActorHud.prototype.hpColor = function(actor) {
	    if (actor.isDead()) {
	        return TextManager.colorCHP;
	    } else if (actor.isDying()) {
	        return TextManager.colorCHP;
	    } else {
	        return TextManager.colorNHP;
	    }
	};

	Sprite_ActorHud.prototype.setHpGauge = function(x,y,width,actor,bitmap){
		var color = TextManager.colorNHP;
	    this.drawGauge(bitmap,x, y, width, actor.hpRate(), this.hpColor(actor), this.hpColor(actor));
	    bitmap.textColor = '#ffffff';
	    bitmap.fontSize = 14;
	    bitmap.drawText(TextManager.hpA, x, y, 44, 16, 'left');
	    bitmap.fontSize = 16;
	    this.drawCurrentAndMax(bitmap,actor.hp, actor.mhp, x, y, width, 16,this.hpColor(actor), color);
	};

	Sprite_ActorHud.prototype.setMpGauge = function(x,y,width,actor,bitmap){
		var color = TextManager.colorMP;
	    this.drawGauge(bitmap,x, y, width, actor.mpRate(), color, color);
	    bitmap.textColor = '#ffffff';
	    bitmap.fontSize = 14;
	    bitmap.drawText(TextManager.mpA, x, y, 44, 16, 'left');
	    bitmap.fontSize = 16;
	    this.drawCurrentAndMax(bitmap,actor.mp, actor.mmp, x, y, width, 16,color, color);
	};

	Sprite_ActorHud.prototype.setSpGauge = function(x,y,width,actor,bitmap){
		if(actor.currentClass().msp_eval!=='0'){
		    var color = TextManager.colorSP;
		    this.drawGauge(bitmap,x, y, width, actor.spRate(), color, color);
		    bitmap.textColor = '#ffffff';
		    bitmap.fontSize = 14;
		    bitmap.drawText(TextManager.spA, x, y, 44, 16, 'left');
		    bitmap.fontSize = 16;
		    this.drawCurrentAndMax(bitmap,actor.sp, actor.msp, x, y, width, 16,color, color);
		}
	};

	Sprite_ActorHud.prototype.drawGauge = function(parentBmp,x, y, width, rate, color1, color2) {
	    var fillW = Math.floor(width * rate);
	    var gaugeY = y + 15;
	    var height = 8;
	    parentBmp.fillRect(x, gaugeY, width, height, TextManager.colorGaugeBack);
	    parentBmp.gradientFillRect(x, gaugeY, fillW, height, color1, color2);
	};

	Sprite_ActorHud.prototype.drawCurrentAndMax = function(parentBmp,current, max, x, y,width, height,color1, color2) {
		var labelWidth = parentBmp.measureTextWidth('HP');
		var maxValueWidth = parentBmp.measureTextWidth(max);
		var curValueWidth = parentBmp.measureTextWidth(current);
		var slashWidth = parentBmp.measureTextWidth('/');
		var x1 = x + width - maxValueWidth;
		var x2 = x1 - slashWidth;
		var x3 = x2 - curValueWidth;

		parentBmp.textColor = color1;
		parentBmp.drawText(current, x3, y, curValueWidth, height, 'right');
		parentBmp.textColor = color2;
		parentBmp.drawText('/', x2, y, slashWidth, height, 'right');
		parentBmp.drawText(max, x1, y, maxValueWidth, height, 'right');

		/*
		if (x3 >= x + labelWidth) {
			parentBmp.textColor = color1;
			parentBmp.drawText(current, x3, y, curValueWidth, height, 'right');
			parentBmp.textColor = color2;
			parentBmp.drawText('/', x2, y, slashWidth, height, 'right');
			parentBmp.drawText(max, x1, y, maxValueWidth, height, 'right');
		} else {
			parentBmp.textColor = color1;
			parentBmp.drawText(current, x1, y, valueWidth, 'right');
		}
		*/
	};

	Sprite_ActorHud.prototype.drawStatusIcons = function(x, y, actor, bitmap){
		var iconNum = Math.min( 5, actor.allIcons().length );

		for(var i=0; i<iconNum; i++){
			this.drawIcon(bitmap, actor.allIcons()[i],x,y);
			x-=Window_Base._iconWidth;
		}

	};

	Sprite_ActorHud.prototype.drawIcon = function(parentbmp,iconIndex, x, y ) {
		var bitmap = ImageManager.loadSystem('IconSet');
	    var pw = Window_Base._iconWidth;
	    var ph = Window_Base._iconHeight;
	    var sx = iconIndex % 16 * pw;
	    var sy = Math.floor(iconIndex / 16) * ph;
	    parentbmp.blt(bitmap, sx, sy, pw, ph, x, y);
	};

	Sprite_ActorHud.prototype.setPosition = function(){
		var mnum = $gameParty.battleMembers().length;
		var width = Graphics.boxWidth/(mnum*2);
		var x = width + width*this._index*2;
		this.move(x, Graphics.boxHeight );
	};




	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_Enemy
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_BattleUI_Sprite_Enemy_initMembers = Sprite_Enemy.prototype.initMembers;
	Sprite_Enemy.prototype.initMembers = function() {
		_JPT_BattleUI_Sprite_Enemy_initMembers.call(this);
		this._arrowSprite = null;
		this._targetLineSprites = null;
		this._targetLineSeq = 0;
		this._actorHuds = [];
	};

	Sprite_Enemy.prototype.addSelectArrowSprite = function(arrow_sprite) {
	    this._arrowSprite = arrow_sprite;
	    this._arrowSprite.hide();
	};

	Sprite_Enemy.prototype.addTargetLineSprite = function(line_sprites) {
	    this._targetLineSprites = line_sprites;
	    this.targetLinesHide();
	};

	Sprite_Enemy.prototype.targetLinesHide = function() {
		if(!this._targetLineSprites)return;
		this._targetLineSprites.forEach(function(sprite) {
	        sprite.hide();
	    }, this);
	};

	Sprite_Enemy.prototype.setActorHuds = function(huds) {
		this._actorHuds = huds;
	};

	Sprite_Enemy.prototype.targetLinesShow = function() {
		if(!this._targetLineSprites)return;
		this._targetLineSprites.forEach(function(sprite) {
	        sprite.show();
	    }, this);
	};

	var _JPT_BattleUI_Sprite_Enemy_startEffect = Sprite_Enemy.prototype.startEffect;
	Sprite_Enemy.prototype.startEffect = function(effectType) {
	    if(this._arrowSprite)this._arrowSprite.hide();
		_JPT_BattleUI_Sprite_Enemy_startEffect.call(this,effectType);
	};

	var _JPT_BattleUI_Sprite_Enemy_startWhiten = Sprite_Enemy.prototype.startWhiten;
	Sprite_Enemy.prototype.startWhiten = function() {
		_JPT_BattleUI_Sprite_Enemy_startWhiten.call(this);

	};

	var _JPT_BattleUI_Sprite_Enemy_update = Sprite_Enemy.prototype.update;
	Sprite_Enemy.prototype.update = function() {
		_JPT_BattleUI_Sprite_Enemy_update.call(this);
		this.updateSelectArrow();
		this.updateTragetLine();
	};

	Sprite_Enemy.prototype.updateSelectArrow = function() {
	    if (this._enemy) {
	    	if(this._enemy.isSelected()){
	    		if(this._arrowSprite){
	    			this._arrowSprite.setPosition(this);
	    		}
	    	}else{
	    		if(this._arrowSprite){
	    			this._arrowSprite.hide();
	    		}
	    	}
	    }
	};

	Sprite_Enemy.prototype.updateTragetLine = function() {
	    if (this._enemy) {
		    if(this._enemy.isShowTrgetLine()){
		    	if(this._targetLineSeq==0){
		    		this._targetLineSeq = 1;
		    		this.setupTargetLine();
		    	}
		    	this.updateTarget();
		    }else{
		    	this.targetLinesHide();
		    	this._targetLineSeq = 0;
		    }
	    }
	};

	Sprite_Enemy.prototype.setupTargetLine = function() {
		var action = this._enemy.currentAction();

		if(action){
			var tagets = action.preTaragets();
			if(tagets.length > 0){
				for(var i=0; i<tagets.length; i++){
					if(tagets[i].isActor()){
						if(tagets[i].index() >= 0 && tagets[i].index() < this._targetLineSprites.length ){
							this.setupTargetQuad( tagets[i].index() );
						}
					}
				}
			}
		}
	};

	Sprite_Enemy.prototype.setupTargetQuad = function(index){
		if(this._actorHuds.length < index)return;

		var line = this._targetLineSprites[index];
		var hud = this._actorHuds[index];

		line.show();

		var start = new PIXI.Point(this._homeX,this._homeY - this.height/2);
		var end = new PIXI.Point(hud.x,hud.y - hud.touchRect().height );

		line.setQuad(start,end);
	};

	Sprite_Enemy.prototype.updateTarget = function(){
		for(var i=0; i<this._targetLineSprites.length; i++){
			if(this._targetLineSprites[i].visible){
				this._targetLineSprites[i].updatemapSprite();
				this._targetLineSprites[i].z = this.z + 10;
			}
		}
	}

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_SelectArrow
	////////////////////////////////////////////////////////////////////////////////////

	function Sprite_SelectArrow() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_SelectArrow.prototype = Object.create(Sprite_Base.prototype);
	Sprite_SelectArrow.prototype.constructor = Sprite_SelectArrow;

	Sprite_SelectArrow.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
	    this.bitmap = ImageManager.loadBitmap("img/battlehud/", "battle_target_cursol");
	    this.anchor.set(0.5,1);
	};

	Sprite_SelectArrow.prototype.setPosition = function(eSprite) {
		this.move(eSprite.x, eSprite.y - eSprite.height - 4);
		this.z = eSprite.z + 1;
		this.show();
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_TargetLine
	////////////////////////////////////////////////////////////////////////////////////

	function Sprite_TargetLine() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_TargetLine.prototype = Object.create(ProjectionSprite2d.prototype);
	Sprite_TargetLine.prototype.constructor = Sprite_TargetLine;

	Sprite_TargetLine.prototype.initialize = function() {
		ProjectionSprite2d.prototype.initialize.call(this);
		this._image = ImageManager.loadBitmap("img/battlehud/", "battle_target_line");
	    this.bitmap = this._image;
	    this.anchor.set(0.5,0);
	    this._quad = [];
	    this.setFrame(0,0,16,292);
	    this.hide();

	    this._highlight = new ProjectionSprite2d(this._image);
	    this._highlight.setFrame(20,0,16,292);
	    this._highlight.anchor.set(0.5,0);
	    this.addChild(this._highlight);
	    this._highlight.blendMode = PIXI.BLEND_MODES.ADD;

	    this._hl_alpha = 0;
	    this._hl_alpha_seq = 0;
	};

	Sprite_TargetLine.prototype.setQuad = function(startPoint,endPoint){
		endPoint.y = endPoint.y + 30;

		var rad = Math.atan2(endPoint.y - startPoint.y, endPoint.x - startPoint.x);

		var deg0 = ((rad * 180 / Math.PI) - 90) * (30/90);
		var deg1 = deg0; //R
		var deg2 = deg0 + 180; //L

		var rad1 = deg1 * Math.PI / 180;
		var rad2 = deg2 * Math.PI / 180;

		var sr = 6;
		var er = 18;

		var ul_rs_x = startPoint.x + (Math.cos(rad2) * ( sr ) );
		var ul_rs_y = startPoint.y + (Math.sin(rad2) * ( sr ) );
		var ur_rs_x = startPoint.x + (Math.cos(rad1) * ( sr ) );
		var ur_rs_y = startPoint.y + (Math.sin(rad1) * ( sr ) );

		var bl_rs_x = endPoint.x + (Math.cos(rad2) * ( er ) );
		var bl_rs_y = endPoint.y + (Math.sin(rad2) * ( er ) );
		var br_rs_x = endPoint.x + (Math.cos(rad1) * ( er ) );
		var br_rs_y = endPoint.y + (Math.sin(rad1) * ( er ) );

		var p0 = new PIXI.Point(ul_rs_x , ul_rs_y);//UL
		var p1 = new PIXI.Point(ur_rs_x , ur_rs_y);//UR
		var p2 = new PIXI.Point(br_rs_x , br_rs_y);//BR
		var p3 = new PIXI.Point(bl_rs_x , bl_rs_y);//BL

		this._quad = [p0,p1,p2,p3];

		this._hl_alpha = Math.floor(Math.random()*255);
		this._hl_alpha_seq = 0;
	};

	Sprite_TargetLine.prototype.updatemapSprite = function () {
		this.proj.mapSprite(this,this._quad );
		this.updateHighlight();
	}

	Sprite_TargetLine.prototype.updateHighlight = function () {

		if(this._hl_alpha_seq==0){
			this._hl_alpha += Math.floor(255/60);
			if(this._hl_alpha>255){
				this._hl_alpha==255;
				this._hl_alpha_seq = 1;
			}
		}else{
			this._hl_alpha -= Math.floor(255/30);
			if(this._hl_alpha<0){
				this._hl_alpha=0;
				this._hl_alpha_seq = 0
			}
		}

		this._highlight.opacity = this._hl_alpha;
	}

	Sprite_TargetLine.prototype.hide = function() {
	    this._hiding = true;
	    this.updateVisibility();
	};

	Sprite_TargetLine.prototype.show = function() {
	    this._hiding = false;
	    this.updateVisibility();
	};

	Sprite_TargetLine.prototype.updateVisibility = function() {
	    this.visible = !this._hiding;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_BattleFace
	////////////////////////////////////////////////////////////////////////////////////
	function Game_HudFace() {
	    this.initialize.apply(this, arguments);
	}

	Game_HudFace.prototype.initialize = function(actor) {
		this._actor = actor;
	    this.initMembers();
	};

	Game_HudFace.prototype.initMembers = function() {
	    this._ptn_index = 0;
	    this.initZoom();
	    this.initShake();
	};

	Game_HudFace.prototype.initZoom = function() {
		this._zoom_proc = 0;
	    this._zoom_frame = 0;
	    this._zoom_frame_cnt = 0;
	    this._zoom_max = 0;
	    this._zoom_speed = 0;
	};

	Game_HudFace.prototype.initShake = function() {
		this._shakeDuration = 0;
    	this._shakePower = 0;
    	this._shakeSpeed = 0;
    	this._shake = 0;
    	this._shakeDirection = 0;
	};

	Game_HudFace.prototype.facePattern = function(){
		if(this._ptn_index==0 && this._actor.hpRate() * 100 < 30){
			return 2;
		}
		return this._ptn_index;
	};

	Game_HudFace.prototype.setZoom = function(){
		this._zoom_proc = 1;
		this._zoom_frame = 15;
		this._zoom_frame_cnt = 0;
		this._zoom_max = 1.2;
		this._zoom_speed = (this._zoom_max - 1)/this._zoom_frame;
	    this._ptn_index = 1;
	    this.initShake();
	};

	Game_HudFace.prototype.zoom = function(){
		var scale = 1;

		if(this._zoom_proc==1){
			this._zoom_frame_cnt+=2;
			if(this._zoom_frame_cnt >= this._zoom_frame){
				this._zoom_proc=2;
				this._zoom_frame_cnt = this._zoom_frame;
			}
			scale = 1 + this._zoom_speed*this._zoom_frame_cnt;
			if(scale > this._zoom_max)scale = this._zoom_max;

		}else if(this._zoom_proc==2){

			this._zoom_frame_cnt--;
			if(this._zoom_frame_cnt <= 0 ){
				this.initZoom();
				this._ptn_index = 0;
			}
			scale = 1 + this._zoom_speed*this._zoom_frame_cnt;
			if(scale < 1)scale = 1;
		}
		return scale;
	};

	Game_HudFace.prototype.setShake = function(){
		this._shakeDuration = 15;
    	this._shakePower = 4;
    	this._shakeSpeed = 15;
    	this._shake = 0;
    	this._shakeDirection = 1;
    	this._ptn_index = 2;
	    this.initZoom();
	};

	Game_HudFace.prototype.shake = function() {

		var rect = new Rectangle();

    	if (this._shakeDuration > 0 || this._shake !== 0) {
            var delta = (this._shakePower * this._shakeSpeed * this._shakeDirection) / 10;
            if (this._shakeDuration <= 1 && this._shake * (this._shake + delta) < 0) {
                this._shake = 0;
            } else {
                this._shake += delta;
            }
            if (this._shake > this._shakePower * 2) {
                this._shakeDirection = -1;
            }
            if (this._shake < - this._shakePower * 2) {
                this._shakeDirection = 1;
            }
            this._shakeDuration--;

        	if (this._shakeDuration == 0) {
        		this.initShake();
        		this._ptn_index = 0;
            }

        	rect.x = this._shake;
        }
    	return rect;
    };



	////////////////////////////////////////////////////////////////////////////////////
	// Game_Actor
	////////////////////////////////////////////////////////////////////////////////////

    /*
    var _JPT_BattleUI_Game_Actor_initMembers = Game_Actor.prototype.initMembers;
	Game_Actor.prototype.initMembers = function() {
		_JPT_BattleUI_Game_Actor_initMembers.call(this);
		this._hud_face = new Game_HudFace(this);
	};



    var _JPT_BattleUI_Game_Actor_setup = Game_Actor.prototype.setup;
	Game_Actor.prototype.setup = function(actorId) {
		_JPT_BattleUI_Game_Actor_setup.call(this,actorId);
		this._hud_face = new Game_HudFace(this);
	};
	*/

	Game_Actor.prototype.setHudFace = function(hudFace) {
		return this._hud_face = hudFace;
	};

	Game_Actor.prototype.hudFace = function() {
		return this._hud_face;
	};

	Game_Actor.prototype.performDamage = function() {
	    Game_Battler.prototype.performDamage.call(this);
	    if (this.isSpriteVisible()) {
	        this.requestMotion('damage');
	    } else {
	        //$gameScreen.startShake(5, 5, 10);
	    	if(this.hudFace())this.hudFace().setShake();
	    }
	    SoundManager.playActorDamage();
	};


    Game_Actor.prototype.EquippableSlot = function() {
        var result = 0;
    	for(var index=0; index < this.menuEquipSlots().length; index++){
	        var etype = this.menuEquipSlots()[index];
	        var tos = this.traitObjects();
	        for (var i=0,max=tos.length;i<max;i++) {
	            var to = tos[i];
	            if ( DataManager.battleEquippableTrait(to).contains(etype)){
	            	result = etype;
	            	break;
	            }
	        }
    	}
        return result;
    };

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Action
	////////////////////////////////////////////////////////////////////////////////////
	/*
	var _JPT_BattleUI_Game_Action_apply = Game_Action.prototype.apply;
	Game_Action.prototype.apply = function(target) {
		 var oldhp = target.hp
		 _JPT_BattleUI_Game_Action_apply.call(this,target);
		 if (target.isActor()) {
			 if (oldhp > target.hp) {
				 //ダメージ
			 }else if (oldhp < target.hp) {
				 //回復
			//	 target.hudFace().zoom();// = [0,20,1,30];
			 }
		 };
	};
	*/

	var _JPT_BattleUI_Game_Action_prepare = Game_Action.prototype.prepare;
	Game_Action.prototype.prepare = function() {
		_JPT_BattleUI_Game_Action_prepare.call(this);
		if (this.subject().isActor()){
			this.subject().hudFace().setZoom();
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleLog
	////////////////////////////////////////////////////////////////////////////////////
	Window_BattleLog.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_BattleLog.prototype.lineHeight = function() {
	    return 28;
	};

	Window_BattleLog.prototype.standardFontSize = function() {
	    return 16;
	};


	Window_BattleLog.prototype.standardPadding = function() {
	    return 20;
	};

	Window_BattleLog.prototype.textPadding = function() {
	    return 6;
	};


	Window_BattleLog.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_BattleLog.prototype.drawSimpleActionLine = function(index) {
	    var text = this._lines[index].replace('<SIMPLE>', '');
	    var rect = this.itemRectForText(index);
	    this.contents.clearRect(rect.x, rect.y, rect.width, rect.height);
	    if (this._actionIcon) {
	      var tw = this.textWidth(text);
	      var ix = (rect.width - tw) / 2 - 4;
	      //this.drawIcon(this._actionIcon, ix, rect.y + 2);
	      this.drawIcon20px(this.contents,this._actionIcon, ix , rect.y + Math.floor(this.lineHeight()/2) -10 );
	    }
	    this.drawText(text, rect.x, rect.y, Graphics.boxWidth, 'center');
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleMessage
	////////////////////////////////////////////////////////////////////////////////////

	function Window_BattleMessage() {
	    this.initialize.apply(this, arguments);
	}

	Window_BattleMessage.prototype = Object.create(Window_Message.prototype);
	Window_BattleMessage.prototype.constructor = Window_BattleMessage;

	Window_BattleMessage.prototype.initialize = function() {
		Window_Message.prototype.initialize.call(this);
		this._battleMode = true;
		this._refresh = false;
		this._windowFrameSprite.alpha = 0;
	};

	Window_BattleMessage.prototype.standardFontFace = function() {
		 return (this._battleMode) ? 'GameFont' : 'GameFont3';
	};

	Window_BattleMessage.prototype.lineHeight = function() {
	    return (this._battleMode) ? 28 : 36;
	};

	Window_BattleMessage.prototype.standardFontSize = function() {
	    return (this._battleMode) ? 16 : 24;
	};


	Window_BattleMessage.prototype.standardPadding = function() {
	    return (this._battleMode) ? 20 : 18;
	};

	Window_BattleMessage.prototype.textPadding = function() {
	    return 6;
	};

	Window_BattleMessage.prototype.setBattleMode = function(mode){
		this._battleMode = mode;
		this.width = this.windowWidth();
		this.height = this.windowHeight();
		this.resetFontSettings();

		if(mode){
			this._windowFrameSprite.alpha = 0;
		}else{
			this._windowFrameSprite.alpha = 1;
		}
	}

	Window_BattleMessage.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


	Window_BattleMessage.prototype.terminateMessage = function() {
		Window_Message.prototype.terminateMessage.call(this);
		$gameMessage.setPositionType(0);
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_SkillWindowBase
	////////////////////////////////////////////////////////////////////////////////////

	function Sprite_SkillWindowBase() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_SkillWindowBase.prototype = Object.create(Sprite_Base.prototype);
	Sprite_SkillWindowBase.prototype.constructor = Sprite_SkillWindowBase;

	Sprite_SkillWindowBase.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
		this.actor = null;
		this._sprWindow = new Sprite_SkillWindow();
		this.addChild(this._sprWindow);

		this._sprAName = new Sprite_ActorName();
		this.addChild(this._sprAName);
		this._sprAName.move(8,28);

		this._sprANameL = new Sprite_ActorNameLine();
		this.addChild(this._sprANameL);
		this._sprANameL.move(8,64);

		this._sprFootL = new Sprite_FooterLine();
		this.addChild(this._sprFootL);
		this._sprFootL.move(16,64 + 190 + 4);
	};

	Sprite_SkillWindowBase.prototype.setActor = function(actor) {
		//this.actor = actor;
		var index = actor.actor().class_icon + 1;
		this._sprAName.setFrame(0,36*index,160,36);
	}


	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_SkillWindow
	////////////////////////////////////////////////////////////////////////////////////
	function Sprite_SkillWindow() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_SkillWindow.prototype = Object.create(Sprite_Base.prototype);
	Sprite_SkillWindow.prototype.constructor = Sprite_SkillWindow;

	Sprite_SkillWindow.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
	    this.anchor.set(0.5,0);
	    this.z = 20;
	    this.bitmap = ImageManager.loadBitmap("img/battlehud/", "battle_bg_skill");
		this.setFrame(0,0,1,380);
		this.scale.x = Graphics.boxWidth + 200;
		this.move(Graphics.boxWidth/2, 10);
		//this.alpha = 198/255;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_FooterLine
	////////////////////////////////////////////////////////////////////////////////////
	function Sprite_FooterLine() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_FooterLine.prototype = Object.create(Sprite_Base.prototype);
	Sprite_FooterLine.prototype.constructor = Sprite_FooterLine;


	Sprite_FooterLine.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
		var w = Graphics.boxWidth-32;
		var h = 1;
		this.bitmap = new Bitmap(w,h);
		this.setFrame(0,0,w,h);
		this.move(0, 0);
		this._bmpLine = ImageManager.loadBitmap("img/battlehud/", "battle_line");
	    this.updateBitmap();
	    this._refresh = true;
	}

	Sprite_FooterLine.prototype.update = function() {
	    Sprite_Base.prototype.update.call(this);
	    if(this._refresh){
	    	this.updateBitmap();
	    	this._refresh = false;
		}
	};

	Sprite_FooterLine.prototype.updateBitmap = function() {
		var bitmap = this.bitmap;
	    var lineThick = 1;
	    bitmap.fillRect( 0, 0, this.width, 1, '#788484');
	}

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_ActorName
	////////////////////////////////////////////////////////////////////////////////////

	function Sprite_ActorName() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_ActorName.prototype = Object.create(Sprite_Base.prototype);
	Sprite_ActorName.prototype.constructor = Sprite_ActorName;

	Sprite_ActorName.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
	    this.z = 21;
	    this.bitmap = ImageManager.loadBitmap("img/battlehud/", "battle_name");
		this.setFrame(0,0,160,36);
		this.move(0, 0);
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_ActorNameLine
	////////////////////////////////////////////////////////////////////////////////////

	function Sprite_ActorNameLine() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_ActorNameLine.prototype = Object.create(Sprite_Base.prototype);
	Sprite_ActorNameLine.prototype.constructor = Sprite_ActorNameLine;

	Sprite_ActorNameLine.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
		var w = Graphics.boxWidth-16;
		var h = 8;
		this.bitmap = new Bitmap(w,h);
		this.setFrame(0,0,w,h);
		this.move(0, 0);
		this._bmpLine = ImageManager.loadBitmap("img/battlehud/", "battle_line");
	    this.updateBitmap();
	    this._refresh = true;
	}

	Sprite_ActorNameLine.prototype.update = function() {
	    Sprite_Base.prototype.update.call(this);
	    if(this._refresh){
	    	this.updateBitmap();
	    	this._refresh = false;
		}
	};

	Sprite_ActorNameLine.prototype.updateBitmap = function() {
		var bitmap = this.bitmap;
		var dotW = 6;
	    var lineH = 8;
	    bitmap.blt(this._bmpLine, dotW, 0, 10, lineH, dotW, 0, this.width - dotW*2, lineH);
	    bitmap.blt(this._bmpLine, 0, 0, dotW, lineH, 0, 0, dotW, lineH);
	    bitmap.blt(this._bmpLine, 60-dotW, 0, dotW, lineH, this.width - dotW, 0, dotW, lineH);
	}


	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_EquipWindowBase
	////////////////////////////////////////////////////////////////////////////////////

	function Sprite_EquipWindowBase() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_EquipWindowBase.prototype = Object.create(Sprite_Base.prototype);
	Sprite_EquipWindowBase.prototype.constructor = Sprite_EquipWindowBase;

	Sprite_EquipWindowBase.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
		this.actor = null;
		this._sprWindow = new Sprite_EquipWindow();
		this.addChild(this._sprWindow);

		this._sprAName = new Sprite_ActorName();
		this.addChild(this._sprAName);
		this._sprAName.move(8,28);

		this._sprANameL = new Sprite_ActorNameLine();
		this.addChild(this._sprANameL);
		this._sprANameL.move(8,64);

		this._sprFootL_1 = new Sprite_FooterLine();
		this.addChild(this._sprFootL_1);
		this._sprFootL_1.move(16,70 + 38);

		this._sprFootL_2 = new Sprite_FooterLine();
		this.addChild(this._sprFootL_2);
		this._sprFootL_2.move(16,70 + 38 + 80);

		this._sprFootL_3 = new Sprite_FooterLine();
		this.addChild(this._sprFootL_3);
		this._sprFootL_3.move(16,70 + 38 + 80 + 170);
	};

	Sprite_EquipWindowBase.prototype.setActor = function(actor) {
		//this.actor = actor;
		var index = actor.actor().class_icon + 1;
		this._sprAName.setFrame(0,36*index,160,36);
	}


	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_EquipWindow
	////////////////////////////////////////////////////////////////////////////////////
	function Sprite_EquipWindow() {
	    this.initialize.apply(this, arguments);
	}

	Sprite_EquipWindow.prototype = Object.create(Sprite_Base.prototype);
	Sprite_EquipWindow.prototype.constructor = Sprite_EquipWindow;

	Sprite_EquipWindow.prototype.initialize = function() {
		Sprite_Base.prototype.initialize.call(this);
	    this.anchor.set(0.5,0);
	    this.z = 20;
	    this.bitmap = ImageManager.loadBitmap("img/battlehud/", "battle_bg_equip");
		this.setFrame(0,0,1,432);
		this.scale.x = Graphics.boxWidth + 200;
		this.move(Graphics.boxWidth/2, 10);
		//this.alpha = 198/255;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Sprite_Damage
	////////////////////////////////////////////////////////////////////////////////////
	Sprite_Damage.prototype.digitHeight = function() {
	    return this._damageBitmap ? this._damageBitmap.height / 7 : 0;
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleHelp
	////////////////////////////////////////////////////////////////////////////////////

	function Window_BattleHelp() {
	    this.initialize.apply(this, arguments);
	}

	Window_BattleHelp.prototype = Object.create(Window_Help.prototype);
	Window_BattleHelp.prototype.constructor = Window_BattleHelp;


	Window_BattleHelp.prototype.initialize = function() {
	    var width = Graphics.boxWidth;
	    var height = this.windowHeight();
	    Window_Base.prototype.initialize.call(this, 0, 0, width, height);

	    this._windowFrameSprite.alpha = 0;
	    this._windowBackSprite.alpha = 0;
	};

	Window_BattleHelp.prototype.windowHeight = function() {
	    return 90;
	};

	Window_BattleHelp.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_BattleHelp.prototype.lineHeight = function() {
	    return 38;
	};

	Window_BattleHelp.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_BattleHelp.prototype.standardPadding = function() {
	    return 8;
	};

	Window_BattleHelp.prototype.textPadding = function() {
	    return 24;
	};

	Window_BattleHelp.prototype.setLogWindow = function(logwindow) {
		this._logWindow = logwindow;
	};


	var _JPT_BattleUI_Window_BattleHelp_show = Window_BattleHelp.prototype.show;
	Window_BattleHelp.prototype.show = function() {
		_JPT_BattleUI_Window_BattleHelp_show.call(this);
		if(this._logWindow && this._logWindow.numLines() > 0){
			this._logWindow.hide();
		}
	};

	var _JPT_BattleUI_Window_BattleHelp_hide = Window_BattleHelp.prototype.hide;
	Window_BattleHelp.prototype.hide = function() {
		_JPT_BattleUI_Window_BattleHelp_hide.call(this);
		if(this._logWindow && this._logWindow.numLines() > 0){
			this._logWindow.show();
		}
	};

	Window_BattleHelp.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleSkill
	////////////////////////////////////////////////////////////////////////////////////

	Window_BattleSkill.prototype.lineHeight = function() {
	    return 30;
	};

	Window_BattleSkill.prototype.standardPadding = function() {
	    return 12;
	};

	Window_BattleSkill.prototype.textPadding = function() {
	    return 16;
	};

	Window_BattleSkill.prototype.itemPadding = function() {
	    return 4;
	};

	Window_BattleSkill.prototype.spacing = function() {
	    return 20;
	};

	Window_BattleSkill.prototype.includes = function(item) {
	    return item && item.stypeId === this._stypeId && (item.occasion === 0 || item.occasion === 1);
	};


	var _JPT_BattleUI_Window_BattleSkill_show = Window_BattleSkill.prototype.show;
	Window_BattleSkill.prototype.show = function() {
	    _JPT_BattleUI_Window_BattleSkill_show.call(this);
	    this._windowFrameSprite.alpha = 0;
	    this._windowBackSprite.alpha = 0;
	    this._helpWindow.x = this.x;
	    this._helpWindow.y = this.y + this.height;
	    if(this._sprBackWindow)this._sprBackWindow.show();
	};

	var _JPT_BattleUI_Window_BattleSkill_setActor = Window_BattleSkill.prototype.setActor;
	Window_BattleSkill.prototype.setActor = function(actor) {
		_JPT_BattleUI_Window_BattleSkill_setActor.call(this,actor);
	    if (this._sprBackWindow._actor !== actor) {
	    	this._sprBackWindow.setActor( actor );
	    }
	};

	var _JPT_BattleUI_Window_BattleSkill_hide = Window_BattleSkill.prototype.hide;
	Window_BattleSkill.prototype.hide = function() {
		_JPT_BattleUI_Window_BattleSkill_hide.call(this);
	    if(this._sprBackWindow)this._sprBackWindow.hide();
	};

	Window_BattleSkill.prototype.setBackWindow = function(sprBackWindow) {
		this._sprBackWindow = sprBackWindow;
	};

	Window_BattleSkill.prototype.drawItem = function(index) {
	    var skill = this._data[index];
	    if (skill) {
	        var costWidth = this.costWidth();
	        var rect = this.itemRectForText(index);
	        this.changePaintOpacity(this.isEnabled(skill));
	        this.drawItemName(skill, rect.x, rect.y, rect.width - Math.floor(costWidth*1.2) );
	        this.drawSkillCost(skill, rect.x, rect.y, rect.width);
	        this.changePaintOpacity(1);
	    }
	};

	Window_BattleSkill.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRectForText(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleItem
	////////////////////////////////////////////////////////////////////////////////////

	Window_BattleItem.prototype.lineHeight = function() {
	    return 30;
	};

	Window_BattleItem.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_BattleItem.prototype.standardPadding = function() {
	    return 12;
	};

	Window_BattleItem.prototype.textPadding = function() {
	    return 16;
	};

	Window_BattleItem.prototype.itemPadding = function() {
	    return 4;
	};

	Window_BattleItem.prototype.spacing = function() {
	    return 20;
	};

	Window_BattleItem.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_BattleItem.prototype.includes = function(item) {
	    return item && item.stypeId === this._stypeId && (item.occasion === 0 || item.occasion === 1);
	};


	var _JPT_BattleUI_Window_BattleItem_show = Window_BattleItem.prototype.show;
	Window_BattleItem.prototype.show = function() {
	    _JPT_BattleUI_Window_BattleSkill_show.call(this);
	    this._windowFrameSprite.alpha = 0;
	    this._windowBackSprite.alpha = 0;
	    this._helpWindow.x = this.x;
	    this._helpWindow.y = this.y + this.height;
	    if(this._sprBackWindow)this._sprBackWindow.show();
	};

	Window_BattleItem.prototype.setActor = function(actor) {
	    if (this._sprBackWindow._actor !== actor) {
	    	this._sprBackWindow.setActor( actor );
	    }
	};

	var _JPT_BattleUI_Window_BattleItem_hide = Window_BattleItem.prototype.hide;
	Window_BattleItem.prototype.hide = function() {
		_JPT_BattleUI_Window_BattleSkill_hide.call(this);
	    if(this._sprBackWindow)this._sprBackWindow.hide();
	};

	Window_BattleItem.prototype.setBackWindow = function(sprBackWindow) {
		this._sprBackWindow = sprBackWindow;
	};

	Window_BattleItem.prototype.drawItem = function(index) {
	    var item = this._data[index];
	    if (item) {
	        var numberWidth = this.numberWidth();
	        var rect = this.itemRectForText(index);
	        this.changePaintOpacity(this.isEnabled(item));
	        this.drawItemName(item, rect.x, rect.y, rect.width - numberWidth);
	        this.drawItemNumber(item, rect.x, rect.y, rect.width);
	        this.changePaintOpacity(1);
	    }
	};

	Window_BattleItem.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};


	Window_BattleItem.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRectForText(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleEquipSlot
	////////////////////////////////////////////////////////////////////////////////////

	function Window_BattleEquipSlot() {
	    this.initialize.apply(this, arguments);
	}

	Window_BattleEquipSlot.prototype = Object.create(Window_EquipSlot.prototype);
	Window_BattleEquipSlot.prototype.constructor = Window_BattleEquipSlot;

	Window_BattleEquipSlot.prototype.standardPadding = function() {
	    return 0;
	};

	Window_BattleEquipSlot.prototype.textPadding = function() {
	    return 50;
	};

	Window_BattleEquipSlot.prototype.maxItems = function() {
	    return 1;
	};

	var _JPT_BattleUI_Window_BattleEquipSlot_setActor = Window_BattleEquipSlot.prototype.setActor;
	Window_BattleEquipSlot.prototype.setActor = function(actor) {
		_JPT_BattleUI_Window_BattleEquipSlot_setActor.call(this,actor);
	    if (this._sprBackWindow._actor !== actor) {
	    	this._sprBackWindow.setActor( actor );
	    }
	};

	var _JPT_BattleUI_Window_BattleEquipSlot_show = Window_BattleEquipSlot.prototype.show;
	Window_BattleEquipSlot.prototype.show = function() {

		this._slot_index = this._actor.EquippableSlot();
		_JPT_BattleUI_Window_BattleEquipSlot_show.call(this);
	    this._windowFrameSprite.alpha = 0;
	    this._windowBackSprite.alpha = 0;
	    this._helpWindow.x = this.x;
	    this._helpWindow.y = this.y + this.height;
	    if(this._sprBackWindow)this._sprBackWindow.show();

	    this.refresh();

	};

	var _JPT_BattleUI_Window_BattleEquipSlot_hide = Window_BattleEquipSlot.prototype.hide;
	Window_BattleEquipSlot.prototype.hide = function() {
		_JPT_BattleUI_Window_BattleEquipSlot_hide.call(this);
	    if(this._sprBackWindow)this._sprBackWindow.hide();
	};

	Window_BattleEquipSlot.prototype.setBackWindow = function(sprBackWindow) {
		this._sprBackWindow = sprBackWindow;
	};


	Window_BattleEquipSlot.prototype.drawItem = function(e_index) {
		if(this._actor){
			var index = this._slot_index;
	        var rect = this.itemRectForText(e_index);
	        this.changeTextColor(this.systemColor());
	        this.changePaintOpacity(this.isEnabled(index));
	        this.drawText(this.slotName(index), rect.x, rect.y, 90, this.lineHeight());
	        this.drawItemName(this._actor.menuEquipItem(index), rect.x + 90, rect.y);
	        this.changePaintOpacity(true);
		}
	};


	Window_BattleEquipSlot.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRectForText(this.index());
	        this.setCursorRect(rect.x + 80, rect.y + this.itemPadding(), rect.width - 80, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	Window_BattleEquipSlot.prototype.update = function() {
	    Window_Selectable.prototype.update.call(this);
	    if (this._itemWindow && this._slot_index >= 0) {
	        this._itemWindow.setSlotId(this._slot_index);
	    }
	};

	Window_BattleEquipSlot.prototype.slotIndex = function() {
		return this._actor.menuEquips(this._slot_index);
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleEquipStatus
	////////////////////////////////////////////////////////////////////////////////////

	function Window_BattleEquipStatus() {
	    this.initialize.apply(this, arguments);
	}

	Window_BattleEquipStatus.prototype = Object.create(Window_EquipStatus.prototype);
	Window_BattleEquipStatus.prototype.constructor = Window_BattleEquipStatus;

	Window_BattleEquipStatus.prototype.windowWidth = function() {
	    return Graphics.boxWidth;
	};

	Window_BattleEquipStatus.prototype.windowHeight = function() {
	    return this.fittingHeight(this.numVisibleRows());
	};

	Window_BattleEquipStatus.prototype.numVisibleRows = function() {
	    return 3;
	};

	Window_BattleEquipStatus.prototype.refresh = function() {
	    this.contents.clear();
	    if (this._actor) {
	    	this.contents.fontSize = 18;
	        for (var i = 0; i < 6; i++) {
	        	var x = 30 + Math.floor(i/3)*400;
	        	var y = this.lineHeight() * (i%3);
	            this.drawItem(x, y, 2 + i);
	        }
	    }
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleEquipItem
	////////////////////////////////////////////////////////////////////////////////////

	function Window_BattleEquipItem() {
	    this.initialize.apply(this, arguments);
	}

	Window_BattleEquipItem.prototype = Object.create(Window_EquipItem.prototype);
	Window_BattleEquipItem.prototype.constructor = Window_BattleEquipItem;

	Window_BattleEquipItem.prototype.standardPadding = function() {
	    return 12;
	};

	var _JPT_BattleUI_Window_BattleEquipItem_show = Window_BattleEquipItem.prototype.show;
	Window_BattleEquipItem.prototype.show = function() {
		_JPT_BattleUI_Window_BattleEquipItem_show.call(this);
	    this._helpWindow.x = this.x;
	    this._helpWindow.y = this.y + this.height;
	};

	Window_BattleEquipItem.prototype.drawItem = function(index) {
	    var item = this._data[index];
	    if (item) {
	        var numberWidth = this.numberWidth();
	        var rect = this.itemRectForText(index);
	        this.changePaintOpacity(this.isEnabled(item));
	        this.drawItemName(item, rect.x + this.textPadding(), rect.y, rect.width - numberWidth);
	        this.drawItemNumber(item, rect.x, rect.y, rect.width);
	        this.changePaintOpacity(1);
	    }else{
	    	if(this._slotId==1){
		    	var rect = this.itemRect(index);
		        rect.width -= this.textPadding();
		    	this.drawText('', rect.x + this.textPadding(), rect.y, rect.width );
	    	}else{
		    	var rect = this.itemRect(index);
		        rect.width -= this.textPadding();
		    	this.drawText('はずす', rect.x + this.textPadding(), rect.y, rect.width );
	    	}

	    }
	};

	Window_BattleEquipItem.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRectForText(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

}());



