//=============================================================================
// JPT_CampMenu
//=============================================================================

var Imported = Imported || {};
Imported['JPT_CampMenu'] = 1.00;

/*:ja
 * @plugindesc 野営メニュー
 * @author Kawamura
 *
 * @param Preparation Command Name
 * @desc 調合 項目名
 * @default 調合
 *
 * @param Make Accessory Command Name
 * @desc アクセサリー作成 項目名
 * @default アクセサリ作成
 *
 * @param Language Learning Command Name
 * @desc 言語習得 項目名
 * @default 言語習得
 *
 * @param Recipe Learning Command Name
 * @desc レシピ習得 項目名
 * @default レシピ習得
 *
 * @param Magic Learning Command Name
 * @desc 魔法習得 項目名
 * @default 魔法習得
 *
 * @param Rest Command Name
 * @desc 寝る 項目名
 * @default 寝る
 *
 * @param Camp Header
 * @desc キャンプヘッダ
 * @default 何をしますか？
 *
 * @param Make Accessory Open Switch ID
 * @desc アクセサリ作成解放スイッチID
 * @default 19
 *
 * @param Window Width
 * @desc ウィンドウ幅
 * @default 240
 *
 * @help
 * キャンプメニューを実装します
 *
 * 依存関係
 * JPT_Synthesis.js
 *
 *-------------------------------------------
 * 野営呼び出しコマンド
 *-------------------------------------------
 * CampMenu
 *
 */



(function() {

	'use strict';

	var parameters = PluginManager.parameters('JPT_CampMenu');

	var preparationCommandName		= parameters['Preparation Command Name'] || '調合';
	var makeAccessoryCommandName	= parameters['Make Accessory Command Name'] || 'アクセサリ作成';
	var languageLearningCommandName	= parameters['Language Learning Command Name'] || '言語習得';
	var recipeLearningCommandName	= parameters['Recipe Learning Command Name'] || 'レシピ習得';
	var magicLearningCommandName	= parameters['Magic Learning Command Name'] || '魔法習得';
	var restCommandName				= parameters['Rest Command Name'] || '寝る';
	var campHeader					= parameters['Camp Header'] || '何をしますか？';
	var makeAccessoryOpenSwitchID	= parseInt(parameters['Make Accessory Open Switch ID'] || 19);
	var campWindowWidth				= parseInt(parameters['Window Width'] || 240);


	////////////////////////////////////////////////////////////////////////////////////
    // Game_Interpreter
	////////////////////////////////////////////////////////////////////////////////////

    var _JPT_CampMenu_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
    	_JPT_CampMenu_Game_Interpreter_pluginCommand.call(this, command, args);

    	if(command==='CampMenu'){
    		SceneManager.snapBlurOff();
    		SceneManager.push(Scene_Camp);
    	}
    };

    ////////////////////////////////////////////////////////////////////////////////////
    // TextManager
	////////////////////////////////////////////////////////////////////////////////////

	Object.defineProperties(TextManager, {
		preparation			: { get: function() { return preparationCommandName; },		configurable: true },
		makeAccessory		: { get: function() { return makeAccessoryCommandName; },		configurable: true },
		languageLearning	: { get: function() { return languageLearningCommandName; },	configurable: true },
		recipeLearning		: { get: function() { return recipeLearningCommandName; },	configurable: true },
		magicLearning		: { get: function() { return magicLearningCommandName; },		configurable: true },
		rest				: { get: function() { return restCommandName; },				configurable: true },
	});


	////////////////////////////////////////////////////////////////////////////////////
    // DataManager
	////////////////////////////////////////////////////////////////////////////////////
    DataManager.getMakeAccessorySwitch = function() {
		if(makeAccessoryOpenSwitchID > 0){
			return $gameSwitches.value(makeAccessoryOpenSwitchID);
		}
		return false;
	}

    ////////////////////////////////////////////////////////////////////////////////////
    // SceneManager
	////////////////////////////////////////////////////////////////////////////////////
    SceneManager._blur = true;

    SceneManager.snapBlurOff = function(){
    	SceneManager._blur = false;
    }

	var _JPT_CampMenu_SceneManager_snapForBackground = SceneManager.snapForBackground;
    SceneManager.snapForBackground = function() {
        if(SceneManager._blur){
        	_JPT_CampMenu_SceneManager_snapForBackground.call(this);
        }else{
        	this._backgroundBitmap = this.snap();
        }
        SceneManager._blur = true;
    };

    ////////////////////////////////////////////////////////////////////////////////////
    // Scene_Camp
	////////////////////////////////////////////////////////////////////////////////////
    function Scene_Camp() {
        this.initialize.apply(this, arguments);
    }

    Scene_Camp.prototype = Object.create(Scene_Base.prototype);
    Scene_Camp.prototype.constructor = Scene_Camp;

    Scene_Camp.prototype.initialize = function() {
    	Scene_Base.prototype.initialize.call(this);
    };

    Scene_Camp.prototype.create = function() {
    	Scene_Base.prototype.create.call(this);
    	this.createBackground();
    	this.createWindowLayer();
    	this.createCommandWindow();
    };

    Scene_Camp.prototype.start = function() {
    	Scene_Base.prototype.start.call(this);
    	this._commandWindow.refresh();
    };

    Scene_Camp.prototype.createBackground = function() {
        this._backgroundSprite = new Sprite();
        this._backgroundSprite.bitmap = SceneManager.backgroundBitmap();
        this.addChild(this._backgroundSprite);
    };

    Scene_Camp.prototype.createCommandWindow = function() {
        this._commandWindow = new Window_CampCommand(50, 40);
        this._commandWindow.setHandler('preparation',		this.commandPreparation.bind(this));
        if(DataManager.getMakeAccessorySwitch()){
        	this._commandWindow.setHandler('makeAccessory',		this.commandMakeAccessory.bind(this));
        }
        //this._commandWindow.setHandler('recipeLearning',	this.popScene.bind(this));
        //this._commandWindow.setHandler('languageLearning',	this.popScene.bind(this));
        //this._commandWindow.setHandler('magicLearning',		this.popScene.bind(this));
        this._commandWindow.setHandler('rest',				this.popScene.bind(this));
        //this._commandWindow.setHandler('cancel',    this.popScene.bind(this));
        this.addWindow(this._commandWindow);
    };

    Scene_Camp.prototype.commandPreparation = function() {
        SceneManager.push(Scene_Preparation);
    };

    Scene_Camp.prototype.commandMakeAccessory = function() {
        SceneManager.push(Scene_MakeAccessory);
    };


    ////////////////////////////////////////////////////////////////////////////////////
    // Window_CampCommand
	////////////////////////////////////////////////////////////////////////////////////
    function Window_CampCommand() {
        this.initialize.apply(this, arguments);
    }

    Window_CampCommand.prototype = Object.create(Window_Command.prototype);
    Window_CampCommand.prototype.constructor = Window_CampCommand;

    Window_CampCommand.prototype.initialize = function(x, y) {
        this.iconIndexes = [];

    	Window_Command.prototype.initialize.call(this, x, y);


        //this.selectLast();
    };

    Window_CampCommand._lastCommandSymbol = null;

    Window_CampCommand.initCommandPosition = function() {
        this._lastCommandSymbol = null;
    };

    Window_CampCommand.prototype.windowWidth = function() {
        return campWindowWidth;
    };

    Window_CampCommand.prototype.windowHeight = function() {
        return 200;//320;
    };

    Window_CampCommand.prototype.numVisibleRows = function() {
        return this.maxItems();
    };

    Window_CampCommand.prototype.makeCommandList = function() {
    	this.addMainCommands();
    };

    Window_CampCommand.prototype.standardFontSize = function() {
	    return 17;
	};

	Window_CampCommand.prototype.standardPadding = function() {
	    return 24;
	};

	Window_CampCommand.prototype.lineHeight = function() {
	    return 40;
	};

	Window_CampCommand.prototype.textPadding = function() {
	    return 0;
	};

	Window_CampCommand.prototype.itemPadding = function() {
	    return 6;
	};

    Window_CampCommand.prototype.standardFontFace = function() {
		 return 'GameFont';
	};


	Window_CampCommand.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


    Window_CampCommand.prototype.addMainCommands = function() {
    	var i = 0;
        this.addCommand(TextManager.preparation,		'preparation', true);
        this.iconIndexes[i] = 0;
        i++;
        if(DataManager.getMakeAccessorySwitch()){
        	this.addCommand(TextManager.makeAccessory,		'makeAccessory', true);
            this.iconIndexes[i] = 1;
            i++;
        }
        //this.addCommand(TextManager.recipeLearning,		'recipeLearning', true);
        //this.addCommand(TextManager.languageLearning,	'languageLearning', true);
        //this.addCommand(TextManager.magicLearning,		'magicLearning', true);
        this.addCommand(TextManager.rest,				'rest', true);
        this.iconIndexes[i] = 5;
    };

	Window_CampCommand.prototype.refresh = function(){
		Window_Command.prototype.refresh.call(this);
		this.drawTitle(0,0);
	}

	Window_CampCommand.prototype.drawTitle = function(x,y){
		var width = this.windowWidth();
		var height = this.contents.fontSize;

		this.contents.fontSize = 18;
		var texWidth = this.contents.measureTextWidth(campHeader);
		this.contents.drawText(campHeader, Math.floor(this.contents.width/2) - Math.floor(texWidth/2) , y, texWidth, this.contents.fontSize, 'left');

		var lineThick = 1;
	    var lineY = y + height + 3;
	    this.contents.fillRect( x, lineY, width, lineThick, '#788484');
	}


	Window_CampCommand.prototype.itemRect = function(index) {
	    var rect = new Rectangle();
	    var maxCols = this.maxCols();
	    rect.width = this.itemWidth();
	    rect.height = this.itemHeight();
	    rect.x = index % maxCols * (rect.width + this.spacing()) - this._scrollX;
	    rect.y = 34 + Math.floor(index / maxCols) * rect.height - this._scrollY;
	    return rect;
	};


    Window_CampCommand.prototype.drawItem = function(index) {
    	var rect = this.itemRect(index);
    	var bitmap = ImageManager.loadBitmap("img/menus/", "menu_inn_icon");
    	this.contents.blt( bitmap, 24*this.iconIndexes[index],0, 24,24, rect.x + 18, rect.y + 8 );
    	this.drawText(this.commandName(index), rect.x + 60, rect.y, rect.width - 60, 'left');
	};


	Window_CampCommand.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRect(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};



}());
