//=============================================================================
// JPT_EnemyClassification
//=============================================================================

var Imported = Imported || {};
Imported['JPT_EnemyClassification'] = 1.00;

/*:ja
 * @plugindesc モンスター系統の設定
 * 系統によるダメージ、非ダメージ補正の実装
 * @author Kawamura
 *
 * @help
 * メモ記述の際はスペースを入れないでください
 *
 *----------------------------------------------------------------------------
 * 敵に系統設定する
 *----------------------------------------------------------------------------
 * 敵キャラのメモに以下を記述
 *
 * <EnemyClass:（系統名）>
 *
 * ex.ドラゴン系統を設定
 * <EnemyClass:Dragon>
 *
 * ex.ドラゴン,悪魔系統を設定
 * <EnemyClass:Dragon,Demon>
 *
 *----------------------------------------------------------------------------
 * 敵に対する効果設定
 *----------------------------------------------------------------------------
 *
 *----------------------------------------------------------------------------
 * スキル、ステート、武器、防具、職業で自動効果発動
 *
 * スキルの場合、習得していれば
 * ステートは付加時
 * 武器、防具、職業は装備時
 *
 * 系統からのダメージ軽減(マイナス設定の場合増加)
 * <EC_ReduceDamage=（系統名）:（軽減率）>
 *
 * ex.ドラゴン系統からのダメージ20%軽減
 * <EC_ReduceDamage=Dragon:20>
 *
 * 系統へのダメージ増加 パッシブ (マイナス設定の場合軽減)
 * <EC_IncreaseDamage=（系統名）:（増加率）>
 *
 * ex.ドラゴン系統へのダメージ20%増加
 * <EC_IncreaseDamage=Dragon:20>
 *
 * 複数の系統を列挙できます
 *
 * ----------------------------------------------------------------------------
 * スキル、アイテムで使用時に効果を出したい場合
 *
 * 系統へのダメージ増加 使用時 (マイナス設定の場合軽減)
 * <EC_IncreaseDamage_Use=（系統名）:（増加率）>
 *
 * ex.ドラゴン系統へのスキルダメージ20%増加
 * <EC_IncreaseDamage_Use=Dragon:20>
 *
 * 複数の系統を列挙できます
 *
 * ----------------------------------------------------------------------------
 * ※ 全ての効果率を加算してダメージ計算に使用します
 *
 * 100%を基準に軽減効果の下限を0%とします
 * 増加による上限はありません
 *
 */

/*
 *
 */

(function() {

	'use strict';

	//var parameters = PluginManager.parameters('JPT_EnemyClassification');

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Action
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_EnemyClass_Game_Action_makeDamageValue = Game_Action.prototype.makeDamageValue;
	Game_Action.prototype.makeDamageValue = function(target, critical) {
		var damage = _JPT_EnemyClass_Game_Action_makeDamageValue.call(this,target, critical);

		var reduce = 100;
		if(this.subject().isEnemy() && target.isActor()){
			//enemyからの攻撃の場合
			if(this.subject().enemy().meta['EnemyClass']){
				var ec = this.subject().enemy().meta['EnemyClass'].split(',');
				for (var i=0,max=ec.length;i<max;i++) {
					reduce -= target.getECReduceDamage(ec[i]);
				}
				if(reduce < 0){
					reduce = 0;
				}
			}
		}else if(this.subject().isActor() && target.isEnemy()){
			//enemyへの攻撃の場合
			if(target.enemy().meta['EnemyClass']){
				var ec = target.enemy().meta['EnemyClass'].split(',');
				for (var i=0,max=ec.length;i<max;i++) {
					reduce += this.subject().getECIncreaseDamage(ec[i],this);
				}
			}
		}
		damage = damage*reduce/100;
		return Math.round(damage);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Actor
	////////////////////////////////////////////////////////////////////////////////////

	Game_Actor.prototype.getECReduceDamage = function(enemyClass) {
		var tos = this.traitObjects().concat(this.skills());
		var reduce = 0;
		var mName = 'EC_ReduceDamage='+enemyClass;
		for (var i=0,max=tos.length;i<max;i++) {
            var to = tos[i];
            if(to.meta[mName]){
           		reduce += parseInt( to.meta[mName],10 );
            }
		}
		return reduce;
	}

	Game_Actor.prototype.getECIncreaseDamage = function(enemyClass,action) {
		var tos = this.traitObjects().concat(this.skills());
		var reduce = 0;
		var mName = 'EC_IncreaseDamage='+enemyClass;
		for (var i=0,max=tos.length;i<max;i++) {
            var to = tos[i];
            if(to.meta[mName]){
           		reduce += parseInt( to.meta[mName],10 );
            }
		}
		if(action.item()){
			mName = 'EC_IncreaseDamage_Use='+enemyClass;
			if(	action.item().meta[mName] ){
           		reduce += parseInt( action.item().meta[mName],10 );
            }
		}
		return reduce;

	}

}());
