﻿//=============================================================================
// JPT_EventSight.js
//=============================================================================

/*:ja
 * @plugindesc マップでプレイヤーがイベントの一定範囲内(ひし形)に入るとスイッチをON
 * @author
 *
 * @param in range tag name
 * @desc イベントのメモ欄で使用するタグ名
 * 一定範囲内に入った場合
 * @default range
 *
 * @param out range tag name
 * @desc イベントのメモ欄で使用するタグ名
 * 一定範囲外に出た場合
 * @default out_range
 *
 * @help このプラグインにはプラグインコマンドはありません。
 *
 * イベントのメモ:
 *   <range:n,s>
 *   # 距離n以内にプレイヤーを発見するとsスイッチがON
 *	 # n : 距離 (タイル単位)
 *	 # s : A-D (セルフスイッチ), 1-999 (スイッチ)
 *
 */

//-----------------------------------------------------------------------------

(function(_global) {
	var N = 'JPT_EventSight';

	var param = PluginManager.parameters(N);
	var jpt_event_sight_in_tag_name = param['in range tag name'] || "range";

	var jpt_event_sight_out_tag_name = param['out range tag name'] || "out_range";


	function _canPass(x, y, d) {
		var x2 = $gameMap.roundXWithDirection(x, d);
		var y2 = $gameMap.roundYWithDirection(y, d);
		if (!$gameMap.isValid(x2, y2)) {
			return false;
		}
		if (!this.isMapPassable(x, y, d)) {
			return false;
		}
		if (Game_CharacterBase.prototype.isCollidedWithEvents.call(this, x2, y2) || this.isCollidedWithVehicles(x2, y2)) {
			return false;
		}
		return true;
	};

	function _check(_d, _x, _y, type) {

		var range =Math.abs(this.x - _x) + Math.abs(this.y - _y);
		var dir = this._directionFix ? this._originalDirection : this._direction;

		if(type==='in'){
			if(range <= _d){
				return true;
			}
		}else if(type==='out'){
			if(range > _d){
				return true;
			}
		}

		return false;
	};

	function _checkAll(_d, type) {
		return _check.call(this, _d, $gamePlayer.x, $gamePlayer.y, type);
	}

	var _Game_Event_updateSelfMovement = Game_Event.prototype.updateSelfMovement;
	Game_Event.prototype.updateSelfMovement = function() {
		_Game_Event_updateSelfMovement.call(this);
		if(!this._erased ){
			if(!this.eventIn()){
				this.eventOut();
			}
		}
	};


	Game_Event.prototype.eventIn = function() {
		var m = this.event().meta[jpt_event_sight_in_tag_name];
		if (m) {
			var r = m.match(/\s*(\d+),\s*([A-D])\s*/);
			if (r) {
				var k = [this._mapId, this._eventId, r[2]];
				if (!$gameSelfSwitches._data[k]) {
					if (_checkAll.call(this, Number(r[1]), 'in' ) ) {
						$gameSelfSwitches.setValue(k, r);
						return true;
					}
				}
				return false;
			}
			r = m.match(/\s*(\d+),\s*(\d+)\s*/);
			if (r) {
				var k = Number(r[2]);
				if (!$gameSwitches.value(k)) {
					if (_checkAll.call(this, Number(r[1]), 'in' ) ) {
						$gameSwitches.setValue(k, r);
						return true;
					}
				}
			}
		}
		return false;
	};

	Game_Event.prototype.eventOut = function() {
		var m = this.event().meta[jpt_event_sight_out_tag_name];
		if (m) {
			var r = m.match(/\s*(\d+),\s*([A-D])\s*/);
			if (r) {
				var k = [this._mapId, this._eventId, r[2]];
				if (!$gameSelfSwitches._data[k]) {
					if (_checkAll.call(this, Number(r[1]), 'out' ) ) {
						$gameSelfSwitches.setValue(k, r);
						return true;
					}
				}
				return false;
			}
			r = m.match(/\s*(\d+),\s*(\d+)\s*/);
			if (r) {
				var k = Number(r[2]);
				if (!$gameSwitches.value(k)) {
					if (_checkAll.call(this, Number(r[1]), 'out' ) ) {
						$gameSwitches.setValue(k, r);
						return true;
					}
				}
			}
		}
		return false;
	};

})(this);
