﻿//=============================================================================
// JPT_EventSightArea.js
//=============================================================================

/*:ja
 * @plugindesc マップでプレイヤーがイベントの一定範囲内(矩形)に入るとスイッチをON
 * @author
 *
 * @param in area tag name
 * @desc イベントのメモ欄で使用するタグ名
 * 一定範囲内に入った場合
 * @default in_area
 *
 * @param out area tag name
 * @desc イベントのメモ欄で使用するタグ名
 * 一定範囲外に出た場合
 * @default out_area
 *
 * @help このプラグインにはプラグインコマンドはありません。
 *
 * イベントのメモ:
 *   <range:x,y,s>
 *   # イベントを中心に ±x, ±y の範囲に入ったら(出たら)スイッチON
 *	 # x : x距離 (タイル単位)
 *	 # y : y距離 (タイル単位)
 *	 # s : A-D (セルフスイッチ), 1-999 (スイッチ)
 *
 *   <in_area:2,2,s> の場合の矩形
 *
 *	□□□□□
 *	□□□□□
 *	□□■□□
 *	□□□□□
 *	□□□□□
 *
 *
 */

//-----------------------------------------------------------------------------

(function(_global) {
	var N = 'JPT_EventSightArea';

	var param = PluginManager.parameters(N);
	var jpt_event_sight_in_area_tag_name = param['in area tag name'] || "in_area";

	var jpt_event_sight_out_area_tag_name = param['out area tag name'] || "out_area";


	function _check(_x, _y, _px, _py, type) {

		var ex = Math.abs(this.x - _px);
		var ey = Math.abs(this.y - _py);

		if( ex > _x || ey > _y ){
			if(type==='out'){
				return true;
			}
		}else{
			if(type==='in'){
				return true;
			}
		}
		return false;
	};

	function _checkAll(_x, _y , type) {
		return _check.call(this, _x, _y, $gamePlayer.x, $gamePlayer.y, type);
	}

	var _JPT_EventSightArea_Game_Event_updateSelfMovement = Game_Event.prototype.updateSelfMovement;
	Game_Event.prototype.updateSelfMovement = function() {
		_JPT_EventSightArea_Game_Event_updateSelfMovement.call(this);
		if(!this._erased ){
			if(!this.eventInArea()){
				this.eventOutArea();
			}
		}
	};


	Game_Event.prototype.eventInArea = function() {
		var m = this.event().meta[jpt_event_sight_in_area_tag_name];
		if (m) {
			var r = m.match(/\s*(\d+),\s*(\d+),\s*([A-D])\s*/);
			if (r) {
				var k = [this._mapId, this._eventId, r[3]];
				if (!$gameSelfSwitches._data[k]) {
					if (_checkAll.call(this, Number(r[1]), Number(r[2]), 'in' ) ) {
						$gameSelfSwitches.setValue(k, r);
						return true;
					}
				}
				return false;
			}
			r = m.match(/\s*(\d+),\s*(\d+),\s*(\d+)\s*/);
			if (r) {
				var k = Number(r[3]);
				if (!$gameSwitches.value(k)) {
					if (_checkAll.call(this, Number(r[1]), Number(r[2]), 'in' ) ) {
						$gameSwitches.setValue(k, r);
						return true;
					}
				}
			}
		}
		return false;
	};

	Game_Event.prototype.eventOutArea = function() {
		var m = this.event().meta[jpt_event_sight_out_area_tag_name];
		if (m) {
			var r = m.match(/\s*(\d+),\s*(\d+),\s*([A-D])\s*/);
			if (r) {
				var k = [this._mapId, this._eventId, r[3]];
				if (!$gameSelfSwitches._data[k]) {
					if (_checkAll.call(this, Number(r[1]), Number(r[2]), 'out' ) ) {
						$gameSelfSwitches.setValue(k, r);
						return true;
					}
				}
				return false;
			}
			r = m.match(/\s*(\d+),\s*(\d+),\s*(\d+)\s*/);
			if (r) {
				var k = Number(r[3]);
				if (!$gameSwitches.value(k)) {
					if (_checkAll.call(this, Number(r[1]), Number(r[2]), 'out' ) ) {
						$gameSwitches.setValue(k, r);
						return true;
					}
				}
			}
		}
		return false;
	};

})(this);
