//=============================================================================
// JPT_MainMenu
//=============================================================================

var Imported = Imported || {};
Imported['JPT_MainMenu'] = 1.00;

/*:ja
 * @plugindesc カスタムメインメニュー
 * @author Kawamura
 *
 * @param FiroMove_Unlock_SwitchID
 * @desc フィーロに任せて移動 アンロック用のスイッチID
 * @default 500
 *
 * @param FiroMove_Enable_SwitchID
 * @desc フィーロに任せて移動 有効化スイッチID
 * @default 501
 *
 * @param FiroMove_MapID_VariableID
 * @desc フィーロに任せて移動 移動先マップID用の変数ID
 * @default 198
 *
 * @param FiroMove_X_VariableID
 * @desc フィーロに任せて移動 移動先X座標用の変数ID
 * @default 199
 *
 * @param FiroMove_Y_VariableID
 * @desc フィーロに任せて移動 移動先Y座標用の変数ID
 * @default 200
 *
 * @param FiroMove_MenuName
 * @desc フィーロに任せて移動 メニュー項目名
 * @default フィーロに任せて移動
 *
 * @param Magic_MenuName
 * @desc 魔法 メニュー項目名
 * @default 魔法
 *
 * @param None_ItemName
 * @desc なし アイテム無し項目名
 * @default なし
 *
 * @param Off_EquipName
 * @desc はずす 装備を外す項目名
 * @default はずす
 *
 * @param Auto Save
 * @type boolean
 * @on ON
 * @off OFF
 * @desc オートセーブ機能のON/OFF
 * OFF - false     ON - true
 * @default false
 *
 * @help
 * 「盾の勇者」カスタムメインメニュープラグインです
 * 依存関係
 * JPT_SkillPoint.js
 * YEP_MessageCore.js
 *
 *--------------------------------------------------------------------------------
 * アクターへの記述
 *--------------------------------------------------------------------------------
 * メモにバストアップとキャラクターのアイコンの指定を記述します
 *
 * バストアップ
 * <menu_bust_up:(ファイル名)>
 *
 * 変数の値に応じて表示するバストアップを変更する場合
 * <menu_bust_up>
 * vid=12
 * images=menu_chara_004,menu_chara_001
 * </menu_bust_up>
 *
 * ステータスイメージ
 * <menu_status_image:(ファイル名)>
 *
 * 変数の値に応じて表示するステータスイメージを変更する場合
 * <menu_status_image>
 * vid=12
 * images=menu_status_chara_004,menu_status_chara_001
 * </menu_status_image>
 *
 * キャラクターアイコン
 * <menu_class_icon_index:(キャラインデックス)>
 * 尚文=0
 * ラフタリア=1
 * フィーロ=2
 * メルティ=3
 *
 *--------------------------------------------------------------------------------
 * フィーロに任せて移動用アイテムへの記述
 *--------------------------------------------------------------------------------
 * <FiroMove:MapID,X,Y>
 *
 * マップID 17, X座標 2, Y座標 3 の場合
 * <FiroMove:17,2,3>
 *
 */

function Window_Title() {
    this.initialize.apply(this, arguments);
}

function Window_MenuHelp() {
    this.initialize.apply(this, arguments);
}

function Window_MenuGold() {
    this.initialize.apply(this, arguments);
}

function Window_MenuItemList() {
    this.initialize.apply(this, arguments);
}

function Window_MenuItemCategory() {
    this.initialize.apply(this, arguments);
}

(function() {


	'use strict';

	var parameters = PluginManager.parameters('JPT_MainMenu');
	var fMove_Unlock_SID = parseInt(parameters['FiroMove_Unlock_SwitchID'] || 500);
	var fMove_Enable_SID = parseInt(parameters['FiroMove_Enable_SwitchID'] || 501);
	var fMove_M_VID = parseInt(parameters['FiroMove_MapID_VariableID'] || 198);
	var fMove_X_VID = parseInt(parameters['FiroMove_X_VariableID'] || 199);
	var fMove_Y_VID = parseInt(parameters['FiroMove_Y_VariableID'] || 200);

	var firoMoveMenuName = parameters['FiroMove_MenuName'] || 'フィーロに任せて移動';
	var magicMenuName = parameters['Magic_MenuName'] || '魔法';
	var noneItemName = parameters['None_ItemName'] || 'なし';
	var offEquipName = parameters['Off_EquipName'] || 'はずす';

	var autoSaveOn = JSON.parse(parameters["Auto Save"]);

	function JPT_MainMenu(){
		throw new Error('This is a static class');
	}

	JPT_MainMenu.isFiroMoveUnlock = function(){
		return $gameSwitches.value(fMove_Unlock_SID);
	}

	JPT_MainMenu.isFiroMoveEnable = function(){
		return $gameSwitches.value(fMove_Enable_SID);
	}


	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_MainMenu_DataManager_makeSavefileInfo = DataManager.makeSavefileInfo;
	DataManager.makeSavefileInfo = function() {
	    var info = _JPT_MainMenu_DataManager_makeSavefileInfo.call(this);
	    info.gold = $gameParty.gold();
	    return info;
	};

	var _JPT_MainMenu_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {
		if (!_JPT_MainMenu_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processActorNotetagsMm($dataActors);
	    this.processItemsNotetagsMm($dataItems);
		return true;
	};


	//メモからデータ取得
	DataManager.processActorNotetagsMm = function(group) {

		var noteBustUp = /<menu_bust_up:(.+)>/i;
		var noteClassIcon = /<menu_class_icon_index:(.+)>/i;
		var noteStatusImage = /<menu_status_image:(.+)>/i;
		var noteEquip = /<menu_equip_types:(.+)>/i;

		var noteBustUpStart = /<menu_bust_up>/i;
		var noteBustUpEnd = /<\/menu_bust_up>/i;

		var noteStatusImageStart = /<menu_status_image>/i;
		var noteStatusImageEnd = /<\/menu_status_image>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			var mode = 'none';
			obj.bust_up = '';
			obj.class_icon = -1;
			obj.status_image = '';
			obj.menu_equip = [];

			obj.status_image_array = [];
			obj.status_image_vid = 0;

			obj.bust_up_array = [];
			obj.bust_up_vid = 0;

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if(line.match(noteBustUp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.bust_up = tagStr.trim();
				}else if(line.match(noteClassIcon)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.class_icon = parseInt( tagStr.trim() ,10 );
				}else  if(line.match(noteStatusImage)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.status_image = tagStr.trim();
				}else  if(line.match(noteEquip)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0].trim();
					obj.menu_equip = tagStr.split(/,/);

				}if(line.match(noteBustUpStart)){
					mode = 'menu_bust_up';
				}else if(line.match(noteBustUpEnd)){
					mode = 'none';
				}else if(mode == 'menu_bust_up'){
					var tag = line.split(/=/)[0];
					var val = line.split(/=/)[1];

					switch(tag){
					case 'vid':
						obj.bust_up_vid = parseInt(val,10);
						break;
					case 'images':
						obj.bust_up_array = val.split(/,/);
						break;
					}
				}if(line.match(noteStatusImageStart)){
					mode = 'menu_status_image';
				}else if(line.match(noteStatusImageEnd)){
					mode = 'none';
				}else if(mode == 'menu_status_image'){
					var tag = line.split(/=/)[0];
					var val = line.split(/=/)[1];

					switch(tag){
					case 'vid':
						obj.status_image_vid = parseInt(val,10);
						break;
					case 'images':
						obj.status_image_array = val.split(/,/);
						break;
					}
				}
			}
		}
	};

	DataManager.processItemsNotetagsMm = function(group) {
		var noteFiroMove = /<FiroMove:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.firo_move = '';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteFiroMove)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.firo_move = tagStr.trim();
				}
			}
		}
	}



	//golobalinfo を受け渡しで参照させる
	DataManager.isThisGameFile_G = function(savefileId,ginfo) {
	    var globalInfo = ginfo;
	    if (globalInfo && globalInfo[savefileId]) {
	        if (StorageManager.isLocalMode()) {
	            return true;
	        } else {
	            var savefile = globalInfo[savefileId];
	            return (savefile.globalId === this._globalId &&
	                    savefile.title === $dataSystem.gameTitle);
	        }
	    } else {
	        return false;
	    }
	};

	DataManager.latestSavefileId_G = function(ginfo) {
	    var globalInfo = ginfo;
	    var savefileId = 1;
	    var timestamp = 0;
	    if (globalInfo) {
	        for (var i = 1; i < globalInfo.length; i++) {
	            if (this.isThisGameFile_G(i,ginfo) && globalInfo[i].timestamp > timestamp) {
	                timestamp = globalInfo[i].timestamp;
	                savefileId = i;
	            }
	        }
	    }
	    return savefileId;
	};

	DataManager.loadAllSavefileImages_G = function(ginfo) {
	    var globalInfo = ginfo;
	    if (globalInfo) {
	        for (var i = 1; i < globalInfo.length; i++) {
	            if (this.isThisGameFile_G(i,ginfo)) {
	                var info = globalInfo[i];
	                this.loadSavefileImages(info);
	            }
	        }
	    }
	};

	DataManager.loadSavefileInfo_G = function(savefileId,ginfo) {
	    var globalInfo = ginfo;
	    return (globalInfo && globalInfo[savefileId]) ? globalInfo[savefileId] : null;
	};

	////////////////////////////////////////////////////////////////////////////////////
    // TextManager
	////////////////////////////////////////////////////////////////////////////////////

	Object.defineProperties(TextManager, {
		menuMagic			: { get: function() { return magicMenuName; }, configurable: true },
		menuFiroMove		: { get: function() { return firoMoveMenuName; }, configurable: true },


		colorN				: { get: function() { return '#ffffff' ; }, configurable: true },
		colorNHP			: { get: function() { return '#c6442d' ; }, configurable: true },
		colorCHP			: { get: function() { return '#ffff40' ; }, configurable: true },
		colorMP				: { get: function() { return '#27aac4' ; }, configurable: true },
		colorSP				: { get: function() { return '#9e57a4' ; }, configurable: true },
		colorEXP			: { get: function() { return '#89760f' ; }, configurable: true },
		colorGaugeBack		: { get: function() { return '#575757' ; }, configurable: true },
	});

	TextManager.element = function(elementId) {
	    return $dataSystem.elements[elementId] || '';
	};


	////////////////////////////////////////////////////////////////////////////////////
    // Game_Actor
	////////////////////////////////////////////////////////////////////////////////////

	Game_Actor.prototype.menuEquipSlots = function() {
	    return this.actor().menu_equip.length > 0 ? this.actor().menu_equip : null;
	};

	Game_Actor.prototype.menuEquips = function(index) {
	    return this.actor().menu_equip.length > 0 ? this.actor().menu_equip[index]-1 : 0;
	};

	Game_Actor.prototype.menuEquipItem = function(index) {
	    return this.actor().menu_equip.length > 0 ? this.equips()[ this.menuEquips(index)] : null;
	};

	////////////////////////////////////////////////////////////////////////////////////
    // Game_Party
	////////////////////////////////////////////////////////////////////////////////////

	Game_Party.prototype.charactersForSavefile = function() {
	    return this.battleMembers().map(function(actor) {
	        return [actor.characterName(), actor.characterIndex(), actor.level];
	    });
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Scene_MenuBase
	////////////////////////////////////////////////////////////////////////////////////
	Scene_MenuBase.prototype.createMaskFrame = function(ww,wh,uw,dw,lw,rw,color) {
		var bitmap = new Bitmap(ww, wh);
		bitmap.fillAll(color);

		bitmap.clearRect( lw, uw, ww-lw-rw, wh-uw-dw );

		return bitmap;
	}

	Scene_MenuBase.prototype.createHelpWindow = function() {
	    this._helpWindow = new Window_MenuHelp();
	    this.addWindow(this._helpWindow);
	};

	Scene_MenuBase.prototype.createGoldWindow = function() {
		this._goldWindow = new Window_MenuGold(0, 0);
	    this._goldWindow.x = Graphics.boxWidth -  this._goldWindow.width;
	    this._goldWindow.y = Graphics.boxHeight - this._goldWindow.height;
	    this.addWindow(this._goldWindow);
	};

	Scene_MenuBase.prototype.createMenuTitle = function(title_index,title_name) {
		this._titleWindow = new Window_Title(title_index,title_name);
		this.addWindow(this._titleWindow);
	}




	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Menu
	////////////////////////////////////////////////////////////////////////////////////
	Scene_Menu.prototype.createBackground = function() {
		Scene_MenuBase.prototype.createBackground.call(this);
		this._backgroundSprite.bitmap.clear();
	}

	var _JPT_MainMenu_Scene_Menu_start = Scene_Menu.prototype.start;
	Scene_Menu.prototype.start = function() {
		_JPT_MainMenu_Scene_Menu_start.call(this);
	    this._commandWindow.refresh();
	    this._goldWindow.refresh();
	};

	Scene_Menu.prototype.createGoldWindow = function() {
		Scene_MenuBase.prototype.createGoldWindow.call(this);
	};

	Scene_Menu.prototype.createCommandWindow = function() {
	    this._commandWindow = new Window_MenuCommand(8, 8);
	    this._commandWindow.setHandler('status',		this.commandPersonal.bind(this));
	    this._commandWindow.setHandler('skill',			this.commandPersonal.bind(this));
	    this._commandWindow.setHandler('magic', 		this.commandPersonal.bind(this));
	    this._commandWindow.setHandler('item',			this.commandItem.bind(this));
	    this._commandWindow.setHandler('equip',			this.commandPersonal.bind(this));
    	this._commandWindow.setHandler('firoMove',		this.commandFiroMove.bind(this));
	    this._commandWindow.setHandler('save',			this.commandSave.bind(this));
	    this._commandWindow.setHandler('options',		this.commandOptions.bind(this));
	    this._commandWindow.setHandler('cancel',		this.popScene.bind(this));
	    this.addWindow(this._commandWindow);
	};

	Scene_Menu.prototype.createStatusWindow = function() {
	    this._statusWindow = new Window_MenuStatus(0, this._commandWindow.height);
	    this._statusWindow.reserveFaceImages();
	    this.addWindow(this._statusWindow);
	};

	Scene_Menu.prototype.onPersonalOk = function() {
	    switch (this._commandWindow.currentSymbol()) {
	    case 'skill':
	        SceneManager.push(Scene_Skill);
	        break;
	    case 'equip':
	        SceneManager.push(Scene_Equip);
	        break;
	    case 'status':
	        SceneManager.push(Scene_Status);
	        break;
	    case 'magic':
	    	SceneManager.push(Scene_Magic);
	    	break;
	    }
	};

	Scene_Menu.prototype.commandFiroMove = function() {
	    SceneManager.push(Scene_FiroMove);
	};

	Scene_Menu.prototype.commandOptions = function() {
	    SceneManager.push(Scene_MenuOptions);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_ItemBase
	////////////////////////////////////////////////////////////////////////////////////

	Scene_ItemBase.prototype.createActorWindow = function() {
	    this._actorWindow = new Window_MenuActor();
	    this._actorWindow.y = 72;
	    this._actorWindow.setHandler('ok',     this.onActorOk.bind(this));
	    this._actorWindow.setHandler('cancel', this.onActorCancel.bind(this));
	    this.addWindow(this._actorWindow);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Skill
	////////////////////////////////////////////////////////////////////////////////////
	Scene_Skill.prototype.titleID = function() {
		return 1;
	};

	Scene_Skill.prototype.skillTypeID = function() {
		return 1;
	};

	Scene_Skill.prototype.titleName = function() {
		return TextManager.skill;
	};

	var _JPT_MainMenu_Scene_Skill_createBackground = Scene_Skill.prototype.createBackground;
	Scene_Skill.prototype.createBackground = function() {
		_JPT_MainMenu_Scene_Equip_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_list_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 72;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,72,177,8,8,'#000000ff');
		this.addChild(this._sprMask);
	};

	Scene_Skill.prototype.create = function() {
	    Scene_ItemBase.prototype.create.call(this);
	    this.createNameWindow();
	    this.createHelpWindow();
	    this.createItemWindow();
	    this.createGoldWindow();
	    this.createActorWindow();
	    this.createWindowFrame();
	    this.createMenuTitle(this.titleID(),this.titleName());
	};

	Scene_Skill.prototype.createItemWindow = function() {
	    var wx = this._nameWindow.x;
	    var wy = this._nameWindow.y + this._nameWindow.height;
	    var ww = this._nameWindow.width;
	    var wh = this._helpWindow.y - wy - 8;
	    this._itemWindow = new Window_SkillList(wx, wy, ww, wh);
	    this._itemWindow.setHelpWindow(this._helpWindow);
	    this._itemWindow.setHandler('ok',     this.onItemOk.bind(this));
	    this._itemWindow.setHandler('cancel', this.popScene.bind(this));
	    this._itemWindow.opacity = 0;
	    this.addWindow(this._itemWindow);
	};

	Scene_Skill.prototype.createNameWindow = function() {
		var wx = 8;
	    var wy = 72;
	    var ww = Graphics.boxWidth - 16;
	    var wh = 40;
		this._nameWindow = new Window_Name(wx, wy, ww, wh);
	    this.addWindow(this._nameWindow);
	};

	Scene_Skill.prototype.createWindowFrame = function(){
		var x = this._nameWindow.x;
		var y = this._nameWindow.y;
		var w = this._nameWindow.width;
	    var h = this._itemWindow.height + this._nameWindow.height;
	    var m = 24;
	    var bitmap = this._sprMask.bitmap;
        var p = 96;
        var q = 96;
        var skin = this._itemWindow.windowskin;
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+m, y+0, w-m*2, m);
        bitmap.blt(skin, p+m, 0+q-m, p-m*2, m, x+m, y+h-m, w-m*2, m);
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, x+0, y+m, m, h-m*2);
        bitmap.blt(skin, p+q-m, 0+m, m, p-m*2, x+w-m, y+m, m, h-m*2);
        bitmap.blt(skin, p+0, 0+0, m, m, x+0, y+0, m, m);
        bitmap.blt(skin, p+q-m, 0+0, m, m, x+w-m, y+0, m, m);
        bitmap.blt(skin, p+0, 0+q-m, m, m, x+0, y+h-m, m, m);
        bitmap.blt(skin, p+q-m, 0+q-m, m, m, x+w-m, y+h-m, m, m);

        //横
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+5, y + 40 , w - 10, m);
        bitmap.blt(skin, p+m, 0+1, p-m*2, 1, x+4, y + 40+1, w - 8, 3);

        m = this._itemWindow._margin;
        w = w - m * 2;
        h = h - m * 2;
        bitmap = new Bitmap(w, h);
        this._sprBack.bitmap = bitmap;
        this._sprBack.setFrame(0, 0, w, h);
        this._sprBack.move(x + m, y + m);

        bitmap.blt(skin, 0, 0, p, p, 0, 0, w, h);

        for (y = 0; y < h; y += p) {
            for (x = 0; x < w; x += p) {
                bitmap.blt(skin, 0, p, p, p, x, y, p, p);
            }
        }

        var tone = this._itemWindow._colorTone;
        bitmap.adjustTone(tone[0], tone[1], tone[2]);
	}


	Scene_Skill.prototype.refreshActor = function() {
	    var actor = this.actor();
	    this._itemWindow.setActor(actor);
	    this._nameWindow.setActor(actor);
	};

	Scene_Skill.prototype.start = function() {
	    Scene_ItemBase.prototype.start.call(this);
	    this.createWindowFrame();
	    this.refreshActor();
	    this._itemWindow.activate();
	    this._itemWindow.setStypeId(this.skillTypeID());
	    this._itemWindow.selectLast();
	    this._titleWindow.refresh();
	    this._goldWindow.refresh();
	};


	Scene_Skill.prototype.useItem = function() {
	    Scene_ItemBase.prototype.useItem.call(this);
	    this._itemWindow.refresh();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Magic
	////////////////////////////////////////////////////////////////////////////////////

	function Scene_Magic() {
	    this.initialize.apply(this, arguments);
	}

	Scene_Magic.prototype = Object.create(Scene_Skill.prototype);
	Scene_Magic.prototype.constructor = Scene_Magic;

	Scene_Magic.prototype.titleID = function() {
		return 0;
	};

	Scene_Magic.prototype.skillTypeID = function() {
		return 2;
	};

	Scene_Magic.prototype.titleName = function() {
		return TextManager.menuMagic;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Status
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_MainMenu_Scene_Status_createBackground = Scene_Status.prototype.createBackground;
	Scene_Status.prototype.createBackground = function() {
		_JPT_MainMenu_Scene_Status_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_status_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 8;

		var sprMask = new Sprite();
		sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,8,32,8,8,'#000000ff');
		this.addChild(sprMask);
	};

	Scene_Status.prototype.create = function() {
	    Scene_MenuBase.prototype.create.call(this);
	    this.createGoldWindow();
	    this._statusWindow = new Window_Status();
	    this._statusWindow.setHandler('cancel',   this.popScene.bind(this));
	    this._statusWindow.setHandler('right', this.nextActor.bind(this));
	    this._statusWindow.setHandler('left',   this.previousActor.bind(this));
	    this.addWindow(this._statusWindow);
	};

	Scene_Status.prototype.start = function() {
	    Scene_MenuBase.prototype.start.call(this);
	    this.refreshActor();
	    this._statusWindow.refresh();
	    this._goldWindow.refresh();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Equip
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_MainMenu_Scene_Equip_createBackground = Scene_Equip.prototype.createBackground;
	Scene_Equip.prototype.createBackground = function() {
		_JPT_MainMenu_Scene_Equip_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_list_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 72;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,72,177,8,8,'#000000ff');
		this.addChild(this._sprMask);
	};

	Scene_Equip.prototype.createWindowFrame = function(){
		var x = this._statusWindow.x;
		var y = this._statusWindow.y;
		var w = Graphics.boxWidth - 16;
	    var h = this._statusWindow.height + this._itemWindow.height - 8;
	    var m = 24;
	    var bitmap = this._sprMask.bitmap;
        var p = 96;
        var q = 96;
        var skin = this._statusWindow.windowskin;
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+m, y+0, w-m*2, m);
        bitmap.blt(skin, p+m, 0+q-m, p-m*2, m, x+m, y+h-m, w-m*2, m);
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, x+0, y+m, m, h-m*2);
        bitmap.blt(skin, p+q-m, 0+m, m, p-m*2, x+w-m, y+m, m, h-m*2);
        bitmap.blt(skin, p+0, 0+0, m, m, x+0, y+0, m, m);
        bitmap.blt(skin, p+q-m, 0+0, m, m, x+w-m, y+0, m, m);
        bitmap.blt(skin, p+0, 0+q-m, m, m, x+0, y+h-m, m, m);
        bitmap.blt(skin, p+q-m, 0+q-m, m, m, x+w-m, y+h-m, m, m);

        //横
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+5, y + 40 , w - 10, m);
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+5, this._itemWindow.y , w - 10, m);
        //縦
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, this._slotWindow.x, y+5+40, m, this._slotWindow.height - 13 );
        //中横
        bitmap.blt(skin, p+m, 0+1, p-m*2, 1, x+4, y + 40+1, w - 8, 3);
        bitmap.blt(skin, p+m, 0+1, p-m*2, 1, x+4, this._itemWindow.y+1, w - 8, 3);
        //中縦
        bitmap.blt(skin, p+1, 0+m, 1, p-m*2, this._slotWindow.x + 1, y+4+40, 3, this._slotWindow.height - 10 );

        m = this._statusWindow._margin;
        w = w - m * 2;
        h = h - m * 2;
        bitmap = new Bitmap(w, h);
        this._sprBack.bitmap = bitmap;
        this._sprBack.setFrame(0, 0, w, h);
        this._sprBack.move(x + m, y + m);

        bitmap.blt(skin, 0, 0, p, p, 0, 0, w, h);
        for (y = 0; y < h; y += p) {
            for (x = 0; x < w; x += p) {
                bitmap.blt(skin, 0, p, p, p, x, y, p, p);
            }
        }
        var tone = this._statusWindow._colorTone;
        bitmap.adjustTone(tone[0], tone[1], tone[2]);
	}

	Scene_Equip.prototype.create = function() {
	    Scene_MenuBase.prototype.create.call(this);
	    this.createHelpWindow();
	    this.createStatusWindow();
	    this.createSlotWindow();
	    this.createItemWindow();
	    this.createGoldWindow();
	    this.createWindowFrame();
	    this.createMenuTitle(3,TextManager.equip);
	    this.refreshActor();
	};

	Scene_Equip.prototype.start = function() {
	    Scene_MenuBase.prototype.start.call(this);
	    this._statusWindow.refresh();
	    this._slotWindow.activate();
	    this._slotWindow.select(0);
	    this._slotWindow.refresh();
	    this._titleWindow.refresh();
	    this._goldWindow.refresh();
	    this.createWindowFrame();
	};

	Scene_Equip.prototype.createStatusWindow = function() {
	    this._statusWindow = new Window_EquipStatus(8, 72);
	    this._statusWindow.opacity = 0;
	    this.addWindow(this._statusWindow);
	};

	Scene_Equip.prototype.createSlotWindow = function() {
	    var wx = this._statusWindow.x + this._statusWindow.width - 8;
	    var wy = this._statusWindow.y + 40;
	    var ww = Graphics.boxWidth - this._statusWindow.width - 8;
	    var wh = this._statusWindow.height - 40;
	    this._slotWindow = new Window_EquipSlot(wx, wy, ww, wh);
	    this._slotWindow.setHelpWindow(this._helpWindow);
	    this._slotWindow.setStatusWindow(this._statusWindow);
	    this._slotWindow.setHandler('ok',       this.onSlotOk.bind(this));
	    this._slotWindow.setHandler('cancel',   this.popScene.bind(this));
	    this._slotWindow.opacity = 0;
	    this.addWindow(this._slotWindow);
	};

	Scene_Equip.prototype.createItemWindow = function() {
	    var wx = this._statusWindow.x;
	    var wy = this._statusWindow.y + this._statusWindow.height - 8;
	    var ww = Graphics.boxWidth - 16;
	    var wh = this._helpWindow.y - this._statusWindow.y - this._statusWindow.height;
	    this._itemWindow = new Window_EquipItem(wx, wy, ww, wh);
	    this._itemWindow.setHelpWindow(this._helpWindow);
	    this._itemWindow.setStatusWindow(this._statusWindow);
	    this._itemWindow.setHandler('ok',     this.onItemOk.bind(this));
	    this._itemWindow.setHandler('cancel', this.onItemCancel.bind(this));
	    this._itemWindow.opacity = 0;
	    this._slotWindow.setItemWindow(this._itemWindow);
	    this.addWindow(this._itemWindow);
	};

	Scene_Equip.prototype.onItemOk = function() {

		var check = true;
		if(this._itemWindow._slotId==1){
			if(!this._itemWindow.item()){
				check = false;
			}
		}
	    SoundManager.playEquip();
	    if(check)this.actor().changeEquip(this._slotWindow.slotIndex(), this._itemWindow.item());
	    this._slotWindow.activate();
	    this._slotWindow.refresh();
	    this._itemWindow.deselect();
	    this._itemWindow.refresh();
	    this._statusWindow.refresh();

	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Item
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_MainMenu_Scene_Item_createBackground = Scene_Item.prototype.createBackground;
	Scene_Item.prototype.createBackground = function() {
		_JPT_MainMenu_Scene_Item_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_list_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 72;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,72+50,177,8,8,'#000000ff');
		this.addChild(this._sprMask);


	};

	Scene_Item.prototype.create = function() {
	    Scene_ItemBase.prototype.create.call(this);
	    this.createHelpWindow();
	    this.createCategoryWindow();
	    this.createItemWindow();
	    this.createActorWindow();
	    this.createGoldWindow();
	    this.createMenuTitle(2,TextManager.item);
	};

	Scene_Item.prototype.createCategoryWindow = function() {
	    this._categoryWindow = new Window_MenuItemCategory();
	    this._categoryWindow.setHelpWindow(this._helpWindow);
	    this._categoryWindow.x = 8;
	    this._categoryWindow.y = 72;
	    this._categoryWindow.setHandler('ok',     this.onCategoryOk.bind(this));
	    this._categoryWindow.setHandler('cancel', this.popScene.bind(this));
	    this.addWindow(this._categoryWindow);
	};

	Scene_Item.prototype.createItemWindow = function() {
		var wx = this._categoryWindow.x;
	    var wy = this._categoryWindow.y + this._categoryWindow.height + 8;
	    var ww = Graphics.boxWidth - 16;
	    var wh = this._helpWindow.y - wy - 8;
	    this._itemWindow = new Window_MenuItemList(wx, wy, ww, wh);
	    this._itemWindow.setHelpWindow(this._helpWindow);
	    this._itemWindow.setHandler('ok',     this.onItemOk.bind(this));
	    this._itemWindow.setHandler('cancel', this.onItemCancel.bind(this));
	    this.addWindow(this._itemWindow);
	    this._categoryWindow.setItemWindow(this._itemWindow);
	};

	Scene_Item.prototype.start = function() {
		Scene_ItemBase.prototype.start.call(this);
		this._titleWindow.refresh();
		this._goldWindow.refresh();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_FiroMove
	////////////////////////////////////////////////////////////////////////////////////
	function Scene_FiroMove() {
	    this.initialize.apply(this, arguments);
	}

	Scene_FiroMove.prototype = Object.create(Scene_Item.prototype);
	Scene_FiroMove.prototype.constructor = Scene_Item;

	Scene_FiroMove.prototype.initialize = function() {
	    Scene_ItemBase.prototype.initialize.call(this);
	};
	Scene_FiroMove.prototype.createBackground = function() {
		Scene_ItemBase.prototype.createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_list_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 72;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,72,177,8,8,'#000000ff');
		this.addChild(this._sprMask);


	};

	Scene_FiroMove.prototype.create = function() {
		Scene_ItemBase.prototype.create.call(this);
	    this.createHelpWindow();
	    this.createItemWindow();
	    this.createGoldWindow();
	    this.createActorWindow();
	    this.createMenuTitle(5,TextManager.menuFiroMove);
	};

	Scene_FiroMove.prototype.start = function() {
		Scene_ItemBase.prototype.start.call(this);
	    this._itemWindow.activate();
	    this._itemWindow.setCategory('firomove');
	    this._itemWindow.select(0);
	    this._titleWindow.refresh();
	    this._goldWindow.refresh();
	};

	Scene_FiroMove.prototype.createItemWindow = function() {
		var wx = 8;
	    var wy = 72;
	    var ww = Graphics.boxWidth - 16;
	    var wh = this._helpWindow.y - wy - 8;
	    this._itemWindow = new Window_MenuFiroMoveList(wx, wy, ww, wh);
	    this._itemWindow.setHelpWindow(this._helpWindow);
	    this._itemWindow.setHandler('ok',     this.onItemOk.bind(this));
	    this._itemWindow.setHandler('cancel', this.popScene.bind(this));
	    this.addWindow(this._itemWindow);
	};

	Scene_FiroMove.prototype.onItemOk = function() {
	    this.determineItem();
	};

	Scene_FiroMove.prototype.useItem = function() {
		if(this.item()){
			var mov = this.item().firo_move.split(/,/);
			$gameVariables.setValue( fMove_M_VID, parseInt( mov[0].trim() || 0) );
			$gameVariables.setValue( fMove_X_VID, parseInt( mov[1].trim() || 0) );
			$gameVariables.setValue( fMove_Y_VID, parseInt( mov[2].trim() || 0) );
		}
		Scene_Item.prototype.useItem.call(this);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_File
	////////////////////////////////////////////////////////////////////////////////////

	Scene_File.prototype.initialize = function() {
	    Scene_MenuBase.prototype.initialize.call(this);
	    this._globalInfo = DataManager.loadGlobalInfo();
	};

	Scene_File.prototype.create = function() {
	    Scene_MenuBase.prototype.create.call(this);
	    DataManager.loadAllSavefileImages_G(this._globalInfo);
	    this.createListWindow();
	};

	Scene_File.prototype.createListWindow = function() {
	    var x = 0;
	    var y = 72;
	    this._listWindow = new Window_SavefileList(x, y,0,0,this._globalInfo);
	    this._listWindow.setHandler('ok',     this.onSavefileOk.bind(this));
	    this._listWindow.setHandler('cancel', this.popScene.bind(this));
	    this._listWindow.select(this.firstSavefileIndex());
	    this._listWindow.setTopRow(this.firstSavefileIndex() - 2);
	    this._listWindow.setMode(this.mode());
	    this._listWindow.refresh();
	    this.addWindow(this._listWindow);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Save
	////////////////////////////////////////////////////////////////////////////////////
	Scene_Save.prototype.createBackground = function() {
		Scene_File.prototype.createBackground.call(this);
		this._backgroundSprite.bitmap.clear();
	}

	Scene_Save.prototype.create = function() {
		Scene_File.prototype.create.call(this);
		this.createMenuTitle(6,TextManager.save);
	}

	Scene_Save.prototype.start = function() {
		Scene_File.prototype.start.call(this);
	    this._titleWindow.refresh();
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Load
	////////////////////////////////////////////////////////////////////////////////////
	Scene_Load.prototype.createBackground = function() {
		Scene_File.prototype.createBackground.call(this);
		this._backgroundSprite.bitmap.clear();
	}

	Scene_Load.prototype.create = function() {
		Scene_File.prototype.create.call(this);
		this.createMenuTitle(6,TextManager.continue_);
	}

	Scene_Load.prototype.start = function() {
		Scene_File.prototype.start.call(this);
	    this._titleWindow.refresh();
	};

	Scene_Load.prototype.firstSavefileIndex = function() {
	    return DataManager.latestSavefileId_G(this._globalInfo) - 1;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_MenuOptions
	////////////////////////////////////////////////////////////////////////////////////

	function Scene_MenuOptions() {
	    this.initialize.apply(this, arguments);
	}

	Scene_MenuOptions.prototype = Object.create(Scene_Options.prototype);
	Scene_MenuOptions.prototype.constructor = Scene_MenuOptions;

	var _JPT_MainMenu_Scene_MenuOptions_createBackground = Scene_MenuOptions.prototype.createBackground;
	Scene_MenuOptions.prototype.createBackground = function() {
		_JPT_MainMenu_Scene_MenuOptions_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_status_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 72;

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,72,32,8,8,'#000000ff');
		this.addChild(this._sprMask);


	};

	Scene_MenuOptions.prototype.create = function() {
	    Scene_MenuBase.prototype.create.call(this);
	    this.createOptionsWindow();
	    this.createMenuTitle(7,TextManager.options);
	};

	Scene_MenuOptions.prototype.start = function() {
		Scene_MenuBase.prototype.start.call(this);
	    this._titleWindow.refresh();
	    this._goldWindow.refresh();
	};

	Scene_MenuOptions.prototype.createOptionsWindow = function() {
	    this._optionsWindow = new Window_MenuOptions();
	    this._optionsWindow.setHandler('cancel', this.popScene.bind(this));
	    this.addWindow(this._optionsWindow);

	    this.createGoldWindow();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_Base
	////////////////////////////////////////////////////////////////////////////////////

	Window_Base.prototype.NormalColor = function() {
		return TextManager.colorN;
	}

	Window_Base.prototype.hpNormalColor = function() {
		return TextManager.colorNHP;
	}

	Window_Base.prototype.mpColor = function() {
		return TextManager.colorMP;
	}

	Window_Base.prototype.spColor = function() {
		return TextManager.colorSP;
	}

	Window_Base.prototype.expColor = function() {
		return TextManager.colorEXP;
	}

	Window_Base.prototype.gaugeBackColor = function() {
	    return TextManager.colorGaugeBack;
	};

	Window_Base.prototype.mpCostColor = function() {
	    return this.textColor(23);
	};

	Window_Base.prototype.spCostColor = function() {
	    return this.textColor(23);
	};


	Window_Base.prototype.hpGaugeColor = function() {
	    return this.hpNormalColor();
	};

	Window_Base.prototype.mpGaugeColor = function() {
	    return this.mpColor();
	};

	Window_Base.prototype.spGaugeColor = function() {
	    return this.spColor();
	};

	Window_Base.prototype.expGaugeColor = function() {
	    return this.expColor();
	};

	Window_Base.prototype.crisisColor = function() {
	    return TextManager.colorCHP;
	};


	Window_Base.prototype.hpColor = function(actor) {
	    if (actor.isDead()) {
	        return this.deathColor();
	    } else if (actor.isDying()) {
	        return this.crisisColor();
	    } else {
	        return this.hpNormalColor();
	    }
	};

	Window_Base.prototype.drawStatusMiniIcon = function(parentbmp,iconIndex, x, y ) {
		var bitmap = ImageManager.loadBitmap("img/menus/", "menu_status_miniicon");
		var pw = 20;
		var ph = 20;
		var sx = iconIndex % 6 * pw;
		var sy = Math.floor(iconIndex / 6) * ph;
		parentbmp.blt(bitmap, sx, sy, pw, ph, x, y);
	}

	Window_Base.prototype.drawIcon20px = function(parentbmp,iconIndex, x, y ) {
		var bitmap = ImageManager.loadSystem('IconSet20px');
	    var pw = 20;
	    var ph = 20;
	    var sx = iconIndex % 16 * pw;
	    var sy = Math.floor(iconIndex / 16) * ph;
	    this.contents.blt(bitmap, sx, sy, pw, ph, x, y);
	}

	Window_Base.prototype.drawNameWithIcon = function(parentbmp,actor,x,y) {
		var bitmap = ImageManager.loadBitmap("img/menus/", "menu_charaicon");
		var ic = actor.actor().class_icon;
		parentbmp.blt(bitmap, 32*ic,0, 32,28, x, y );
		var name = actor.nickname();
		parentbmp.drawText(name, x + 32 ,y + 6, 150, parentbmp.fontSize, 'left' );
	}

	Window_Base.prototype.drawGold = function(parent, value, x, y, width, fontSize){
		fontSize = fontSize || 18;
		var buf_fsize = parent.fontSize;
		parent.fontSize = fontSize;
		parent.textColor = this.NormalColor();
		parent.outlineColor = 'rgba(0, 0, 0, 0)';
		parent.outlineWidth = 0;

		var digitWidth = parent.measureTextWidth('0');
		var iconWidth = 16 + 6;

		var textHeight = 18;
		var textPaddingY = 3;

		var bronze = value%100;
		var silver = Math.floor(value/100);
		silver = silver%100;
		var gold = Math.floor(value/10000);

		gold = Math.min(gold, 999);

	    this.drawGoldIcon(parent, 0, x + width - digitWidth*7 - iconWidth*3, y + 2);
	    parent.drawText(('000' + gold).slice(-3), x - digitWidth*4 - iconWidth*2 , y + textPaddingY, width - 3, textHeight,'right');
	    this.drawGoldIcon(parent, 1, x + width - digitWidth*4 - iconWidth*2, y + 2);
	    parent.drawText(('00' + silver).slice(-2), x - digitWidth*2 - iconWidth, y + textPaddingY, width - 3, textHeight,'right');
	    this.drawGoldIcon(parent,2, x + width - digitWidth*2 - iconWidth, y + 2);
	    parent.drawText(('00' + bronze).slice(-2), x, y + textPaddingY, width - 3 , textHeight,'right');

		parent.fontSize = buf_fsize;
	}

	Window_Base.prototype.drawGoldIcon = function(parent,iconIndex, x, y) {
		var bitmap = ImageManager.loadBitmap("img/menus/", "menu_money");
	    var pw = 16;
	    var ph = 16;
	    var sx = iconIndex * pw;
	    var sy = 0;
	    parent.blt(bitmap, sx, sy, pw, ph, x, y);
	}

	Window_Base.prototype.drawHorzLine = function(parentbmp, x, y, width, height) {
		var lineThick = 1;
	    var lineY = y + height / 2 - 1;
	    parentbmp.fillRect( x, lineY, width, lineThick, '#788484');
	};

	Window_Base.prototype.makeFontBigger = function() {
	    if (this.contents.fontSize <= 96) {
	        this.contents.fontSize += 6;
	    }
	};

	Window_Base.prototype.makeFontSmaller = function() {
	    if (this.contents.fontSize >= 18) {
	        this.contents.fontSize -= 6;
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_Title
	////////////////////////////////////////////////////////////////////////////////////
	Window_Title.prototype = Object.create(Window_Base.prototype);
	Window_Title.prototype.constructor = Window_Title;

	Window_Title.prototype.initialize = function( title_index, title_name ) {
		this.title_index = title_index;
		this.title_name = title_name;
	    Window_Base.prototype.initialize.call(this, 0, 0, Graphics.boxWidth, 72);
	    this.opacity = 0;
	    this.refresh();
	};


	Window_Title.prototype.standardFontFace = function() {
		 return 'GameFont2';
	};


	Window_Title.prototype.standardFontSize = function() {
	    return 24;
	};

	Window_Title.prototype.standardPadding = function() {
	    return 0;
	};

	Window_Title.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		//this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_Title.prototype.refresh = function() {
    	this.contents.clear();

    	var bitmap = ImageManager.loadBitmap("img/menus/", "menu_iconplate");
    	this.contents.blt(bitmap,0,0,800,56,8,8);

    	bitmap = ImageManager.loadBitmap("img/menus/", "menu_title_icon");

    	var sx = this.title_index % 5 * 48;
		var sy = Math.floor(this.title_index / 5) * 44;
    	this.contents.blt(bitmap, sx, sy, 48, 44,16,12)

    	this.contents.textColor = '#00000040';
    	this.contents.outlineWidth = 3;
    	this.contents.outlineColor = '#00000040';
    	this.contents.drawText(this.title_name, 70 + 2, 22 + 2, 252 - 48, this.contents.fontSize, 'left' );
    	this.contents.textColor = '#ffffffff';
    	this.contents.outlineWidth = 0;
    	this.contents.outlineColor = '#00000000';
    	this.contents.drawText(this.title_name, 70 , 22, 252 - 48, this.contents.fontSize, 'left' );
    };

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_Name
	////////////////////////////////////////////////////////////////////////////////////
    function Window_Name() {
	    this.initialize.apply(this, arguments);
	}

    Window_Name.prototype = Object.create(Window_Base.prototype);
    Window_Name.prototype.constructor = Window_Name;

    Window_Name.prototype.initialize = function( x, y, width, height ) {
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this._actor = null;
	    this.opacity = 0;
	};

	Window_Name.prototype.standardFontFace = function() {
		 return 'GameFont';
	};


	Window_Name.prototype.standardFontSize = function() {
	    return 18;
	};

	Window_Name.prototype.standardPadding = function() {
	    return 0;
	};

	Window_Name.prototype.textPadding = function() {
	    return 20;
	};

	Window_Name.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		//this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';


	};

	Window_Name.prototype.refresh = function() {
		this.contents.clear();
		if (this._actor) {
			this.drawNameWithIcon(this.contents,this._actor, this.textPadding(), 8 );
		}
	}

	Window_Name.prototype.setActor = function(actor) {
	    if (this._actor !== actor) {
	        this._actor = actor;
	        this.refresh();
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_Selectable
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_MainMenu_Window_Selectable_initialize = Window_Selectable.prototype.initialize;
	Window_Selectable.prototype.initialize = function(x, y, width, height) {
		_JPT_MainMenu_Window_Selectable_initialize.call(this, x, y, width, height);
		this.createOtherParts();
	};

	Window_Selectable.prototype.createOtherParts = function(){
	}

	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuCommand
	////////////////////////////////////////////////////////////////////////////////////
	Window_MenuCommand.prototype.initialize = function(x, y) {
		this.loadImage();
		this.createSprite();
	    Window_Command.prototype.initialize.call(this, x, y);
	    this.selectLast();
	    this._befSelectIndex = -1;
	    this.opacity = 0;
	    this.contentsOpacity = 0;
	    this._curAnimationCount = 0;
	};

	Window_MenuCommand.prototype.maxCols = function() {
	    return 8;
	};

	Window_MenuCommand.prototype.maxItems = function() {
	    return (JPT_MainMenu.isFiroMoveUnlock()) ? 8 : 7;
	};

	Window_MenuCommand.prototype.numVisibleRows = function() {
	    return 1;
	};

	Window_MenuCommand.prototype.windowWidth = function() {
	    return Graphics.boxWidth - 16;
	};

	Window_MenuCommand.prototype.windowHeight = function() {
	    return 80;
	};

	Window_MenuCommand.prototype.standardPadding = function() {
	    return 0;
	};

	Window_MenuCommand.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_MenuCommand.prototype.createSprite = function() {
		this._commandBackSpr = new Sprite();
		this._commandIconSpr = [];
		for(var i=0; i<this.maxItems(); i++){
			this._commandIconSpr[i] = new Sprite();
			this._commandBackSpr.addChild(this._commandIconSpr[i]);
		}
	}

	Window_MenuCommand.prototype.createOtherParts = function(){
		this.addChild(this._commandBackSpr);
	}

	Window_MenuCommand.prototype.loadImage = function() {
		this._bmpPlate = ImageManager.loadBitmap("img/menus/", "menu_iconplate");
		this._bmpIcon = ImageManager.loadBitmap("img/menus/", "menu_icon");
	}

	Window_MenuCommand.prototype.makeCommandIconList = function() {
		this._commandBackSpr.bitmap = this._bmpPlate;
		this._commandBackSpr.x = 0;
		this._commandBackSpr.y = 0;
	}

	Window_MenuCommand.prototype.refresh = function(){
		this.makeCommandIconList();
		Window_Command.prototype.refresh.call(this);
	}

	Window_MenuCommand.prototype.makeCommandList = function() {
		var enabled = true;
		this.addCommand('',		'status', enabled);
		this.addCommand('',		'skill', enabled);
		this.addCommand('',		'magic', enabled);
		this.addCommand('',		'item', enabled);
		this.addCommand('',		'equip', enabled);
		if(JPT_MainMenu.isFiroMoveUnlock()){
			this.addCommand('',		'firoMove', JPT_MainMenu.isFiroMoveEnable());
		}
		this.addCommand('',		'save', this.isSaveEnabled());
		this.addCommand('',		'options', enabled);
	};

	Window_MenuCommand.prototype.itemRect = function(index) {
		var rect = new Rectangle();

		var ww = Math.floor((this.windowWidth()-52)/this.maxItems());
	    rect.width = 72;
	    rect.height = 64;
	    rect.x = 26 + Math.floor((ww-72)/2) + ww*index;
	    rect.y = 0;
	    return rect;
	}


	Window_MenuCommand.prototype.getIconIndex = function(index) {
		var Indexs = [4,1,0,2,3,5,6,7];
		return Indexs[index];
	}

	Window_MenuCommand.prototype.drawItem = function(index) {

		var frameIndex = index;
		if(!JPT_MainMenu.isFiroMoveUnlock()&&index>=5){
			frameIndex++;
		}

		var iconId = this.getIconIndex(frameIndex);

		var rect = this.itemRect(index);
		this._commandIconSpr[index].bitmap = this._bmpIcon;
		this._commandIconSpr[index].setFrame(0,64*iconId,72,64);
		this._commandIconSpr[index].x = rect.x + 36;
		this._commandIconSpr[index].y = rect.y + 32;
		this._commandIconSpr[index].anchor.set(0.5,0.5);

		if(JPT_MainMenu.isFiroMoveUnlock()&&index==5){
			if(!JPT_MainMenu.isFiroMoveEnable()){
				this._commandIconSpr[index].alpha = 160/255;
			}
		}
	};


	Window_MenuCommand.prototype.updateCursor = function() {
		var rect = this.itemRect(this.index());
        this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
	};


	Window_MenuCommand.prototype.update = function() {
		Window_Selectable.prototype.update.call(this);
		this.updateSelectIcon();
	};

	Window_MenuCommand.prototype.updateSelectIcon = function() {

		if(this._befSelectIndex!=this.index()){
			this._befSelectIndex = this.index();
			this._curAnimationCount = 0;
			for(var i=0; i<this.maxItems(); i++){
				var frameIndex = i;
				if(!JPT_MainMenu.isFiroMoveUnlock()&&i>=5){
					frameIndex++;
				}

				var iconId = this.getIconIndex(frameIndex);

				if(i==this.index()){
					this._commandIconSpr[i].setFrame(72*1,64*iconId,72,64);
				}else{
					this._commandIconSpr[i].setFrame(72*0,64*iconId,72,64);
					this._commandIconSpr[i].scale.set(1,1);
				}
			}
		}

		if(JPT_MainMenu.isFiroMoveUnlock()&&this.index()==5){
			if(!JPT_MainMenu.isFiroMoveEnable()){
				return;
			}
		}

		var scale = 1;
		var frame = 40;
		var spd = 0.005;
		var blinkCount = this._curAnimationCount % frame;
		if (blinkCount < Math.floor(frame / 2)) {
			scale += blinkCount * spd;
		}else{
			scale += (frame - blinkCount) * spd;
		}

		this._curAnimationCount++;

		this._commandIconSpr[this.index()].scale.set(scale,scale);
	}


	Window_MenuCommand.prototype.cursorRight = function(wrap) {
		var index = this.index();
	    var maxItems = this.maxItems();
	    var maxCols = this.maxCols();

	    if(index < maxItems - 1){
	    	this.select((index + 1) % maxItems);
	    }else{
	    	this.select(0);
	    }
	};

	Window_MenuCommand.prototype.cursorLeft = function(wrap) {
		var index = this.index();
	    var maxItems = this.maxItems();
	    var maxCols = this.maxCols();

	    if(index > 0){
	        this.select((index - 1 + maxItems) % maxItems);
	    }else{
	    	this.select(maxItems - 1);
	    }
	};



	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuStatus
	////////////////////////////////////////////////////////////////////////////////////
	Window_MenuStatus.prototype.initialize = function(x, y) {

		this.loadImage();
		this.createSprite();

	    var width = this.windowWidth();
	    var height = this.windowHeight();
	    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
	    this._formationMode = false;
	    this._pendingIndex = -1;
	    this.opacity = 0;
	    this.contentsOpacity = 0;
	    this._selectable = false;
	    this._befSelectIndex = -1;

	    this.refresh();
	};

	Window_MenuStatus.prototype.windowWidth = function() {
	    return Graphics.boxWidth;
	};

	Window_MenuStatus.prototype.windowHeight = function() {
	    return (252*2)+8;
	};

	Window_MenuStatus.prototype.numVisibleRows = function() {
	    return 2;
	};

	Window_MenuStatus.prototype.maxCols = function() {
	    return 2;
	};

	Window_MenuStatus.prototype.standardPadding = function() {
	    return 0;
	};

	Window_MenuStatus.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_MenuStatus.prototype.loadImage = function() {

		ImageManager.reserveSystem('IconSet20px');

		this._bmpFrame = ImageManager.loadBitmap("img/menus/", "menu_charaplate_flame");
	    this._bmpPlate = ImageManager.loadBitmap("img/menus/", "menu_charaplate");

	    this._bmpBustUp = [];
	    for(var i=0; i<$gameParty.battleMembers().length; i++){
	    	var actor = $gameParty.battleMembers()[i].actor();
	    	var bName = this.getBustUpName(actor);
	    	this._bmpBustUp[bName] = ImageManager.loadBitmap("img/menus/", bName);
	    }

	};

	Window_MenuStatus.prototype.getBustUpName = function(actor){
		var ret = actor.bust_up;
		if(!ret && actor.bust_up_vid > 0){
			var vid = $gameVariables.value(actor.bust_up_vid);
			ret = actor.bust_up_array[ vid ];
		}
		return ret;
	}


	Window_MenuStatus.prototype.createSprite = function() {
		this._sprBack = new Sprite();
		this._ItemContents = [];
		this._sprPlate = [];
		this._sprFrame = [];
		this._sprCursor = [];
		this._sprStatus = [];

		for(var i=0; i<$gameParty.maxBattleMembers(); i++){
			this._ItemContents[i] = new PIXI.Container();
			this._sprPlate[i] = new Sprite();
			this._sprFrame[i] = new Sprite();
			this._sprCursor[i] = new Sprite();
			this._sprStatus[i] = new Sprite();
		}
	}


	Window_MenuStatus.prototype.createOtherParts = function() {
		this.addChild(this._sprBack);
		for(var i=0; i<$gameParty.maxBattleMembers(); i++){
			this._ItemContents[i].addChild(this._sprPlate[i]);

			this._ItemContents[i].addChild(this._sprCursor[i]);

			this._ItemContents[i].addChild(this._sprStatus[i]);

			this._ItemContents[i].addChild(this._sprFrame[i]);
			this.addChild(this._ItemContents[i]);
	    }

	}

	Window_MenuStatus.prototype.itemRect = function(index) {

		var pos_array = [[8,0],[412,0],[8,260],[412,260]];

	    var rect = new Rectangle();

	    if(index < 0)return rect;

	    rect.width = 396;
	    rect.height = 252;
	    rect.x = pos_array[index][0];
	    rect.y = pos_array[index][1];

	    return rect;
	};

	Window_MenuStatus.prototype.update = function() {
		Window_Selectable.prototype.update.call(this);
		this.updateSelectItem();
	};

	Window_MenuStatus.prototype.updateSelectItem = function() {
		if (this.isCursorMovable()) {

			if(this.index() !== this._befSelectIndex){
				this._befSelectIndex = this.index();
				for(var i=0; i<$gameParty.battleMembers().length; i++){
					if(i==this.index()){
						this._sprCursor[i].visible = true;
					}else{
						this._sprCursor[i].visible = false;
					}
				}
			}

			this.updateCursorAlpha();

		}else{
			this._befSelectIndex = -1;
			for(var i=0; i<$gameParty.battleMembers().length; i++){
				this._sprCursor[i].visible = false;
			}
		}
	}

	Window_MenuStatus.prototype.updateCursorAlpha = function() {
		var blinkCount = this._animationCount % 40;
	    var cursorOpacity = 255;
	    if (this.active) {
	        if (blinkCount < 20) {
	            cursorOpacity -= blinkCount * 8;
	        } else {
	            cursorOpacity -= (40 - blinkCount) * 8;
	        }
	    }
	    this._sprCursor[this.index()].alpha = cursorOpacity / 255;
	}

	Window_MenuStatus.prototype.drawAllPendingItem = function() {

		for(var i=0; i<$gameParty.maxBattleMembers(); i++){
			if(i >= $gameParty.battleMembers().length){
				this.drawPendingItem(i);
			}
		}

	}

	Window_MenuStatus.prototype.drawPendingItem = function(index) {
		var rect = this.itemRect(index);

		this._ItemContents[index].x = rect.x;
		this._ItemContents[index].y = rect.y;

		this._sprPlate[index].bitmap = this._bmpPlate;
		this._sprPlate[index].setFrame(rect.width*2,0,rect.width,rect.height);
		this._sprPlate[index].x = 0;
		this._sprPlate[index].y = 0;
		this._sprPlate[index].width = rect.width;
		this._sprPlate[index].height = rect.height;

		this._sprFrame[index].bitmap = this._bmpFrame;
		this._sprFrame[index].x = 0;
		this._sprFrame[index].y = 0;

	}

	Window_MenuStatus.prototype.drawItem = function(index) {

		var rect = this.itemRect(index);

		this._ItemContents[index].x = rect.x;
		this._ItemContents[index].y = rect.y;

		this._sprPlate[index].bitmap = this._bmpPlate;
		this._sprPlate[index].setFrame(rect.width*0,0,rect.width,rect.height);
		this._sprPlate[index].x = 0;
		this._sprPlate[index].y = 0;
		this._sprPlate[index].width = rect.width;
		this._sprPlate[index].height = rect.height;

		this._sprFrame[index].bitmap = this._bmpFrame;
		this._sprFrame[index].x = 0;
		this._sprFrame[index].y = 0;

		this._sprCursor[index].bitmap = this._bmpPlate;
		this._sprCursor[index].setFrame(rect.width*1,0,rect.width,rect.height);
		this._sprCursor[index].x = 0;
		this._sprCursor[index].y = 0;
		this._sprCursor[index].width = rect.width;
		this._sprCursor[index].height = rect.height;
		this._sprCursor[index].visible = false;

		this.drawItemStatus(index);
	};



	Window_MenuStatus.prototype.drawItemStatus = function(index) {
	    var rect = this.itemRect(index);

	    this._sprStatus[index].bitmap = new Bitmap(rect.width,rect.height);
	    this._sprStatus[index].bitmap.fontFace = 'GameFont';
		this._sprStatus[index].bitmap.textColor = '#ffffff';
		this._sprStatus[index].bitmap.fontSize = 18;
		this._sprStatus[index].bitmap.outlineColor = 'rgba(0, 0, 0, 0)';
		this._sprStatus[index].bitmap.outlineWidth = 0;

	    var actor = $gameParty.battleMembers()[index];

	    //バストアップ
	    this.drawBustUp(index,actor);
	    //名前
	    this.drawNameWithIcon(this._sprStatus[index].bitmap,actor,16,28 );
	    //レベル
	    this.drawLevel(index,actor);
	    //ステートアイコン
	    this.drawStatesIcon(index,actor);
	    //ステータスバー
	    this.drawStatesBar(index,actor);
	};

	Window_MenuStatus.prototype.drawBustUp = function(index,actor) {
		var bName = this.getBustUpName(actor.actor());
	    var bitmap = this._bmpBustUp[bName];
	    this._sprStatus[index].bitmap.blt(bitmap,
	    		0,0,
	    		208,244,
	    		184,4
	    		);
	};

	Window_MenuStatus.prototype.drawLevel = function(index,actor) {
		var name = actor.name();
		var x = 220;
		var y = 34;
		var fontSize = 12;
		this._sprStatus[index].bitmap.fontSize = fontSize;

		this._sprStatus[index].bitmap.drawText('Lv',
	    		x,y+7,
	    		24,fontSize,
	    		'left'
	    		);

		fontSize = 16;
		this._sprStatus[index].bitmap.fontSize = fontSize;

		this._sprStatus[index].bitmap.drawText(actor._level,
	    		x+16,y+3,
	    		100,fontSize,
	    		'left'
	    		);
	}

	Window_MenuStatus.prototype.drawStatesIcon = function(index,actor) {
	    var icons = actor.allIcons();
	    for (var i = 0; i < icons.length; i++) {
	        this.drawIcon(this._sprStatus[index].bitmap,icons[i], 21 + 32 * i, 84);
	    }
	}

	Window_MenuStatus.prototype.drawIcon = function(parentBmp,iconIndex, x, y) {
	    var bitmap = ImageManager.loadSystem('IconSet');
	    var pw = Window_Base._iconWidth;
	    var ph = Window_Base._iconHeight;
	    var sx = iconIndex % 16 * pw;
	    var sy = Math.floor(iconIndex / 16) * ph;
	    parentBmp.blt(bitmap, sx, sy, pw, ph, x, y);
	};

	Window_MenuStatus.prototype.drawStatesBar = function(index,actor) {
		var y = 130;
		var oy = 38;
		var width = 200;
		this.drawActorHp(this._sprStatus[index].bitmap,actor,20,y,width);
		this.drawActorMp(this._sprStatus[index].bitmap,actor,20,y + oy,width);
		this.drawActorSp(this._sprStatus[index].bitmap,actor,20,y + oy*2,width);
	}



	Window_MenuStatus.prototype.drawActorHp = function(parentBmp,actor, x, y, width) {
	    var color = this.hpGaugeColor();
	    this.drawGauge(parentBmp,x, y, width, actor.hpRate(), this.hpColor(actor), this.hpColor(actor));
	    parentBmp.textColor = '#ffffff';
	    parentBmp.fontSize = 16;
	    parentBmp.drawText(TextManager.hpA, x, y, 44, 16, 'left');
	    this.drawCurrentAndMax(parentBmp,actor.hp, actor.mhp, x, y, width, 16,this.hpColor(actor), this.hpNormalColor());
	};

	Window_MenuStatus.prototype.drawActorMp = function(parentBmp,actor, x, y, width) {
	    var color = this.mpGaugeColor();
	    this.drawGauge(parentBmp,x, y, width, actor.mpRate(), color, color);
	    parentBmp.textColor = '#ffffff';
	    parentBmp.fontSize = 16;
	    parentBmp.drawText(TextManager.mpA, x, y, 44, 16, 'left');
	    this.drawCurrentAndMax(parentBmp,actor.mp, actor.mmp, x, y, width, 16,this.mpColor(), this.mpColor());
	};

	Window_MenuStatus.prototype.drawActorSp = function(parentBmp,actor, x, y, width) {
		if(actor.currentClass().msp_eval!=='0'){
		    var color = this.spGaugeColor();
		    this.drawGauge(parentBmp,x, y, width, actor.spRate(), color, color);
		    parentBmp.textColor = '#ffffff';
		    parentBmp.fontSize = 16;
		    parentBmp.drawText(TextManager.spA, x, y, 44, 16, 'left');
		    this.drawCurrentAndMax(parentBmp,actor.sp, actor.msp, x, y, width, 16,this.spColor(), this.spColor());
		}
	};

	Window_MenuStatus.prototype.drawGauge = function(parentBmp,x, y, width, rate, color1, color2) {
	    var fillW = Math.floor(width * rate);
	    var gaugeY = y + 15;
	    var height = 8;
	    parentBmp.fillRect(x, gaugeY, width, height, this.gaugeBackColor());
	    parentBmp.gradientFillRect(x, gaugeY, fillW, height, color1, color2);
	};

	Window_MenuStatus.prototype.drawCurrentAndMax = function(parentBmp,current, max, x, y,width, height,color1, color2) {
		var labelWidth = parentBmp.measureTextWidth('HP');
		var maxValueWidth = parentBmp.measureTextWidth(max);
		var curValueWidth = parentBmp.measureTextWidth(current);
		var slashWidth = parentBmp.measureTextWidth('/');
		var x1 = x + width - maxValueWidth;
		var x2 = x1 - slashWidth;
		var x3 = x2 - curValueWidth;

		parentBmp.textColor = color1;
		parentBmp.drawText(current, x3, y, curValueWidth, height, 'right');
		parentBmp.textColor = color2;
		parentBmp.drawText('/', x2, y, slashWidth, height, 'right');
		parentBmp.drawText(max, x1, y, maxValueWidth, height, 'right');

		/*
		if (x3 >= x + labelWidth) {
			parentBmp.textColor = color1;
			parentBmp.drawText(current, x3, y, curValueWidth, height, 'right');
			parentBmp.textColor = color2;
			parentBmp.drawText('/', x2, y, slashWidth, height, 'right');
			parentBmp.drawText(max, x1, y, maxValueWidth, height, 'right');
		} else {
			parentBmp.textColor = color1;
			parentBmp.drawText(current, x1, y, curValueWidth, height,'right');
		}
		*/
	};


	Window_MenuStatus.prototype.drawBack = function() {
		this._sprBack.bitmap = new Bitmap(this.windowWidth(),this.windowHeight());
		this._sprBack.bitmap.fillAll('#000000');
	}


	Window_MenuStatus.prototype.refresh = function() {
		Window_Selectable.prototype.refresh.call(this);
		this._befSelectIndex = -1;
		this.drawAllPendingItem();
		this.drawBack();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuActor
	////////////////////////////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuGold
	////////////////////////////////////////////////////////////////////////////////////
	Window_MenuGold.prototype = Object.create(Window_Gold.prototype);
	Window_MenuGold.prototype.constructor = Window_MenuGold;

	Window_MenuGold.prototype.initialize = function(x, y) {
	    var width = this.windowWidth();
	    var height = this.windowHeight();
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this.refresh();
	};

	Window_MenuGold.prototype.initialize = function(x, y) {
		//var width = this.windowWidth();
	    //var height = this.windowHeight();
	    Window_Gold.prototype.initialize.call(this, x, y);
	    this.opacity = 0;
	    this.refresh();
	};

	Window_MenuGold.prototype.standardPadding = function() {
	    return 0;
	};

	Window_MenuGold.prototype.windowWidth = function() {
	    return 160;
	};

	Window_MenuGold.prototype.windowHeight = function() {
	    return 32;
	};

	Window_MenuGold.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_MenuGold.prototype.drawCurrencyValue = function(value, unit, x, y, width) {
		this.drawGold(this.contents ,value, x, y, width);
	};

	Window_MenuGold.prototype.refresh = function() {
	    var x = this.textPadding();
	    var width = this.contents.width - this.textPadding() * 2;
	    this.contents.clear();
	    this.drawCurrencyValue(this.value(), this.currencyUnit(), x, 6, width);
	}


	////////////////////////////////////////////////////////////////////////////////////
	// Window_Gold
	////////////////////////////////////////////////////////////////////////////////////

	Window_Gold.prototype.initialize = function(x, y) {
	    var width = this.windowWidth();
	    var height = this.windowHeight();
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this.refresh();
	};

	Window_Gold.prototype.initialize = function(x, y) {
		var width = this.windowWidth();
	    var height = this.windowHeight();
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this.refresh();
	};

	Window_Gold.prototype.standardPadding = function() {
	    return 0;
	};

	Window_Gold.prototype.windowWidth = function() {
	    return 200;
	};

	Window_Gold.prototype.windowHeight = function() {
	    return 40;
	};

	Window_Gold.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_Gold.prototype.drawCurrencyValue = function(value, unit, x, y, width) {
		this.drawGold(this.contents ,value, x, y, width);
	};

	Window_Gold.prototype.refresh = function() {
	    //var x = this.textPadding();
	    //var width = this.contents.width - this.textPadding() * 2;
	    this.contents.clear();
	    this.drawCurrencyValue(this.value(), this.currencyUnit(), 0, 10, 178);
	}


	////////////////////////////////////////////////////////////////////////////////////
	// Window_Status
	////////////////////////////////////////////////////////////////////////////////////
	Window_Status.prototype.initialize = function() {

		this.loadImage();
		this.createSprite();

	    var width = Graphics.boxWidth;
	    var height = this.windowHeight();
	    Window_Selectable.prototype.initialize.call(this, 0, 0, width, height);
	    this._actor = null;
	    this._actorBef = null;
	    this.opacity = 0;
	    this.isMoving = false;
	    this.contentsOpacity = 0;
	    this.direction = 1;
	    this.charaInitX = 40;
	    this.charaOfsetX = 0;
	    this.charaOfsetXInit = 100;
	    this.lineH = 24;
	    this.refresh();
	    this.activate();

	};

	Window_Status.prototype.windowHeight = function() {
	    return 584+8;
	};

	Window_Status.prototype.numVisibleRows = function() {
	    return 1;
	};

	Window_Status.prototype.maxCols = function() {
	    return 2;
	};

	Window_Status.prototype.maxItems = function() {
	    return ($gameParty.battleMembers().length <= 1) ? 0 : 2 ;
	};

	Window_Status.prototype.standardPadding = function() {
	    return 0;
	};


	Window_Status.prototype.loadImage = function() {
		ImageManager.loadBitmap("img/menus/", "menu_status_miniicon");
	    this._bmpTitle = ImageManager.loadBitmap("img/menus/", "menu_title_icon");
	    this._bmpArrow = ImageManager.loadBitmap("img/menus/", "menu_arrow");
	    this._bmpGradation = ImageManager.loadBitmap("img/menus/", "menu_status_gradation");
	    this._bmpClassIcon = ImageManager.loadBitmap("img/menus/", "menu_status_icon");

	    this._bmpChara = [];
	    this._bmpNameLine = ImageManager.loadBitmap("img/menus/", "menu_status_line");

	    //this._bmpNamePlate = [];
	    for(var i=0; i<$gameParty.battleMembers().length; i++){
	    	var actor = $gameParty.battleMembers()[i].actor();
	    	var siName = this.getStatusImageName(actor);
	    	this._bmpChara[siName] = ImageManager.loadBitmap("img/menus/", siName);
	    }


	};


	Window_Status.prototype.getStatusImageName = function(actor){
		var ret = actor.status_image;
		if(!ret && actor.status_image_vid > 0){
			var vid = $gameVariables.value(actor.status_image_vid);
			ret = actor.status_image_array[ vid ];
		}
		return ret;
	}

	Window_Status.prototype.createSprite = function() {

		this._sprTitle = new Sprite();
		this._sprArrowR = new Sprite();
		this._sprArrowL = new Sprite();
		this._sprGradation = new Sprite();
		this._sprNamePalate = new Sprite();
		this._sprStatusPalate = new Sprite();
		this._sprChara = [];

		for(var i=0; i<$gameParty.battleMembers().length; i++){
			var id = $gameParty.battleMembers()[i].actorId();
			this._sprChara[id] = new Sprite();
		}

		this._sprMask = new Sprite();
	}


	Window_Status.prototype.createOtherParts = function() {

		for(var i=0; i<$gameParty.battleMembers().length; i++){
			var id = $gameParty.battleMembers()[i].actorId();
			this.addChild(this._sprChara[id]);
		}

		this.addChild(this._sprTitle);
		this.addChild(this._sprArrowR);
		this.addChild(this._sprArrowL);
		this.addChild(this._sprGradation);

		this.addChild(this._sprNamePalate);
		this.addChild(this._sprStatusPalate);
		this.addChild(this._sprMask);
	}


	Window_Status.prototype.refresh = function() {
	    this.contents.clear();
	    if (this._actor) {
	    	this.CharaImage();
	    	this.StatusPlateImage();

	    }

	    this.NamePlateImage();

	    this.ArrowImage();
	    this.GradationImage();
	    this.TitleImage();
	    this.MaskImage();
	};


	Window_Status.prototype.MaskImage = function() {
		var bitmap = new Bitmap(8, this.windowHeight());
		bitmap.fillAll('#000000ff');
		this._sprMask.bitmap = bitmap;
	}

	Window_Status.prototype.CharaImage = function() {

		for(var i=0; i<$gameParty.battleMembers().length; i++){
			var actor = $gameParty.battleMembers()[i].actor();
			var id = $gameParty.battleMembers()[i].actorId();
			var siName = this.getStatusImageName(actor);
			this._sprChara[id].bitmap = this._bmpChara[siName];
			this._sprChara[id].anchor.set(0,1);
			this._sprChara[id].x = this.charaInitX;
			this._sprChara[id].y = this.windowHeight();
			if(this._actor.actorId()==id){
				this._sprChara[id].alpha = 0;
				this._sprChara[id].visible = true;
				this.charaOfsetX = this.charaOfsetXInit;
			}else{
				this._sprChara[id].alpha = 0;
				this._sprChara[id].visible = false;
			}
		}
	}

	Window_Status.prototype.ArrowImage = function() {

		if($gameParty.battleMembers().length <= 1)return;

		var rect = this.itemRect(0)
		this._sprArrowL.bitmap = this._bmpArrow;
		this._sprArrowL.setFrame(0,0,rect.width,rect.height);
		this._sprArrowL.x = rect.x;
		this._sprArrowL.y = rect.y;
		this._sprArrowL.width = rect.width;
		this._sprArrowL.height = rect.height;

		this._sprArrowR.bitmap = this._bmpArrow;
		this._sprArrowR.setFrame(rect.width,0,rect.width,rect.height);
		//this._sprArrowR.setFrame(0,0,rect.width,rect.height);
		//this._sprArrowR.texture.rotate = 12;
		this._sprArrowR.x = rect.x;
		this._sprArrowR.y = rect.y;
		this._sprArrowR.width = rect.width;
		this._sprArrowR.height = rect.height;
	}

	Window_Status.prototype.GradationImage = function() {
		this._sprGradation.bitmap = this._bmpGradation;
		this._sprGradation.x = 8;
		this._sprGradation.y = 408;
		this._sprGradation.scale.set(10,1);
	}

	Window_Status.prototype.TitleImage = function() {

		this._sprTitle.bitmap = new Bitmap(252,44);
		var parent = this._sprTitle.bitmap;

		parent.fontFace = 'GameFont2';
		parent.fontSize = 24;
		parent.fontBold = false;
		parent.fontItalic = false;

		parent.blt( this._bmpTitle, 48*4,0, 48,44, 0,0 );

		var x = 54;
		var y = 10;
		parent.textColor = '#00000040';
		parent.outlineWidth = 3;
		parent.outlineColor = '#00000040';
		parent.drawText(TextManager.status, x + 2, y + 2, 252 - 48 + 4, parent.fontSize, 'left' );
		parent.textColor = '#ffffffff';
		parent.outlineWidth = 0;
		parent.outlineColor = '#00000000';
		parent.drawText(TextManager.status, x , y, 252 - 48, parent.fontSize, 'left' );

		//this._sprTitle.bitmap = this._bmpTitle;
		//this._sprTitle.setFrame(0,44*4,252,44);
		this._sprTitle.x = 16;
		this._sprTitle.y = 16;
	}

	Window_Status.prototype.NamePlateImage = function() {
		//this._sprNamePalate.bitmap = this._bmpNamePlate[this._actor.actor().class_icon];

		this._sprNamePalate.bitmap = new Bitmap(356,80);
		this._sprNamePalate.x = 8;
		this._sprNamePalate.y = 512;

		var parent = this._sprNamePalate.bitmap;
		parent.fontFace = 'GameFont2';
		parent.fontBold = false;
		parent.fontItalic = false;
		parent.textColor = '#ffffffff';
		parent.outlineWidth = 0;
		//parent.outlineColor = '#00000000';

		parent.blt( this._bmpNameLine, 0,0, 356,8, 0,40 );

		var name = '';
		var icon = 0;
		var cname = '';
		if(this._actor){
			name = this._actor.nickname();
			icon = this._actor.actor().class_icon;
			cname = this._actor.currentClass().name;
		}

		var x = 48;
		var y = 4;
		var width = 356;
		parent.fontSize = 34;
		parent.drawText(name, x , y, width, parent.fontSize, 'left' );

		x = -20;
		y = 54;
		parent.fontSize = 18;
		parent.drawText(cname, x , y, width, parent.fontSize, 'right' );

		x = width -20 - parent.measureTextWidth(cname) - 26;
		y = 52;
		parent.blt(this._bmpClassIcon, 24*icon ,0, 24,20, x, y );
	}


	Window_Status.prototype.StatusPlateImage = function() {
		var startY = 8;
		var width = 300;
		var lineH = this.lineH;
		var horzH = 20;
		var center = Math.floor(width/2);
		var actor = this._actor;
		var lineY = 0;
		var col_width = center -10;
		var next_col_x = center + 10;

		this._sprStatusPalate.bitmap = new Bitmap(width,480);
		this._sprStatusPalate.x = 440;
		this._sprStatusPalate.y = 116 - lineH - startY;

		this._sprStatusPalate.bitmap.fontFace = 'GameFont';
		this._sprStatusPalate.bitmap.outlineWidth = 0;

		var parent = this._sprStatusPalate.bitmap;

		lineY += startY;
		this.drawLv(parent,actor,lineY,width);
		lineY += lineH;
		this.drawExp(parent,actor,lineY,width);
		lineY += lineH;
		this.drawNextExp(parent,actor,lineY,width);

		lineY += horzH;
		this.drawHorzLine(parent,lineY,width,horzH);
		lineY += horzH;

		this.drawHp(parent,actor,lineY,width);
		lineY += lineH;
		this.drawMp(parent,actor,lineY,width);
		lineY += lineH;
		if(actor.currentClass().msp_eval!=='0'){
			this.drawSp(parent,actor,lineY,width);
		}

		lineY += horzH;
		this.drawHorzLine(parent,lineY,width,horzH);
		lineY += horzH;

		this.drawStatusBlock(parent,actor,lineY, col_width, next_col_x );
		lineY += lineH*2;

		lineY += horzH;
		this.drawHorzLine(parent,lineY,width,horzH);
		lineY += horzH;

		this.drawEquipBlock(parent,actor,lineY,width);
		lineY += lineH*2;

		lineY += horzH;
		this.drawHorzLine(parent,lineY,width,horzH);
		lineY += horzH;

		this.drawElementalBlock(parent,actor,lineY, col_width, next_col_x );

	}

	Window_Status.prototype.drawHorzLine = function(parentbmp,y, width, height) {
		Window_Base.prototype.drawHorzLine.call(this,parentbmp,0,y,width, height);
	};

	Window_Status.prototype.drawTextL = function(parentbmp,text,x,y,width) {
		parentbmp.drawText(text,x,y,width,this.lineH,'left');
	}
	Window_Status.prototype.drawTextR = function(parentbmp,text,x,y,width) {
		parentbmp.drawText(text,x,y,width,this.lineH,'right');
	}

	Window_Status.prototype.drawGauge = function(parentbmp,x, y, width, rate, color) {
	    var fillW = Math.floor(width * rate);
	    var gaugeY = y + 15;
	    var height = 8;
	    parentbmp.fillRect(x, gaugeY, width, height, this.gaugeBackColor());
	    parentbmp.gradientFillRect(x, gaugeY, fillW, height, color, color);
	};

	Window_Status.prototype.drawCurrentAndMax = function(parentbmp,current, max, x, y,width, height,color1, color2) {
		var maxValueWidth = parentbmp.measureTextWidth(max);
		var curValueWidth = parentbmp.measureTextWidth(current);
		var slashWidth = parentbmp.measureTextWidth('/');
		var x1 = x + width - maxValueWidth;
		var x2 = x1 - slashWidth;
		var x3 = x2 - curValueWidth;

		parentbmp.textColor = color1;
		parentbmp.drawText(current, x3, y, curValueWidth, height, 'right');
		parentbmp.textColor = color2;
		parentbmp.drawText('/', x2, y, slashWidth, height, 'right');
		parentbmp.drawText(max, x1, y, maxValueWidth, height, 'right');
		parentbmp.textColor = this.NormalColor();
	};

	Window_Status.prototype.drawIconTextParam = function(parentbmp,iconid,text,param, x, y, width) {
		this.drawStatusMiniIcon(parentbmp,iconid,x,y);
		var paramW = parentbmp.measureTextWidth('0000');
		this.drawTextL(parentbmp,text,x + 22,y, width - paramW - 22);
		this.drawTextR(parentbmp,param,x + width - paramW,y, paramW);
	}



	Window_Status.prototype.drawLv = function(parentbmp,actor,y,width){
		parentbmp.fontSize = 22;
		this.drawTextR(parentbmp,actor._level,0,y, width);

		parentbmp.fontSize = 16;
		this.drawTextL(parentbmp,TextManager.levelA,0,y, 24);
	}

	Window_Status.prototype.drawExp = function(parentbmp,actor,y,width){
		var exp = actor.currentExp();
		if (actor.isMaxLevel()) {
			exp = '--';
		}
		parentbmp.fontSize = 16;
		this.drawTextL(parentbmp,TextManager.expA,0,y, 24);
		this.drawTextR(parentbmp,exp,0,y, width);
	}

	Window_Status.prototype.drawNextExp = function(parentbmp,actor,y,width){
		var exp = actor.nextRequiredExp();
		var gauge = actor.currentExp()/actor.nextLevelExp();
		if (actor.isMaxLevel()) {
			exp = '--';
			gauge = 0;
		}
		parentbmp.fontSize = 16;
		this.drawTextL(parentbmp,'NEXT',0,y, 24);
		parentbmp.textColor = this.expColor();
		this.drawTextR(parentbmp,exp,0,y, width);
		this.drawGauge(parentbmp,34,y-8,160, gauge,this.expGaugeColor());
		parentbmp.textColor = this.NormalColor();
	}

	Window_Status.prototype.drawHp = function(parentbmp,actor,y,width){
		parentbmp.fontSize = 16;
		this.drawTextL(parentbmp,TextManager.hpA,0,y, 24);
		this.drawCurrentAndMax(parentbmp,actor.hp, actor.mhp, 0, y, width, this.lineH,
				this.hpColor(actor), this.hpNormalColor());
		this.drawGauge(parentbmp,34,y-8,160, actor.hpRate(),this.hpGaugeColor());
	}

	Window_Status.prototype.drawMp = function(parentbmp,actor,y,width){
		parentbmp.fontSize = 16;
		this.drawTextL(parentbmp,TextManager.mpA,0,y, 24);
		this.drawCurrentAndMax(parentbmp,actor.mp, actor.mmp, 0, y, width, this.lineH,
				this.mpColor(), this.mpColor());
		this.drawGauge(parentbmp,34,y-8,160, actor.mpRate(),this.mpGaugeColor());
	}

	Window_Status.prototype.drawSp = function(parentbmp,actor,y,width){
		parentbmp.fontSize = 16;
		this.drawTextL(parentbmp,TextManager.spA,0,y, 24);
		this.drawCurrentAndMax(parentbmp,actor.sp, actor.msp, 0, y, width, this.lineH,
				this.spColor(), this.spColor());
		this.drawGauge(parentbmp,34,y-8,160, actor.spRate(),this.spGaugeColor());
	}

	Window_Status.prototype.drawStatusBlock = function(parentbmp,actor,y, col_width, next_col_x ){
		this.drawIconTextParam(parentbmp,0,TextManager.param(2),actor.atk,0,y,col_width);
		this.drawIconTextParam(parentbmp,1,TextManager.param(3),actor.def,next_col_x,y,col_width);
		y += this.lineH;
		this.drawIconTextParam(parentbmp,6,TextManager.param(4),actor.mat,0,y,col_width);
		this.drawIconTextParam(parentbmp,7,TextManager.param(5),actor.mdf,next_col_x,y,col_width);
		y += this.lineH;
		this.drawIconTextParam(parentbmp,12,TextManager.param(6),actor.agi,0,y,col_width);
		this.drawIconTextParam(parentbmp,13,TextManager.param(7),actor.luk,next_col_x,y,col_width);
	}


	Window_Status.prototype.drawEquipBlock = function(parentbmp,actor,y,width){
		var icons = {1:0,2:1,3:0,4:2,5:8};
		var equip_ids = actor.actor().menu_equip;
		var x = 0;
		for(var i=0; i<equip_ids.length; i++){
			var eq = actor.equips()[ equip_ids[i] - 1 ];
			this.drawStatusMiniIcon(parentbmp,icons[equip_ids[i]],x,y);
			if(eq){
				this.drawTextL(parentbmp,eq.name,x + 22,y, width - 22);
			}else{
				this.drawTextL(parentbmp,noneItemName,x + 22,y, width - 22);
			}
			y+=this.lineH;
		}
	}

	Window_Status.prototype.drawElementalBlock = function(parentbmp,actor,y,col_width, next_col_x){
		//炎
		this.drawIconTextParam(parentbmp,3,TextManager.element(2),this.elementRate(2) ,0,y,col_width);
		//水
		this.drawIconTextParam(parentbmp,10,TextManager.element(5),this.elementRate(5),next_col_x,y,col_width);
		y += this.lineH;
		//氷
		this.drawIconTextParam(parentbmp,9,TextManager.element(3),this.elementRate(3),0,y,col_width);
		//風
		this.drawIconTextParam(parentbmp,16,TextManager.element(7),this.elementRate(7),next_col_x,y,col_width);
		y += this.lineH;
		//土
		this.drawIconTextParam(parentbmp,15,TextManager.element(6),this.elementRate(6),0,y,col_width);
		//雷
		this.drawIconTextParam(parentbmp,5,TextManager.element(4),this.elementRate(4),next_col_x,y,col_width);
		y += this.lineH;
		//光
		this.drawIconTextParam(parentbmp,4,TextManager.element(8),this.elementRate(8),0,y,col_width);
		//闇
		this.drawIconTextParam(parentbmp,11,TextManager.element(9),this.elementRate(9),next_col_x,y,col_width);
	}

	Window_Status.prototype.elementRate = function(elementid){
		return Math.min( (100 - Math.floor(this._actor.elementRate(elementid)*100 ) ), 100) + '%';
	}





	Window_Status.prototype.update = function() {
		Window_Selectable.prototype.update.call(this);
		this.ArrowUpdate();
		this.CharaUpdate();
	};

	Window_Status.prototype.ArrowUpdate = function() {
		if($gameParty.battleMembers().length <= 1)return;
		var LX = this.itemRect(0).x;
		var RX = this.itemRect(1).x;
		var frame = 40;
		var spd = 0.2;
		var blinkCount = this._animationCount % frame;
		if (blinkCount < Math.floor(frame / 2)) {
			LX -= blinkCount * spd;
			RX += blinkCount * spd;
		}else{
			LX -= (frame - blinkCount) * spd;
			RX += (frame - blinkCount) * spd;
		}

		this._sprArrowL.x = Math.floor( LX );
		this._sprArrowR.x = Math.floor( RX );
	}


	Window_Status.prototype.CharaUpdate = function() {
		if(this.charaOfsetX > 0){
			var frame = 15;
			var id = this._actor.actorId();

			this.charaOfsetX -= this.charaOfsetXInit/frame;

			if(this.charaOfsetX <= 0)this.charaOfsetX = 0;

			this._sprChara[id].x = this.charaInitX + (this.charaOfsetX*this.direction);
			this._sprChara[id].alpha = 1 - this.charaOfsetX/this.charaOfsetXInit;

			this.isMoving = true;

		}else{
			this.isMoving = false;
		}
	}

	Window_Status.prototype.itemRect = function(index) {

		var pos_array = [[12,256],[752,256]];

	    var rect = new Rectangle();

	    if(index < 0)return rect;

	    rect.width = 52;
	    rect.height = 76;
	    rect.x = pos_array[index][0];
	    rect.y = pos_array[index][1];

	    return rect;
	};

	Window_Status.prototype.processTouch = function() {
		if (!this.isMoving) {
			Window_Selectable.prototype.processTouch.call(this);
		}
	};


	Window_Status.prototype.onTouch = function(triggered) {
	    var lastIndex = this.index();
	    var x = this.canvasToLocalX(TouchInput.x);
	    var y = this.canvasToLocalY(TouchInput.y);
	    var hitIndex = this.hitTest(x, y);

	    if (hitIndex >= 0) {
		    if (triggered) {
			    if(hitIndex==0){
			    	this.processPageLeft();
			    }else if(hitIndex==1){
			    	this.processPageRight();
			    }
		    	SoundManager.playCursor();
		    }
	    }
	};


	Window_Status.prototype.processHandling = function() {
	    if (this.isOpenAndActive() && !this.isMoving) {
	        if (this.isOkEnabled() && this.isOkTriggered()) {
	            this.processOk();
	        } else if (this.isCancelEnabled() && this.isCancelTriggered()) {
	            this.processCancel();
	        } else if (this.isHandled('right') && Input.isTriggered('right')) {
	            this.processPageRight();
	        } else if (this.isHandled('left') && Input.isTriggered('left')) {
	            this.processPageLeft();
	        }
	    }
	};

	Window_Status.prototype.processPageRight = function() {
	    this.direction = 1;

		SoundManager.playCursor();
	    this.updateInputData();
	    this.deactivate();
	    this.callHandler('right');

	};

	Window_Status.prototype.processPageLeft = function() {
	    this.direction = -1;

		SoundManager.playCursor();
	    this.updateInputData();
	    this.deactivate();
	    this.callHandler('left');
	};



	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuHelp
	////////////////////////////////////////////////////////////////////////////////////
	Window_MenuHelp.prototype = Object.create(Window_Help.prototype);
	Window_MenuHelp.prototype.constructor = Window_MenuHelp;

	Window_MenuHelp.prototype.initialize = function() {
	    var width = Graphics.boxWidth;
	    var height = this.windowHeight();
	    var x = 8;
	    var y = Graphics.boxHeight - 32 - height;
	    Window_Base.prototype.initialize.call(this, x, y, width, height);

	    this._windowFrameSprite.alpha = 0;
	    this._windowBackSprite.alpha = 0;

	    var sprWinBack = new Sprite();
	    sprWinBack.bitmap = ImageManager.loadBitmap("img/menus/", "menu_commentary");
	    this._windowSpriteContainer.addChild(sprWinBack);
	    this._text = '';
	};

	Window_MenuHelp.prototype.windowWidth = function() {
	    return Graphics.boxWidth - 16;
	};

	Window_MenuHelp.prototype.windowHeight = function() {
	    return 136;
	};


	Window_MenuHelp.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};



	Window_MenuHelp.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_MenuHelp.prototype.lineHeight = function() {
	    return 48;
	};

	Window_MenuHelp.prototype.standardFontSize = function() {
	    return 20;
	};

	Window_MenuHelp.prototype.standardPadding = function() {
	    return 18;
	};

	Window_MenuHelp.prototype.textPadding = function() {
	    return 18;
	};

	Window_MenuHelp.prototype.calcTextHeight = function(textState, all) {
		return this.lineHeight();
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_EquipStatus
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_MainMenu_Window_EquipStatus_initialize = Window_EquipStatus.prototype.initialize;
	Window_EquipStatus.prototype.initialize = function(x, y) {
		this.loadImage();
		_JPT_MainMenu_Window_EquipStatus_initialize.call(this,x,y);
	};

	Window_EquipStatus.prototype.loadImage = function() {

		ImageManager.reserveSystem('IconSet20px');

		ImageManager.loadBitmap("img/menus/", "menu_status_miniicon");
		ImageManager.loadBitmap("img/menus/", "menu_charaicon");

	};

	Window_EquipStatus.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_EquipStatus.prototype.windowWidth = function() {
	    return 324;
	};

	Window_EquipStatus.prototype.windowHeight = function() {
	    return 206;
	};


	Window_EquipStatus.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_EquipStatus.prototype.lineHeight = function() {
	    return 24;
	};

	Window_EquipStatus.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_EquipStatus.prototype.standardPadding = function() {
	    return 4;
	};

	Window_EquipStatus.prototype.textPadding = function() {
	    return 20;
	};


	Window_EquipStatus.prototype.refresh = function() {
	    this.contents.clear();
	    if (this._actor) {
	    	this.contents.fontSize = 18;
	    	this.drawNameWithIcon(this.contents,this._actor, this.textPadding() - 4, 4 );
	        for (var i = 0; i < 6; i++) {
	            this.drawItem(4, this.lineHeight() * (1 + i) + 22, 2 + i);
	        }
	    }
	};

	Window_EquipStatus.prototype.drawItem = function(x, y, paramId) {
		this.contents.fontSize = 16;
		var iconid = {0:0,1:1,2:6,3:7,4:12,5:13};
		this.drawStatusMiniIcon(this.contents,iconid[paramId - 2],x + 16,y);
	    this.drawParamName(x + this.textPadding() + 23, y, paramId);
	    if (this._actor) {
	        this.drawCurrentParam(x + 160, y, paramId);
	    }
	    if (this._tempActor) {
		    this.drawRightArrow(x + 210, y, paramId);
	        this.drawNewParam(x + 232, y, paramId);
	    }
	};

	Window_EquipStatus.prototype.drawParamName = function(x, y, paramId) {
	    this.changeTextColor(this.NormalColor());
	    this.drawText(TextManager.param(paramId), x, y, 120);
	};

	Window_EquipStatus.prototype.drawCurrentParam = function(x, y, paramId) {
		this.changeTextColor(this.NormalColor());
	    this.drawText(this._actor.param(paramId), x, y, 48, 'right');
	};

	Window_EquipStatus.prototype.drawRightArrow = function(x, y, paramId) {
		var newValue = this._tempActor.param(paramId);
	    var diffvalue = newValue - this._actor.param(paramId);
	    this.changeTextColor(this.paramchangeTextColor(diffvalue));
	    this.drawText('\u2192', x, y, 32, 'center');
	};

	Window_EquipStatus.prototype.drawNewParam = function(x, y, paramId) {
	    var newValue = this._tempActor.param(paramId);
	    var diffvalue = newValue - this._actor.param(paramId);
	    this.changeTextColor(this.paramchangeTextColor(diffvalue));
	    this.drawText(newValue, x, y, 48, 'right');
	};

	Window_EquipStatus.prototype.paramchangeTextColor = function(change) {
	    if (change > 0) {
	        return this.powerUpColor();
	    } else if (change < 0) {
	        return this.powerDownColor();
	    } else {
	        return this.NormalColor();
	    }
	};

	Window_EquipStatus.prototype.powerUpColor = function() {
	    return '#74b57b';
	};

	Window_EquipStatus.prototype.powerDownColor = function() {
	    return '#ff0000';
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_EquipSlot
	////////////////////////////////////////////////////////////////////////////////////
	Window_EquipSlot.prototype.maxItems = function() {
	    return this._actor ? this._actor.menuEquipSlots().length : 0;
	};


	Window_EquipSlot.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_EquipSlot.prototype.lineHeight = function() {
	    return 40;
	};

	Window_EquipSlot.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_EquipSlot.prototype.standardPadding = function() {
	    return 22;
	};

	Window_EquipSlot.prototype.textPadding = function() {
	    return 0;
	};

	Window_EquipSlot.prototype.itemPadding = function() {
	    return 7;
	};

	Window_EquipSlot.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_EquipSlot.prototype.item = function() {
	    return this._actor && this._actor.menuEquipSlots().length > 0 ? this._actor.equips()[ this._actor.menuEquips(this.index())] : null;
	};

	Window_EquipSlot.prototype.slotName = function(index) {
	    var slots = this._actor.menuEquipSlots();
	    return this._actor && slots ? $dataSystem.equipTypes[slots[index]] : '';
	};

	Window_EquipSlot.prototype.isEnabled = function(index) {
	    return this._actor ? this._actor.isEquipChangeOk( this._actor.menuEquips(index) ) : false;
	};

	Window_EquipSlot.prototype.drawItem = function(index) {
	    if (this._actor) {
	        var rect = this.itemRectForText(index);

	        console.log(this.textPadding());
	        console.log(rect);

	        this.changeTextColor(this.systemColor());
	        this.changePaintOpacity(this.isEnabled(index));
	        this.drawText(this.slotName(index), rect.x, rect.y, 90, this.lineHeight());
	        this.drawItemName(this._actor.menuEquipItem(index), rect.x + 90, rect.y);
	        this.changePaintOpacity(true);
	    }
	};

	Window_EquipSlot.prototype.slotIndex = function() {
		return this._actor.menuEquips(this.index());
	};


	Window_EquipSlot.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};

	Window_EquipSlot.prototype.drawEquipIcon = function(parentbmp,equip_id,x,y){
		var icons = {1:0,2:1,3:0,4:2,5:8};
		this.drawStatusMiniIcon(parentbmp,icons[equip_id],x,y);
	}

	Window_EquipSlot.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRect(this.index());
	        var nameWidth = 80;
	        this.setCursorRect(rect.x + nameWidth, rect.y + this.itemPadding(), rect.width - nameWidth, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_EquipItem
	////////////////////////////////////////////////////////////////////////////////////


	Window_EquipItem.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_EquipItem.prototype.lineHeight = function() {
	    return 34;
	};

	Window_EquipItem.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_EquipItem.prototype.standardPadding = function() {
	    return 20;
	};

	Window_EquipItem.prototype.textPadding = function() {
	    return 16;
	};

	Window_EquipItem.prototype.itemPadding = function() {
	    return 4;
	};

	Window_EquipItem.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_EquipItem.prototype.setSlotId = function(slotId) {
		if(this._actor){
			var sid = this._actor.menuEquips(slotId);
		    if (this._slotId !== sid) {
		        this._slotId = sid;
		        this.refresh();
		        this.resetScroll();
		    }
		}
	};

	Window_EquipItem.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};

	Window_EquipItem.prototype.drawItem = function(index) {
	    var item = this._data[index];
	    if (item) {
	        var numberWidth = this.numberWidth();
	        var rect = this.itemRect(index);
	        rect.width -= this.textPadding();
	        this.changePaintOpacity(this.isEnabled(item));
	        this.drawItemName(item, rect.x + this.textPadding(), rect.y, rect.width - numberWidth);
	        this.drawItemNumber(item, rect.x, rect.y, rect.width);
	        this.changePaintOpacity(1);
	    }else{
	    	if(this._slotId==1){
		    	var rect = this.itemRect(index);
		        rect.width -= this.textPadding();
		    	this.drawText('', rect.x + this.textPadding(), rect.y, rect.width );
	    	}else{
		    	var rect = this.itemRect(index);
		        rect.width -= this.textPadding();
		    	this.drawText(offEquipName, rect.x + this.textPadding(), rect.y, rect.width );
	    	}
	    }
	};

	Window_EquipItem.prototype.makeItemList = function() {
	    this._data = $gameParty.allItems().filter(function(item) {
	        return this.includes(item);
	    }, this);
   		if (this.includes(null)) {
    		this._data.push(null);
    	}
	};

	Window_EquipItem.prototype.drawItemNumber = function(item, x, y, width) {

		var num = $gameParty.numItems(item);

	    if (this.needsNumber() && num > 1) {
	        this.drawText('×', x, y, width - this.textWidth('00'), 'right');
	        this.drawText(num, x, y, width, 'right');
	    }
	};

	Window_EquipItem.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRect(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_SkillList
	////////////////////////////////////////////////////////////////////////////////////

	Window_SkillList.prototype.initialize = function(x, y, width, height) {
	    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
	    this._actor = null;
	    this._stypeId = 0;
	    this._data = [];
	};

	Window_SkillList.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_SkillList.prototype.lineHeight = function() {
	    return 34;
	};

	Window_SkillList.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_SkillList.prototype.standardPadding = function() {
	    return 20;
	};

	Window_SkillList.prototype.textPadding = function() {
	    return 8;
	};

	Window_SkillList.prototype.itemPadding = function() {
	    return 4;
	};

	Window_SkillList.prototype.spacing = function() {
	    return 20;
	};

	Window_SkillList.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_SkillList.prototype.costWidth = function() {
	    return this.textWidth('000'+TextManager.mpA);
	};

	Window_SkillList.prototype.drawItem = function(index) {
	    var skill = this._data[index];
	    if (skill) {
	        var costWidth = this.costWidth();
	        var rect = this.itemRect(index);
	        rect.width -= this.textPadding();
	        this.changePaintOpacity(this.isEnabled(skill));
	        this.drawItemName(skill, rect.x, rect.y, rect.width - Math.floor(costWidth*1.2) );
	        this.drawSkillCost(skill, rect.x, rect.y, rect.width);
	        this.changePaintOpacity(1);
	    }
	};

	Window_SkillList.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};

	Window_SkillList.prototype.drawSkillCost = function(skill, x, y, width) {
	    if (this._actor.skillSpCost(skill) > 0) {
	        this.changeTextColor(this.spCostColor());
	        this.drawText(this._actor.skillSpCost(skill)+TextManager.spA, x+4, y, width, 'right');
	    } else if (this._actor.skillMpCost(skill) > 0) {
	        this.changeTextColor(this.mpCostColor());
	        this.drawText(this._actor.skillMpCost(skill)+TextManager.mpA, x+4, y, width, 'right');
	    }
	};

	Window_SkillList.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRect(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};





	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuItemCategory
	////////////////////////////////////////////////////////////////////////////////////


	Window_MenuItemCategory.prototype = Object.create(Window_ItemCategory.prototype);
	Window_MenuItemCategory.prototype.constructor = Window_MenuItemCategory;

	Window_MenuItemCategory.prototype.initialize = function() {
		this.loadImage();
		this.createSprite();
		Window_ItemCategory.prototype.initialize.call(this, 0, 0);
	    this.opacity = 0;
	    this.contentsOpacity = 0;
	    this._curAnimationCount = 0;
	    this._befSelectIndex = -1;

	};

	Window_MenuItemCategory.prototype.windowWidth = function() {
	    return Graphics.boxWidth - 16;
	};

	Window_MenuItemCategory.prototype.windowHeight = function() {
	    return 40;
	};

	Window_MenuItemCategory.prototype.standardPadding = function() {
	    return 0;
	};

	Window_MenuItemCategory.prototype.loadImage = function() {
		ImageManager.reserveSystem('IconSet20px');
		this._bmpCategory = ImageManager.loadBitmap("img/menus/", "menu_item_category");
	};

	Window_MenuItemCategory.prototype.createSprite = function() {
		this._commandIconSpr = [];
		for(var i=0; i<this.maxCols(); i++){
			this._commandIconSpr[i] = new Sprite();
		}
		this._commandSelectSpr = new Sprite();
	}

	Window_MenuItemCategory.prototype.createOtherParts = function(){
		for(var i=0; i<this.maxCols(); i++){
			this.addChild(this._commandIconSpr[i]);
		}
		this.addChild(this._commandSelectSpr);
	}

	Window_MenuItemCategory.prototype.itemRect = function(index) {
		var rect = new Rectangle();

	    rect.width = 168;
	    rect.height = 40;
	    rect.x = 28+ 192*index;
	    rect.y = 4;
	    return rect;
	}

	Window_MenuItemCategory.prototype.drawItem = function(index) {
		var rect = this.itemRect(index);
		this._commandIconSpr[index].bitmap = this._bmpCategory;
		this._commandIconSpr[index].setFrame(168*index,0,168,40);
		this._commandIconSpr[index].x = rect.x;
		this._commandIconSpr[index].y = rect.y-2;
	};

	Window_MenuItemCategory.prototype.drawSelectIcon = function() {
		this._commandSelectSpr.bitmap = this._bmpCategory;
		this._commandSelectSpr.setFrame(168*this.index(),40,168,40);
		if(this.index()>=0){
			this._commandSelectSpr.x = this._commandIconSpr[this.index()].x;
			this._commandSelectSpr.y = this._commandIconSpr[this.index()].y;
		}
	};


	Window_MenuItemCategory.prototype.updateCursor = function() {
		var rect = this.itemRect(this.index());
        this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
	};

	Window_MenuItemCategory.prototype.update = function() {
		Window_ItemCategory.prototype.update.call(this);
		this.updateSelectIcon();
	};

	Window_MenuItemCategory.prototype.updateSelectIcon = function() {

		if(this._befSelectIndex != this.index() ){
			this._commandSelectSpr.setFrame(168*this.index(),40,168,40);
			this._commandSelectSpr.x = this._commandIconSpr[this.index()].x;
			this._commandSelectSpr.y = this._commandIconSpr[this.index()].y;
			this._befSelectIndex = this.index();
		}

		var blinkCount = this._curAnimationCount % 40;
	    var cursorOpacity = 255;
        if (blinkCount < 20) {
            cursorOpacity -= blinkCount * 8;
        } else {
            cursorOpacity -= (40 - blinkCount) * 8;
        }

	    this._curAnimationCount++;

	    this._commandSelectSpr.alpha = cursorOpacity / 255;
	}

	Window_MenuItemCategory.prototype.refresh = function() {
		Window_ItemCategory.prototype.refresh.call(this);
		this.drawSelectIcon();
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuItemList
	////////////////////////////////////////////////////////////////////////////////////


	Window_MenuItemList.prototype = Object.create(Window_ItemList.prototype);
	Window_MenuItemList.prototype.constructor = Window_MenuItemList;

	Window_MenuItemList.prototype.initialize = function(x, y, width, height) {
		Window_ItemList.prototype.initialize.call(this, x, y, width, height);
	};

	Window_MenuItemList.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_MenuItemList.prototype.lineHeight = function() {
	    return 34;
	};

	Window_MenuItemList.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_MenuItemList.prototype.standardPadding = function() {
	    return 20;
	};

	Window_MenuItemList.prototype.textPadding = function() {
	    return 8;
	};

	Window_MenuItemList.prototype.itemPadding = function() {
	    return 4;
	};

	Window_MenuItemList.prototype.spacing = function() {
	    return 20;
	};

	Window_MenuItemList.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_MenuItemList.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};

	Window_MenuItemList.prototype.drawItemNumber = function(item, x, y, width) {

		var num = $gameParty.numItems(item);

	    //if (this.needsNumber() && num > 1) {
	        this.drawText('×', x, y, width - this.textWidth('00'), 'right');
	        this.drawText(num, x, y, width, 'right');
	    //}
	};

	Window_MenuItemList.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	        var rect = this.itemRect(this.index());
	        this.setCursorRect(rect.x, rect.y + this.itemPadding(), rect.width, rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuFiroMoveList
	////////////////////////////////////////////////////////////////////////////////////

	function Window_MenuFiroMoveList() {
	    this.initialize.apply(this, arguments);
	}

	Window_MenuFiroMoveList.prototype = Object.create(Window_MenuItemList.prototype);
	Window_MenuFiroMoveList.prototype.constructor = Window_MenuFiroMoveList;

	Window_MenuFiroMoveList.prototype.initialize = function(x, y, width, height) {
		Window_MenuItemList.prototype.initialize.call(this, x, y, width, height);
	};

	Window_MenuFiroMoveList.prototype.includes = function(item) {
		return DataManager.isItem(item) && item.itypeId === 4;
	};

	Window_MenuFiroMoveList.prototype.drawItemNumber = function(item, x, y, width) {
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_SavefileList
	////////////////////////////////////////////////////////////////////////////////////

	Window_SavefileList.prototype.initialize = function(x, y, width, height,ginfo) {

		this.loadImage();
		this.createSprite();
		this.setGlobalInfo(ginfo)

		width = this.windowWidth();
		height = this.windowHeight();
	    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
	    this.activate();
	    this._mode = null;
	    this._befSelectIndex = -1;
	    this.opacity = 0;
	};

	Window_SavefileList.prototype.maxVisibleItems = function() {
	    return 3;
	};

	Window_SavefileList.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_SavefileList.prototype.windowWidth = function() {
	    return Graphics.boxWidth;
	};

	Window_SavefileList.prototype.windowHeight = function() {
	    return this.lineHeight()*3 + this.standardPadding()*2;
	};

	Window_SavefileList.prototype.lineHeight = function() {
	    return 170;
	};

	Window_SavefileList.prototype.standardFontSize = function() {
	    return 18;
	};

	Window_SavefileList.prototype.standardPadding = function() {
	    return 16;
	};

	Window_SavefileList.prototype.spacing = function() {
	    return 0;
	};

	Window_SavefileList.prototype.loadImage = function() {
		this._bmpSlot = ImageManager.loadBitmap("img/menus/", "menu_save_slot");
		ImageManager.loadBitmap("img/menus/", "menu_money");
	};

	Window_SavefileList.prototype.setGlobalInfo = function(ginfo) {
		this._globalInfo = ginfo;
	}

	Window_SavefileList.prototype.createSprite = function() {
		this._ItemContents = [];
		this._sprPlate = [];
		this._sprCursor = [];
		this._sprStatus = [];

		for(var i=0; i<this.maxItems(); i++){
			this._ItemContents[i] = new PIXI.Container();
			this._sprPlate[i] = new Sprite();
			this._sprCursor[i] = new Sprite();
			this._sprStatus[i] = new Sprite();
		}
	}

	Window_SavefileList.prototype.createOtherParts = function() {
		for(var i=0; i<this.maxItems(); i++){
			this._ItemContents[i].addChild(this._sprPlate[i]);
			this._ItemContents[i].addChild(this._sprCursor[i]);
			this._ItemContents[i].addChild(this._sprStatus[i]);
			this._windowContentsSprite.addChild(this._ItemContents[i]);
	    }
	}

	Window_SavefileList.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


	Window_SavefileList.prototype.resetBitmapFontSettings = function(bitmap) {
		bitmap.fontFace = this.standardFontFace();
		bitmap.fontSize = this.standardFontSize();
		bitmap.fontBold = false;
		bitmap.fontItalic = false;
		bitmap.outlineWidth = 0;
		bitmap.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_SavefileList.prototype.itemRect = function(index) {
	    var rect = new Rectangle();
	    var maxCols = this.maxCols();
	    rect.width = this.itemWidth();
	    rect.height = this.itemHeight();
	    rect.x = index % maxCols * (rect.width + this.spacing()) - this._scrollX;
	    rect.y = Math.floor(index / maxCols) * rect.height - this._scrollY;
	    return rect;
	};

	Window_SavefileList.prototype.clearItem = function(index) {
		this._ItemContents[index].visible = false;
	};

	Window_SavefileList.prototype.drawAllItems = function() {
		for(var i=0; i<this.maxItems(); i++){
			this.clearItem(i);
		}
		Window_Selectable.prototype.drawAllItems.call(this);
	};


	Window_SavefileList.prototype.drawItem = function(index) {

		var id = index + 1;
	    var rect = this.itemRect(index);

	    this._ItemContents[index].visible = true;
	    this._ItemContents[index].x = rect.x - 8;
	    this._ItemContents[index].y = rect.y + 5;

	    var info = DataManager.loadSavefileInfo_G(id,this._globalInfo);

    	if (this._mode === 'load') {
	    	if(!DataManager.isThisGameFile_G(id,this._globalInfo)){
	    		this._ItemContents[index].alpha = 160/255;
	    	}
	    }
	    this.plateImage(index);
	    this.statusImage(index,info);
	};

	Window_SavefileList.prototype.plateImage = function(index){
		this._sprPlate[index].bitmap = this._bmpSlot;
		this._sprPlate[index].setFrame(0,0,800,160);

		this._sprCursor[index].bitmap = this._bmpSlot;
		this._sprCursor[index].setFrame(0,160,800,160);
	}

	Window_SavefileList.prototype.statusImage = function(index,info){
		this._sprStatus[index].bitmap = new Bitmap(800,160);
		var parent = this._sprStatus[index].bitmap;
		this.resetBitmapFontSettings(parent);
		this.drawFileID(parent,index+1,48,14,180);

		if(info){
			this.drawInfoPlaytime(parent,info, 540, 66, 200);
			this.drawInfoGold(parent,info, 483, 111, 260);
			this.drawInfoCharacters(parent,info, 76, 138);
		}
	}



	Window_SavefileList.prototype.drawInfoPlaytime = function(parent,info, x, y, width) {
	    if (info.playtime) {
	    	var twN = parent.measureTextWidth('00');
	    	var twS = parent.measureTextWidth(' : ');
	    	var time = info.playtime.split(/:/);

			parent.drawText(time[2], x, y, width ,parent.fontSize , 'right');
			x -= twN;
			parent.drawText(' : ', x, y, width ,parent.fontSize , 'right');
			x -= twS;
			parent.drawText(time[1], x, y, width ,parent.fontSize , 'right');
			x -= twN;
			parent.drawText(' : ', x, y, width ,parent.fontSize , 'right');
			x -= twS;
			parent.drawText(time[0], x, y, width ,parent.fontSize , 'right');
	    }
	};

	Window_SavefileList.prototype.drawInfoGold = function(parent,info, x, y, width) {
	    if (info.gold >= 0) {
	    	this.drawGold(parent,info.gold,x,y,width);
	    }
	};

	Window_SavefileList.prototype.drawInfoCharacters = function(parent, info, x, y) {
		if (info.characters) {
			for (var i = 0; i < info.characters.length; i++) {
	            var data = info.characters[i];
	            this.drawInfoCharacter(parent, data[0], data[1], x + i * 84, y);
	            this.drawInfoLevel(parent, data[2], x + i * 84, y - 78);
	        }
		}
	}

	Window_SavefileList.prototype.drawInfoCharacter = function(parent, characterName, characterIndex, x, y) {
		var bitmap = ImageManager.loadCharacter(characterName);
		var big = ImageManager.isBigCharacter(characterName);
		var pw = bitmap.width / (big ? 3 : 12);
		var ph = bitmap.height / (big ? 4 : 8);
		var n = characterIndex;
		var sx = (n % 4 * 3 + 1) * pw;
		var sy = (Math.floor(n / 4) * 4) * ph;
		parent.blt(bitmap, sx, sy, pw, ph, x - pw / 2, y - ph);
	};

	Window_SavefileList.prototype.drawInfoLevel = function(parent, level, x, y) {

		parent.fontSize = 14;
		var lv_w = parent.measureTextWidth('Lv ');
		var width = lv_w;
		parent.fontSize = 18;
		var num_w = parent.measureTextWidth(level);
		width += num_w;

		x -= Math.floor(width/2);

		parent.fontSize = 14;
		parent.drawText('Lv ', x, y+3, lv_w ,parent.fontSize , 'left');

		parent.fontSize = 18;
		parent.drawText(level, x + lv_w, y, num_w ,parent.fontSize , 'left');
	}


	Window_SavefileList.prototype.update = function() {
		Window_Selectable.prototype.update.call(this);
		this.updateSelectItem();
	};

	Window_SavefileList.prototype.updateSelectItem = function() {
		if(this.index() !== this._befSelectIndex){
			this._befSelectIndex = this.index();
			for(var i=0; i<this.maxItems(); i++){
				if(i==this.index()){
					this._sprCursor[i].visible = true;
				}else{
					this._sprCursor[i].visible = false;
				}
			}
		}

		this.updateCursorAlpha();
	}

	Window_SavefileList.prototype.updateCursorAlpha = function() {
		var blinkCount = this._animationCount % 40;
	    var cursorOpacity = 255;
	    if (this.active) {
	        if (blinkCount < 20) {
	            cursorOpacity -= blinkCount * 8;
	        } else {
	            cursorOpacity -= (40 - blinkCount) * 8;
	        }
	    }
	    this._sprCursor[this.index()].alpha = cursorOpacity / 255;
	}


	Window_SavefileList.prototype.updateCursor = function() {
		this.setCursorRect(0, 0, 0, 0);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// AutoSave
	////////////////////////////////////////////////////////////////////////////////////

	if(autoSaveOn){
		//オートセーブ機能有効時
		var runAutoSave = false;
		var AutoSaveSlot = 1;


		var _JPT_MainMenu_Scene_Map_update = Scene_Map.prototype.update;
		Scene_Map.prototype.update = function() {
			if(runAutoSave){
		    	$gameSystem.onBeforeSave();
			    if (DataManager.saveGame(AutoSaveSlot)) {
			    	StorageManager.cleanBackup(AutoSaveSlot);
			    }
		    	runAutoSave = false;
		    }

			_JPT_MainMenu_Scene_Map_update.call(this);

		};

		Scene_Save.prototype.onSavefileOk = function() {
		    Scene_File.prototype.onSavefileOk.call(this);
		    if( this.savefileId() == AutoSaveSlot){
		    	this.onSaveFailure();
		    }else{
			    $gameSystem.onBeforeSave();
			    if (DataManager.saveGame(this.savefileId())) {
			        this.onSaveSuccess();
			    } else {
			        this.onSaveFailure();
			    }
		    }
		};


		var _JPT_MainMenu_BattleManager_updateBattleEnd = BattleManager.updateBattleEnd;
		BattleManager.updateBattleEnd = function() {
			if (this.isBattleTest()) {
		    	runAutoSave = false;
		    } else if (!this._escaped && $gameParty.isAllDead()) {
		    	runAutoSave = false;
		    } else {
		    	runAutoSave = true;
		    }
			_JPT_MainMenu_BattleManager_updateBattleEnd.call(this);
		};


		Game_Player.prototype.performTransfer = function() {
		    if (this.isTransferring()) {
		        this.setDirection(this._newDirection);
		        if (this._newMapId !== $gameMap.mapId() || this._needsMapReload) {
		            $gameMap.setup(this._newMapId);
		            this._needsMapReload = false;
		        }
		        this.locate(this._newX, this._newY);
		        this.refresh();
		        this.clearTransferInfo();
		        runAutoSave = true;
		    }
		};

		Window_SavefileList.prototype.drawFileID = function(parent,id, x, y, width) {

			if(id==AutoSaveSlot){
				parent.drawText(TextManager.file + ' Auto', x, y, width ,parent.fontSize , 'left');
			}else{
				parent.drawText(TextManager.file + ' ' + (id - 1), x, y, width ,parent.fontSize , 'left');
			}
		}

		DataManager.saveGameWithoutRescue = function(savefileId) {
		    var json = JsonEx.stringify(this.makeSaveContents());
		    if (json.length >= 200000) {
		        console.warn('Save data too big!');
		    }
		    StorageManager.save(savefileId, json);
		    if(savefileId!=AutoSaveSlot){
		    	this._lastAccessedId = savefileId;
		    }
		    var globalInfo = this.loadGlobalInfo() || [];
		    globalInfo[savefileId] = this.makeSavefileInfo();
		    this.saveGlobalInfo(globalInfo);
		    return true;
		};

		DataManager.loadGameWithoutRescue = function(savefileId) {
		    var globalInfo = this.loadGlobalInfo();
		    if (this.isThisGameFile(savefileId)) {
		        var json = StorageManager.load(savefileId);
		        this.createGameObjects();
		        this.extractSaveContents(JsonEx.parse(json));
		        if(savefileId!=AutoSaveSlot){
		        	this._lastAccessedId = savefileId;
		        }
		        return true;
		    } else {
		        return false;
		    }
		};

		DataManager.selectSavefileForNewGame = function() {
		    var globalInfo = this.loadGlobalInfo();
		    this._lastAccessedId = 2;
		    if (globalInfo) {
		        var numSavefiles = Math.max(0, globalInfo.length - 1);
		        if (numSavefiles < this.maxSavefiles()) {
		            if(numSavefiles>0)this._lastAccessedId = numSavefiles + 1;
		        } else {
		            var timestamp = Number.MAX_VALUE;
		            for (var i = 2; i < globalInfo.length; i++) {
		                if (!globalInfo[i]) {
		                    this._lastAccessedId = i;
		                    break;
		                }
		                if (globalInfo[i].timestamp < timestamp) {
		                    timestamp = globalInfo[i].timestamp;
		                    this._lastAccessedId = i;
		                }
		            }
		        }
		    }
		};

	}else{

		Window_SavefileList.prototype.drawFileID = function(parent,id, x, y, width) {
			parent.drawText(TextManager.file + ' ' +id, x, y, width ,parent.fontSize , 'left');
		}

	}


	////////////////////////////////////////////////////////////////////////////////////
	// Window_Options
	////////////////////////////////////////////////////////////////////////////////////
	Window_Options.prototype.initialize = function() {
	    Window_Command.prototype.initialize.call(this, 0, 0);
	    this.updatePlacement();
	};

	Window_Options.prototype.windowWidth = function() {
	    return 634;
	};

	Window_Options.prototype.windowHeight = function() {
	    return 336;
	};
	Window_Options.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_Options.prototype.standardPadding = function() {
	    return 56;
	};

	Window_Options.prototype.lineHeight = function() {
	    return 32;
	};

	Window_Options.prototype.lineYOffset = function() {
	    return 38;
	};

	Window_Options.prototype.textPadding = function() {
	    return 36;
	};

	Window_Options.prototype.itemPadding = function() {
	    return 4;
	};

	Window_Options.prototype.standardFontFace = function() {
		 return 'GameFont';
	};


	Window_Options.prototype.addGeneralOptions = function() {
	    this.addCommand(TextManager.alwaysDash, 'alwaysDash');
	    //this.addCommand(TextManager.commandRemember, 'commandRemember');
	};


	Window_Options.prototype.addVolumeOptions = function() {
	    this.addCommand(TextManager.bgmVolume, 'bgmVolume');
	    this.addCommand(TextManager.bgsVolume, 'bgsVolume');
	    //this.addCommand(TextManager.meVolume, 'meVolume');
	    this.addCommand(TextManager.seVolume, 'seVolume');
	};


	Window_Options.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_Options.prototype.itemRect = function(index) {
	    var rect = new Rectangle();
	    var maxCols = this.maxCols();
	    rect.width = this.itemWidth();
	    rect.height = this.itemHeight();
	    rect.x = index % maxCols * (rect.width + this.spacing()) - this._scrollX;
	    rect.y = Math.floor(index / maxCols) * rect.height - this._scrollY;
	    rect.y += this.lineYOffset();
	    return rect;
	};

	Window_Options.prototype.statusWidth = function() {
	    return 140;
	};

	Window_Options.prototype.drawItem = function(index) {
	    var rect = this.itemRectForText(index);
	    var statusWidth = this.statusWidth();
	    var titleWidth = rect.width - statusWidth;
	    this.resetTextColor();
	    this.changePaintOpacity(this.isCommandEnabled(index));
	    this.drawText(this.commandName(index), rect.x, rect.y, titleWidth, 'left');
	    this.drawStatusText(index);
	    this.drawHorzLine(this.contents, rect.x-4, rect.y+11, rect.width+4, rect.height);
	};

	Window_Options.prototype.itemRectCol = function(index){
		var symbol = this.commandSymbol(index);
		var value = this.getConfigValue(symbol);

		var padding = Math.floor(this.textPadding()/1.5);

		if (this.isVolumeSymbol(symbol)) {
			var rect = this.itemRectForText(index);
			rect.x = rect.width - this.statusWidth() + padding;
			rect.width = this.textWidth('000%');
			rect.x += ( Math.floor(this.statusWidth()/2) - Math.floor(rect.width/2) );
			return [ rect, rect, true ];
		}else{
			var rect1 = this.itemRectForText(index);
			rect1.x = rect1.width - this.statusWidth() + padding;
			rect1.width = this.textWidth('OFF');
			var rect2 = this.itemRectForText(index);
			var w2 = this.textWidth('OFF');
			rect2.x = rect2.width - w2 + padding;
			rect2.width = w2;
			return [ rect1, rect2, value ];
		}
	}

	Window_Options.prototype.drawStatusText = function(index) {
		var symbol = this.commandSymbol(index);
	    var value = this.getConfigValue(symbol);

	    var rectCol = this.itemRectCol(index);

	    if (this.isVolumeSymbol(symbol)) {
	    	this.drawText(this.volumeStatusText(value), rectCol[0].x, rectCol[0].y, rectCol[0].width, 'right');
	    }else{
	    	this.booleanTextColor(value);
	    	this.drawText('ON', rectCol[0].x + ( Math.floor(rectCol[0].width/2) - Math.floor(this.textWidth('ON')/2) ),
	    			rectCol[0].y, rectCol[0].width, 'left');
	    	this.booleanTextColor(!value);
	    	this.drawText('OFF', rectCol[1].x, rectCol[1].y, rectCol[1].width, 'right');
	    }
	}

	Window_Options.prototype.booleanTextColor = function(value) {
		if (value) {
			this.contents.textColor = this.NormalColor();
		}else{
			this.contents.textColor = '#788484';
		}
	}

	Window_Options.prototype.updateCursor = function() {
	    if (this._cursorAll) {
	        var allRowsHeight = this.maxRows() * this.itemHeight();
	        this.setCursorRect(0, 0, this.contents.width, allRowsHeight);
	        this.setTopRow(0);
	    } else if (this.isCursorVisible()) {
	    	var curWidth = 80;
	    	var rectCol = this.itemRectCol(this.index());
	    	var rect = (rectCol[2]) ? rectCol[0] : rectCol[1];
	    	this.setCursorRect(
	    			rect.x + ( Math.floor(rect.width/2) - Math.floor(curWidth/2) ),
	    			rect.y + this.itemPadding()-2,
	    			curWidth,
	    			rect.height - this.itemPadding()*2 );
	    } else {
	        this.setCursorRect(0, 0, 0, 0);
	    }
	};

	Window_Options.prototype.cursorRight = function(wrap) {
	    var index = this.index();
	    var symbol = this.commandSymbol(index);
	    var value = this.getConfigValue(symbol);
	    if (this.isVolumeSymbol(symbol)) {
	        value += this.volumeOffset();
	        value = value.clamp(0, 100);
	        this.changeValue(symbol, value);
	    } else {
	        this.changeValue(symbol, false);
	    }
	    this.updateCursor();
	};

	Window_Options.prototype.cursorLeft = function(wrap) {
	    var index = this.index();
	    var symbol = this.commandSymbol(index);
	    var value = this.getConfigValue(symbol);
	    if (this.isVolumeSymbol(symbol)) {
	        value -= this.volumeOffset();
	        value = value.clamp(0, 100);
	        this.changeValue(symbol, value);
	    } else {
	        this.changeValue(symbol, true);
	    }
	    this.updateCursor();
	};


	Window_Options.prototype.processOk = function() {
	    var index = this.index();
	    var symbol = this.commandSymbol(index);
	    var value = this.getConfigValue(symbol);
	    if (this.isVolumeSymbol(symbol)) {
	        value += this.volumeOffset();
	        if (value > 100) {
	            value = 0;
	        }
	        value = value.clamp(0, 100);
	        this.changeValue(symbol, value);
	    } else {
	        this.changeValue(symbol, !value);
	    }
	    this.updateCursor();
	};

	Window_Options.prototype.processOkCol = function(col) {
	    var index = this.index();
	    var symbol = this.commandSymbol(index);
	    var value = this.getConfigValue(symbol);
	    if (this.isVolumeSymbol(symbol)) {
	        value += this.volumeOffset();
	        if (value > 100) {
	            value = 0;
	        }
	        value = value.clamp(0, 100);
	        this.changeValue(symbol, value);
	    } else {
	        this.changeValue(symbol, (col==0));
	    }
	    this.updateCursor();
	};

	Window_Options.prototype.onTouch = function(triggered) {
	    var lastIndex = this.index();
	    var x = this.canvasToLocalX(TouchInput.x);
	    var y = this.canvasToLocalY(TouchInput.y);
	    var hitArray = this.hitTestCol(x, y);
	    var hitIndex = hitArray[0];
	    if (hitIndex >= 0) {
	    	if (hitArray[1] < 0) {
		        if (hitIndex === this.index()) {
		            if (triggered && this.isTouchOkEnabled()) {
		                this.processOk();
		            }
		        } else if (this.isCursorMovable()) {
		            this.select(hitIndex);
		        }
	    	}else{
	    		if (triggered) {
	    			this.select(hitIndex);
	                this.processOkCol(hitArray[1]);
	            }
	    	}
	    } else if (this._stayCount >= 10) {
	        if (y < this.padding) {
	            this.cursorUp();
	        } else if (y >= this.height - this.padding) {
	            this.cursorDown();
	        }
	    }
	    if (this.index() !== lastIndex) {
	        SoundManager.playCursor();
	    }
	};

	Window_Options.prototype.hitTestCol = function(x, y) {
	    if (this.isContentsArea(x, y)) {
	        var cx = x - this.padding;
	        var cy = y - this.padding;
	        var topIndex = this.topIndex();
	        for (var i = 0; i < this.maxPageItems(); i++) {
	            var index = topIndex + i;
	            if (index < this.maxItems()) {
	                var rect = this.itemRect(index);
	                var right = rect.x + rect.width;
	                var bottom = rect.y + rect.height;
	                if (cx >= rect.x && cy >= rect.y && cx < right && cy < bottom) {
	                	for (var j = 0; j < 2; j++) {
	                		var rectCol = this.itemRectCol(index);
	                		 var rightC = rectCol[j].x + rectCol[j].width;
	                		if (cx >= rectCol[j].x  && cx < rightC) {
	                			return [index, j];
	                		}
	                	}
	                    return [index, -1];
	                }
	            }
	        }
	    }
	    return [-1,-1];
	};





	////////////////////////////////////////////////////////////////////////////////////
	// Window_MenuOptions
	////////////////////////////////////////////////////////////////////////////////////
	function Window_MenuOptions() {
	    this.initialize.apply(this, arguments);
	}

	Window_MenuOptions.prototype = Object.create(Window_Options.prototype);
	Window_MenuOptions.prototype.constructor = Window_Options;

	Window_MenuOptions.prototype.windowWidth = function() {
	    return 800;
	};

	Window_MenuOptions.prototype.windowHeight = function() {
	    return Graphics.boxHeight - 32 - 72;
	};

	Window_MenuOptions.prototype.standardPadding = function() {
	    return 105;
	};

	Window_MenuOptions.prototype.lineHeight = function() {
	    return 44;
	};

	Window_MenuOptions.prototype.textPadding = function() {
	    return 58;
	};

	Window_MenuOptions.prototype.itemPadding = function() {
	    return 11;
	};

	Window_MenuOptions.prototype.updatePlacement = function() {
	    this.x = (Graphics.boxWidth - this.width) / 2;
	    this.y = 72;
	};

	Window_MenuOptions.prototype.lineYOffset = function() {
	    return 60;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_Message
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_MainMenu_Window_Message_initialize = Window_Message.prototype.initialize;
	Window_Message.prototype.initialize = function() {
		_JPT_MainMenu_Window_Message_initialize.call(this);
		//this.move(this.x, this.y, this.width, this.height);
	};


	Window_Message.prototype.windowWidth = function() {
	    return Graphics.boxWidth - 16;
	};

	Window_Message.prototype.windowHeight = function() {
	    return this.fittingHeight(this.numVisibleRows());
	};

	Window_Message.prototype.standardFontSize = function() {
	    return 24;
	};

	Window_Message.prototype.standardFontFace = function() {
		 return 'GameFont3';
	};

	var _JPT_MainMenu_Window_Message_resetFontSettings = Window_Message.prototype.resetFontSettings;
	Window_Message.prototype.resetFontSettings = function() {
		_JPT_MainMenu_Window_Message_resetFontSettings.call(this);
	    this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	    this.contents.outlineWidth = 0;
	};

	Window_Message.prototype.updatePlacement = function() {
	    this._positionType = $gameMessage.positionType();
	    this.y = this._positionType * (Graphics.boxHeight - this.height) / 2 + ( 8 - this._positionType * 8 );
	    this._goldWindow.y = this.y > 0 ? 0 : Graphics.boxHeight - this._goldWindow.height;
	};


	/*
	Window_Message.prototype.refreshDimmerBitmap = function() {
	    if (this._dimmerSprite) {
	        var bitmap = this._dimmerSprite.bitmap;
	        var w = this.width;
	        var h = this.height;
	        var m = this.padding;
	        var c1 = this.dimColor1();
	        var c2 = this.dimColor2();
	        bitmap.resize(w, h);
	        bitmap.gradientFillRect(0, 0, w, m, c2, c1, true);
	        bitmap.fillRect(m, m, w - m * 2, h - m * 2, c1);
	        bitmap.gradientFillRect(0, h - m, w, m, c1, c2, true);

	        bitmap.gradientFillRect(0, m, m, h-m*2, c2, c1, false);
	        bitmap.gradientFillRect(w-m, m, m, h-m*2, c1, c2, false);
	        this._dimmerSprite.setFrame(0, 0, w, h);
	    }
	};
	*/


	////////////////////////////////////////////////////////////////////////////////////
	// Window_NameBox
	////////////////////////////////////////////////////////////////////////////////////


	Window_NameBox.prototype.initialize = function(parentWindow) {

		this.loadImage();
		this.createSprite();

	    this._parentWindow = parentWindow;
		this._ignoreMask = true
	    Window_Base.prototype.initialize.call(this, 0, 0, 240, this.windowHeight());
		this._text = '';
		this._openness = 0;
		this._closeCounter = 0;
		this.deactivate();
		this.createOtherParts();
		this.opacity = 0;
	};


	Window_NameBox.prototype.standardFontFace = function() {
	    return 'GameFont3';
	};


	Window_NameBox.prototype.standardFontSize = function() {
	    return 24;
	};

	Window_NameBox.prototype.windowHeight = function() {
	    return 48;
	};

	Window_NameBox.prototype.windowWidth = function() {
		this.resetFontSettings();
	    var dw = this.textWidthEx(this._text);
		dw += this.standardPadding() * 2;
		return Math.max( 216-16, dw );
	};

	Window_NameBox.prototype.standardPadding = function() {
	    return 8;
	};

	Window_NameBox.prototype.textPadding = function() {
	    return 0;
	};


	Window_NameBox.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
	    this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	    this.contents.outlineWidth = 0;
	};

	Window_NameBox.prototype.createSprite = function() {
		this._backSpr = new Sprite();
	}

	Window_NameBox.prototype.createOtherParts = function(){
		this.addChildToBack(this._backSpr);
	}

	Window_NameBox.prototype.loadImage = function() {
		this._bmpPlate = ImageManager.loadBitmap("img/menus/", "menu_wordwindow_name");
	}

	var _JPT_MainMenu_Window_NameBox_refresh = Window_NameBox.prototype.refresh;
	Window_NameBox.prototype.refresh = function(text, position) {

		this._text = Yanfly.Param.MSGNameBoxText + text;

		var winWith = this.windowWidth();
		var textWith = this.drawTextEx(this._text, 0, 0);

		if(winWith > textWith){
			var text_x = Math.floor(winWith/2)-Math.floor(textWith/2)-this.standardPadding();
			Yanfly.Param.MSGNameBoxPadding = text_x*2;
		}else{
			Yanfly.Param.MSGNameBoxPadding = 0;
		}

		this.drawNamePlate(winWith);

		var ret =_JPT_MainMenu_Window_NameBox_refresh.call(this,text, position);

		return ret;
	};

	Window_NameBox.prototype.drawNamePlate = function(width){
		var w = width;
		var h = this.windowHeight();
		this._backSpr.bitmap = new Bitmap(w + 16,this.windowHeight());
		this._backSpr.bitmap.blt(this._bmpPlate, 0,0, 32 ,this.windowHeight(), 0,0 ,w,h);
		this._backSpr.bitmap.blt(this._bmpPlate, 200,0, 16 ,this.windowHeight(), w,0 ,16,h);
		this._backSpr.setFrame(0, 0, w + 16, h);
	}

	var _JPT_MainMenu_Window_NameBox_update = Window_NameBox.prototype.update;
	Window_NameBox.prototype.update = function() {

		this.openNamePlate();

		_JPT_MainMenu_Window_NameBox_update.call(this);
	};

	Window_NameBox.prototype.openNamePlate = function() {
		//スライド
		this.x = -255 + this.openness;

		//αフェード
		this._backSpr.alpha = this.openness/255;

		//拡縮
		this._backSpr.scale.y = this.openness / 255;
        this._backSpr.y = this._backSpr.height / 2 * (1 - this.openness / 255);
	}


	Window_NameBox.prototype.adjustPositionY = function() {
		if ($gameMessage.positionType() === 0) {
			this.y = this._parentWindow.y + this._parentWindow.height;
			this.y -= this._backSpr.height - this._backSpr.height/2 - 12;
		} else {
			this.y = this._parentWindow.y;
			this.y -= this._backSpr.height/2 + 12;
		}
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_ChoiceList
	////////////////////////////////////////////////////////////////////////////////////

	Window_ChoiceList.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_ChoiceList.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
	    this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	    this.contents.outlineWidth = 0;
	};

	var _JPT_MainMenu_Window_ChoiceList_updatePlacement = Window_ChoiceList.prototype.updatePlacement;
	Window_ChoiceList.prototype.updatePlacement = function() {
		_JPT_MainMenu_Window_ChoiceList_updatePlacement.call(this);
		var messagePosType = $gameMessage.positionType();
		if (messagePosType === 0) {
			this.y = this._messageWindow.y + this._messageWindow.height;
		} else if (messagePosType === 2) {
			this.y = this._messageWindow.y - this.height;
		}
	};

}());
