//=============================================================================
// JPT_PassiveSkillPlusParam
//=============================================================================

var Imported = Imported || {};
Imported['JPT_PassiveSkillPlusParam'] = 1.00;

/*:ja
 * @plugindesc パッシブスキルでのステータス補正
 * @author Kawamura
 *
 * @help
 * パーティー内のアクターがステータス補正に設定したスキルを所持していた場合
 * タグで紐づけられたアクターのパラメータに補正を加えることができます
 *
 * ※JPT_SkillPointが必要です
 *
 * バトル開始時、メニュー開始時に再計算しますので
 * それ以外でパラメータを参照する場合は注意してください
 *
 * スキルのメモに以下の様に記述
 * <pas_pp>
 * tag:(タグ名)
 * atk:(計算式)
 * mdf:(計算式)
 * element:(属性ID),(加算割合)
 * state:(ステートID),(加算割合)
 * </pas_pp>
 *
 * tagは補正をかけるアクターを探すために使うので必ず記入してください
 * 補正がかけられるパラメータは
 * mhp(最大HP)
 * mmp(最大MP)
 * atk(攻撃)
 * def(防御)
 * mat(魔法攻撃)
 * mdf(魔法防御)
 * agi(敏捷)
 * luk(幸運)
 * msp(最大SP)
 * spr(SPリジェネ率)
 *
 * 各計算式はevalで評価可能な形で記述してください
 *
 * <pas_pp>
 * tag:naofumi
 * atk:5 * a.level
 * spr:5
 * </pas_pp>
 *
 * 属性有効度の指定
 * 炎 20%減の場合
 * element:2,-20
 *
 * 属性有効度の指定はeval使えません
 *
 * 属性ID
 * 1=物理
 * 2=炎
 * 3=氷
 * 4=雷
 * 5=水
 * 6=土
 * 7=風
 * 8=光
 * 9=闇
 *
 *
 * ステート有効度の指定
 * 毒 20%減の場合
 * state:4,-20
 *
 * アクターのメモに以下の様に記述
 * <pas_pp_tag:(タグ名) >
 *
 */

/*
 *
 */

(function() {

	'use strict';

	//var parameters = PluginManager.parameters('JPT_PassiveSkillPlusParam');


	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

	DataManager.evalCondition = function(formula, user) {
		var a = user;
		var s = $gameSwitches;
		var v = $gameVariables;
		var ret = eval(formula);
		return ret;
	};

	var _JPT_SkillLevelUpPlusParam_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_SkillLevelUpPlusParam_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processSkillNotetagsPaspp($dataSkills);
	    this.processActorNotetagsPaspp($dataActors);
		return true;
	};


	//メモからデータ取得
	DataManager.processSkillNotetagsPaspp = function(group) {
		var noteStart = /<pas_pp>/i;
		var noteEnd = /<\/pas_pp>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.pas_pp_tag = '';
			obj.pas_pp_mhp = '0';
			obj.pas_pp_mmp = '0';
			obj.pas_pp_atk = '0';
			obj.pas_pp_def = '0';
			obj.pas_pp_mat = '0';
			obj.pas_pp_mdf = '0';
			obj.pas_pp_agi = '0';
			obj.pas_pp_luk = '0';
			obj.pas_pp_msp = '0';
			obj.pas_pp_spr = '0';
			obj.pas_pp_element = [];
			obj.pas_pp_state = [];

			var mode = 'none';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteStart)) {
					mode = 'pas_pp';
				}else if(line.match(noteEnd)){
					break;
				}else if(mode == 'pas_pp'){
					var state = line.split(/:/)[0];
					var strEval = line.split(/:/)[1];

					switch(state){
					case 'tag':
						obj.pas_pp_tag = strEval;
						break;
					case 'mhp':
						obj.pas_pp_mhp = strEval;
						break;
					case 'mmp':
						obj.pas_pp_mmp = strEval;
						break;
					case 'atk':
						obj.pas_pp_atk = strEval;
						break;
					case 'def':
						obj.pas_pp_def = strEval;
						break;
					case 'mat':
						obj.pas_pp_mat = strEval;
						break;
					case 'mdf':
						obj.pas_pp_mdf = strEval;
						break;
					case 'agi':
						obj.pas_pp_agi = strEval;
						break;
					case 'luk':
						obj.pas_pp_luk = strEval;
						break;
					case 'msp':
						obj.pas_pp_msp = strEval;
						break;
					case 'spr':
						obj.pas_pp_spr = strEval;
						break;
					case 'element':
						obj.pas_pp_element.push(strEval.split(/,/));
						break;
					case 'state':
						obj.pas_pp_state.push(strEval.split(/,/));
						break;
					}
				}
			}
		}
	};

	DataManager.processActorNotetagsPaspp = function(group) {

		var noteStart = /<pas_pp_tag:(.+)/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);
			obj.pas_pp_tag = '';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteStart)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.pas_pp_tag = tagStr.trim();
				}
			}
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Actor
	////////////////////////////////////////////////////////////////////////////////////

	Game_BattlerBase.prototype.clearPassiveSkillParamPlus = function() {
	    this._passiveParamPlus = [0,0,0,0,0,0,0,0,0,0];
	    this._passiveElementPlus = [];
	    this._passiveStatePlus = [];
	};

	Game_BattlerBase.prototype.clearPassiveSkillParamPlusElementState = function() {
	    this._passiveElementPlus = [];
	    this._passiveStatePlus = [];
	}

	var _JPT_PsPp_Game_BattlerBase_clearParamPlus = Game_BattlerBase.prototype.clearParamPlus;
	Game_BattlerBase.prototype.clearParamPlus = function() {
		_JPT_PsPp_Game_BattlerBase_clearParamPlus.call(this);
		this.clearPassiveSkillParamPlus();
	};

	Game_BattlerBase.prototype.addPsParam = function(index,param) {
	    this._passiveParamPlus[index] += param ;
	};

	var _JPT_PsPp_Game_BattlerBase_paramPlus = Game_BattlerBase.prototype.paramPlus;
	Game_BattlerBase.prototype.paramPlus = function(paramId) {
		var pp = _JPT_PsPp_Game_BattlerBase_paramPlus.call(this,paramId);
	    return Math.floor( pp + this._passiveParamPlus[paramId] );
	};

	var _JPT_PsPp_Game_BattlerBase_elementRate = Game_BattlerBase.prototype.elementRate;
	Game_BattlerBase.prototype.elementRate = function(elementId) {
	    var rate = _JPT_PsPp_Game_BattlerBase_elementRate.call(this,elementId);
	    var eplus = 1;
	    if(this._passiveElementPlus['eid'+elementId]){
	    	eplus =Math.max( 0, (100 + this._passiveElementPlus['eid'+elementId])/100);
		}else{
			this._passiveElementPlus['eid'+elementId] = 0;
		}
	    return rate*eplus;
	};

	var _JPT_PsPp_Game_BattlerBase_stateRate = Game_BattlerBase.prototype.stateRate;
	Game_BattlerBase.prototype.stateRate = function(stateId) {
		var rate = _JPT_PsPp_Game_BattlerBase_stateRate.call(this,stateId);
	    var splus = 1;
	    if(this._passiveStatePlus['sid'+stateId]){
	    	splus =Math.max( 0, (100 + this._passiveStatePlus['sid'+stateId])/100);
		}
	    return rate*splus;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Actor
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_PsPp_Game_Actor_levelUp = Game_Actor.prototype.levelUp;
	Game_Actor.prototype.levelUp = function() {
		_JPT_PsPp_Game_Actor_levelUp.call(this);
		this.calculatePsParam();
	};

	var _JPT_PsPp_Game_Actor_levelDown = Game_Actor.prototype.levelDown;
	Game_Actor.prototype.levelDown = function() {
		_JPT_PsPp_Game_Actor_levelDown.call(this);
		this.calculatePsParam();
	};

	var _JPT_PsPp_Game_Actor_learnSkill = Game_Actor.prototype.learnSkill;
	Game_Actor.prototype.learnSkill = function(skillId) {
		_JPT_PsPp_Game_Actor_learnSkill.call(this,skillId);
		$gameParty.allMembersCalcPsParam();
	};

	Game_Actor.prototype.calculatePsParam = function() {
		this.clearPassiveSkillParamPlus();

		//パーティメンバーのスキルチェック
		for(var i=0; i<$gameParty.allMembers().length; i++){
			var skills = $gameParty.allMembers()[i].skills();

			for(var j=0; j<skills.length; j++){
				var skill = skills[j];
				//tagが一致していれば
				if(this.actor().pas_pp_tag==skill.pas_pp_tag){

					var num = DataManager.evalCondition(skill.pas_pp_mhp,this);
					if(num != 0)this.addPsParam(0,num);
					num = DataManager.evalCondition(skill.pas_pp_mmp,this);
					if(num != 0)this.addPsParam(1,num);
					num = DataManager.evalCondition(skill.pas_pp_atk,this);
					if(num != 0)this.addPsParam(2,num);
					num = DataManager.evalCondition(skill.pas_pp_def,this);
					if(num != 0)this.addPsParam(3,num);
					num = DataManager.evalCondition(skill.pas_pp_mat,this);
					if(num != 0)this.addPsParam(4,num);
					num = DataManager.evalCondition(skill.pas_pp_mdf,this);
					if(num != 0)this.addPsParam(5,num);
					num = DataManager.evalCondition(skill.pas_pp_agi,this);
					if(num != 0)this.addPsParam(6,num);
					num = DataManager.evalCondition(skill.pas_pp_luk,this);
					if(num != 0)this.addPsParam(7,num);
					num = DataManager.evalCondition(skill.pas_pp_msp,this);
					if(num != 0)this.addPsParam(JPT_SkillPoint.spParamIndex(),num);
					num = DataManager.evalCondition(skill.pas_pp_spr,this);
					if(num != 0)this.addPsParam(JPT_SkillPoint.srgXParamIndex(),num);

					for(var e = 0; e < skill.pas_pp_element.length; e++){
						var ep = skill.pas_pp_element[e];
						if(!this._passiveElementPlus['eid'+ep[0]]){
							this._passiveElementPlus['eid'+ep[0]] = 0;
						}
						this._passiveElementPlus['eid'+ep[0]] += parseInt(ep[1],10);
					}

					for(var s = 0; s < skill.pas_pp_state.length; s++){
						var st = skill.pas_pp_state[s];
						if(!this._passiveStatePlus['sid'+st[0]]){
							this._passiveStatePlus['sid'+st[0]] = 0;
						}
						this._passiveStatePlus['sid'+st[0]] += parseInt(st[1],10);
					}
				}
			}
		}
	};

	Game_Actor.prototype.calculatePsParamElementState = function() {
		this.clearPassiveSkillParamPlusElementState();

		//パーティメンバーのスキルチェック
		for(var i=0; i<$gameParty.allMembers().length; i++){
			var skills = $gameParty.allMembers()[i].skills();

			for(var j=0; j<skills.length; j++){
				var skill = skills[j];
				//tagが一致していれば
				if(this.actor().pas_pp_tag==skill.pas_pp_tag){

					for(var e = 0; e < skill.pas_pp_element.length; e++){
						var ep = skill.pas_pp_element[e];
						if(!this._passiveElementPlus['eid'+ep[0]]){
							this._passiveElementPlus['eid'+ep[0]] = 0;
						}
						this._passiveElementPlus['eid'+ep[0]] += parseInt(ep[1],10);
					}

					for(var s = 0; s < skill.pas_pp_state.length; s++){
						var st = skill.pas_pp_state[s];
						if(!this._passiveStatePlus['sid'+st[0]]){
							this._passiveStatePlus['sid'+st[0]] = 0;
						}
						this._passiveStatePlus['sid'+st[0]] += parseInt(st[1],10);
					}
				}
			}
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Party
	////////////////////////////////////////////////////////////////////////////////////
	Game_Party.prototype.allMembersCalcPsParam = function() {
		this.members().forEach(function(actor) {
			actor.calculatePsParam();
	    });
	};

	Game_Party.prototype.allMembersCalcPsParamElementState = function() {
		this.members().forEach(function(actor) {
			actor.calculatePsParamElementState();
	    });
	};


	Game_Party.prototype.onBattleStart = function() {
		Game_Unit.prototype.onBattleStart.call(this);
		this.allMembersCalcPsParamElementState();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Menu
	////////////////////////////////////////////////////////////////////////////////////


	var _JPT_PsPp_Scene_Menu_start = Scene_Menu.prototype.start;
	Scene_Menu.prototype.start = function() {
		$gameParty.allMembersCalcPsParamElementState();
		_JPT_PsPp_Scene_Menu_start.call(this);
	};



}());
