//=============================================================================
// JPT_PeddlingMenu.js
//=============================================================================

var Imported = Imported || {};
Imported['JPT_PeddlingMenu'] = 1.00;

/*:ja
 * @plugindesc 行商メニュー
 * @author Kawamura
 *
 * @param Peddling Start Switch ID
 * @desc 行商開始スイッチID
 * @default 85
 *
 *
 * @param Peddling Command Name
 * @desc 行商 項目名
 * @default 行商
 *
 * @param Possessed Items Command Name
 * @desc 所持アイテム 項目名
 * @default 所持アイテム
 *
 * @param Num of Sales Command Name
 * @desc 販売数 項目名
 * @default 販売数
 *
 * @param Total Command Name
 * @desc 合計 項目名
 * @default 合計
 *
 * @param Max Command Name
 * @desc 最大数 項目名
 * @default 最大数
 *
 * @param Sales Amount Command Name
 * @desc 売上額 項目名
 * @default 売上額
 *
 * @param Max Variable Start ID
 * @desc 販売個数最大値設定変数の開始ID
 * @default 250
 *
 * @param Now Variable Start ID
 * @desc 販売個数現在値設定変数の開始ID
 * @default 276
 *
 * @param Town Num
 * @desc 街の数
 * @default 22
 *
 * @param Recover Rate
 * @desc 販売個数の回復レート
 * @default 10
 *
 * @param Bef Place Variable ID
 * @desc 前回行商を行った場所の変数ID
 * @default 241
 *
 * @help
 * 行商メニューを実装します
 *
 * 依存関係
 *
 *-------------------------------------------
 * 行商呼び出しコマンド
 *-------------------------------------------
 * PeddlingMenu (変数IDのオフセット値)
 *
 *
 *
 *-------------------------------------------
 * 行商用アイテムの設定
 *-------------------------------------------
 *アイテムのメモに以下記入
 *
 * <PeddlingItem>
 *
 *
 */



(function() {

	'use strict';

	var parameters = PluginManager.parameters('JPT_PeddlingMenu');

	var peddlingCommandName			= parameters['Peddling Command Name'] || '行商';
	var possessedItemsCommandName	= parameters['Possessed Items Command Name'] || '所持アイテム';
	var numOfSalesCommandName		= parameters['Num of Sales Command Name'] || '販売数';
	var totalCommandName			= parameters['Total Command Name'] || '合計';
	var maxCommandName				= parameters['Max Command Name'] || '最大数';
	var salesAmountCommandName		= parameters['Sales Amount Command Name'] || '売上額';

	var peddlingStartSwitchID		= parseInt( parameters['Peddling Start Switch ID'] || 85, 10);

	var maxVariableStartID			= parseInt(parameters['Max Variable Start ID'] || 250, 10);
	var nowVariableStartID			= parseInt(parameters['Now Variable Start ID'] || 276, 10);
	var townNum						= parseInt(parameters['Town Num'] || 22, 10);

	var recoverRate					= parseInt(parameters['Recover Rate'] || 10, 10);

	var befPlaceVariableID			= parseInt(parameters['Bef Place Variable ID'] || 241, 10);


	////////////////////////////////////////////////////////////////////////////////////
    // Game_Interpreter
	////////////////////////////////////////////////////////////////////////////////////

	var peddlingNum = 0;
	var peddlingMaxNum = 0;
	var peddlingSpecialItems = [];
	var peddlingSpecialRate = 1;

	var townID = -1;

    var _JPT_PeddlingMenu_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
    	_JPT_PeddlingMenu_Game_Interpreter_pluginCommand.call(this, command, args);

    	if(command==='PeddlingMenu'){

			$gameSwitches.setValue( peddlingStartSwitchID, false );

			townID = -1;
			peddlingNum = 999;
    		peddlingMaxNum = 999;

    		if(args.length > 0){
    			if(args[0] >= 0 && args[0]<=999){
    				townID = parseInt(args[0]);
    				peddlingNum = $gameVariables.value( nowVariableStartID + townID);
    			}
    		}

    		/*
    		peddlingSpecialItems = [];
    		if(args.length > 1){
    			if(args[1]){
    				peddlingSpecialItems = args[1].line.split(/,/);
    			}
    		}

    		peddlingSpecialRate = 1;
    		if(args.length > 2){
    			if(args[2] > 0){
    				peddlingSpecialRate = args[2];
    			}
    		}
    		*/


    		SceneManager.push(Scene_Peddling);
    	}
    };

    ////////////////////////////////////////////////////////////////////////////////////
    // TextManager
	////////////////////////////////////////////////////////////////////////////////////

	Object.defineProperties(TextManager, {
		peddling			: { get: function() { return peddlingCommandName; },		configurable: true },
	});

    ////////////////////////////////////////////////////////////////////////////////////
    // DataManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_PeddlingMenu_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_PeddlingMenu_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processItemsNotetagsPm($dataItems);
	    this.processAromorsNotetagsPm($dataArmors);
		return true;
	};

	DataManager.processItemsNotetagsPm = function(group) {
		var notePeddling = /<PeddlingItem>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.peddling = false;

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(notePeddling)) {
					obj.peddling = true;
				}
			}
		}
	}


	DataManager.processAromorsNotetagsPm = function(group) {
		var notePeddling = /<PeddlingItem>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.peddling = false;

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(notePeddling)) {
					obj.peddling = true;
				}
			}
		}
	}


    ////////////////////////////////////////////////////////////////////////////////////
    // Scene_Camp
	////////////////////////////////////////////////////////////////////////////////////
    function Scene_Peddling() {
        this.initialize.apply(this, arguments);
    }

    Scene_Peddling.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Peddling.prototype.constructor = Scene_Peddling;

    Scene_Peddling.prototype.initialize = function() {
    	Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_Peddling.prototype.titleID = function() {
		return 2;
	};

	Scene_Peddling.prototype.titleName = function() {
		return TextManager.peddling;
	};

    Scene_Peddling.prototype.create = function() {
    	Scene_MenuBase.prototype.create.call(this);
    	this.createCategoryWindow();
    	this.createHeaderWindow();
    	this.createFooterWindow();
    	this.createItemWindow();
    	this.createWindowFrame();
    	this.createMenuTitle(this.titleID(),this.titleName());
    };

    Scene_Peddling.prototype.start = function() {
    	Scene_MenuBase.prototype.start.call(this);
    	this._titleWindow.refresh();
    	this._headerWindow.refresh();
    	this._itemWindow.refresh();
    	this._footerWindow.refresh();
    };

    var _JPT_PeddlingMenu_Scene_Peddling_createBackground = Scene_Peddling.prototype.createBackground;
    Scene_Peddling.prototype.createBackground = function() {
    	_JPT_PeddlingMenu_Scene_Peddling_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_peddling_bg");
		this._backgroundSprite.x = 10;
		this._backgroundSprite.y = Graphics.boxHeight - 486 - 16;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,this._backgroundSprite.y,8,8,8,'#000000ff');
		this.addChild(this._sprMask);
	};

	Scene_Peddling.prototype.createWindowFrame = function(){

		var x = this._headerWindow.x;
		var y = this._headerWindow.y;
		var w = this._headerWindow.width;
	    var h = this._headerWindow.height + this._itemWindow.height + this._footerWindow.height;
	    var m = 24;
	    var bitmap = this._sprMask.bitmap;
        var p = 96;
        var q = 96;
        var skin = this._itemWindow.windowskin;
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+m, y+0, w-m*2, m);
        bitmap.blt(skin, p+m, 0+q-m, p-m*2, m, x+m, y+h-m, w-m*2, m);
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, x+0, y+m, m, h-m*2);
        bitmap.blt(skin, p+q-m, 0+m, m, p-m*2, x+w-m, y+m, m, h-m*2);
        bitmap.blt(skin, p+0, 0+0, m, m, x+0, y+0, m, m);
        bitmap.blt(skin, p+q-m, 0+0, m, m, x+w-m, y+0, m, m);
        bitmap.blt(skin, p+0, 0+q-m, m, m, x+0, y+h-m, m, m);
        bitmap.blt(skin, p+q-m, 0+q-m, m, m, x+w-m, y+h-m, m, m);

        //横
        var fy = y + this._headerWindow.height + this._itemWindow.height;
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+5, fy , w - 10, m);
        bitmap.blt(skin, p+m, 0+1, p-m*2, 1, x+4, fy + 1, w - 8, 3);
	}

	Scene_Peddling.prototype.createCategoryWindow = function() {
	    this._categoryWindow = new Window_PeddlingCategory();
	    this._categoryWindow.x = 8;
	    this._categoryWindow.y = 72;
	    this._categoryWindow.setHandler('ok',     this.onCategoryOk.bind(this));
	    this._categoryWindow.setHandler('cancel', this.popScene.bind(this));
	    this.addWindow(this._categoryWindow);
	};

	Scene_Peddling.prototype.createHeaderWindow = function() {
		var wx = this._categoryWindow.x;
	    var wy = this._categoryWindow.y + this._categoryWindow.height + 8;
	    var ww = Graphics.boxWidth - 16;
	    var wh = 40;
	    this._headerWindow = new Window_PeddlingHeader(wx, wy, ww, wh);
	    this.addWindow(this._headerWindow);
	};


	Scene_Peddling.prototype.createItemWindow = function() {
		var wx = this._headerWindow.x;
	    var wy = this._headerWindow.y + this._headerWindow.height;
	    var ww = Graphics.boxWidth - 16;
	    var wh = this._footerWindow.y - wy;
	    this._itemWindow = new Window_PeddlingItem(wx, wy, ww, wh);
	    //this._itemWindow.setHandler('ok',     this.onItemCancel.bind(this));
	    this._itemWindow.setHandler('cancel', this.onItemCancel.bind(this));
	    this.addWindow(this._itemWindow);
	    this._itemWindow.setFooterWindow(this._footerWindow);
	};

	Scene_Peddling.prototype.createFooterWindow = function() {
	    var ww = Graphics.boxWidth - 16;
		var wh = 70;
		var wx = this._categoryWindow.x;
	    var wy = Graphics.boxHeight - 8 - wh;
	    this._footerWindow = new Window_PeddlingFooter(wx, wy, ww, wh);
	    this.addWindow(this._footerWindow);
	};



	Scene_Peddling.prototype.onCategoryOk = function(){
		if(this._categoryWindow.index()==0){
			this._itemWindow.activate();
		    this._itemWindow.selectLast();
		}else{
			var ret = this._itemWindow.sellItem();
			if(ret[0] > 0){
				this.applySellNum(ret[0]);
				$gameSwitches.setValue( peddlingStartSwitchID, true );
				this.popScene();
			}else{
				this._categoryWindow.activate();
			}
		}
	}

	Scene_Peddling.prototype.onItemCancel = function() {
	    this._itemWindow.deselect();
	    this._categoryWindow.activate();
	};


	Scene_Peddling.prototype.applySellNum = function(sellNum) {

		if(townID >= 0){
			var now = $gameVariables.value( nowVariableStartID + townID);
			now -= sellNum;
			if(now < 0)now = 0;
			$gameVariables.setValue( nowVariableStartID + townID, now);
		}

		var befTownID = $gameVariables.value( befPlaceVariableID );

		//前回と同じ場所で行商した場合は回復させない
		if(befTownID!=townID){
			for(var i=0; i < townNum; i++){
				if(i!=townID){
					var max = $gameVariables.value( maxVariableStartID + i);
					var now = $gameVariables.value( nowVariableStartID + i);
					now += Math.floor( max*(recoverRate/100) );
					if(now > max ) now = max;
					$gameVariables.setValue( nowVariableStartID + i, now);
				}
			}
		}

		$gameVariables.setValue( befPlaceVariableID, townID);
	}

	////////////////////////////////////////////////////////////////////////////////////
	// Window_PeddlingCategory
	////////////////////////////////////////////////////////////////////////////////////
	function Window_PeddlingCategory() {
	    this.initialize.apply(this, arguments);
	}

	Window_PeddlingCategory.prototype = Object.create(Window_HorzCommand.prototype);
	Window_PeddlingCategory.prototype.constructor = Window_PeddlingCategory;

	Window_PeddlingCategory.prototype.initialize = function() {
		this.loadImage();
		this.createSprite();
		Window_HorzCommand.prototype.initialize.call(this, 0, 0);
	    this.opacity = 0;
	    this.contentsOpacity = 0;
	    this._curAnimationCount = 0;
	    this._befSelectIndex = -1;

	};

	Window_PeddlingCategory.prototype.windowWidth = function() {
	    return 448+24;
	};

	Window_PeddlingCategory.prototype.windowHeight = function() {
	    return 40;
	};

	Window_PeddlingCategory.prototype.standardPadding = function() {
	    return 0;
	};

	Window_PeddlingCategory.prototype.maxCols = function() {
	    return 2;
	};

	Window_PeddlingCategory.prototype.loadImage = function() {
		ImageManager.reserveSystem('IconSet20px');
		this._bmpCategory = ImageManager.loadBitmap("img/menus/", "menu_peddling_category");
	};

	Window_PeddlingCategory.prototype.makeCommandList = function() {
	    this.addCommand('item select',		'select');
	    this.addCommand('item sell',		'sell');
	};

	Window_PeddlingCategory.prototype.createSprite = function() {
		this._commandIconSpr = [];
		for(var i=0; i<this.maxCols(); i++){
			this._commandIconSpr[i] = new Sprite();
		}
		this._commandSelectSpr = new Sprite();
	}

	Window_PeddlingCategory.prototype.createOtherParts = function(){
		for(var i=0; i<this.maxCols(); i++){
			this.addChild(this._commandIconSpr[i]);
		}
		this.addChild(this._commandSelectSpr);
	}

	Window_PeddlingCategory.prototype.itemRect = function(index) {
		var rect = new Rectangle();

	    rect.width = 224;
	    rect.height = 40;
	    rect.x = (224 + 24)*index;
	    rect.y = 4;
	    return rect;
	}

	Window_PeddlingCategory.prototype.drawItem = function(index) {
		var rect = this.itemRect(index);
		this._commandIconSpr[index].bitmap = this._bmpCategory;
		this._commandIconSpr[index].setFrame(224*index,0,224,40);
		this._commandIconSpr[index].x = rect.x;
		this._commandIconSpr[index].y = rect.y-2;
	};

	Window_PeddlingCategory.prototype.drawSelectIcon = function() {
		this._commandSelectSpr.bitmap = this._bmpCategory;
		this._commandSelectSpr.setFrame(224*this.index(),40,224,40);
		if(this.index()>=0){
			this._commandSelectSpr.x = this._commandIconSpr[this.index()].x;
			this._commandSelectSpr.y = this._commandIconSpr[this.index()].y;
		}
	};


	Window_PeddlingCategory.prototype.updateCursor = function() {
		var rect = this.itemRect(this.index());
        this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
	};

	Window_PeddlingCategory.prototype.update = function() {
		Window_ItemCategory.prototype.update.call(this);
		this.updateSelectIcon();
	};

	Window_PeddlingCategory.prototype.updateSelectIcon = function() {

		if(this._befSelectIndex != this.index() ){
			this._commandSelectSpr.setFrame(224*this.index(),40,224,40);
			this._commandSelectSpr.x = this._commandIconSpr[this.index()].x;
			this._commandSelectSpr.y = this._commandIconSpr[this.index()].y;
			this._befSelectIndex = this.index();
		}

		var blinkCount = this._curAnimationCount % 40;
	    var cursorOpacity = 255;
        if (blinkCount < 20) {
            cursorOpacity -= blinkCount * 8;
        } else {
            cursorOpacity -= (40 - blinkCount) * 8;
        }

	    this._curAnimationCount++;

	    this._commandSelectSpr.alpha = cursorOpacity / 255;
	}

	Window_PeddlingCategory.prototype.refresh = function() {
		Window_ItemCategory.prototype.refresh.call(this);
		this.drawSelectIcon();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_PeddlingItem
	////////////////////////////////////////////////////////////////////////////////////
    function Window_PeddlingItem() {
        this.initialize.apply(this, arguments);
    }

    Window_PeddlingItem.prototype = Object.create(Window_MenuItemList.prototype);
    Window_PeddlingItem.prototype.constructor = Window_PeddlingItem;

    Window_PeddlingItem.prototype.initialize = function(x, y, width, height) {
    	this.createSprite();
    	Window_MenuItemList.prototype.initialize.call(this, x, y, width, height);
    	this._footerWindow = null;
    	this.opacity = 0;
    	this._sellNums = false;
    	this.refresh();
	};

	Window_PeddlingItem.prototype.maxCols = function() {
	    return 1;
	};


	Window_PeddlingItem.prototype.createSprite = function() {
		this._sprBtnLArrow = new Sprite_Button();
		this._sprBtnRArrow = new Sprite_Button();
	}

	Window_PeddlingItem.prototype.createOtherParts = function() {
		this.addChild(this._sprBtnLArrow);
		this.addChild(this._sprBtnRArrow);

		this._sprBtnLArrow.setClickHandler(this.onClickLeft.bind(this));
		this._sprBtnRArrow.setClickHandler(this.onClickRight.bind(this));
	};

	Window_PeddlingItem.prototype.setFooterWindow = function(window){
		this._footerWindow = window;
	}

	Window_PeddlingItem.prototype.includes = function(item) {

		if(DataManager.isItem(item) || DataManager.isArmor(item)){
			if(item.peddling){
				return item.peddling;
			}
		}
		return false;
	};

	Window_PeddlingItem.prototype.drawItem = function(index) {
	    var item = this._data[index];
	    if (item) {
	        var numberWidth = this.numberWidth();
	        var rect = this.itemRect(index);
	        rect.width -= this.textPadding();
	        var itemNameW = 240;
	        this.changePaintOpacity(this.isEnabled(item));
	        this.drawItemName(item, rect.x, rect.y, itemNameW);
	        this.drawItemNumber(item, index,rect.x, rect.y, rect.width - 150);
	        this.drawGold(this.contents,item.price,rect.x + itemNameW, rect.y + 6, 260, 16);
	        this.drawSellItemNumber(index, rect.x, rect.y, rect.width - 30);
	        this.changePaintOpacity(1);
	    }
	};

	Window_PeddlingItem.prototype.setArrows = function(x,y,width) {

	    var p = 24;
	    var q = p/2;
	    var sx = 96+p;
	    var sy = 0+p;

	    this._sprBtnLArrow.bitmap = this._windowskin;
	    this._sprBtnLArrow.anchor.x = 0.5;
	    this._sprBtnLArrow.anchor.y = 0.5;
	    this._sprBtnLArrow.setFrame(sx, sy+q, q, p);

	    this._sprBtnRArrow.bitmap = this._windowskin;
	    this._sprBtnRArrow.anchor.x = 0.5;
	    this._sprBtnRArrow.anchor.y = 0.5;
	    this._sprBtnRArrow.setFrame(sx+q+p, sy+q, q, p);

	    this._sprBtnLArrow.move(x,y);

	    this._sprBtnRArrow.move( x + width, y );

	    if(this.isContentsArea(x,y)){
	    	this._sprBtnLArrow.visible = this.active;
			this._sprBtnRArrow.visible = this.active;
	    }else{
	    	this._sprBtnLArrow.visible = false;
			this._sprBtnRArrow.visible = false;
	    }

	};


	Window_PeddlingItem.prototype.onClickLeft = function() {
		this.changeNumber(-1);

	};

	Window_PeddlingItem.prototype.onClickRight = function() {
		this.changeNumber(1);
	};

	Window_PeddlingItem.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
    	var icon_index = item.iconIndex;
    	var item_name = item.name;
        var iconBoxWidth = 20 + 4;
        this.resetTextColor();
        this.drawIcon20px(this.contents,icon_index, x , y  + Math.floor(this.lineHeight()/2) -10 );
        this.drawText(item_name, x + iconBoxWidth, y, width - iconBoxWidth);
	};


	Window_PeddlingItem.prototype.drawItemNumber = function(item, index,x, y, width) {
		var num = $gameParty.numItems(item) - this._sellNums[index];
        this.drawText(num, x, y, width, 'right');
	};

	Window_PeddlingItem.prototype.drawSellItemNumber = function(index, x, y, width) {
		var num = this._sellNums[index];
        this.drawText(num, x, y, width, 'right');
	};

	Window_PeddlingItem.prototype.isEnabled = function(item) {
	    return true;
	};

	Window_PeddlingItem.prototype.updateHelp = function() {
	};

	Window_PeddlingItem.prototype.initSellNums = function() {
		if(!this._sellNums){
			this._sellNums = [];
			for(var i=0; i<this._data.length; i++ ){
				this._sellNums[i] = 0;
			}
		}
	}


	Window_PeddlingItem.prototype.update = function() {
		Window_MenuItemList.prototype.update.call(this);
	    this.processKeyPress();

    	this._sprBtnLArrow.visible = this.active;
		this._sprBtnRArrow.visible = this.active;

	    if(this.active){
	    	var rect = this.itemRect(this.index());
	    	this.setArrows(rect.width - 58, rect.y + 37, 60);
	    }
	};

	Window_PeddlingItem.prototype.refresh = function() {
		this.makeItemList();
		this.initSellNums();

	    this.createContents();
	    this.drawAllItems();
	};


	Window_PeddlingItem.prototype.changeNumber = function(amount) {

		var i = this.index();
		var item = this._data[i];
		var numMax = $gameParty.numItems(item);

		var ct = this.calcTotal();

		var cMax = Math.min( numMax, peddlingNum - ct[0]);

		this._sellNums[i] += amount;

		if(amount>0){
			if( cMax <= 0 || numMax < this._sellNums[i]){
				this._sellNums[i] = 0;
			}
		}else{
			if(this._sellNums[i] < 0){
				this._sellNums[i] = cMax;
			}
		}

		ct = this.calcTotal();
		if(this._footerWindow){
			this._footerWindow.setTotal(ct[0],ct[1]);
		}
		this.refresh();
	};


	Window_PeddlingItem.prototype.processKeyPress = function() {
	    if (this.isOpenAndActive()) {
	        if (Input.isRepeated('right')) {
	            this.changeNumber(1);
	        }
	        if (Input.isRepeated('left')) {
	            this.changeNumber(-1);
	        }
	    }
	};

	Window_PeddlingItem.prototype.calcTotal = function() {
		var totalG = 0;
		var totalNum = 0;
		for(var i=0; i<this._data.length; i++ ){
			totalNum += this._sellNums[i];
			totalG += (this._data[i].price * this._sellNums[i]);
		}
		return [totalNum,totalG];
	}

	Window_PeddlingItem.prototype.sellItem = function() {

		var totalG = 0;
		var totalNum = 0;
		for(var i=0; i<this._data.length; i++ ){
			var item = this._data[i];
			totalNum += this._sellNums[i];
			totalG += (item.price * this._sellNums[i]);
			$gameParty.loseItem(item, this._sellNums[i]);
		}
		$gameParty.gainGold(totalG);
		return [totalNum,totalG];
	}

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_PeddlingHeader
	////////////////////////////////////////////////////////////////////////////////////
	function Window_PeddlingHeader() {
	    this.initialize.apply(this, arguments);
	}

	Window_PeddlingHeader.prototype = Object.create(Window_Base.prototype);
	Window_PeddlingHeader.prototype.constructor = Window_PeddlingHeader;

	Window_PeddlingHeader.prototype.initialize = function(x, y, width, height) {
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this.opacity = 0;
	    this.refresh();
	};

	Window_PeddlingHeader.prototype.standardFontFace = function() {
		 return 'GameFont';
	};


	Window_PeddlingHeader.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_PeddlingHeader.prototype.standardPadding = function() {
	    return 0;
	};

	Window_PeddlingHeader.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


	Window_PeddlingHeader.prototype.refresh = function() {
    	this.contents.clear();

    	var width = 616;
    	this.contents.drawText(possessedItemsCommandName,24,19,this.width,this.contents.fontSize,'left');
   		this.drawHorzLine(this.contents,16,22,width,this.contents.fontSize*2);

   		this.contents.drawText(numOfSalesCommandName,width + 42+8,19,this.width - width - 28,this.contents.fontSize,'left');
   		this.drawHorzLine(this.contents,width + 42,22,this.width - width - 58,this.contents.fontSize*2);
    };


    ////////////////////////////////////////////////////////////////////////////////////
	// Window_PeddlingFooter
	////////////////////////////////////////////////////////////////////////////////////
	function Window_PeddlingFooter() {
	    this.initialize.apply(this, arguments);
	}

	Window_PeddlingFooter.prototype = Object.create(Window_Base.prototype);
	Window_PeddlingFooter.prototype.constructor = Window_PeddlingFooter;

	Window_PeddlingFooter.prototype.initialize = function(x, y, width, height) {
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this.opacity = 0;
	    this._totalG = 0;
	    this._totalNum = 0;
	    this.refresh();
	};

	Window_PeddlingFooter.prototype.standardFontFace = function() {
		 return 'GameFont';
	};


	Window_PeddlingFooter.prototype.standardFontSize = function() {
	    return 18;
	};

	Window_PeddlingFooter.prototype.standardPadding = function() {
	    return 0;
	};

	Window_PeddlingFooter.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


	Window_PeddlingFooter.prototype.setTotal = function(totalNum,totalG) {
		this._totalG = totalG;
		this._totalNum = totalNum;
		this.refresh();
    };

    Window_PeddlingFooter.prototype.refresh = function() {
    	this.contents.clear();

    	var x = this.width - 278;
    	this.drawGold(this.contents,this._totalG, x, 24, 260);

    	x -= 10;
    	this.contents.textColor = TextManager.colorSP;
    	this.contents.drawText(salesAmountCommandName,x,26,100,this.contents.fontSize,'right');


    	x -= 100;
    	var numW = this.textWidth('000/000');
    	var numT = this._totalNum + '/' + peddlingNum;
    	this.contents.fontSize = 24;
       	this.contents.textColor = TextManager.colorN;
    	this.contents.drawText(numT,x,22,numW,this.contents.fontSize,'right');

    	x -= numW + 130;
    	this.contents.fontSize = this.standardFontSize();
    	this.contents.textColor = TextManager.colorSP;
    	this.contents.drawText( totalCommandName+'/'+maxCommandName,x,26,200,this.contents.fontSize,'right');


    };

}());
