//=============================================================================
// JPT_ProjectionSprite.js
//=============================================================================

/*:ja
 * @plugindesc プロジェクションスプライト
 * @author Kawamura
 *
 * @help
 * pixi-projection.js
 *
 *
 */



////////////////////////////////////////////////////////////////////////////////////
// ProjectionSprite2d
////////////////////////////////////////////////////////////////////////////////////

function ProjectionSprite2d() {
    this.initialize.apply(this, arguments);
}

ProjectionSprite2d.prototype = Object.create(PIXI.projection.Sprite2d.prototype);
ProjectionSprite2d.prototype.constructor = ProjectionSprite2d;

ProjectionSprite2d.prototype.initialize = function(bitmap) {
	var texture = new PIXI.Texture(new PIXI.BaseTexture());
	PIXI.projection.Sprite2d.call(this, texture);

	this._bitmap = null;
	this._frame = new Rectangle();
    this._realFrame = new Rectangle();

    this.bitmap = bitmap;
};

Object.defineProperty(ProjectionSprite2d.prototype, 'bitmap', {
    get: function() {
        return this._bitmap;
    },
    set: function(value) {
        if (this._bitmap !== value) {
            this._bitmap = value;

            if(value){
                this._refreshFrame = true;
                value.addLoadListener(this._onBitmapLoad.bind(this));
            }else{
                this._refreshFrame = false;
                this.texture.frame = Rectangle.emptyRectangle;
            }
        }
    },
    configurable: true
});

Object.defineProperty(ProjectionSprite2d.prototype, 'opacity', {
    get: function() {
        return this.alpha * 255;
    },
    set: function(value) {
        this.alpha = value.clamp(0, 255) / 255;
    },
    configurable: true
});

/*
Object.defineProperty(ProjectionSprite2d.prototype, 'width', {
    get: function() {
        return this._frame.width;
    },
    set: function(value) {
        this._frame.width = value;
        this._refresh();
    },
    configurable: true
});

Object.defineProperty(ProjectionSprite2d.prototype, 'height', {
    get: function() {
        return this._frame.height;
    },
    set: function(value) {
        this._frame.height = value;
        this._refresh();
    },
    configurable: true
});
*/

ProjectionSprite2d.prototype._onBitmapLoad = function(bitmapLoaded) {
    if(bitmapLoaded === this._bitmap){
        if (this._refreshFrame && this._bitmap) {
            this._refreshFrame = false;
            this._frame.width = this._bitmap.width;
            this._frame.height = this._bitmap.height;
        }
    }

    this._refresh();
};

ProjectionSprite2d.prototype.update = function() {
    this.children.forEach(function(child) {
        if (child.update) {
            child.update();
        }
    });
};

/*
ProjectionSprite2d.prototype.move = function(x, y) {
    this.x = x;
    this.y = y;
};
*/

ProjectionSprite2d.prototype.setFrame = function(x, y, width, height) {
    this._refreshFrame = false;
    var frame = this._frame;
    if (x !== frame.x || y !== frame.y ||
            width !== frame.width || height !== frame.height) {
        frame.x = x;
        frame.y = y;
        frame.width = width;
        frame.height = height;
        this._refresh();
    }
};

ProjectionSprite2d.prototype._refresh = function() {
    var frameX = Math.floor(this._frame.x);
    var frameY = Math.floor(this._frame.y);
    var frameW = Math.floor(this._frame.width);
    var frameH = Math.floor(this._frame.height);
    var bitmapW = this._bitmap ? this._bitmap.width : 0;
    var bitmapH = this._bitmap ? this._bitmap.height : 0;
    var realX = frameX.clamp(0, bitmapW);
    var realY = frameY.clamp(0, bitmapH);
    var realW = (frameW - realX + frameX).clamp(0, bitmapW - realX);
    var realH = (frameH - realY + frameY).clamp(0, bitmapH - realY);

    this._realFrame.x = realX;
    this._realFrame.y = realY;
    this._realFrame.width = realW;
    this._realFrame.height = realH;
    this.pivot.x = frameX - realX;
    this.pivot.y = frameY - realY;

    if (realW > 0 && realH > 0) {
    	if (this._bitmap) {
            this.texture.baseTexture = this._bitmap.baseTexture;
        }
        this.texture.frame = this._realFrame;
    } else if (this._bitmap) {
        this.texture.frame = Rectangle.emptyRectangle;
    } else {
        this.texture.baseTexture.width = Math.max(this.texture.baseTexture.width, this._frame.x + this._frame.width);
        this.texture.baseTexture.height = Math.max(this.texture.baseTexture.height, this._frame.y + this._frame.height);
        this.texture.frame = this._frame;
    }
    this.texture._updateID++;
};

