//=============================================================================
// JPT_ShakeEvent
//=============================================================================

var Imported = Imported || {};
Imported['JPT_ShakeEvent'] = 1.00;

/*:ja
 * @plugindesc イベントを揺らす
 * @author Kawamura
 *
 * @help
 *-------------------------------------------
 * ShakeScreen
 *-------------------------------------------
 * パラメータ
 * イベントID 縦横 強さ 速さ 時間(フレーム) 待つか
 *
 * ShakeEv 3 X 3 10 60 TRUE
 *
 */

(function() {

	'use strict';

	var parameters = PluginManager.parameters('JPT_ShakeEvent');

	////////////////////////////////////////////////////////////////////////////////////
    // Game_Interpreter
	////////////////////////////////////////////////////////////////////////////////////

    var _JPT_ShakeEvent_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
    	_JPT_ShakeEvent_Game_Interpreter_pluginCommand.call(this, command, args);

    	if(command==='ShakeEv'){
    		this.JPT_ShakeEvent(args);
    	}
    };

    Game_Interpreter.prototype.JPT_ShakeEvent = function(params) {
    	$gameMap.event(params[0]).startShake(params[1].toUpperCase(), params[2], params[3], params[4]);
        if (params[5].toUpperCase() === 'TRUE') {
            this.wait(params[4]);
        }
        return true;
    };

    ////////////////////////////////////////////////////////////////////////////////////
    // Game_CharacterBase
	////////////////////////////////////////////////////////////////////////////////////
    var _JPT_ShakeEvent_Game_CharacterBase_initMembers = Game_CharacterBase.prototype.initMembers;
    Game_CharacterBase.prototype.initMembers = function() {
    	_JPT_ShakeEvent_Game_CharacterBase_initMembers.call(this);
    	this.clearShake();
    };


    var _JPT_ShakeEvent_Game_CharacterBase_update = Game_CharacterBase.prototype.update;
    Game_CharacterBase.prototype.update = function() {
    	_JPT_ShakeEvent_Game_CharacterBase_update.call(this);

    	if(this.isShaking()){
    		this.updateShake();
    	}
    };


    Game_CharacterBase.prototype.startShake = function( type, power, speed, duration) {
    	this._shake_type = type;
    	this._shakePower = power;
        this._shakeSpeed = speed;
        this._shakeDuration = duration;
    };

    Game_CharacterBase.prototype.clearShake = function() {
    	this._shakeDuration = 0;
    	this._shakePower = 0;
    	this._shakeSpeed = 0;
    	this._shake_type = '';
    	this._shake = 0;
    	this._shakeDirection = 1;
    };

    Game_CharacterBase.prototype.isShaking = function() {
    	return this._shakeDuration > 0;
    }

    Game_CharacterBase.prototype.updateShake = function() {

    	if (this._shakeDuration > 0 || this._shake !== 0) {
            var delta = (this._shakePower * this._shakeSpeed * this._shakeDirection) / 10;
            if (this._shakeDuration <= 1 && this._shake * (this._shake + delta) < 0) {
                this._shake = 0;
            } else {
                this._shake += delta;
            }
            if (this._shake > this._shakePower * 2) {
                this._shakeDirection = -1;
            }
            if (this._shake < - this._shakePower * 2) {
                this._shakeDirection = 1;
            }
            this._shakeDuration--;
        }

    	switch(this._shake_type){
    	case 'Y':
    		this._realY += (this._shake/100);
    		break;
    	case 'X':
    		this._realX += (this._shake/100);
    		break;
   		default:
   			this._shake = 0;
    		break;
    	}

    	if (this._shakeDuration == 0) {
    		this.clearShake();
        }
    };

}());
