//=============================================================================
// JPT_ShopMenu
//=============================================================================

var Imported = Imported || {};
Imported['JPT_ShopMenu'] = 1.00;

/*:ja
 * @plugindesc ショップメニュー
 * @author Kawamura
 *
 * @param Shop_MenuName
 * @desc ショップ メニュー項目名
 * @default ショップ
 *
 * @help
 * 「盾の勇者」カスタムショップメニュープラグインです
 * 依存関係
 *
 */


(function() {


	'use strict';

	var parameters = PluginManager.parameters('JPT_ShopMenu');
	var shopMenuName = parameters['Shop_MenuName'] || 'ショップ';

	////////////////////////////////////////////////////////////////////////////////////
    // TextManager
	////////////////////////////////////////////////////////////////////////////////////

	Object.defineProperties(TextManager, {
		menuShop			: { get: function() { return shopMenuName; }, configurable: true },
	});

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Shop
	////////////////////////////////////////////////////////////////////////////////////

	Scene_Shop.prototype.titleID = function() {
		return 2;
	};

	Scene_Shop.prototype.titleName = function() {
		return TextManager.menuShop;
	};

	var _JPT_ShopMenu_Scene_Shop_createBackground = Scene_Shop.prototype.createBackground;
	Scene_Shop.prototype.createBackground = function() {
		_JPT_ShopMenu_Scene_Shop_createBackground.call(this);
		this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_list_bg");
		this._backgroundSprite.x = 10;
		this._backgroundSprite.y = Graphics.boxHeight - 486 - 16;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,this._backgroundSprite.y,177,8,8,'#000000ff');
		this.addChild(this._sprMask);

		this._sprFrame = new Sprite();
		this._sprFrame.bitmap = new Bitmap(Graphics.boxWidth,Graphics.boxHeight);
		this.addChild(this._sprFrame);
    };

	Scene_Shop.prototype.create = function() {
	    Scene_MenuBase.prototype.create.call(this);

	    this.createHelpWindow();
	    this.createGoldWindow();

	    this.createCommandWindow();
	    this.createDummyWindow();
	    this.createBuyWindow();
	    this.createStatusWindow();
	    this.createCategoryWindow();
	    this.createSellWindow();
	    this.createNumberWindow();

	    this.createWindowFrame();
	    this.createMenuTitle(this.titleID(),this.titleName());

        this._sprFrame.visible = false;
	};

	Scene_Shop.prototype.start = function() {
    	Scene_MenuBase.prototype.start.call(this);

    	this._titleWindow.refresh();
	    this._goldWindow.refresh();
	    this.createWindowFrame();
    };


    Scene_Shop.prototype.createWindowFrame = function(){

		var x = this._dummyWindow.x;
		var y = this._dummyWindow.y;
		var w = this._dummyWindow.width;
	    var h = this._dummyWindow.height;
	    var m = 24;
	    var bitmap = this._sprMask.bitmap;
        var p = 96;
        var q = 96;
        var skin = this._dummyWindow.windowskin;
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+m, y+0, w-m*2, m);
        bitmap.blt(skin, p+m, 0+q-m, p-m*2, m, x+m, y+h-m, w-m*2, m);
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, x+0, y+m, m, h-m*2);
        bitmap.blt(skin, p+q-m, 0+m, m, p-m*2, x+w-m, y+m, m, h-m*2);
        bitmap.blt(skin, p+0, 0+0, m, m, x+0, y+0, m, m);
        bitmap.blt(skin, p+q-m, 0+0, m, m, x+w-m, y+0, m, m);
        bitmap.blt(skin, p+0, 0+q-m, m, m, x+0, y+h-m, m, m);
        bitmap.blt(skin, p+q-m, 0+q-m, m, m, x+w-m, y+h-m, m, m);

        //縦
        bitmap = this._sprFrame.bitmap;
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, this._statusWindow.x, y+5, m, this._statusWindow.height - 10);
        bitmap.blt(skin, p+1, 0+m, 1, p-m*2, this._statusWindow.x + 1, y+4, 3, this._statusWindow.height - 8 );

        m = this._dummyWindow._margin;
        w = w - m * 2;
        h = h - m * 2;
        bitmap = new Bitmap(w, h);
        this._sprBack.bitmap = bitmap;
        this._sprBack.setFrame(0, 0, w, h);
        this._sprBack.move(x + m, y + m);

        bitmap.blt(skin, 0, 0, p, p, 0, 0, w, h);
        for (y = 0; y < h; y += p) {
            for (x = 0; x < w; x += p) {
                bitmap.blt(skin, 0, p, p, p, x, y, p, p);
            }
        }
        var tone = this._dummyWindow._colorTone;
        bitmap.adjustTone(tone[0], tone[1], tone[2]);
	}


    Scene_Shop.prototype.createGoldWindow = function() {
    	Scene_MenuBase.prototype.createGoldWindow.call(this);
    };

    Scene_Shop.prototype.createCommandWindow = function() {
        this._commandWindow = new Window_ShopCommand(this._purchaseOnly);
        this._commandWindow.y = 72;
        this._commandWindow.setHandler('buy',    this.commandBuy.bind(this));
        this._commandWindow.setHandler('sell',   this.commandSell.bind(this));
        this._commandWindow.setHandler('cancel', this.popScene.bind(this));
        this.addWindow(this._commandWindow);
    };

    Scene_Shop.prototype.createDummyWindow = function() {
    	var wx = 8;
        var wy = this._commandWindow.y + this._commandWindow.height + 8;
        var wh = this._helpWindow.y - wy - 8;
        this._dummyWindow = new Window_ShopDummy(wx, wy, Graphics.boxWidth - 16, wh);
        this._dummyWindow.setWinFrame(this._sprFrame);
        this.addWindow(this._dummyWindow);
        this._dummyWindow.opacity = 0;
    };

    Scene_Shop.prototype.createBuyWindow = function() {
        var wy = this._dummyWindow.y;
        var wh = this._dummyWindow.height;
        this._buyWindow = new Window_ShopBuy(this._dummyWindow.x, wy, wh, this._goods);
        this._buyWindow.setHelpWindow(this._helpWindow);
        this._buyWindow.hide();
        this._buyWindow.setHandler('ok',     this.onBuyOk.bind(this));
        this._buyWindow.setHandler('cancel', this.onBuyCancel.bind(this));
        this.addWindow(this._buyWindow);
        this._buyWindow.opacity = 0;
    };

    Scene_Shop.prototype.createStatusWindow = function() {
        var wx = this._buyWindow.width;
        var wy = this._dummyWindow.y;
        var ww = Graphics.boxWidth - wx - 8;
        var wh = this._dummyWindow.height;
        this._statusWindow = new Window_ShopStatus(wx, wy, ww, wh);
        this._statusWindow.hide();
        this.addWindow(this._statusWindow);
        this._statusWindow.opacity = 0;

        this._buyWindow.setStatusWindow(this._statusWindow);

    };

    Scene_Shop.prototype.createCategoryWindow = function() {
        this._categoryWindow = new Window_ShopItemCategory();
        this._categoryWindow.setHelpWindow(this._helpWindow);
        this._categoryWindow.y = this._commandWindow.y;
        this._categoryWindow.hide();
        this._categoryWindow.deactivate();
        this._categoryWindow.setHandler('ok',     this.onCategoryOk.bind(this));
        this._categoryWindow.setHandler('cancel', this.onCategoryCancel.bind(this));
        this.addWindow(this._categoryWindow);
    };

    Scene_Shop.prototype.createSellWindow = function() {
    	var wx = this._dummyWindow.x;
        var wy = this._categoryWindow.y + this._categoryWindow.height + 8;
        var wh = this._helpWindow.y - wy - 8;
        this._sellWindow = new Window_ShopSell(wx, wy, Graphics.boxWidth-16, wh);
        this._sellWindow.setHelpWindow(this._helpWindow);
        this._sellWindow.setWinFrame(this._sprFrame);
        this._sellWindow.hide();
        this._sellWindow.setHandler('ok',     this.onSellOk.bind(this));
        this._sellWindow.setHandler('cancel', this.onSellCancel.bind(this));
        this._categoryWindow.setItemWindow(this._sellWindow);
        this.addWindow(this._sellWindow);
        this._sellWindow.opacity = 0;
    };

    Scene_Shop.prototype.createNumberWindow = function() {
    	var wx = this._dummyWindow.x;
        var wy = this._dummyWindow.y;
        var wh = this._dummyWindow.height;
        this._numberWindow = new Window_ShopNumber(wx, wy, wh);
        this._numberWindow.hide();
        this._numberWindow.setHandler('ok',     this.onNumberOk.bind(this));
        this._numberWindow.setHandler('cancel', this.onNumberCancel.bind(this));
        this.addWindow(this._numberWindow);
        this._numberWindow.opacity = 0;
    };

    Scene_Shop.prototype.onCategoryCancel = function() {
        this._categoryWindow.hide();
        this._sellWindow.hide();
    	this._commandWindow.activate();
        this._dummyWindow.show();
    };


    ////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopDummy
	////////////////////////////////////////////////////////////////////////////////////

    function Window_ShopDummy() {
	    this.initialize.apply(this, arguments);
	}

    Window_ShopDummy.prototype = Object.create(Window_Base.prototype);
    Window_ShopDummy.prototype.constructor = Window_ShopDummy;

    Window_ShopDummy.prototype.setWinFrame = function(sprFrame){
    	this._sprFrame = sprFrame;
    };

    Window_ShopDummy.prototype.show = function(){
    	Window_Base.prototype.show.call(this);
        this._sprFrame.visible = false;
    }

    Window_ShopDummy.prototype.hide = function(){
    	Window_Base.prototype.hide.call(this);
        this._sprFrame.visible = true;
    }

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopCommand
	////////////////////////////////////////////////////////////////////////////////////

    Window_ShopCommand.prototype = Object.create(Window_MenuItemCategory.prototype);
    Window_ShopCommand.prototype.constructor = Window_ShopCommand;

    Window_ShopCommand.prototype.initialize = function(purchaseOnly) {
        this._purchaseOnly = purchaseOnly;
        Window_MenuItemCategory.prototype.initialize.call(this);
    };

    Window_ShopCommand.prototype.maxCols = function() {
        return 3;
    };

    Window_ShopCommand.prototype.makeCommandList = function() {
        this.addCommand(TextManager.buy,    'buy');
        this.addCommand(TextManager.sell,   'sell',   !this._purchaseOnly);
        this.addCommand(TextManager.cancel, 'cancel');
    };

    Window_ShopCommand.prototype.loadImage = function() {
		ImageManager.reserveSystem('IconSet20px');
		this._bmpCategory = ImageManager.loadBitmap("img/menus/", "menu_shop_category");
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopItemCategory
	////////////////////////////////////////////////////////////////////////////////////
	function Window_ShopItemCategory() {
	    this.initialize.apply(this, arguments);
	}

	Window_ShopItemCategory.prototype = Object.create(Window_MenuItemCategory.prototype);
	Window_ShopItemCategory.prototype.constructor = Window_ShopItemCategory;

	Window_ShopItemCategory.prototype.maxCols = function() {
        return 3;
    };

    Window_ShopItemCategory.prototype.makeCommandList = function() {
        this.addCommand(TextManager.item,    'item');
        this.addCommand(TextManager.weapon,  'weapon');
        this.addCommand(TextManager.armor,   'armor');
    };

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopBuy
	////////////////////////////////////////////////////////////////////////////////////

	Window_ShopBuy.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_ShopBuy.prototype.windowWidth = function() {
	    return 506;
	};

	Window_ShopBuy.prototype.lineHeight = function() {
	    return 34;
	};

	Window_ShopBuy.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_ShopBuy.prototype.standardPadding = function() {
	    return 20;
	};

	Window_ShopBuy.prototype.textPadding = function() {
	    return 8;
	};

	Window_ShopBuy.prototype.itemPadding = function() {
	    return 4;
	};

	Window_ShopBuy.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


	Window_ShopBuy.prototype.drawItem = function(index) {
	    var item = this._data[index];
	    var rect = this.itemRect(index);
	    var priceWidth = 96;
	    rect.width -= this.textPadding();
	    var itemNameW = 300;
	    this.changePaintOpacity(this.isEnabled(item));
	    this.drawItemName(item, rect.x, rect.y, itemNameW);
	    this.drawGold(this.contents,this.price(item),this.width - 260 , rect.y + 6, 220, 16);
	    this.changePaintOpacity(true);
	};

	Window_ShopBuy.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopStatus
	////////////////////////////////////////////////////////////////////////////////////

	Window_ShopStatus.prototype.standardFontFace = function() {
		 return 'GameFont';
	};

	Window_ShopStatus.prototype.lineHeight = function() {
	    return 26;
	};

	Window_ShopStatus.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_ShopStatus.prototype.standardPadding = function() {
	    return 20;
	};

	Window_ShopStatus.prototype.textPadding = function() {
	    return 8;
	};

	Window_ShopStatus.prototype.itemPadding = function() {
	    return 4;
	};

	Window_ShopStatus.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_ShopStatus.prototype.drawItemName = function(item, x, y) {
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, this.width - iconBoxWidth - 40);

	        console.log(this.width);
	    }
	};

	Window_ShopStatus.prototype.drawEquipInfo = function(x, y) {
	    var members = this.statusMembers();
	    for (var i = 0; i < members.length; i++) {
	        this.drawActorEquipInfo(x, y + this.lineHeight() * (i * 2.2), members[i]);
	    }
	};

	var _JPT_ShopMenu_Window_ShopStatus_drawPossession = Window_ShopStatus.prototype.drawPossession;
	Window_ShopStatus.prototype.drawPossession = function(x, y) {
		_JPT_ShopMenu_Window_ShopStatus_drawPossession.call(this,x, y);
		this.drawHorzLine(this.contents,8,this.lineHeight()-4,this.width,this.contents.fontSize*2);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopNumber
	////////////////////////////////////////////////////////////////////////////////////

	Window_ShopNumber.prototype.windowWidth = function() {
	    return 506;
	};

	Window_ShopNumber.prototype.drawTotalPrice = function() {
	    var total = this._price * this._number;
	    var width = this.contentsWidth() - this.textPadding();
	    this.drawGold(this.contents,total,this.width - 260, this.priceY(), 220, 22);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_ShopSell
	////////////////////////////////////////////////////////////////////////////////////

	Window_ShopSell.prototype = Object.create(Window_MenuItemList.prototype);
	Window_ShopSell.prototype.constructor = Window_ShopSell;

	Window_ShopSell.prototype.setWinFrame = function(sprFrame){
    	this._sprFrame = sprFrame;
    };

    Window_ShopSell.prototype.show = function(){
    	Window_Base.prototype.show.call(this);
        this._sprFrame.visible = false;
    }

    Window_ShopSell.prototype.hide = function(){
    	Window_Base.prototype.hide.call(this);
        this._sprFrame.visible = true;
    }

    Window_ShopSell.prototype.isEnabled = function(item) {
        return item && item.price > 0;
    };
}());



