//=============================================================================
// JPT_SkillLevelUpPlusParam
//=============================================================================

var Imported = Imported || {};
Imported['JPT_SkillLevelUpPlusParam'] = 1.00;

/*:ja
 * @plugindesc レベルアップ時のステータス補正
 * @author Kawamura
 *
 * @help
 * パーティー内のアクターがステータス補正に設定したスキルを所持していた場合
 * タグで紐づけられたアクターがレベルアップした場合にパラメータに補正を加えることができます
 *
 * ※JPT_SkillPointが必要です
 *
 * スキルのメモに以下の様に記述
 * <lvup_pp>
 * tag:(タグ名)
 * atk:(計算式)
 * mdf:(計算式)
 * </lvup_pp>
 *
 * tagは補正をかけるアクターを探すために使うので必ず記入してください
 * 補正がかけられるパラメータは
 * mhp(最大HP)
 * mmp(最大MP)
 * atk(攻撃)
 * def(防御)
 * mat(魔法攻撃)
 * mdf(魔法防御)
 * agi(敏捷)
 * luk(幸運)
 * msp(最大SP)
 *
 * 各計算式はevalで評価可能な形で記述してください
 *
 * アクターのメモに以下の様に記述
 * <lvup_pp_tag:(タグ名) >
 *
 */

/*
 *
 */

(function() {

	'use strict';

	//var parameters = PluginManager.parameters('JPT_SkillLevelUpPlusParam');


	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

	DataManager.evalCondition = function(formula, user) {
		var a = user;
		var s = $gameSwitches;
		var v = $gameVariables;
		var ret = eval(formula);
		return ret;
	};

	var _JPT_SkillLevelUpPlusParam_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_SkillLevelUpPlusParam_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processSkillNotetagsLvupp($dataSkills);
	    this.processActorNotetagsLvupp($dataActors);
		return true;
	};


	//メモからデータ取得
	DataManager.processSkillNotetagsLvupp = function(group) {
		var noteStart = /<lvup_pp>/i;
		var noteEnd = /<\/lvup_pp>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.lvup_pp_tag = '';
			obj.lvup_pp_mhp = '0';
			obj.lvup_pp_mmp = '0';
			obj.lvup_pp_atk = '0';
			obj.lvup_pp_def = '0';
			obj.lvup_pp_mat = '0';
			obj.lvup_pp_mdf = '0';
			obj.lvup_pp_agi = '0';
			obj.lvup_pp_luk = '0';
			obj.lvup_pp_msp = '0';

			var mode = 'none';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteStart)) {
					mode = 'lvup_pp';
				}else if(line.match(noteEnd)){
					break;
				}else if(mode == 'lvup_pp'){
					var state = line.split(/:/)[0];
					var strEval = line.split(/:/)[1];

					switch(state){
					case 'tag':
						obj.lvup_pp_tag = strEval;
						break;
					case 'mhp':
						obj.lvup_pp_mhp = strEval;
						break;
					case 'mmp':
						obj.lvup_pp_mmp = strEval;
						break;
					case 'atk':
						obj.lvup_pp_atk = strEval;
						break;
					case 'def':
						obj.lvup_pp_def = strEval;
						break;
					case 'mat':
						obj.lvup_pp_mat = strEval;
						break;
					case 'mdf':
						obj.lvup_pp_mdf = strEval;
						break;
					case 'agi':
						obj.lvup_pp_agi = strEval;
						break;
					case 'luk':
						obj.lvup_pp_luk = strEval;
						break;
					case 'msp':
						obj.lvup_pp_msp = strEval;
						break;
					}
				}
			}
		}
	};

	DataManager.processActorNotetagsLvupp = function(group) {

		var noteStart = /<lvup_pp_tag:(.+)/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);
			obj.lvup_pp_tag = '';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteStart)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.lvup_pp_tag = tagStr.trim();
				}
			}
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_BattlerBase
	////////////////////////////////////////////////////////////////////////////////////

	/*
	var _JPT_SkillLevelUpPlusParam_Game_BattlerBase_addParam = Game_BattlerBase.prototype.addParam;
	Game_BattlerBase.prototype.addParam = function(paramId, value) {
	    _JPT_SkillLevelUpPlusParam_Game_BattlerBase_addParam.call(this,paramId, Math.floor(value) );
	};
	*/

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Actor
	////////////////////////////////////////////////////////////////////////////////////


	var _JPT_SkillLevelUpPlusParam_Game_Actor_levelUp = Game_Actor.prototype.levelUp;
	Game_Actor.prototype.levelUp = function() {
		_JPT_SkillLevelUpPlusParam_Game_Actor_levelUp.call(this);

		//パーティメンバーのスキルチェック
		for(var i=0; i<$gameParty.allMembers().length; i++){
			var skills = $gameParty.allMembers()[i].skills();

			for(var j=0; j<skills.length; j++){
				var skill = skills[j];
				//tagが一致していれば
				if(this.actor().lvup_pp_tag==skill.lvup_pp_tag){

					var num = DataManager.evalCondition(skill.lvup_pp_mhp,this);
					if(num != 0)this.addParam(0,num);
					num = DataManager.evalCondition(skill.lvup_pp_mmp,this);
					if(num != 0)this.addParam(1,num);
					num = DataManager.evalCondition(skill.lvup_pp_atk,this);
					if(num != 0)this.addParam(2,num);
					num = DataManager.evalCondition(skill.lvup_pp_def,this);
					if(num != 0)this.addParam(3,num);
					num = DataManager.evalCondition(skill.lvup_pp_mat,this);
					if(num != 0)this.addParam(4,num);
					num = DataManager.evalCondition(skill.lvup_pp_mdf,this);
					if(num != 0)this.addParam(5,num);
					num = DataManager.evalCondition(skill.lvup_pp_agi,this);
					if(num != 0)this.addParam(6,num);
					num = DataManager.evalCondition(skill.lvup_pp_luk,this);
					if(num != 0)this.addParam(7,num);
					num = DataManager.evalCondition(skill.lvup_pp_msp,this);
					if(num != 0)this.addPsParam(JPT_SkillPoint.spParamIndex(),num);
				}
			}
		}
	};


}());
