//=============================================================================
// JPT_SkillPoint
//=============================================================================

var Imported = Imported || {};
Imported['JPT_SkillPoint'] = 1.00;

/*:ja
 * @plugindesc スキルポイントの実装
 * @author Kawamura
 *
 * @param SkillPointName
 * @desc スキルポイントの名前
 * @default ＳＰ
 *
 * @param SkillPointAbbreviation
 * @desc スキルポイントの略称
 * @default SP
 *
 * @param SkillPointParamIndex
 * @desc スキルポイント パラメータインデックス
 * @default 8
 *
 * @param SkillPointRegenerateXParamIndex
 * @desc SP再生率 パラメータインデックス
 * @default 10
 *
 * @param SkillPointParamMax
 * @desc スキルポイントパラメータ最大値
 * @default 999
 *
 * @param RecoverSpEffectID
 * @desc SP回復効果ID
 * @default 101
 *
 * @help
 * スキルポイントを実装します
 * MPとは別枠の消費リソースです
 *
 *--------------------------------------------------------------------------------
 * パラメータ
 *--------------------------------------------------------------------------------
 * SkillPointName, SkillPointAbbreviation
 * ステータスなどの表記を定義してください
 *
 * SkillPointParamIndex, SkillPointRegenerateXParamIndex
 * プログラム上の参照インデックスです
 * 他のプラグインと競合しない場合はデフォルトのまま使用してください
 *
 * SkillPointParamMax
 * SPの最大値です msp_eval で設定した計算式もこれ以上の値にはなりません
 *
 * RecoverSpEffectID
 * スキルやアイテムなどでSP回復効果を表すIDです
 * 他のプラグインと競合しない場合はデフォルトのまま使用してください
 *
 *--------------------------------------------------------------------------------
 * 職業への記述
 *--------------------------------------------------------------------------------
 * 職業のメモに以下の様に記述されていればその職業に就いている間SPが利用できます
 *
 * sp最大値
 * <msp_eval:(計算式)>
 *
 * evalで評価可能な形で記述してください
 * msp_eval:10 + a._level*3
 *
 *--------------------------------------------------------------------------------
 * スキルへの記述
 *--------------------------------------------------------------------------------
 * スキルのメモに以下の様に記述すればコストとしてSPを設定できます
 *
 * <cost_sp_eval:(計算式)>
 * 計算式はevalで評価可能な形で記述してください
 *
 * 固定消費
 * <cost_sp_eval:5>
 *
 * 最大SPの割合
 * <cost_sp_eval:a.msp/10>
 *
 * 現在の残りSP全部
 * <cost_sp_eval:a.sp>
 *
 *--------------------------------------------------------------------------------
 *--------------------------------------------------------------------------------
 *--------------------------------------------------------------------------------
 * HP,MP同様にSP再生率が設定できます
 *
 * SP再生率を設定しておくと毎ターンSPが回復するようになり
 * マップ上でも一定歩数ごとに回復します(規定は20歩)
 * ※SP再生率がマイナス値の場合SPが減っていきます
 *
 *
 *--------------------------------------------------------------------------------
 * アクター、職業効果
 *--------------------------------------------------------------------------------
 * SP最大値補正*
 * <trait_msp:120%>
 *
 * SP再生率補正＋
 * <trait_srg:5%>
 *
 *--------------------------------------------------------------------------------
 * 装備効果
 *--------------------------------------------------------------------------------
 * 武器、防具に対して
 * SP最大値補正
 * 固定値+
 * <param_msp:5>
 * 割合*
 * <trait_msp:120%>
 *
 * SP再生率補正＋
 * <trait_srg:5%>
 *
 *--------------------------------------------------------------------------------
 * 使用効果
 *--------------------------------------------------------------------------------
 * スキル、アイテムによる回復、ダメージ
 *
 * SP回復<割合,固定値>
 * ※割合,固定値とも記述してください。設定したくない場合は0にしてください
 * ※ダメージの場合はマイナス値にしてください
 * <recover_sp:10%,30>
 *
 * SP最大値のみ 強化、弱体化バフ レベルは2まで
 * レベル、持続ターンの指定
 *
 * SP最大値バフ
 * <buff_msp:1,5>
 * <debuff_msp:2,3>
 *
 * 強化解除
 * <rel_buff_msp>
 *
 * 弱体解除
 * <rel_debuff_msp>
 *
 *
 *--------------------------------------------------------------------------------
 * ステート効果
 *--------------------------------------------------------------------------------
 * SP最大値補正*
 * <trait_msp:120%>
 *
 * SP再生率補正+
 * <trait_srg:10%>
 *
 * SP最大値 弱体有効度*
 * <debuff_rate_msp:150%>
 *
 *--------------------------------------------------------------------------------
 * 敵キャラ
 *--------------------------------------------------------------------------------
 * 敵キャラにもSPを設定できます
 *
 * SP最大値
 * <param_msp:50>
 *
 * SP再生率補正+
 * <trait_srg:10%>
 *
 *--------------------------------------------------------------------------------
 * イベントコマンド
 *--------------------------------------------------------------------------------
 * SP_Change ActorID vale_eval
 *
 * ActorIDは0でパーティー全体になります
 * vale_evalはevalで評価可能な形で記述してください
 *
 * アクター1のSPを最大値の10%減らす
 * SP_Change 1 -(a.msp/10)
 *
 *
 *
 */

function JPT_SkillPoint(){
	throw new Error('This is a static class');
}

(function() {

	'use strict';

	var parameters = PluginManager.parameters('JPT_SkillPoint');
	var SkillPointName = parameters['SkillPointName'];
	var SkillPointAbbreviation = parameters['SkillPointAbbreviation'];
	var SkillPointParamIndex = parseInt(parameters['SkillPointParamIndex'] || 8);
	var SkillPointRegenerateXParamIndex = parseInt(parameters['SkillPointRegenerateXParamIndex'] || 10);
	var SkillPointParamMax = parseInt(parameters['SkillPointParamMax'] || 999);
	var RecoverSpEffectID = parseInt(parameters['RecoverSpEffectID'] || 101);

	JPT_SkillPoint.spParamIndex = function(){
		return SkillPointParamIndex;
	}

	JPT_SkillPoint.srgXParamIndex = function(){
		return SkillPointRegenerateXParamIndex;
	}

	////////////////////////////////////////////////////////////////////////////////////
    // TextManager
	////////////////////////////////////////////////////////////////////////////////////

	Object.defineProperties(TextManager, {
		sp	: { get: function() { return SkillPointName; }, configurable: true },
		spA	: { get: function() { return SkillPointAbbreviation; }, configurable: true },
		srg : { get: function() { return SkillPointAbbreviation + "回復率"; }, configurable: true },
	});

	////////////////////////////////////////////////////////////////////////////////////
    // Game_Interpreter
	////////////////////////////////////////////////////////////////////////////////////

    var _JPT_SkillPoint_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
    	_JPT_SkillPoint_Game_Interpreter_pluginCommand.call(this, command, args);

    	switch(command){
    	case 'SP_Change':
    		this.changeSP(args);
    		break;
    	}
    };

    // Change SP

    Game_Interpreter.prototype.changeSP = function(params) {
        this.iterateActorId(params[0], function(actor) {
            //actor.gainSp( Math.round( eval(params[1]) ) );
        	actor.gainSp( Math.round( DataManager.evalCondition(params[1], actor) ) );
        }.bind(this));
        return true;
    };

	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

    DataManager.evalCondition = function(formula, user) {
		var a = user;
		var s = $gameSwitches;
		var v = $gameVariables;
		var ret = eval(formula);
		return ret;
	};

	var _JPT_SkillPoint_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_SkillPoint_DataManager_isDatabaseLoaded.call(this)) return false;

		this.processClassNotetagsSp($dataClasses);
	    this.processActorNotetagsSp($dataActors);
	    this.processWeaponsNotetagsSp($dataWeapons);
	    this.processArmorsNotetagsSp($dataArmors);
		this.processSkillNotetagsSp($dataSkills);
	    this.processItemsNotetagsSp($dataItems);
	    this.processStatesNotetagsSp($dataStates);
	    this.processEnemiesNotetagsSp($dataEnemies);

	    //console.log('SkillPoint isDatabaseLoaded');
		return true;
	};


	//メモからデータ取得
	DataManager.processClassNotetagsSp = function(group) {

		var noteMspEval = /<msp_eval:(.+)>/i;
		var noteTraitSp = /<trait_msp:(.+)>/i;
		var noteTraitSrg = /<trait_srg:(.+)>/i;


		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.msp_eval = '0';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];

				if(line.match(noteMspEval)){
					var tagStr = line.split(/:/)[1];
					obj.msp_eval = tagStr.split(/>/)[0];
				}else if(line.match(noteTraitSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_PARAM, dataId: SkillPointParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}else if(line.match(noteTraitSrg)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_XPARAM, dataId: SkillPointRegenerateXParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}
			}
		}
	};

	DataManager.processActorNotetagsSp = function(group) {
		var noteTraitSp = /<trait_msp:(.+)>/i;
		var noteTraitSrg = /<trait_srg:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if(line.match(noteTraitSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_PARAM, dataId: SkillPointParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}else if(line.match(noteTraitSrg)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_XPARAM, dataId: SkillPointRegenerateXParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}
			}
		}
	}

	DataManager.processWeaponsNotetagsSp = function(group) {

		var noteParamSp = /<param_msp:(.+)>/i;
		var noteTraitSp = /<trait_msp:(.+)>/i;
		var noteTraitSrg = /<trait_srg:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			if(obj.params.length <= SkillPointParamIndex){
				for(var i=0, m=(SkillPointParamIndex + 1 - obj.params.length); i<m; i++ ){
					obj.params.push( 0 );
				}
			}

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];

				if(line.match(noteParamSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.params[SkillPointParamIndex] = parseInt(tagStr);
				}else if(line.match(noteTraitSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_PARAM, dataId: SkillPointParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}else if(line.match(noteTraitSrg)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_XPARAM, dataId: SkillPointRegenerateXParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}
			}
		}
	}

	DataManager.processArmorsNotetagsSp = function(group) {

		var noteParamSp = /<param_msp:(.+)>/i;
		var noteTraitSp = /<trait_msp:(.+)>/i;
		var noteTraitSrg = /<trait_srg:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			if(obj.params.length <= SkillPointParamIndex){
				for(var i=0, m=(SkillPointParamIndex + 1 - obj.params.length); i<m; i++ ){
					obj.params.push( 0 );
				}
			}

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];

				if(line.match(noteParamSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.params[SkillPointParamIndex] = parseInt(tagStr);
				}else if(line.match(noteTraitSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_PARAM, dataId: SkillPointParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}else if(line.match(noteTraitSrg)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_XPARAM, dataId: SkillPointRegenerateXParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}
			}
		}
	}

	DataManager.processSkillNotetagsSp = function(group) {

		var noteCost = /<cost_sp_eval:(.+)>/i;
		var noteBuff = /<buff_msp:(.+)>/i;
		var noteDeBuff = /<debuff_msp:(.+)>/i;
		var noteRelBuff = /<rel_buff_msp>/i;
		var noteRelDeBuff = /<rel_debuff_msp>/i;
		var noteRecover = /<recover_sp:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.cost_sp_eval = '0';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteCost)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.cost_sp_eval = tagStr.trim();
				}else if (line.match(noteBuff)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					var val = tagStr.split(/,/);
					var item = {code: Game_Action.EFFECT_ADD_BUFF, dataId: SkillPointParamIndex, value1: parseInt(val[1]), value2: 0 };
					for(var j=0; j<parseInt(val[0]); j++){
						obj.effects.push(item);
					}
				}else if (line.match(noteDeBuff)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					var val = tagStr.split(/,/);
					var item = {code: Game_Action.EFFECT_ADD_DEBUFF, dataId: SkillPointParamIndex, value1: parseInt(val[1]), value2: 0 };
					for(var j=0; j<parseInt(val[0]); j++){
						obj.effects.push(item);
					}
				}else if (line.match(noteRelBuff)) {
					var item = {code: Game_Action.EFFECT_REMOVE_BUFF, dataId: SkillPointParamIndex, value1: 1, value2: 0 };
					obj.effects.push(item);
				}else if (line.match(noteRelDeBuff)) {
					var item = {code: Game_Action.EFFECT_REMOVE_DEBUFF, dataId: SkillPointParamIndex, value1: 1, value2: 0 };
					obj.effects.push(item);
				}else if (line.match(noteRecover)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					var val = tagStr.split(/,/);
					val[0] = val[0].replace(/%/g, "");
					var item = {code: RecoverSpEffectID, dataId: 0, value1: parseInt(val[0]), value2: parseInt(val[1]) };
					obj.effects.push(item);
				}
			}
		}
	};

	DataManager.processItemsNotetagsSp = function(group) {
		var noteBuff = /<buff_msp:(.+)>/i;
		var noteDeBuff = /<debuff_msp:(.+)>/i;
		var noteRelBuff = /<rel_buff_msp>/i;
		var noteRelDeBuff = /<rel_debuff_msp>/i;
		var noteRecover = /<recover_sp:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteBuff)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					var val = tagStr.split(/,/);
					var item = {code: Game_Action.EFFECT_ADD_BUFF, dataId: SkillPointParamIndex, value1: parseInt(val[1]), value2: 0 };
					for(var j=0; j<parseInt(val[0]); j++){
						obj.effects.push(item);
					}
				}else if (line.match(noteDeBuff)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					var val = tagStr.split(/,/);
					var item = {code: Game_Action.EFFECT_ADD_DEBUFF, dataId: SkillPointParamIndex, value1: parseInt(val[1]), value2: 0 };
					for(var j=0; j<parseInt(val[0]); j++){
						obj.effects.push(item);
					}
				}else if (line.match(noteRelBuff)) {
					var item = {code: Game_Action.EFFECT_REMOVE_BUFF, dataId: SkillPointParamIndex, value1: 1, value2: 0 };
					obj.effects.push(item);
				}else if (line.match(noteRelDeBuff)) {
					var item = {code: Game_Action.EFFECT_REMOVE_DEBUFF, dataId: SkillPointParamIndex, value1: 1, value2: 0 };
					obj.effects.push(item);
				}else if (line.match(noteRecover)) {
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					var val = tagStr.split(/,/);
					val[0] = val[0].replace(/%/g, "");
					var item = {code: RecoverSpEffectID, dataId: 0, value1: parseInt(val[0]), value2: parseInt(val[1]) };
					obj.effects.push(item);
				}
			}
		}
	}

	DataManager.processStatesNotetagsSp = function(group) {
		var noteTraitSp = /<trait_msp:(.+)>/i;
		var noteTraitSrg = /<trait_srg:(.+)>/i;
		var noteDeBuffRate = /<debuff_rate_msp:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];

				if(line.match(noteDeBuffRate)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_DEBUFF_RATE, dataId: SkillPointParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}else if(line.match(noteTraitSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_PARAM, dataId: SkillPointParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}else if(line.match(noteTraitSrg)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_XPARAM, dataId: SkillPointRegenerateXParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}
			}
		}
	}

	DataManager.processEnemiesNotetagsSp = function(group) {
		var noteParamSp = /<param_msp:(.+)>/i;
		var noteTraitSrg = /<trait_srg:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			if(obj.params.length <= SkillPointParamIndex){
				for(var i=0, m=(SkillPointParamIndex + 1 - obj.params.length); i<m; i++ ){
					obj.params.push( 0 );
				}
			}

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];

				if(line.match(noteParamSp)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					obj.params[SkillPointParamIndex] = parseInt(tagStr);
				}else if(line.match(noteTraitSrg)){
					var tagStr = line.split(/:/)[1];
					tagStr = tagStr.split(/>/)[0];
					tagStr = tagStr.replace(/%/g, "");
					var item = {code: Game_BattlerBase.TRAIT_XPARAM, dataId: SkillPointRegenerateXParamIndex, value: tagStr/100 };
					obj.traits.push(item);
				}
			}
		}
	}




	////////////////////////////////////////////////////////////////////////////////////
	// Game_BattlerBase
	////////////////////////////////////////////////////////////////////////////////////

	Object.defineProperties(Game_BattlerBase.prototype, {
		// 現在のSP
	    sp: { get: function() { return this._sp; }, configurable: true },
	    // SP最大値
	    msp: { get: function() { return this.param(SkillPointParamIndex); }, configurable: true },
	    // SP回復率
	    srg: { get: function() { return this.xparam(SkillPointRegenerateXParamIndex); }, configurable: true },
	});


	var _JPT_SkillPoint_Game_BattlerBase_initMembers = Game_BattlerBase.prototype.initMembers;
	Game_BattlerBase.prototype.initMembers = function() {
		_JPT_SkillPoint_Game_BattlerBase_initMembers.call(this);
	    this._sp = 0;
	};

	var _JPT_SkillPoint_Game_BattlerBase_clearParamPlus = Game_BattlerBase.prototype.clearParamPlus;
	Game_BattlerBase.prototype.clearParamPlus = function() {
		_JPT_SkillPoint_Game_BattlerBase_clearParamPlus.call(this);

		if(this._paramPlus.length <= SkillPointParamIndex){
			for(var i=0, n=(SkillPointParamIndex + 1 - this._paramPlus.length); i<n; i++ ){
				this._paramPlus.push( 0 );
			}
		}
	};

	var _JPT_SkillPoint_Game_BattlerBase_clearBuffs = Game_BattlerBase.prototype.clearBuffs;
	Game_BattlerBase.prototype.clearBuffs = function() {
		_JPT_SkillPoint_Game_BattlerBase_clearBuffs.call(this);

		if(this._buffs.length <= SkillPointParamIndex){
			for(var i=0, n=(SkillPointParamIndex + 1 - this._buffs.length); i<n; i++ ){
				this._buffs.push( 0 );
			}
		}

		if(this._buffTurns.length <= SkillPointParamIndex){
			for(var i=0, n=(SkillPointParamIndex + 1 - this._buffTurns.length); i<n; i++ ){
				this._buffTurns.push( 0 );
			}
		}
	};


	var _JPT_SkillPoint_Game_BattlerBase_refresh = Game_BattlerBase.prototype.refresh;
	Game_BattlerBase.prototype.refresh = function() {
		_JPT_SkillPoint_Game_BattlerBase_refresh.call(this);
		this._sp = this._sp.clamp(0, this.msp);
	};

	var _JPT_SkillPoint_Game_BattlerBase_recoverAll = Game_BattlerBase.prototype.recoverAll;
	Game_BattlerBase.prototype.recoverAll = function() {
		_JPT_SkillPoint_Game_BattlerBase_recoverAll.call(this);
	    this._sp = this.msp;
	};


	Game_BattlerBase.prototype.spRate = function() {
	    return this.msp > 0 ? this.sp / this.msp : 0;
	};

	Game_BattlerBase.prototype.setSp = function(sp) {
	    this._sp = sp;
	    this.refresh();
	};

	Game_BattlerBase.prototype.skillSpCost = function(skill) {
		return DataManager.evalCondition(skill.cost_sp_eval,this);
	};

	var _JPT_SkillPoint_Game_BattlerBase_canPaySkillCost = Game_BattlerBase.prototype.canPaySkillCost;
	Game_BattlerBase.prototype.canPaySkillCost = function(skill) {
	    return _JPT_SkillPoint_Game_BattlerBase_canPaySkillCost.call(this, skill) && this._sp >= this.skillSpCost(skill);
	};

	var _JPT_SkillPoint_Game_BattlerBase_paySkillCost = Game_BattlerBase.prototype.paySkillCost;
	Game_BattlerBase.prototype.paySkillCost = function(skill) {
		_JPT_SkillPoint_Game_BattlerBase_paySkillCost.call(this,skill);
		this._sp -= this.skillSpCost(skill);
	};

	var _JPT_SkillPoint_Game_BattlerBase_paramMin = Game_BattlerBase.prototype.paramMin;
	Game_BattlerBase.prototype.paramMin = function(paramId) {
	    if (paramId === SkillPointParamIndex) {
	        return 0;
	    } else {
	        return _JPT_SkillPoint_Game_BattlerBase_paramMin.call(this,paramId);
	    }
	};

	var _JPT_SkillPoint_Game_BattlerBase_paramMax = Game_BattlerBase.prototype.paramMax;
	Game_BattlerBase.prototype.paramMax = function(paramId) {
		if (paramId === SkillPointParamIndex) {
	        return SkillPointParamMax;
	    } else {
	        return _JPT_SkillPoint_Game_BattlerBase_paramMax.call(this,paramId);
	    }
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Game_Battler
	////////////////////////////////////////////////////////////////////////////////////
	Game_Battler.prototype.gainSp = function(value) {
	    this._result.spDamage = -value;
	    this.setSp(this.sp + value);
	};

	Game_Battler.prototype.regenerateSp = function() {
	    var value = Math.floor(this.msp * this.srg);
	    if (value !== 0) {
	       if(this.sp < this.msp) this.gainSp(value);
	    }
	};

	/*
	var _JPT_SkillPoint_Game_Battler_regenerateAll = Game_Battler.prototype.regenerateAll;
	Game_Battler.prototype.regenerateAll = function() {
		_JPT_SkillPoint_Game_Battler_regenerateAll.call(this);
	    if (this.isAlive()) {
	        this.regenerateSp();
	    }
	    if ($gameParty.inBattle()) this.startDamagePopup();
	};
	*/


	////////////////////////////////////////////////////////////////////////////////////
	// Game_Actor
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_SkillPoint_Game_Actor_paramBase = Game_Actor.prototype.paramBase;
	Game_Actor.prototype.paramBase = function(paramId) {
		if(paramId==SkillPointParamIndex){
			return DataManager.evalCondition(this.currentClass().msp_eval,this);
		}else{
		    return _JPT_SkillPoint_Game_Actor_paramBase.call(this,paramId);
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_ActionResult
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_SkillPoint_Game_ActionResult_clear = Game_ActionResult.prototype.clear;
	Game_ActionResult.prototype.clear = function() {
		_JPT_SkillPoint_Game_ActionResult_clear.call(this);
	    this.spDamage = 0;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Action
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_SkillPoint_Game_Action_testItemEffect = Game_Action.prototype.testItemEffect;
	Game_Action.prototype.testItemEffect = function(target, effect) {
	    switch (effect.code) {
	    case RecoverSpEffectID:
	    	return target.sp < target.msp || effect.value1 < 0 || effect.value2 < 0;
	    default:
	      return _JPT_SkillPoint_Game_Action_testItemEffect.call(this, target, effect);
	    }
	};

	var _JPT_SkillPoint_Game_Action_applyItemEffect = Game_Action.prototype.applyItemEffect;
	Game_Action.prototype.applyItemEffect = function(target, effect) {
	    switch (effect.code) {
	    case RecoverSpEffectID:
	    	this.itemEffectRecoverSp(target, effect);
	    	break;
	    default:
	    	_JPT_SkillPoint_Game_Action_applyItemEffect.call(this, target, effect);
	        break;
	    }
	};

	Game_Action.prototype.itemEffectRecoverSp = function(target, effect) {
	    var value = (target.mmp * effect.value1 + effect.value2) * target.rec;
	    if (this.isItem()) {
	        value *= this.subject().pha;
	    }
	    value = Math.floor(value);
	    if (value !== 0) {
	        target.gainSp(value);
	        this.makeSuccess(target);
	    }
	};

}());
