//=============================================================================
// JPT_SkillTransform
//=============================================================================

var Imported = Imported || {};
Imported['JPT_SkillTransform'] = 1.00;

/*:ja
 * @plugindesc スキル変身(戻る)
 * @author Kawamura
 *
 * @help
 * 戦闘中アクターを別のアクターへ切り替える機能を追加します
 *
 *---------------------------------------------------------------------------------------
 * スキルの設定について
 *---------------------------------------------------------------------------------------
 * 「変身」「戻る」はステート付加時に実行していますので
 * 「変身」「戻る」のスキルにそれぞれ専用のステートを付加するようにしてください
 *
 *---------------------------------------------------------------------------------------
 * ステートの設定について
 *---------------------------------------------------------------------------------------
 * 「変身」ステートのメモに <StateTransform> を記述してください
 * 「戻る」ステートのメモに <StateReTransform> を記述してください
 *---------------------------------------------------------------------------------------
 * アクターの設定について
 *---------------------------------------------------------------------------------------
 * * 変身元のアクター
 * 「変身」に指定したスキルを使えるようにしてください
 * メモに以下の例のように記述してください
 * <transform>
 * id=1
 * relation=master
 * </transform>
 *---------------------------------------------------------------------------------------
 * * 変身後のアクター
 * 「戻る」に指定したスキルを使えるようにしてください
 * メモに以下の例のように記述してください
 * <transform>
 * id=1
 * relation=slave
 * </transform>
 *---------------------------------------------------------------------------------------
 *
 * 「id」は変身元と変身後を紐づけるために使用します
 * 同じ数値を記述してください
 * 「relation」は主従を設定します
 * 変身元は「master」変身後は「slave」と記述してください
 *
 * 現状、変身元と変身後は対になっています
 * 複数の変身後を設定することはできません
 *
 *---------------------------------------------------------------------------------------
 *
 */

/*
 *
 */

(function() {

	'use strict';

	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_SkillTransform_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_SkillTransform_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processActorNotetagsTs($dataActors);
	    this.processStateNotetagsTs($dataStates);
		return true;
	};


	//メモからデータ取得
	DataManager.processActorNotetagsTs = function(group) {

		var noteStart = /<transform>/i;
		var noteEnd = /<\/transform>/i;

		var noteID = /id=(\d+)/i;
		var noteRelation = /relation=(\w+)/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.transform_id = -1;
			obj.transform_relation = '';

			var mode = 'none';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteStart)) {
					mode = 'transform';
				}else if(line.match(noteEnd)){
					break;
				}else if(mode == 'transform'){
					if(line.match(noteID)){
						obj.transform_id = parseInt( line.split(/=/)[1], 10 );
					}else if(line.match(noteRelation)){
						obj.transform_relation = line.split(/=/)[1];
					}
				}
			}
		}
	};

	//メモからデータ取得
	DataManager.processStateNotetagsTs = function(group) {

		var noteTs = /<StateTransform>/i;
		var noteReTs = /<StateReTransform>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			if(!obj.unique)	obj.unique = '';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteTs)) {
					obj.unique = JPT_SkillTransform.TS_UNIQUE;
				}else if(line.match(noteReTs)){
					obj.unique = JPT_SkillTransform.RETS_UNIQUE;
				}
			}
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// JPT_SkillTransform
	////////////////////////////////////////////////////////////////////////////////////

	function JPT_SkillTransform() {
	    this.initialize.apply(this, arguments);
	}

	JPT_SkillTransform.TS_UNIQUE = 'transform';
	JPT_SkillTransform.RETS_UNIQUE = 're_transform';

	JPT_SkillTransform.swapActorTransform = function (target, relationFrom, relationTo ) {

		var battleMembers = $gameParty.battleMembers();
		var actorIndex = -1;
		for(var i=0; i<$gameParty.battleMembers().length; i++ ){
			if($gameParty.battleMembers()[i].actorId()==target.actorId()){
				actorIndex = i;
				break;
			}
		}

		if(	target.actor().transform_id < 0 &&
			target.actor().transform_relation != relationFrom){
			return;
		}

		var toActor = null;
		for(var i=0; i<$dataActors.length; i++ ){
			if(	$dataActors[i] &&
				$dataActors[i].transform_id==target.actor().transform_id &&
				$dataActors[i].transform_relation==relationTo){
				toActor = $gameActors.actor($dataActors[i].id);
			}
		}

		if(toActor){

			//変身後のパラメータ計算
			if(relationFrom=='master'){
				toActor.changeLevel(target.level,false);
				toActor.copyParamPlus(target);
			}

			//HP割合,MP割合,TPの引継ぎ
			toActor.setHp( Math.round(toActor.mhp * target.hpRate()) );
			toActor.setMp( Math.round(toActor.mmp * target.mpRate()) );
			toActor.setTp( Math.round(target.tp) );

			//ステートの引継ぎ
			toActor.copyState(target);

			//toActor.appear();
			this.setChanged(true);

			$gameParty.addActor(toActor.actorId());
			$gameParty.swapOrder(actorIndex, $gameParty.allMembers().length - 1);
			$gameParty.removeActor(target.actorId());
		}
	}

	var JPT_SkillTransform_change = false;

	JPT_SkillTransform.setChanged = function(change){
		return JPT_SkillTransform_change = change;
	}

	JPT_SkillTransform.isChanged = function(){
		return JPT_SkillTransform_change;
	}

	JPT_SkillTransform.checkTransformed = function() {
		for(var i=0; i<$gameParty.battleMembers().length; i++ ){
			if(	$gameParty.battleMembers()[i].actor().transform_id >= 0 &&
				$gameParty.battleMembers()[i].actor().transform_relation == 'slave'){
				return true;
			}
		}
		return false;
	}

	JPT_SkillTransform.onBattleEnd = function() {
		var check = false;
		for(var i=0; i<$gameParty.battleMembers().length; i++ ){

			if(	$gameParty.battleMembers()[i].actor().transform_id >= 0 &&
				$gameParty.battleMembers()[i].actor().transform_relation == 'slave'){
				check = true;
				//戻す
				this.swapActorTransform($gameParty.battleMembers()[i],'slave','master');
				break;
			}
		}
		if(check){
			//シーケンス「戻る」が終わるまで待つに
		}
	}


	////////////////////////////////////////////////////////////////////////////////////
	// BattleManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_SkillTransform_BattleManager_initMembers = BattleManager.initMembers;
	BattleManager.initMembers = function() {
		_JPT_SkillTransform_BattleManager_initMembers.call(this);
		this._seq_transform = 0;
	};

	var _JPT_SkillTransform_BattleManager_checkBattleEnd = BattleManager.checkBattleEnd;
	BattleManager.checkBattleEnd = function() {
		if (this._phase) {
			if ($gameTroop.isAllDead() && JPT_SkillTransform.checkTransformed() ) {
				JPT_SkillTransform.onBattleEnd();
				return true;
	        }
		}

		_JPT_SkillTransform_BattleManager_checkBattleEnd.call(this);
	}


	var _JPT_SkillTransform_BattleManager_processEscape = BattleManager.processEscape;
	BattleManager.processEscape = function() {
	    var success = _JPT_SkillTransform_BattleManager_processEscape.call(this);
	    if(success && JPT_SkillTransform.checkTransformed()){
	    	JPT_SkillTransform.onBattleEnd();
	    }
	    return success;
	};


	////////////////////////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////////////////////////

	Game_BattlerBase.prototype.statesArray = function() {
	    return this._states;
	};

	Game_BattlerBase.prototype.statesTurns = function() {
	    return this._stateTurns;
	};

	Game_BattlerBase.prototype.paramPlusArray = function() {
	    return this._paramPlus;
	};

	Game_BattlerBase.prototype.buffs = function() {
	    return this._buffs;
	}

	Game_BattlerBase.prototype.buffTurns = function() {
	    return this._buffTurns;
	}


	Game_BattlerBase.prototype.copyState = function(traget) {
	    this._states = traget.statesArray();
	    this._stateTurns = traget.statesTurns();
	    this._buffs = traget.buffs();
	    this._buffTurns = traget.buffTurns();
	};

	Game_BattlerBase.prototype.copyParamPlus = function(traget) {
	    this._paramPlus = traget.paramPlusArray();
	};

	Game_Actor.prototype.stateSteps = function() {
		return this._stateSteps;
	}

	Game_Actor.prototype.copyState = function(traget){
		Game_BattlerBase.prototype.copyState.call(this,traget);
		this._stateSteps = traget.stateSteps();
	}

	//ステート付加時に入れ替え実行
	var _JPT_SkillTransform_Game_Action_itemEffectAddNormalState = Game_Action.prototype.itemEffectAddNormalState;
	Game_Action.prototype.itemEffectAddNormalState = function(target, effect) {
		_JPT_SkillTransform_Game_Action_itemEffectAddNormalState.call(this,target, effect);

		JPT_SkillTransform.setChanged(false);
		if( $dataStates[effect.dataId].unique == JPT_SkillTransform.TS_UNIQUE ){
			//console.log('use transform');
			JPT_SkillTransform.swapActorTransform(target, 'master', 'slave');
		}else if( $dataStates[effect.dataId].unique == JPT_SkillTransform.RETS_UNIQUE ){
			//console.log('use retransform end');
			JPT_SkillTransform.swapActorTransform(target, 'slave', 'master');
		}

	};

	////////////////////////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_SkillTransform_Sprite_Actor_moveToStartPosition = Sprite_Actor.prototype.moveToStartPosition;
	Sprite_Actor.prototype.moveToStartPosition = function() {

		if(JPT_SkillTransform.isChanged())return;

		_JPT_SkillTransform_Sprite_Actor_moveToStartPosition.call(this);
	};


}());
