//=============================================================================
// JPT_SkillVigilance
//=============================================================================

var Imported = Imported || {};
Imported['JPT_SkillVigilance'] = 1.00;

/*:ja
 * @plugindesc スキル警戒
 * @author Kawamura
 *
 * @param Target Default Text
 * @desc スキルにターゲット時のテキスト指定がない場合の表示テキスト指定
 * @default #Oは#Tを狙っている
 *
 * @param Target Self Default Text
 * @desc ターゲットが自身の場合のテキスト
 * @default 自身
 *
 * @help
 * 警戒に指定したスキルの速度補正は最大にしてください
 *
 * 警戒に指定したスキルのメモに<SkillVigilance>を記述してください
 *
 * 警戒中、コマンド再選択時に警戒を選択できなくするために
 * 警戒のスキルを封印するステートを付加するようにしてください
 *
 *
 *
 *--------------------------------------------------------------------------------
 * 警戒中に表示される敵の行動表示について
 *--------------------------------------------------------------------------------
 * 敵スキルのメモに以下の様に記述してください
 * <vig_mess:(表示メッセージ)>
 *
 * 指定なし
 * 「AはBをねらっている」
 * 指定なしの場合ターゲットされたキャラ分すべて表示します
 * 全体攻撃の場合パーティ全員分のメッセージが列挙されます
 *
 * 指定したメッセージをそのまま表示
 * <vig_mess:メッセージ>
 *  *
 * コマンドを埋め込んだメッセージ
 * <vig_mess:#Eは#Sを使おうとしている>
 *
 * #O スキル使用者名
 * #T スキル対象名(複数の場合は「、」区切りで列挙)
 * #S スキル名
 *
 * #O = #T の場合 #Tを「自分」にします
 *
 */

/*
 *
 */
(function() {

	'use strict';

	////////////////////////////////////////////////////////////////////////////////////

	function JPT_SkillVigilance() {
	    this.initialize.apply(this, arguments);
	}

	var parameters = PluginManager.parameters('JPT_SkillVigilance');
	var targetDefaultText = parameters['Target Default Text'];
	var targetSelf = parameters['Target Self Default Text'];

	JPT_SkillVigilance.INIT = 0;
	JPT_SkillVigilance.INVOCATION = 1;
	JPT_SkillVigilance.REINPUT = 2;
	JPT_SkillVigilance.SELECTED = 3;
	JPT_SkillVigilance.UNIQUE = 'vigilance';

	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_SkillVigilance_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_SkillVigilance_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processSkillNotetagsVigilance($dataSkills);
		return true;
	};


	//メモからデータ取得
	DataManager.processSkillNotetagsVigilance = function(group) {
		var noteVig = /<SkillVigilance>/i;
		var noteMessage = /<vig_mess:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			if(!obj.unique)	obj.unique = '';
			obj.vig_mess = '';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteVig)) {
					obj.unique = JPT_SkillVigilance.UNIQUE;
				}else if(line.match(noteMessage)){
					var tagStr = line.split(/:/)[1];
					obj.vig_mess = tagStr.split(/>/)[0];
				}
			}
		}
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Battle
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_SkillVigilance_Scene_Battle_start = Scene_Battle.prototype.start;
	Scene_Battle.prototype.start = function() {
		_JPT_SkillVigilance_Scene_Battle_start.call(this);
	    BattleManager.targetLineOff();
	};

	var _JPT_SkillVigilance_Scene_Battle_selectPreviousCommand = Scene_Battle.prototype.selectPreviousCommand;
	Scene_Battle.prototype.selectPreviousCommand = function() {
		if(BattleManager.getReInput() != JPT_SkillVigilance.REINPUT){
			_JPT_SkillVigilance_Scene_Battle_selectPreviousCommand.call(this);
		}
	};

	var _JPT_SkillVigilance_Scene_Battle_endCommandSelection = Scene_Battle.prototype.endCommandSelection;
	Scene_Battle.prototype.endCommandSelection = function() {
		_JPT_SkillVigilance_Scene_Battle_endCommandSelection.call(this);
		//BattleManager.targetLineOff();
	}


	////////////////////////////////////////////////////////////////////////////////////
	// BattleManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_SkillVigilance_BattleManager_initMembers = BattleManager.initMembers;
	BattleManager.initMembers = function() {
		_JPT_SkillVigilance_BattleManager_initMembers.call(this);
		this.setReInput(JPT_SkillVigilance.INIT);
		this._subjectPre = null;
		this._checkTurn = false;
	};

	var _JPT_SkillVigilance_BattleManager_update = BattleManager.update;
	BattleManager.update = function() {
	    if (!this.isBusy() && !this.updateEvent()) {
	        switch (this._phase) {
	        case 'skillvigilance_targetlog':
	        	this.updateTargetLog();
	        	break;
	        default:
	        	_JPT_SkillVigilance_BattleManager_update.call(this);
	        	break;
	        }
	    }
	};

	var _JPT_SkillVigilance_BattleManager_startTurn = BattleManager.startTurn;
	BattleManager.startTurn = function() {
		if(this.getReInput() == JPT_SkillVigilance.SELECTED){
			//警戒入力後、敵のターンカウントさせない
			this.setReInput(JPT_SkillVigilance.INIT);

			this._checkTurn = true;
		    this._phase = 'turn';
		    this.clearActor();
		    $gameTroop.increaseTurn();
		    this.makeActionOrders();
		    $gameParty.requestMotionRefresh();
		    this._logWindow.startTurn();
		    this._checkTurn = false;
		}else{
			this._checkTurn = false;
			_JPT_SkillVigilance_BattleManager_startTurn.call(this);
		}
	};


	BattleManager.getCheckTurn = function(){
		return this._checkTurn;
	}


	//クールタイム処理
	Game_Troop.prototype.increaseTurn = function() {
	    if(!BattleManager.getCheckTurn())Yanfly.SCD.Game_Troop_increaseTurn.call(this);

	    if(!BattleManager.isCheckSelectVigilance() || BattleManager.getCheckTurn()){
		    if (Imported.YEP_BattleEngineCore) {
		      if (BattleManager.isTurnBased()) {
		        this.updateCooldowns();
		        $gameParty.updateCooldowns();
		      }
		    } else {
		      this.updateCooldowns();
		      $gameParty.updateCooldowns();
		    }
	    }
	};


	BattleManager.isCheckSelectVigilance = function() {
		return (this.useVigilanceActorIndex() != -1);
	}

	var _JPT_SkillVigilance_BattleManager_selectNextCommand = BattleManager.selectNextCommand;
	BattleManager.selectNextCommand = function() {
		if( this.getReInput() != JPT_SkillVigilance.REINPUT ){
			_JPT_SkillVigilance_BattleManager_selectNextCommand.call(this);
		}else{
            this.setReInput(JPT_SkillVigilance.SELECTED);
            this.startTurn();
		}
	};

	var _JPT_SkillVigilance_BattleManager_processTurn = BattleManager.processTurn;
	BattleManager.processTurn = function() {
		if( this.isUseVigilance() || this.getReInput()==JPT_SkillVigilance.INVOCATION ){
		    var subject = this._subject;
		    var action = subject.currentAction();
		    if (action) {
		        action.prepare();
		        if (action.isValid()) {
		            this.startAction();
		        }
		        if(action.item() && action.item().unique == JPT_SkillVigilance.UNIQUE && this.getReInput()==JPT_SkillVigilance.INIT){
		        	this.setReInput(JPT_SkillVigilance.INVOCATION);
		        }
		        subject.removeCurrentAction();
		    } else {
		        subject.onAllActionsEnd();
		        this.refreshStatus();
		        this._logWindow.displayAutoAffectedStatus(subject);
		        this._logWindow.displayCurrentState(subject);
		        this._logWindow.displayRegeneration(subject);

		        if(this.getReInput()==JPT_SkillVigilance.INVOCATION){
		        	this.makePreTarget();
		        	return;
		        }
		        this._subject = this.getNextSubject();
		    }
		}else{
			_JPT_SkillVigilance_BattleManager_processTurn.call(this);
		}
	};

	var _JPT_SkillVigilance_BattleManager_startAction = BattleManager.startAction;
	BattleManager.startAction = function() {
		_JPT_SkillVigilance_BattleManager_startAction.call(this);
		BattleManager.targetLineOff();
	};



	BattleManager.makePreTarget = function() {
		this._phase = 'skillvigilance_targetlog';
		this._battlerIndex = 0;
		this._subjectPre = null;
	}


	BattleManager.getNextSubjectPre = function() {
	    for (;;) {

	    	if(this._battlerIndex >=  this._actionBattlers.length){
	    		return null;
	    	}

	    	var battler = this._actionBattlers[this._battlerIndex];
	        this._battlerIndex++;

	        if (battler.isBattleMember() && battler.isAlive() && battler.isEnemy() ) {
	        	battler.clearActionIndex();
	            return battler;
	        }
	    }
	};


	BattleManager.updateTargetLog = function(){

		if (!this._subjectPre) {
	        this._subjectPre = this.getNextSubjectPre();
	    }
	    if (this._subjectPre) {
	        this.processTargetLog();
	    } else {
	    	this.reInput();
	    }

	}


	BattleManager.processTargetLog = function() {
	    var subject = this._subjectPre;
	    var action = subject.indexOfAction();
	    if (action) {
	        action.prepare();
	        if (action.isValid()) {
	        	action.makePreTaragets();

	        	this._logWindow.targeting(subject, action, action.preTaragets());
	        }
	        subject.nextActionIndex();
	    } else {

	    	/*
	        subject.onAllActionsEnd();
	        this.refreshStatus();
	        this._logWindow.displayAutoAffectedStatus(subject);
	        this._logWindow.displayCurrentState(subject);
	        this._logWindow.displayRegeneration(subject);
	        */

	        this._subjectPre = this.getNextSubjectPre();
	    }
	};



	BattleManager.reInput = function() {
	    this._phase = 'input';
	    this.setReInput(JPT_SkillVigilance.REINPUT);
		this._lastActorIndex = this._actorIndex;
		this._actorIndex = this.useVigilanceActorIndex();
	    var newActor = this.actor();
	    if (newActor) {
	    	newActor.makeActions();
	        newActor.setActionState('inputting');
	    }
	};

	BattleManager.setReInput = function(reinput){
		this._reinput = reinput;
	}

	BattleManager.getReInput = function(){
		return this._reinput;
	}

	BattleManager.useVigilanceActorIndex = function() {
		var no = -1;

		for(var i=0; i<$gameParty.allMembers().length; i++ ){
			var actor = $gameParty.allMembers()[i];
			if(actor.lastBattleSkill() && actor.lastBattleSkill().unique == JPT_SkillVigilance.UNIQUE){
	        	no = i;
	        	break;
	        }
		}
	    return no;
	};

	BattleManager.isUseVigilance = function(){
		var subject = this._subject;
	    var action = subject.currentAction();
	    if (action) {
	    	if(action.item() && action.item().unique == JPT_SkillVigilance.UNIQUE){
	    		return true;
	        }
	    }
	    return false;
	}


	BattleManager.targetLineOff = function(){
		$gameTroop.members().forEach( function(enemy){
			enemy.setShowTrgetLine(false);
		});
	}

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Battler
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_SkillVigilance_Game_Battler_initMembers = Game_Battler.prototype.initMembers;
	Game_Battler.prototype.initMembers = function() {
		_JPT_SkillVigilance_Game_Battler_initMembers.call(this);
		this._showTrgetLine = false;
	};

	Game_Battler.prototype.indexOfAction = function() {
		if(this._actionsIndex >=  this._actions.length){
    		return null;
    	}
	    return this._actions[this._actionsIndex];
	};

	Game_Battler.prototype.nextActionIndex = function() {
	    this._actionsIndex++;
	};

	Game_Battler.prototype.clearActionIndex = function() {
	    this._actionsIndex = 0;
	};

	Game_Battler.prototype.clearActionIndex = function() {
	    this._actionsIndex = 0;
	};

	Game_Battler.prototype.isShowTrgetLine = function() {
	    return this._showTrgetLine;
	};

	Game_Battler.prototype.setShowTrgetLine = function(show) {
	    this._showTrgetLine = show;
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Action
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_SkillVigilance_Game_Action_initialize = Game_Action.prototype.initialize;
	Game_Action.prototype.initialize = function(subject, forcing) {
		_JPT_SkillVigilance_Game_Action_initialize.call(this, subject, forcing);
		this.clearPreTaragets();
	};

	var _JPT_SkillVigilance_Game_Action_makeTargets = Game_Action.prototype.makeTargets;
	Game_Action.prototype.makeTargets = function() {
		if(this._preTartets && this._preTartets.length > 0){
			this.fixPreTaragets(); 	//事前ターゲットが行動時に有効でない場合ターゲットを修正
			return this._preTartets;
		}else{
			return _JPT_SkillVigilance_Game_Action_makeTargets.call(this);
		}
	};

	//表示用に事前にターゲットする
	Game_Action.prototype.preTaragets = function(){
		return this._preTartets;
	}

	Game_Action.prototype.makePreTaragets = function(){
		this.clearPreTaragets();
		this._preTartets = this.makeTargets();

		if(this._preTartets.length > 0){
			for(var i=0; i<this._preTartets.length; i++){
				var target = this._preTartets[i];
				this._preTartetIndexs[i] = target.index();
			}
		}

	}

	Game_Action.prototype.clearPreTaragets = function(){
		this._preTartets = [];
		this._preTartetIndexs = [];
	}

	Game_Action.prototype.fixPreTaragets = function(){
		if(this._preTartets.length > 0){
			var reset = false;
			for(var i=0; i<this._preTartets.length; i++){
				var target = this._preTartets[i];
				if(target.isDead()){
					reset = true;
				}else if(!target.isBattleMember()){
					if(target.isActor()){
						this._preTartets[i] = $gameParty.smoothTarget(this._preTartetIndexs[i]);
					}else{
						this._preTartets[i] = $gameTroop.smoothTarget(this._preTartetIndexs[i]);
					}
				}
			}
			if(reset){
				this._preTartets = _JPT_SkillVigilance_Game_Action_makeTargets.call(this);
			}
		}
	}


	////////////////////////////////////////////////////////////////////////////////////
	// Game_Troop
	////////////////////////////////////////////////////////////////////////////////////
	/*
	var _JPT_SkillVigilance_Game_Troop_increaseTurn = Game_Troop.prototype.increaseTurn;
	Game_Troop.prototype.increaseTurn = function() {
		_JPT_SkillVigilance_Game_Troop_increaseTurn.call(this);


	    if (Imported.YEP_BattleEngineCore) {
	      if (BattleManager.isTurnBased()) {
	        this.updateCooldowns();
	        $gameParty.updateCooldowns();
	      }
	    } else {
	      this.updateCooldowns();
	      $gameParty.updateCooldowns();
	    }
	};
	*/

	////////////////////////////////////////////////////////////////////////////////////
	// Window_BattleLog
	////////////////////////////////////////////////////////////////////////////////////
	//ターゲットを表示

	Window_BattleLog.prototype.targeting = function(subject, action, targets) {
	    var item = action.item();
	    this.displayTarget(subject, item, targets);
	    subject.setShowTrgetLine(true);
	};

	Window_BattleLog.prototype.displayTarget = function(subject, item, targets) {
	    var numMethods = this._methods.length;

	    if (DataManager.isSkill(item)) {

	    	var t_name = '';
	    	if(!item.vig_mess){
		    	var mess = targetDefaultText;
		    	if(targets){
			    	for(var i=0; i<targets.length; i++){
			    		if(subject.name()==targets[i].name()){
		    				t_name = targetSelf;
	    				}else{
		    				t_name = targets[i].name();
	    				}
			    		mess = mess.replace(/#O/g, subject.name());
			    		mess = mess.replace(/#T/g, t_name);
			    		this.push('addText', mess);
			    	}
		    	}
	    	}else{
	    		var mess = item.vig_mess;
	    		mess = mess.replace(/#O/g, subject.name());
	    		mess = mess.replace(/#S/g, item.name);
	    		if(targets){
		    		if(targets.length > 1){
		    			for(var i=0; i<targets.length; i++){
		    				if(subject.name()==targets[i].name()){
			    				t_name += targetSelf;
		    				}else{
			    				t_name += targets[i].name();
		    				}
		    				if(i!=targets.length-1){
		    					t_name += '、';
		    				}
		    			}
		    		}else{
	    				if(subject.name()==targets[0].name()){
		    				t_name = targetSelf;
	    				}else{
	    	    			t_name = targets[0].name();
	    				}
		    		}
	    		}else{
	    			t_name = '';
	    		}

	    		mess = mess.replace(/#T/g, t_name);
	    		this.push('addText', mess);
	    	}
	    }

	    if (this._methods.length === numMethods) {
	        this.push('wait');
	    }
	};



}() );
