//=============================================================================
// JPT_Synthesis
//=============================================================================

var Imported = Imported || {};
Imported['JPT_Synthesis'] = 1.00;

/*:ja
 * @plugindesc アイテム合成の実装
 * @author Kawamura
 *
 * @param List Name
 * @desc 完成品 項目名
 * @default 完成品
 *
 * @param Num Possession Name
 * @desc 所持数 項目名
 * @default 所持数
 *
 * @param Required Material Name
 * @desc 必要素材 項目名
 * @default 必要素材
 *
 * @param Preparation Ask Text
 * @desc 調合確認テキスト
 * @default 調合する
 *
 * @param Make Accessory Ask Text
 * @desc アクセサリ作成確認テキスト
 * @default 作成する
 *
 * @param Cancel Text
 * @desc キャンセルテキスト
 * @default キャンセル
 *
 * @param Unlock Shield Text
 * @desc 盾開放テキスト
 * @default %1が解放された
 *
 * @help
 * 合成メニューUI
 * バトル後の盾自動解放
 *
 * 依存関係
 * JPT_MainMenu.js
 *
 *-------------------------------------------
 * レシピの記述方法
 *-------------------------------------------
 * <synthesis_recipe>
 * type=preparation
 * name=ヒール丸薬
 * default=20
 * dst=20,21,22,23
 * prob=50,80,100,120
 * src=251:3,258:2
 * skill=34,36
 * </synthesis_recipe>
 *
 * ※type
 * preparation	: 調合メニューから合成
 * accessory	: アクセサリ作成メニューから合成
 * shield		: 盾解放としてバトル終了時に自動実行
 *
 * ※name
 * 合成メニューに表示される名前
 *
 * ※default
 * 合成一覧表示用のデフォルトアイテムのID
 * 設定がない場合はdstの0番目が表示
 *
 * ※dst
 * 成果物アイテムID
 * カンマ区切りで複数指定可能
 *
 * ※prob
 * 成果物を複数指定した場合の出現確率
 * カンマ区切りで選出区割りを成果物の指定数分記述
 * 成果物が1つの場合は無効
 * 1～100の間で抽選
 * 例の場合dst=20は1～50,dst=21は51～80が出れば出現
 * 100以上はスキルよって下駄をはかせた場合にのみ出現
 * スキル下駄が20だった場合に抽選が85ならば105になるのでdst=23が出現
 * 抽選結果が指定値以上になった場合は一番後ろの成果物が出現します
 *
 * ※src
 * 必要素材を アイテムID:個数 で指定
 * 複数の場合はカンマ区切りで記述
 *
 * ※skill
 * 盾開放時に習得するスキルIDをカンマ区切りで記述
 *
 *-------------------------------------------
 * 失敗アイテムの記述方法
 *-------------------------------------------
 * アイテムメモに以下を記述
 * <synthesis_garbage>
 *
 * 失敗アイテムはアイテムとして追加されません
 * 失敗アイテムは合成結果と同じデータベース項目上に登録し、それを設定してください
 * 合成結果がアイテムの場合はアイテム、防具の場合は防具
 *
 *
 *-------------------------------------------
 * モンスター追加ドロップの記述方法
 *-------------------------------------------
 * モンスターメモに以下を記述
 * <drop_item:種類ID,アイテムID,確率(1/X),ドロップ時メッセージ表示>
 *
 * 種類ID
 * 1=アイテム
 * 2=武器
 * 3=防具
 *
 *アイテムのID123, 1/16の確率,表示なし
 * <drop_item:1,123,16,false>
 *
 *
 *-------------------------------------------
 * スキルの記述方法
 *-------------------------------------------
 * <synthesis_prob_up>
 * type=preparation
 * up=20
 * </synthesis_prob_up>
 *
 * 習得、装備による付与等でパーティ内の誰かがスキルを所持していた場合
 * レシピprobの抽選結果に下駄をはかせます
 *
 * ※type
 * 下駄をはかせたい合成タイプを指定
 *
 * ※up
 * 下駄の効果値
 *
 *-------------------------------------------
 * 直接合成メニューを呼ぶ方法
 *-------------------------------------------
 *
 * プラグインコマンドで以下を指定
 *
 * 調合
 * CampMenuPreparation
 *
 * アクセサリ作成
 * CampMenuMakeAccessory
 *
 * 引数にレシピ番号を指定するとそのレシピしか表示しません
 * CampMenuPreparation 404,411
 *
 *
 */

function Scene_Preparation() {
    this.initialize.apply(this, arguments);
}

function Scene_MakeAccessory() {
    this.initialize.apply(this, arguments);
}


(function() {

	'use strict';

	var parameters = PluginManager.parameters('JPT_Synthesis');

	var listName				= parameters['List Name'] || '完成品';
	var numPossessionName		= parameters['Num Possession Name'] || '所持数';
	var requiredMaterialName	= parameters['Required Material Name'] || '必要素材';
	var preparationAskText		= parameters['Preparation Ask Text'] || '調合する';
	var makeAccessoryAskText	= parameters['Make Accessory Ask Text'] || '作成する';
	var cancelText				= parameters['Cancel Text'] || 'キャンセル';
	var unlockShieldText		= parameters['Unlock Shield Text'] || '%1が解放された';


	var synthesisIDs = null;

	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////
	Object.defineProperties(TextManager, {
		unlockShield	: { get: function() { return unlockShieldText; }, configurable: true },
	});

	////////////////////////////////////////////////////////////////////////////////////
	// DataManager
	////////////////////////////////////////////////////////////////////////////////////

	var _JPT_MainMenu_DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
	DataManager.isDatabaseLoaded = function() {

		if (!_JPT_MainMenu_DataManager_isDatabaseLoaded.call(this)) return false;

	    this.processItemsNotetagsSynthesis($dataItems);
	    this.processArmorsNotetagsSynthesis($dataArmors);
	    this.processSkillNotetagsSynthesis($dataSkills);
	    this.processEnemiesNotetagsSynthesis($dataEnemies);
		return true;
	};

	//メモからデータ取得
	DataManager.processItemsNotetagsSynthesis = function(group) {
		var noteStart = /<synthesis_recipe>/i;
		var noteEnd = /<\/synthesis_recipe>/i;

		var noteGarbage = /<synthesis_garbage>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			var mode = 'none';

			obj.recipe_type = '';
			obj.recipe_name = '';
			obj.recipe_default = 0;
			obj.recipe_dst = [];
			obj.recipe_prob = [];
			obj.recipe_src = [];
			obj.synthesis_garbage = false;
			obj.recipe_skills = [];

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteGarbage)) {
					obj.synthesis_garbage = true;
				}else if (line.match(noteStart)) {
					mode = 'synthesis_recipe';
				}else if(line.match(noteEnd)){
					break;
				}else if(mode == 'synthesis_recipe'){
					var tag = line.split(/=/)[0];
					var val = line.split(/=/)[1];

					switch(tag){
					case 'type':
						obj.recipe_type = val;
						break;
					case 'name':
						obj.recipe_name = val;
						break;
					case 'default':
						obj.recipe_default = parseInt(val,10);
						break;
					case 'dst':
						obj.recipe_dst = val.split(/,/);
						break;
					case 'prob':
						obj.recipe_prob = val.split(/,/);
						break;
					case 'src':
						obj.recipe_src = val.split(/,/);
						break;
					case 'skill':
						obj.recipe_skills = val.split(/,/);
						break;
					}
				}
			}

		}
	};

	//メモからデータ取得
	DataManager.processArmorsNotetagsSynthesis = function(group) {

		var noteGarbage = /<synthesis_garbage>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			var mode = 'none';

			obj.synthesis_garbage = false;

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteGarbage)) {
					obj.synthesis_garbage = true;
				}
			}

		}
	};


	DataManager.processSkillNotetagsSynthesis = function(group) {
		var noteStart = /<synthesis_prob_up>/i;
		var noteEnd = /<\/synthesis_prob_up>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			var mode = 'none';

			obj.synth_prob_up = 0;
			obj.synth_prob_up_type = '';

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteStart)) {
					mode = 'synthesis_prob_up';
				}else if(line.match(noteEnd)){
					break;
				}else if(mode == 'synthesis_prob_up'){
					var tag = line.split(/=/)[0];
					var val = line.split(/=/)[1];

					switch(tag){
					case 'type':
						obj.synth_prob_up_type = val;
						break;
					case 'up':
						obj.synth_prob_up = val;
						break;
					}
				}
			}

		}
	};

	DataManager.processEnemiesNotetagsSynthesis = function(group) {
		var noteDrop = /<drop_item:(.+)>/i;

		for (var n = 1; n < group.length; n++) {
			var obj = group[n];
			var notedata = obj.note.split(/[\r\n]+/);

			obj.drop_extra = [];

			for (var i = 0; i < notedata.length; i++) {
				var line = notedata[i];
				if (line.match(noteDrop)) {
					var tag = line.split(/:/)[1];
					var str = tag.split(/>/)[0];
					var val = str.split(/,/);
					if(val.length >= 4){
						var dat = {	kind: parseInt(val[0],10),
									dataId: parseInt(val[1],10),
									denominator: parseInt(val[2],10),
									view : (val[3].toLowerCase()==="true")};
						obj.drop_extra.push(dat);
					}
				}
			}

		}
	}


	DataManager.getRecipeSrcItems = function(item) {
		if(this.isItem(item)){
			if(item.recipe_src.length > 0){
				return item.recipe_src.map(function( str ) {
			        return { item : $dataItems[str.split(/:/)[0]] , rnum : str.split(/:/)[1] };
				});
			}
		}
		return null;
	};

	DataManager.getRecipeDstItems = function(item) {
		if(this.isItem(item)){
			if(item.recipe_dst.length > 0){
				switch(item.recipe_type){
				case 'preparation':
					return item.recipe_dst.map(function( id ) {
				        return $dataItems[id];
					});
				case 'accessory':
					return item.recipe_dst.map(function( id ) {
				        return $dataArmors[id];
					});
				case 'shield':
					return item.recipe_dst.map(function( id ) {
				        return $dataArmors[id];
					});
				default:
					return item.recipe_dst.map(function( id ) {
				        return $dataItems[id];
					});
				}
			}
		}
		return null;
	};

	DataManager.getRecipeDefaultItems = function(item) {
		if(this.isItem(item)){
			if(item.recipe_default>0){
				switch(item.recipe_type){
				case 'preparation':
			        return $dataItems[item.recipe_default];
				case 'accessory':
			        return $dataArmors[item.recipe_default];
				case 'shield':
			        return $dataArmors[item.recipe_default];
				default:
			        return $dataItems[item.recipe_default];
				}
			}else{
				return this.getRecipeDstItems(item)[0];
			}
		}
		return null;
	};

	DataManager.synthesisItem = function(item,num,type,max){
		max = max || 99;
		var dsts = DataManager.getRecipeDstItems(item);
		var srcs = DataManager.getRecipeSrcItems(item);

		for(var i=0;i<srcs.length;i++){
			var src = srcs[i];
			if(src.rnum * num > $gameParty.numItems(src.item)){
				return [];
			}
		}

		var base_up = DataManager.SynthesisProbUp(type);	//スキル上昇分

		var result = [];

		if( item.recipe_prob.length > 0 && dsts.length > 1 &&
			item.recipe_prob.length==	dsts.length ){
			for(var j=0; j<num; j++){
				var rand = DataManager.randInt(1,100,true) + base_up;
				var select_index = 0;
				for(var i=0;i<dsts.length;i++){
					var prob = parseInt(item.recipe_prob[i],10);
					select_index = i;
					if(prob>=rand)break;
				}
				var ret = dsts[select_index];
				var id = 'id_' + ret.id;
				if(result[id]){
					if($gameParty.numItems(ret)+result[id].cnt+1 > max)continue;
					result[id].cnt ++;
				}else{
					if($gameParty.numItems(ret)+1 > max)continue;
					result[id] = {'id':ret.id , 'cnt' : 1, 'item' : ret };
				}
			}
		}else{
			var id = 'id_' + dsts[0].id;
			result[id] = {'id':dsts[0].id , 'cnt' : num, 'item' : dsts[0] };
		}
		for (var key in result) {
			var dst = result[key];
			if(!dst.item.synthesis_garbage){
				$gameParty.gainItem(dst.item, dst.cnt);
			}
		}
		for(var i=0;i<srcs.length;i++){
			var src = srcs[i];
			$gameParty.loseItem(src.item, src.rnum * num);
		}

		if(type==='shield' && item.recipe_skills.length > 0){
			//pas_pp_tag:Naofumi のみ
			for(var i=0; i<$gameParty.allMembers().length; i++){
				var actor = $gameParty.allMembers()[i];
				if(actor.actor().pas_pp_tag==='Naofumi'){
					for(var i=0; i<item.recipe_skills.length; i++){
						var sid = item.recipe_skills[i];
						actor.learnSkill(sid);
					}
				}
			}
		}
		return result;
	};

	DataManager.randInt = function(min,max,inclusive){
		inclusive = inclusive || false;
		min = Math.ceil(min);
		max = Math.floor(max);
		var rand;
		if(inclusive){
			rand = Math.floor(Math.random() * (max - min + 1)) + min;
		}else{
			rand = Math.floor(Math.random() * (max - min)) + min;
		}
		return rand;
	};

	DataManager.SynthesisProbUp = function(type){
		var ret = 0;
		for(var i=0; i<$gameParty.allMembers().length; i++){
			var skills = $gameParty.allMembers()[i].skills();

			for(var j=0; j<skills.length; j++){
				var skill = skills[j];
				//tagが一致していれば
				if(type==skill.synth_prob_up_type){
					ret += parseInt(skill.synth_prob_up,10);
				}
			}
		}
		return ret;
	};

	DataManager.SynthesisShield = function(){
		var ret = [];

		//所持盾レシピの中で未開放のものを選出
		var recipe = $gameParty.items().filter(function( item ) {
			 if( DataManager.isItem(item) && item.itypeId === 3 && item.recipe_type==='shield'){
				 var dst = DataManager.getRecipeDefaultItems(item);
				 //return ($gameParty.numItems(dst) < 1);
				 return !$gameParty.hasItem(dst,true);
			 }
			 return false;
		});

		if(recipe.length > 0){
			for(var i=0;i<recipe.length;i++){
				var result = DataManager.synthesisItem(recipe[i],1,'shield');
				if(result){
					for (var key in result) {
						var dst = result[key];
						if(!dst.item.synthesis_garbage){
							ret.push(dst.item);
						}
					}
				}
			}
		}

		return ret;

	};

	////////////////////////////////////////////////////////////////////////////////////
    // Game_Interpreter
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_Synthesis_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
    	_JPT_Synthesis_Game_Interpreter_pluginCommand.call(this, command, args);

    	synthesisIDs = null;
    	if(command==='CampMenuPreparation'){
    		if(args[0]){
    			synthesisIDs = args[0].split(/,/);
    		}
    		SceneManager.push(Scene_Preparation);
    	}
    	if(command==='CampMenuMakeAccessory'){
    		if(args[0]){
    			synthesisIDs = args[0].split(/,/);
    		}
    		SceneManager.push(Scene_MakeAccessory);
    	}
    };

	////////////////////////////////////////////////////////////////////////////////////
    // Game_Enemy
	////////////////////////////////////////////////////////////////////////////////////
	Game_Enemy.prototype.makeExtraDropItems = function() {
	    return this.enemy().drop_extra.reduce(function(r, di) {
	        if (di.kind > 0 && Math.random() * di.denominator < this.dropItemRate()) {
	            return r.concat( {data : this.itemObject(di.kind, di.dataId), view : di.view }  );
	        } else {
	            return r;
	        }
	    }.bind(this), []);
	};

	Game_Troop.prototype.makeExtraDropItems = function() {
	    return this.deadMembers().reduce(function(r, enemy) {
	        return r.concat(enemy.makeExtraDropItems());
	    }, []);
	};

	////////////////////////////////////////////////////////////////////////////////////
    // BattleManager
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_Synthesis_BattleManager_makeRewards = BattleManager.makeRewards;
	BattleManager.makeRewards = function() {
		_JPT_Synthesis_BattleManager_makeRewards.call(this);
	    this._rewards.itemsEx = $gameTroop.makeExtraDropItems();
	};

	var _JPT_Synthesis_BattleManager_displayRewards = BattleManager.displayRewards;
	BattleManager.displayRewards = function() {
		_JPT_Synthesis_BattleManager_displayRewards.call(this);
		this.displayDropItemsEx();
	};

	BattleManager.displayDropItemsEx = function() {
	    var items = this._rewards.itemsEx;
	    if (items.length > 0) {
	        $gameMessage.newPage();
	        items.forEach(function(item) {
	        	if(item.view){
	        		$gameMessage.add(TextManager.obtainItem.format(item.data.name));
	        	}
	        });
	    }
	};

	BattleManager.gainDropItemsEx = function() {
	    var items = this._rewards.itemsEx;
	    items.forEach(function(item) {
	        $gameParty.gainItem(item.data, 1);
	    });
	};


	BattleManager.ShieldUnlock = function() {
	    var items = DataManager.SynthesisShield();
	    items.forEach(function(item) {
	    	$gameMessage.add(TextManager.unlockShield.format(item.name));
	    });
	};

	var _JPT_Synthesis_BattleManager_processVictory = BattleManager.processVictory;
	BattleManager.processVictory = function() {
		_JPT_Synthesis_BattleManager_processVictory.call(this);
		this.gainDropItemsEx();
		this.ShieldUnlock();
	};


	////////////////////////////////////////////////////////////////////////////////////
    // Scene_Synthesis
	////////////////////////////////////////////////////////////////////////////////////
	function Scene_Synthesis() {
	    this.initialize.apply(this, arguments);
	}

    Scene_Synthesis.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Synthesis.prototype.constructor = Scene_Synthesis;

    Scene_Synthesis.prototype.initialize = function() {
    	Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_Synthesis.prototype.titleID = function() {
		return 8;
	};

	Scene_Synthesis.prototype.synthesisType = function() {
		return '';
	};

	Scene_Synthesis.prototype.titleName = function() {
		return '';
	};

	Scene_Synthesis.prototype.synthesisText = function() {
		return '';
	};

    Scene_Synthesis.prototype.create = function() {
    	Scene_MenuBase.prototype.create.call(this);
    	this.createGoldWindow();
    	this.createHelpWindow();
    	this.createHeaderWindow();
    	this.createItemWindow();
    	this.createRecipeWindow();
    	this.createMenuTitle(this.titleID(),this.titleName());
    	this.createWindowFrame();
    	this.createSubWindow();
    };

    Scene_Synthesis.prototype.start = function() {
    	Scene_MenuBase.prototype.start.call(this);
    	this.createWindowFrame();
    	this._titleWindow.refresh();
	    this._goldWindow.refresh();
	    this._headerWindow.refresh();
	    this._itemWindow.activate();
	    this._itemWindow.refresh();
	    this._itemWindow.select(0);
	    this._recipeWindow.refresh();
	    this._numWindow.start();
	    this._resultWindow.start();
    };

    var _JPT_Synthesis_Scene_Synthesis_createBackground = Scene_Synthesis.prototype.createBackground;
    Scene_Synthesis.prototype.createBackground = function() {
    	_JPT_Synthesis_Scene_Synthesis_createBackground.call(this);
    	this._backgroundSprite.bitmap = ImageManager.loadBitmap("img/menus/", "menu_list_bg");
		this._backgroundSprite.x = 8;
		this._backgroundSprite.y = 72;

		this._sprBack = new Sprite();
		this.addChild(this._sprBack);

		this._sprMask = new Sprite();
		this._sprMask.bitmap = this.createMaskFrame(Graphics.boxWidth,Graphics.boxHeight,72,177,8,8,'#000000ff');
		this.addChild(this._sprMask);
    };

    Scene_Synthesis.prototype.createWindowFrame = function(){

		var x = this._headerWindow.x;
		var y = this._headerWindow.y;
		var w = Graphics.boxWidth -16;
	    var h = this._helpWindow.y - this._headerWindow.y - 8;
	    var m = 24;
	    var bitmap = this._sprMask.bitmap;
        var p = 96;
        var q = 96;
        var skin = this._helpWindow.windowskin;
        bitmap.blt(skin, p+m, 0+0, p-m*2, m, x+m, y+0, w-m*2, m);
        bitmap.blt(skin, p+m, 0+q-m, p-m*2, m, x+m, y+h-m, w-m*2, m);
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, x+0, y+m, m, h-m*2);
        bitmap.blt(skin, p+q-m, 0+m, m, p-m*2, x+w-m, y+m, m, h-m*2);
        bitmap.blt(skin, p+0, 0+0, m, m, x+0, y+0, m, m);
        bitmap.blt(skin, p+q-m, 0+0, m, m, x+w-m, y+0, m, m);
        bitmap.blt(skin, p+0, 0+q-m, m, m, x+0, y+h-m, m, m);
        bitmap.blt(skin, p+q-m, 0+q-m, m, m, x+w-m, y+h-m, m, m);

        //縦
        bitmap.blt(skin, p+0, 0+m, m, p-m*2, this._recipeWindow.x, y+5, m, this._recipeWindow.height - 10);
        bitmap.blt(skin, p+1, 0+m, 1, p-m*2, this._recipeWindow.x + 1, y+4, 3, this._recipeWindow.height - 8 );

        m = this._helpWindow._margin;
        w = w - m * 2;
        h = h - m * 2;
        bitmap = new Bitmap(w, h);
        this._sprBack.bitmap = bitmap;
        this._sprBack.setFrame(0, 0, w, h);
        this._sprBack.move(x + m, y + m);

        bitmap.blt(skin, 0, 0, p, p, 0, 0, w, h);

        for (y = 0; y < h; y += p) {
            for (x = 0; x < w; x += p) {
                bitmap.blt(skin, 0, p, p, p, x, y, p, p);
            }
        }

        var tone = this._helpWindow._colorTone;
        bitmap.adjustTone(tone[0], tone[1], tone[2]);
	}

    Scene_Synthesis.prototype.createHeaderWindow = function() {
    	this._headerWindow = new Window_Header(8, 72, Math.floor((Graphics.boxWidth -16)/2) - 22 , 40 );
	    this.addWindow(this._headerWindow);
	    this._headerWindow.setTitle(listName);
    }

    Scene_Synthesis.prototype.createItemWindow = function() {
    	var wx = this._headerWindow.x;
	    var wy = this._headerWindow.y + this._headerWindow.height;
	    var ww = this._headerWindow.width;
	    var wh = this._helpWindow.y - wy - 8;
	    this._itemWindow = new Window_SynthesisItemList(wx, wy, ww, wh);
	    this._itemWindow.setHelpWindow(this._helpWindow);
	    this._itemWindow.setHandler('ok',     this.numSelectOpen.bind(this));
	    this._itemWindow.setHandler('cancel', this.popScene.bind(this));
	    this.addWindow(this._itemWindow);
	    this._itemWindow.setCategory(this.synthesisType());
    }

    Scene_Synthesis.prototype.createRecipeWindow = function() {
    	var wx = this._headerWindow.x + this._headerWindow.width;
	    var wy = this._headerWindow.y;
	    var ww = Graphics.boxWidth -16 - this._headerWindow.width;
	    var wh = this._helpWindow.y - wy - 8;
    	this._recipeWindow = new Window_Recipe(wx, wy, ww, wh );
	    this.addWindow(this._recipeWindow);
	    this._itemWindow.setRecipeWindow(this._recipeWindow);
    }

    Scene_Synthesis.prototype.createSubWindow = function() {
    	this._numWindow = new Window_SynthNumSelect();
    	this._numWindow.setHandler('ok',     this.numSelectYesNo.bind(this));
	    this._numWindow.setHandler('cancel', this.numSelectClose.bind(this));
	    this.addWindow(this._numWindow);
	    this._numWindow.setText(this.synthesisText());
	    this._itemWindow.setNumWindow(this._numWindow);

	    this._resultWindow = new Window_SynthResult();
    	this._resultWindow.setHandler('ok',     this.resultClose.bind(this));
	    this._resultWindow.setHandler('cancel', this.resultClose.bind(this));
	    this.addWindow(this._resultWindow);
    }

    Scene_Synthesis.prototype.numSelectOpen = function(){
    	this._numWindow.show();
    	this._numWindow.activate();
    	this._itemWindow.deactivate();
    	this._numWindow.start();
    	this._numWindow.select(0);
    }

    Scene_Synthesis.prototype.numSelectClose = function(){
    	this._numWindow.hide();
    	this._numWindow.deactivate();
    	this._itemWindow.activate();
    	this._itemWindow.refresh();
	    this._recipeWindow.refresh();
    }

    Scene_Synthesis.prototype.numSelectYesNo = function(){
    	var ret = this._numWindow.synthesisItem(this.synthesisType());
    	this._numWindow.hide();
    	this._numWindow.deactivate();
    	if(ret){
    		this._resultWindow.setRetItems(ret);
	    	this._resultWindow.show();
	    	this._resultWindow.activate();
	    	this._resultWindow.start();
	    	this._resultWindow.select(0);
    	}else{
        	this._itemWindow.activate();
        	this._itemWindow.refresh();
    	    this._recipeWindow.refresh();
    	}
    }

    Scene_Synthesis.prototype.resultClose = function(){

    	this._resultWindow.hide();
    	this._resultWindow.deactivate();
    	this._itemWindow.activate();
    	this._itemWindow.refresh();
	    this._recipeWindow.refresh();
    }


    ////////////////////////////////////////////////////////////////////////////////////
    // Scene_Preparation
	////////////////////////////////////////////////////////////////////////////////////
    Scene_Preparation.prototype = Object.create(Scene_Synthesis.prototype);
    Scene_Preparation.prototype.constructor = Scene_Preparation;

    Scene_Preparation.prototype.titleID = function() {
		return 8;
	};

	Scene_Preparation.prototype.synthesisType = function() {
		return 'preparation';
	};

	Scene_Preparation.prototype.titleName = function() {
		return TextManager.preparation;
	};

	Scene_Preparation.prototype.synthesisText = function() {
		return preparationAskText;
	};

    ////////////////////////////////////////////////////////////////////////////////////
    // Scene_MakeAccessory
	////////////////////////////////////////////////////////////////////////////////////
	Scene_MakeAccessory.prototype = Object.create(Scene_Synthesis.prototype);
	Scene_MakeAccessory.prototype.constructor = Scene_MakeAccessory;

	Scene_MakeAccessory.prototype.titleID = function() {
		return 9;
	};

	Scene_MakeAccessory.prototype.synthesisType = function() {
		return 'accessory';
	};

	Scene_MakeAccessory.prototype.titleName = function() {
		return TextManager.makeAccessory;
	};

	Scene_MakeAccessory.prototype.synthesisText = function() {
		return makeAccessoryAskText;
	};

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_Header
	////////////////////////////////////////////////////////////////////////////////////
	function Window_Header() {
	    this.initialize.apply(this, arguments);
	}

    Window_Header.prototype = Object.create(Window_Base.prototype);
    Window_Header.prototype.constructor = Window_Header;

    Window_Header.prototype.initialize = function(x, y, width, height) {
	    Window_Base.prototype.initialize.call(this, x, y, width, height);
	    this.opacity = 0;
	    this._title = null;
	    this.refresh();
	};

	Window_Header.prototype.standardFontFace = function() {
		 return 'GameFont';
	};


	Window_Header.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_Header.prototype.standardPadding = function() {
	    return 0;
	};

	Window_Header.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_Header.prototype.setTitle = function(title){
		if(this._title!==title){
			this._title = title;
			this.refresh();
		}
	}

	Window_Header.prototype.refresh = function() {
    	this.contents.clear();
    	if(this._title){
    		this.contents.drawText(this._title,20,19,this.width,this.contents.fontSize,'left');
    		this.drawHorzLine(this.contents,16,22,this.width - 28,this.contents.fontSize*2);
    	}
    };

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_SynthesisItemList
	////////////////////////////////////////////////////////////////////////////////////
    function Window_SynthesisItemList() {
        this.initialize.apply(this, arguments);
    }

    Window_SynthesisItemList.prototype = Object.create(Window_MenuItemList.prototype);
    Window_SynthesisItemList.prototype.constructor = Window_SynthesisItemList;

    Window_SynthesisItemList.prototype.initialize = function(x, y, width, height) {
    	Window_MenuItemList.prototype.initialize.call(this, x, y, width, height);
    	this._recipeWindow = null;
    	this._numWindow = null;
    	this.opacity = 0;
	};

	Window_SynthesisItemList.prototype.maxCols = function() {
	    return 1;
	};

	Window_SynthesisItemList.prototype.setRecipeWindow = function(window){
		this._recipeWindow = window;
	}

	Window_SynthesisItemList.prototype.setNumWindow = function(window){
		this._numWindow = window;
	}

	Window_SynthesisItemList.prototype.includes = function(item) {
		var check = false;
		if(DataManager.isItem(item)){
			if(synthesisIDs){
				for(var i=0; i<synthesisIDs.length; i++){
					if(synthesisIDs[i]==item.id){
						check = true;
						break;
					}
				}
			}else{
				check = true;
			}
		}

		return DataManager.isItem(item) && item.itypeId === 3 && item.recipe_type===this._category && check;
	};


	Window_SynthesisItemList.prototype.drawItemNumber = function(item, x, y, width) {
		var num = $gameParty.numItems(item);
	};


	Window_SynthesisItemList.prototype.drawItemName = function(item, x, y, width) {
	    width = width || 312;
		width = 328;
	    if (item) {
	    	var icon_index = item.iconIndex;
	    	var item_name = item.name;
	    	var dst = DataManager.getRecipeDefaultItems( item );
	    	if(dst){
	    		icon_index = dst.iconIndex;
	    		if(item.recipe_name.length > 0){
	    			item_name = item.recipe_name;
	    		}else{
	    			item_name = dst.name;
	    		}
	    	}
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,icon_index, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item_name, x + iconBoxWidth, y, width - iconBoxWidth);
	    }
	};

	Window_SynthesisItemList.prototype.isEnabled = function(item) {
		var items = DataManager.getRecipeSrcItems(item);
		var reqire_item = items.filter(function(src) {
			return src.rnum <= $gameParty.numItems(src.item)
	    });

		var ditem = DataManager.getRecipeDefaultItems(item);
		var dst_item = $gameParty.numItems(ditem);

	    return (items.length==reqire_item.length) && (dst_item < 99);
	};

	Window_SynthesisItemList.prototype.updateHelp = function() {
		if(this.item()){
			this.setHelpWindowItem(DataManager.getRecipeDefaultItems( this.item() ));
		    this.setRecipeWindowItem(this.item());
		    this.setNumWindowItem(this.item());
		}
	};

	Window_SynthesisItemList.prototype.setRecipeWindowItem = function(item) {
	    if (this._recipeWindow) {
	        this._recipeWindow.setItem(item);
	    }
	};

	Window_SynthesisItemList.prototype.setNumWindowItem = function(item) {
	    if (this._numWindow) {
	        this._numWindow.setItem(item);
	    }
	};

    ////////////////////////////////////////////////////////////////////////////////////
	// Window_Recipe
	////////////////////////////////////////////////////////////////////////////////////
	function Window_Recipe() {
	    this.initialize.apply(this, arguments);
	}

	Window_Recipe.prototype = Object.create(Window_Base.prototype);
	Window_Recipe.prototype.constructor = Window_Recipe;

	Window_Recipe.prototype.initialize = function(x, y, width, height) {
		Window_Base.prototype.initialize.call(this, x, y, width, height);
		this._item = '';
	    this.opacity = 0;
	};

	Window_Recipe.prototype.standardFontSize = function() {
	    return 16;
	};

	Window_Recipe.prototype.standardPadding = function() {
	    return 0;
	};

	Window_Recipe.prototype.lineHeight = function() {
	    return 28;
	};

	Window_Recipe.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};


	Window_Recipe.prototype.setItem = function(item) {
		if(this._item !== item){
			this._item = item;
			this.refresh();
		}
	};

	Window_Recipe.prototype.refresh = function() {
    	this.contents.clear();
    	if(this._item){
    		this.drawPossessionItemNum();
    		this.drawRequiredMaterials();
    	}
    };

    Window_Recipe.prototype.drawPossessionItemNum = function(){
    	this.contents.drawText(numPossessionName,20,19,this.width,this.contents.fontSize,'left');

    	var num = $gameParty.numItems(DataManager.getRecipeDefaultItems(this._item));
    	var twidth = this.textWidth('000');
    	this.contents.drawText(num,-18,19,this.width,this.contents.fontSize,'right');

    	this.drawHorzLine(this.contents,16,22,this.width - 32,this.contents.fontSize*2);
    };

    Window_Recipe.prototype.drawRequiredMaterials = function(){
    	this.contents.drawText(requiredMaterialName,20,19 + 44,this.width,this.contents.fontSize,'left');
    	this.drawHorzLine(this.contents,16,22 + 44,this.width - 32,this.contents.fontSize*2);

    	var items = DataManager.getRecipeSrcItems(this._item);
    	for(var i=0; i<items.length; i++ ){
    		this.drawItemName(items[i],20,90+this.lineHeight()*i,this.width);
    	}
    };

    Window_Recipe.prototype.drawItemName = function(src, x, y, width) {
	    width = width || 312;
	    var item = src.item;
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        var numBlockWidth = this.textWidth('00/00');
			this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, width - 40 - iconBoxWidth - numBlockWidth -18);

	        var numWidth = this.textWidth('00');
	        var slaWidth = this.textWidth('/');
	        x = -18;
			y += 7;
	        this.contents.drawText(src.rnum,x,y,width,this.contents.fontSize,'right');
	        x -= numWidth;
	        this.contents.drawText('/',x,y,width,this.contents.fontSize,'right');
	        x -= slaWidth;
	        this.contents.drawText($gameParty.numItems(item),x,y,width,this.contents.fontSize,'right');
	    }
	};


	////////////////////////////////////////////////////////////////////////////////////
	// Window_SubBase
	////////////////////////////////////////////////////////////////////////////////////
	function Window_SubBase() {
	    this.initialize.apply(this, arguments);
	}

	Window_SubBase.prototype = Object.create(Window_Selectable.prototype);
	Window_SubBase.prototype.constructor = Window_SubBase;

	Window_SubBase.prototype.initialize = function() {
		Window_Selectable.prototype.initialize.call(this, 0, 0, 0, 0);
		this.deactivate();
		this.hide();
	};

	Window_SubBase.prototype._refreshBack = function() {
	    var m = this._margin;
	    var w = this._width - m * 2;
	    var h = this._height - m * 2;
	    var bitmap = new Bitmap(w, h);

	    this._windowBackSprite.alpha = 1;
	    this._windowBackSprite.bitmap = bitmap;
	    this._windowBackSprite.setFrame(0, 0, w, h);
	    this._windowBackSprite.move(m, m);
	    this._windowBackSprite.bitmap.gradientFillRect(0,0,w,h,'#012e32','#021113',true);
	};

	Window_SubBase.prototype.resetFontSettings = function() {
		this.contents.fontFace = this.standardFontFace();
		this.contents.fontSize = this.standardFontSize();
		this.contents.fontBold = false;
		this.contents.fontItalic = false;
		this.contents.outlineWidth = 0;
		this.contents.outlineColor = 'rgba(0, 0, 0, 0)';
	};

	Window_SubBase.prototype.updatePlacement = function() {
	    this.width = this.windowWidth();
	    this.height = this.windowHeight();
	    this.x = (Graphics.boxWidth - this.width) / 2;
        this.y = (Graphics.boxHeight - this.height) / 2;
        this.createContents();
	};

	Window_SubBase.prototype.start = function() {
		this.updatePlacement();
		this.refresh();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_SynthNumSelect
	////////////////////////////////////////////////////////////////////////////////////

	function Window_SynthNumSelect() {
	    this.initialize.apply(this, arguments);
	}

	Window_SynthNumSelect.prototype = Object.create(Window_SubBase.prototype);
	Window_SynthNumSelect.prototype.constructor = Window_SynthNumSelect;

	Window_SynthNumSelect.prototype.initialize = function() {

		this.createSprite();

		Window_SubBase.prototype.initialize.call(this);
		this._item = '';
		this._text = '';
		this._makeNum = 1;
		this._makeMax = 0;
	};

	Window_SynthNumSelect.prototype.windowWidth = function() {
	    return Graphics.boxWidth - 70*2;
	};

	Window_SynthNumSelect.prototype.windowHeight = function() {
	    return Math.floor(Graphics.boxHeight/3);
	};

	Window_SynthNumSelect.prototype.standardFontSize = function() {
	    return 18;
	};

	Window_SynthNumSelect.prototype.standardPadding = function() {
	    return 0;
	};

	Window_SynthNumSelect.prototype.textPadding = function(){
		return 32;
	};

	Window_SynthNumSelect.prototype.lineHeight = function() {
	    return 28;
	};

	Window_SynthNumSelect.prototype.maxCols = function() {
	    return 2;
	};

	Window_SynthNumSelect.prototype.maxItems = function() {
	    return 2;
	};

	Window_SynthNumSelect.prototype.setText = function(text){
		this._text = text;

	};

	Window_SynthNumSelect.prototype.createSprite = function() {
		this._sprBtnUpArrow = new Sprite_Button();
		this._sprBtnDwArrow = new Sprite_Button();
	}

	Window_SynthNumSelect.prototype.createOtherParts = function() {
		this.addChild(this._sprBtnUpArrow);
		this.addChild(this._sprBtnDwArrow);

		this._sprBtnUpArrow.setClickHandler(this.onClickUp.bind(this));
		this._sprBtnDwArrow.setClickHandler(this.onClickDown.bind(this));
	};


	Window_SynthNumSelect.prototype.start = function() {
		this._makeNum = 1;
		Window_SubBase.prototype.start.call(this);
	};

	Window_SynthNumSelect.prototype.update = function() {
		Window_SubBase.prototype.update.call(this);
	    this.processKeyPress();
	};

	Window_SynthNumSelect.prototype.changeNumber = function(amount) {
		this._makeNum+=amount;

		if(amount>0){
			if(this._makeNum > this._makeMax){
				this._makeNum = 1;
			}
		}else{
			if(this._makeNum < 1){
				this._makeNum = this._makeMax;
			}
		}

		this.refresh();
	};

	Window_SynthNumSelect.prototype.onClickUp = function() {
		this.changeNumber(1);

	};

	Window_SynthNumSelect.prototype.onClickDown = function() {
		this.changeNumber(-1);
	};

	Window_SynthNumSelect.prototype.processKeyPress = function() {
	    if (this.isOpenAndActive()) {
	        if (Input.isRepeated('up')) {
	            this.changeNumber(1);
	        }
	        if (Input.isRepeated('down')) {
	            this.changeNumber(-1);
	        }
	    }
	};

	Window_SynthNumSelect.prototype.setItem = function(item) {
		this._item = item;
		this._makeMax = this.makeMax();
		this._sprBtnUpArrow.visible = (this._makeMax > 1);
		this._sprBtnDwArrow.visible = (this._makeMax > 1);
		this.refresh();
	};

    Window_SynthNumSelect.prototype.makeMax = function(){
    	var makeMax = 0;
    	if(this._item){
    		var dsts = DataManager.getRecipeDstItems(this._item);
        	var dnum = $gameParty.numItems(dsts[0]);
    		makeMax = 99 - dnum;
    		var items = DataManager.getRecipeSrcItems(this._item);
    		for(var i=0; i<items.length; i++ ){
    			var rnum = items[i].rnum;
    			var pnum = $gameParty.numItems(items[i].item);
    			var mnum = Math.floor(pnum/rnum);
    			if(makeMax > mnum ){
    				makeMax = mnum;
    			}
    		}
    	}
    	return makeMax;
    };

	Window_SynthNumSelect.prototype.refresh = function() {
		Window_SubBase.prototype.refresh.call(this);
    	if(this._item){
    		this.drawMakeItem(40,40);
    	}
    };

    Window_SynthNumSelect.prototype.drawMakeItem = function(x, y) {

    	var nameWidth = this.width/2 + 80;

    	var item = DataManager.getRecipeDefaultItems(this._item);
	    if (item) {
	        var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, nameWidth - iconBoxWidth);
	    }

	    var crossW = this.textWidth('×');
	    x+=nameWidth;
	    this.contents.drawText('×',x ,y,crossW,this.lineHeight(),'left');

	    x+=crossW;
	    var numWidth = this.textWidth('0000');

	    this.contents.drawText(this._makeNum,x,y,numWidth,this.lineHeight(),'right');

	    this.setArrows(x + numWidth/2 + 10,y - 3,this.lineHeight() + 4);


	    this.drawHorzLine(this.contents,16, this.height/2 ,this.width - 32, 0);
	};

	Window_SynthNumSelect.prototype.setArrows = function(x,y,height) {

	    var p = 24;
	    var q = p/2;
	    var sx = 96+p;
	    var sy = 0+p;
	    this._sprBtnDwArrow.bitmap = this._windowskin;
	    this._sprBtnDwArrow.anchor.x = 0.5;
	    this._sprBtnDwArrow.anchor.y = 0.5;
	    this._sprBtnDwArrow.setFrame(sx+q, sy+q+p, p, q);
	    this._sprBtnUpArrow.bitmap = this._windowskin;
	    this._sprBtnUpArrow.anchor.x = 0.5;
	    this._sprBtnUpArrow.anchor.y = 0.5;
	    this._sprBtnUpArrow.setFrame(sx+q, sy, p, q);

	    this._sprBtnUpArrow.move(x,y);

	    this._sprBtnDwArrow.move(x,y+height);
	};

	Window_SynthNumSelect.prototype.itemRect = function(index) {
	    var rect = new Rectangle();
	    var maxCols = this.maxCols();
	    var padding = 48;
	    var boxWidth = this.width - padding*2;
	    rect.width = 200;
	    rect.height = this.itemHeight();
	    //rect.x = index % maxCols * (rect.width + this.spacing());
	    rect.x = padding + boxWidth / maxCols * index + (boxWidth / maxCols / 2 - rect.width / 2);
	    rect.y = this.height / 2 + 40;
	    return rect;
	};

	Window_SynthNumSelect.prototype.drawItem = function(index) {
		if(index < 0)return;

		var texts = [this._text,cancelText];
		var twidth = this.textWidth(texts[index]);
		var rect = this.itemRect(index);

		this.contents.drawText(texts[index],rect.x + rect.width/2 - twidth/2,rect.y,twidth,this.lineHeight(),'left');
	};

	Window_SynthNumSelect.prototype.synthesisItem = function(type){
		if(this.index()!=0)return null;
		return DataManager.synthesisItem(this._item,this._makeNum,type);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Window_SynthResult
	////////////////////////////////////////////////////////////////////////////////////

	function Window_SynthResult() {
	    this.initialize.apply(this, arguments);
	}

	Window_SynthResult.prototype = Object.create(Window_SubBase.prototype);
	Window_SynthResult.prototype.constructor = Window_SynthResult;

	Window_SynthResult.prototype.initialize = function() {

		Window_SubBase.prototype.initialize.call(this);
		this._retItems = null;
	};

	Window_SynthResult.prototype.windowWidth = function() {
	    return Graphics.boxWidth - 70*2;
	};

	Window_SynthResult.prototype.windowHeight = function() {
		var nums = 1;
		if(this._retItems)nums = this._retItems.length;

	    return this.lineHeight()*3 + this.lineHeight()*nums + 60;
	};

	Window_SynthResult.prototype.standardFontSize = function() {
	    return 18;
	};

	Window_SynthResult.prototype.standardPadding = function() {
	    return 0;
	};

	Window_SynthResult.prototype.textPadding = function(){
		return 32;
	};

	Window_SynthResult.prototype.lineHeight = function() {
	    return 28;
	};

	Window_SynthResult.prototype.maxCols = function() {
	    return 1;
	};

	Window_SynthResult.prototype.maxItems = function() {
	    return 1;
	};

	Window_SynthResult.prototype.setRetItems = function(result) {
		this._retItems = result;
		this.refresh();
	};

	Window_SynthResult.prototype.refresh = function() {
		Window_SubBase.prototype.refresh.call(this);
		if(this._retItems){
			var i=0;
			for (var key in this._retItems) {
				var ret = this._retItems[key];
				this.drawItemName(ret,40,20+this.lineHeight()*i,this.width);
				i++;
			}
		}

		this.drawHorzLine(this.contents,16, this.height - this.lineHeight()-24,this.width - 32, 0);
    };

    Window_SynthResult.prototype.drawItemName = function(result, x, y, width) {
	    width = width || 312;
	    var item = result.item;
	    if (item) {
	    	var nameWidth = this.width/2 + 80;

		    var iconBoxWidth = 20 + 4;
	        this.resetTextColor();
	        this.drawIcon20px(this.contents,item.iconIndex, x , y  + Math.floor(this.lineHeight()/2) -10 );
	        this.drawText(item.name, x + iconBoxWidth, y, nameWidth - iconBoxWidth);

		    var crossW = this.textWidth('×');
		    x+=nameWidth;
		    this.contents.drawText('×',x ,y,crossW,this.lineHeight(),'left');

		    x+=crossW;
		    var numWidth = this.textWidth('0000');

		    this.contents.drawText(result.cnt,x,y,numWidth,this.lineHeight(),'right');
	    }
	};


    Window_SynthResult.prototype.itemRect = function(index) {
	    var rect = new Rectangle();
	    var maxCols = this.maxCols();
	    var padding = 48;
	    var boxWidth = this.width - padding*2;
	    rect.width = 200;
	    rect.height = this.itemHeight();
	    rect.x = padding + boxWidth / maxCols * index + (boxWidth / maxCols / 2 - rect.width / 2);
	    rect.y = this.height - this.lineHeight() - 20;
	    return rect;
	};

	Window_SynthResult.prototype.drawItem = function(index) {
		if(index < 0)return;

		var twidth = this.textWidth('OK');
		var rect = this.itemRect(index);

		this.contents.drawText('OK',rect.x + rect.width/2 - twidth/2,rect.y,twidth,this.lineHeight(),'left');
	};


}());
