//=============================================================================
// JPT_TestLoad
//=============================================================================

var Imported = Imported || {};
Imported['JPT_TestLoad'] = 1.00;

/*:ja
 * @plugindesc 素材の先読み
 * @author Kawamura
 *
 * @param fontFaces
 * @desc 利用するフォント
 * @default GameFont,GameFont2
 *
 * @param Loading Image
 * @desc Now Loadingイメージファイル
 * @default Loading.png
 *
 * @param titleSkip
 * @type boolean
 * @on YES
 * @off NO
 * @desc タイトルスキップ
 * NO - false     YES - true
 * @default false
 *
 * @param disabaleRandomEncount
 * @type boolean
 * @on YES
 * @off NO
 * @desc ランダムエンカウント無効
 * NO - false     YES - true
 * @default false
 *
 * @param Show Made With MV
 * @type boolean
 * @on ON
 * @off OFF
 * @desc "Made with MV"のスプラッシュ画面を表示できる/できないようにします。
 * OFF - false     ON - true
 * @default true
 *
 * @param Made with MV Image
 * @desc "Made with MV"を表示する際に使用する画像
 * デフォルト: MadeWithMv
 * @default MadeWithMv
 * @require 1
 * @dir img/system/
 * @type file
 *
 * @param Show Custom Splash
 * @type boolean
 * @on ON
 * @off OFF
 * @desc "Made with MV"のスプラッシュ画面を表示できる/できないようにします。
 * OFF - false     ON - true
 * @default false
 *
 * @param Custom Image
 * @desc "Made with MV"を表示する際に使用する画像
 * デフォルト:
 * @default
 * @require 1
 * @dir img/system/
 * @type file
 *
 * @param Fade Out Time
 * @desc フェードアウトに要する時間（フレーム数）
 * デフォルト: 120
 * @default 120
 *
 * @param Fade In Time
 * @desc フェードインに要する時間（フレーム数）
 * デフォルト: 120
 * @default 120
 *
 * @param Wait Time
 * @desc フェードインからフェードアウトまでに要する時間（フレーム数）
 * デフォルト: 160
 * @default 160
 *
 * @param Mobile Build
 * @type boolean
 * @on ON
 * @off OFF
 * @desc スマホ向けにビルドするときにON
 * OFF - false     ON - true
 * @default false
 *
 * @param Mobile Build Switch ID
 * @desc スマホ向け確認スイッチID
 * デフォルト: 130
 * @default 130
 *
 * @help
 * フォントなどを複数利用する際にあらかじめキャッシュに乗るように
 * タイトル後、裏で画像を表示するシーンを経由させます
 *
 */

(function() {

	'use strict';

	function parseBoolean(str){
		return (str.toLowerCase()==="true");
	}

	var parameters = PluginManager.parameters('JPT_TestLoad');
	var fontFaces = parameters['fontFaces'] || 'GameFont';
	var titleSkip = eval( String(parameters['titleSkip'] || 'false' ) );
	var loadingImage = parameters['Loading Image'] || 'Loading.png';
	var disabaleRandomEncount = eval( String(parameters['disabaleRandomEncount'] || 'false' ) );
	var mobileBuildSwitchID = parseInt(parameters['Mobile Build Switch ID'] || 130);
	var isMobileBuild = JSON.parse(parameters["Mobile Build"]);

	var Liquidize = Liquidize || {};
	Liquidize.MadeWithMV = {};
	Liquidize.MadeWithMV.Parameters = PluginManager.parameters('JPT_TestLoad');

	Liquidize.MadeWithMV.ShowMV = JSON.parse(Liquidize.MadeWithMV.Parameters["Show Made With MV"]);
	Liquidize.MadeWithMV.MVImage = String(Liquidize.MadeWithMV.Parameters["Made with MV Image"]);
	Liquidize.MadeWithMV.ShowCustom = JSON.parse(Liquidize.MadeWithMV.Parameters["Show Custom Splash"]);
	Liquidize.MadeWithMV.CustomImage = String(Liquidize.MadeWithMV.Parameters["Custom Image"]);
	Liquidize.MadeWithMV.FadeOutTime = Number(Liquidize.MadeWithMV.Parameters["Fade Out Time"]) || 120;
	Liquidize.MadeWithMV.FadeInTime = Number(Liquidize.MadeWithMV.Parameters["Fade In Time"]) || 120;
	Liquidize.MadeWithMV.WaitTime = Number(Liquidize.MadeWithMV.Parameters["Wait Time"]) || 160;

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_Boot
	////////////////////////////////////////////////////////////////////////////////////
	Scene_Boot.prototype.start = function() {

	    Scene_Base.prototype.start.call(this);
	    SoundManager.preloadImportantSounds();
	    if (DataManager.isBattleTest()) {
	        DataManager.setupBattleTest();
	        SceneManager.goto(Scene_Battle);
	    } else if (DataManager.isEventTest()) {
	        DataManager.setupEventTest();
	        SceneManager.goto(Scene_Map);
	    } else {
	        this.checkPlayerLocation();
	        DataManager.setupNewGame();
	        SceneManager.goto(Scene_TestLoad);
	        Window_TitleCommand.initCommandPosition();
	    }
	    this.updateDocumentTitle();
	    this.printVersion();
	};

	////////////////////////////////////////////////////////////////////////////////////
	// SceneManager
	////////////////////////////////////////////////////////////////////////////////////
	/*
	SceneManager.initGraphics = function() {
	    var type = this.preferableRendererType();
	    Graphics.initialize(this._screenWidth, this._screenHeight, type);
	    Graphics.boxWidth = this._boxWidth;
	    Graphics.boxHeight = this._boxHeight;
	    Graphics.setLoadingImage('img/system/' + loadingImage );
	    if (Utils.isOptionValid('showfps')) {
	        Graphics.showFps();
	    }
	    if (type === 'webgl') {
	        this.checkWebGL();
	    }
	};
	*/

	////////////////////////////////////////////////////////////////////////////////////
	// Game_Player
	////////////////////////////////////////////////////////////////////////////////////
	var _JPT_TestLoad_Game_Player_executeEncounter = Game_Player.prototype.executeEncounter;
	Game_Player.prototype.executeEncounter = function() {
		if(disabaleRandomEncount)return false;
		return _JPT_TestLoad_Game_Player_executeEncounter.call(this);
	};

	////////////////////////////////////////////////////////////////////////////////////
	// Scene_TestLoad
	////////////////////////////////////////////////////////////////////////////////////
    function Scene_TestLoad() {
        this.initialize.apply(this, arguments);
    }

    Scene_TestLoad.prototype = Object.create(Scene_Base.prototype);
    Scene_TestLoad.prototype.constructor = Scene_TestLoad;

    Scene_TestLoad.prototype.initialize = function() {
    	Scene_Base.prototype.initialize.call(this);
        this._window = null;
    };

    Scene_TestLoad.prototype.create = function() {
    	Scene_Base.prototype.create.call(this);
        this._window = new Window_TestLoad();
        //this.addChild(this._window)
    };

    Scene_TestLoad.prototype.start = function() {
    	Scene_Base.prototype.start.call(this);
        this._window.refresh();

        if(titleSkip){
        	SceneManager.goto(Scene_Map);
        }else{
        	if ((Liquidize.MadeWithMV.ShowMV || Liquidize.MadeWithMV.ShowCustom) && !DataManager.isBattleTest() && !DataManager.isEventTest()) {
                SceneManager.goto(Scene_Splash);
            } else {
            	SceneManager.goto(Scene_Title);
            }
        }

    };

	////////////////////////////////////////////////////////////////////////////////////
	// Window_TestLoad
	////////////////////////////////////////////////////////////////////////////////////
    function Window_TestLoad() {
        this.initialize.apply(this, arguments);
    }

    Window_TestLoad.prototype = Object.create(Window_Base.prototype);
    Window_TestLoad.prototype.constructor = Window_TestLoad;

    Window_TestLoad.prototype.initialize = function() {
        var width = Graphics.boxWidth;
        var height = Graphics.boxHeight;
        Window_Base.prototype.initialize.call(this, 0, 0, width, height);
        this.refresh();
        this.activate();
    };

    Window_TestLoad.prototype.windowWidth = function() {
	    return Graphics.boxWidth;
	};

	Window_TestLoad.prototype.windowHeight = function() {
	    return Graphics.boxHeight;
	};

    Window_TestLoad.prototype.refresh = function() {
    	this.contents.clear();

    	var faces = fontFaces.split(/,/);

    	if(faces){
    		for(var i=0; i<faces.length; i++){
    	        this.drawText(faces[i],'abcd123あいうえお漢字',20,20+30*i,400,'left');
    		}
    	}
    };

    Window_TestLoad.prototype.drawText = function( fontFace, text, x, y, maxWidth, align) {
    	this.contents.fontFace = fontFace;
    	this.contents.drawText(text, x, y, maxWidth, this.lineHeight(), align);
    }

     //-----------------------------------------------------------------------------
	 // Scene_Splash
	 //
	 // This is a constructor, implementation is done in the inner scope.

	 function Scene_Splash() {
	     this.initialize.apply(this, arguments);
	 }
     //-----------------------------------------------------------------------------
     // Scene_Boot
     //
     // The scene class for dealing with the game boot.
     var _Scene_Boot_loadSystemImages = Scene_Boot.prototype.loadSystemImages;
     Scene_Boot.prototype.loadSystemImages = function() {
         _Scene_Boot_loadSystemImages.call(this);
         if (Liquidize.MadeWithMV.ShowMV) {
             ImageManager.loadSystem(Liquidize.MadeWithMV.MVImage);
         }
         if (Liquidize.MadeWithMV.ShowCustom) {
             ImageManager.loadSystem(Liquidize.MadeWithMV.CustomImage);
         }
     };


     var PrintVersionName="0000";

     Scene_Boot.prototype.printVersion = function(){
 		if(PrintVersionName==="0000")return;

		var elp = document.createElement('p');
		elp.id = 'PrintVersion';
		elp.width = window.innerWidth * 0.5;
		elp.height = 20;
		elp.style.textAlign = 'left';
		elp.style.textShadow = '1px 1px 3px #000';
		elp.style.color = 'white';
		elp.style.fontSize = '16px';
		elp.style.zIndex = 99;
		elp.style.position = 'fixed';
		elp.style.margin = 'auto';
		elp.style.top = 0;
		elp.style.left = 0;
		elp.style.width = 100 + 'px';
		elp.style.height = 20 + 'px';

		document.body.appendChild(elp);
		document.getElementById("PrintVersion").innerHTML = 'rev.' + PrintVersionName;

     }


     //-----------------------------------------------------------------------------
     // Scene_Splash
     //
     // The scene class for dealing with the splash screens.

     Scene_Splash.prototype = Object.create(Scene_Base.prototype);
     Scene_Splash.prototype.constructor = Scene_Splash;

     Scene_Splash.prototype.initialize = function() {
         Scene_Base.prototype.initialize.call(this);
         this._mvSplash = null;
         this._customSplash = null;
         this._mvWaitTime = Liquidize.MadeWithMV.WaitTime;
         this._customWaitTime = Liquidize.MadeWithMV.WaitTime;
         this._mvFadeOut = false;
         this._mvFadeIn = false;
         this._customFadeOut = false;
         this._customFadeIn = false;
         this._gotoInit = false;
     };

     Scene_Splash.prototype.create = function() {
         Scene_Base.prototype.create.call(this);
         this.createSplashes();
     };

     Scene_Splash.prototype.start = function() {
         Scene_Base.prototype.start.call(this);
         SceneManager.clearStack();
         if (this._mvSplash != null) {
             this.centerSprite(this._mvSplash);
         }
         if (this._customSplash != null) {
             this.centerSprite(this._customSplash);
         }
     };

     Scene_Splash.prototype.update = function() {
         if (Liquidize.MadeWithMV.ShowMV) {
             if (!this._mvFadeIn) {
                 this.startFadeIn(Liquidize.MadeWithMV.FadeInTime, false);
                 this._mvFadeIn = true;
             } else {
                 if (this._mvWaitTime > 0 && this._mvFadeOut == false) {
                     this._mvWaitTime--;
                 } else {
                     if (this._mvFadeOut == false) {
                         this._mvFadeOut = true;
                         this.startFadeOut(Liquidize.MadeWithMV.FadeOutTime, false);
                     }
                 }
             }
         }

         if (Liquidize.MadeWithMV.ShowCustom) {
             if (Liquidize.MadeWithMV.ShowMV && this._mvFadeOut == true) {
                 if (!this._customFadeIn && this._fadeDuration == 0) {
                     this._customSplash.opacity = 255;
                     this._customWaitTime = Liquidize.MadeWithMV.WaitTime;
                     this.startFadeIn(Liquidize.MadeWithMV.FadeInTime, false);
                     this._customFadeIn = true;
                 } else {
                     if (this._customWaitTime > 0 && this._customFadeOut == false) {
                         this._customWaitTime--;
                     } else {
                         if (this._customFadeOut == false) {
                             this._customFadeOut = true;
                             this.startFadeOut(Liquidize.MadeWithMV.FadeOutTime, false);
                         }
                     }
                 }
             } else if (!Liquidize.MadeWithMV.ShowMV) {
                 if (!this._customFadeIn) {
                     this._customSplash.opacity = 255;
                     this.startFadeIn(Liquidize.MadeWithMV.FadeInTime, false);
                     this._customFadeIn = true;
                 } else {
                     if (this._customWaitTime > 0 && this._customFadeOut == false) {
                         this._customWaitTime--;
                     } else {
                         if (this._customFadeOut == false) {
                             this._customFadeOut = true;
                             this.startFadeOut(Liquidize.MadeWithMV.FadeOutTime, false);
                         }
                     }
                 }
             }
         }

         if (Liquidize.MadeWithMV.ShowCustom) {
             if (Liquidize.MadeWithMV.ShowMV && this._mvFadeOut == true && this._customFadeOut == true) {
                 this.gotoTitleOrTest();
             } else if (!Liquidize.MadeWithMV.ShowMV && this._customFadeOut == true) {
                 this.gotoTitleOrTest();
             }
         } else {
             if (this._mvFadeOut == true) {
                 this.gotoTitleOrTest();
             }
         }

         Scene_Base.prototype.update.call(this);
     };

     Scene_Splash.prototype.createSplashes = function() {
         if (Liquidize.MadeWithMV.ShowMV) {
             this._mvSplash = new Sprite(ImageManager.loadSystem(Liquidize.MadeWithMV.MVImage));
             this.addChild(this._mvSplash);
         }
         if (Liquidize.MadeWithMV.ShowCustom) {
             this._customSplash = new Sprite(ImageManager.loadSystem(Liquidize.MadeWithMV.CustomImage));
             this._customSplash.opacity = 0;
             this.addChild(this._customSplash);
         }
     };

     Scene_Splash.prototype.centerSprite = function(sprite) {
         sprite.x = Graphics.width / 2;
         sprite.y = Graphics.height / 2;
         sprite.anchor.x = 0.5;
         sprite.anchor.y = 0.5;
     };

     Scene_Splash.prototype.gotoTitleOrTest = function() {
    	 if(this._gotoInit)return;
    	 this._gotoInit = true;
         Scene_Base.prototype.start.call(this);
         SceneManager.goto(Scene_Title);
         Window_TitleCommand.initCommandPosition();
     };

    //
	var _JPT_TestLoad_Scene_Title_commandNewGame = Scene_Title.prototype.commandNewGame;
	Scene_Title.prototype.commandNewGame = function() {
		_JPT_TestLoad_Scene_Title_commandNewGame.call(this);

		$gameSwitches.setValue( mobileBuildSwitchID, isMobileBuild );
	};



}());