//=============================================================================
// Move_anime.js
//=============================================================================
//Copyright (c) 2016 Trb
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
//
//twitter https://twitter.com/Trb_surasura
/*:
 * @plugindesc 使用者から相手へアニメーションを飛ばすプラグイン
 * @author Trb
 * 
 * @version 1.16 2016 4/19 このプラグインの機能を使った後にコモンイベントでアニメーションを再生すると動きがおかしくなる不具合を修正。
 * 			1.15 2016 4/19 暫定的に、YEP_BattleCoreと併用する時の設定を追加しました。
 * 			1.1  2016 4/15 放物線状に飛ばす時 乱数補正を掛けられるようにしました。
 * 			1.0  2016 4/11 初版 
 * 
 * 
 * @help スキルのアニメーションが使用者から相手へ向かって飛んで行くようになります。
 * 飛ばしたいスキルのメモ欄に
 * <move:a>
 * と書くと、aフレーム掛けて飛んでいきます。
 * （10フレーム掛けて飛ばしたければ <move:10> と書きます）
 * 
 * moveの設定のみだとまっすぐ直線に飛んでいきますが、放物線状に飛ばしたい場合は
 * <arc:b>
 * と書くと、bの高さの放物線の軌道で飛んでいきます。
 * （設定値が小さすぎるとあまり分からないので
 * 　初めは  <arc:100> 程度でお試し下さい）
 * 
 * ver1.1 追加
 * <deflection:c>
 * と書くと、arcの値にプラスマイナスcの乱数が加算されます。
 * (例 <arc:50> <deflection:100> と書くと実際のarc値は -50～150 の乱数になる)
 * 多段ヒットの技にこの設定を適用することで乱れ撃ちのようになります。
 * 
 * ver1.2 追加
 * <repeat:d>
 * と書くと、同じアニメーションをd回追加再生します。
 * これはYEP_BattleCoreを入れると連続攻撃時にアニメーションが1回しか表示されない問題を
 * 回避するために追加した機能であり、
 * これ単体で使うと本来の再生回数*d回再生されるようになってしまうので注意して下さい。
 * 
 */
(function () {
	var subjectX = 0;//使用者の座標
	var subjectY = 0;
	var moveDuration = 0;//何フレームで移動するか
	var arcRate = 0;//放物線で移動させる時の倍率
	var deflection = 0;
	var repeats = 1;
	var id = 0;

//各設定値の代入用関数
	setParams = function(move,arc,def,rep,animeId){
		moveDuration = move;
		arcRate = arc;
		deflection = def;
		repeats = rep;
	    id = animeId;		
	};


	var SBu = Sprite_Battler.prototype.update;
	Sprite_Battler.prototype.update = function() {
		SBu.call(this);
		//行動中のキャラの座標を取得する
		if(this._battler != null && this._battler.isActing()){
			if(!$gameSystem.isSideView() && this._actor){
				//フロントビューの場合
				subjectX = Graphics.boxWidth / 2;
				subjectY = Graphics.boxHeight * 0.8;				
			}else{
				subjectX = this.x;
				subjectY = this.y;
			}
		}
		
	};
	
	var WBsa = Window_BattleLog.prototype.startAction;
	Window_BattleLog.prototype.startAction = function(subject, action, targets) {
		var item = action.item();
		if(Number(item.meta.move)>0){//スキルのメモにムーブ値が設定されていたら代入
			setParams(Number(item.meta.move),Number(item.meta.arc) || 0,
				Number(item.meta.deflection) || 0,Number(item.meta.repeat) || 0,item.animationId);
		}else{//無かったら0
			setParams(0,0,0,0,0);
		};
		WBsa.call(this,subject, action, targets);
	};

	Window_BattleLog.prototype.showNormalAnimation = function(targets, animationId, mirror) {
		var animation = $dataAnimations[animationId];
		if (animation) {
			var delay = this.animationBaseDelay();
			var nextDelay = this.animationNextDelay();
			targets.forEach(function(target) {
				target.startAnimation(animationId, mirror, delay);
				delay += nextDelay;
				var repeat = repeats;
				while(repeat > 0 && id == animationId){
					target.startAnimation(animationId, mirror, delay);
					delay += nextDelay;
					repeat -= 1;
				}
			});
		}
	};

	var SAim = Sprite_Animation.prototype.initMembers;
	Sprite_Animation.prototype.initMembers = function() {
		SAim.call(this);
		this._x2 = 0;
		this._y2 = 0;
		this._duration2 = 0;
		this._arcRate = 0;
		this._arc = 0;
	};

	var SAs = Sprite_Animation.prototype.setup;
	Sprite_Animation.prototype.setup = function(target, animation, mirror, delay) {
		if(moveDuration > 0 && $gameParty.inBattle()){
			var t =target._battler ? target : target.parent;
			this._x2 = subjectX - t.x;
			this._y2 = subjectY - t.y;
			this._duration2 = moveDuration;
			this._arc = this._duration2;
			this._arcRate = arcRate + (Math.random() - 0.5) * deflection * 2;
			
		}
		SAs.call(this,target, animation, mirror, delay);
	};
	
	var SAup = Sprite_Animation.prototype.updatePosition;
	Sprite_Animation.prototype.updatePosition = function() {
		SAup.call(this);
		//表示位置補正の計算
		if(this._duration2 > 0){
			this._x2 -= this._x2/this._duration2;
			this._y2 -= this._y2/this._duration2;
			this.x += this._x2;
			this.y += this._y2 - Math.sin(this._duration2 / this._arc * Math.PI) * this._arcRate;
			this._duration2 -= 1;
		}
	};


	//コモンイベントでアニメーションを表示させる時、設定値をリセットする
	Game_Interpreter.prototype.command337 = function() {
		setParams(0,0,0,0,0);
		this.iterateEnemyIndex(this._params[0], function(enemy) {
			if (enemy.isAlive()) {
				enemy.startAnimation(this._params[1], false, 0);
			}
		}.bind(this));
		return true;
	};

})();